@@ -18,9 +18,13 @@ set -e
 
 function install_maven {
   (
+    local maven_version="3.9.2"
+    local local_binary="apache-maven-${maven_version}-bin.tar.gz"
+    local mirror_host="https://www.apache.org/dyn/closer.lua"
+    local url="${mirror_host}/maven/maven-3/${maven_version}/binaries/${local_binary}?action=download"
     cd /opt/
-    wget -nv https://downloads.apache.org/maven/maven-3/3.8.8/binaries/apache-maven-3.8.8-bin.tar.gz
-    tar -xvf apache-maven-3.8.8-bin.tar.gz && mv apache-maven-3.8.8 /usr/lib/maven
+    wget -nv -O ${local_binary} ${url}
+    tar -xvf ${local_binary} && mv apache-maven-${maven_version} /usr/lib/maven
   )
   echo "PATH=${PATH}:/usr/lib/maven/bin" >> $GITHUB_ENV
 }
