@@ -77,6 +77,18 @@ public class CHNativeBlock {
     return nativeBlockStats(blockAddress, columnPosition);
   }
 
+  public static native long copyBlock(long blockAddress);
+
+  public ColumnarBatch copyColumnarBatch() {
+    return new CHNativeBlock(copyBlock(blockAddress)).toColumnarBatch();
+  }
+
+  private native long nativeSlice(long blockAddress, int offset, int limit);
+
+  public long nativeSlice(int offset, int limit) {
+    return nativeSlice(blockAddress, offset, limit);
+  }
+
   public void close() {
     if (blockAddress != 0) {
       nativeClose(blockAddress);
@@ -105,4 +117,14 @@ public class CHNativeBlock {
     }
     return new ColumnarBatch(vectors, numRows);
   }
+
+  public static ColumnarBatch slice(ColumnarBatch batch, int offset, int limit) {
+    if (offset + limit > batch.numRows()) {
+      throw new GlutenException(
+          "Parameter out of bound in slice function, offset: " + offset + ", limit: " + limit);
+    }
+    CHNativeBlock block = CHNativeBlock.fromColumnarBatch(batch);
+    long blockAddress = block.nativeSlice(offset, limit);
+    return new CHNativeBlock(blockAddress).toColumnarBatch();
+  }
 }
