@@ -17,6 +17,7 @@
 package org.apache.gluten.backendsapi.clickhouse
 
 import org.apache.gluten.backendsapi.MetricsApi
+import org.apache.gluten.config.ShuffleWriterType
 import org.apache.gluten.logging.LogLevelUtil
 import org.apache.gluten.metrics._
 import org.apache.gluten.substrait.{AggregationParams, JoinParams}
@@ -188,7 +189,7 @@ class CHMetricsApi extends MetricsApi with Logging with LogLevelUtil {
   override def genFilterTransformerMetricsUpdater(
       metrics: Map[String, SQLMetric],
       extraMetrics: Seq[(String, SQLMetric)] = Seq.empty): MetricsUpdater =
-    new FilterMetricsUpdater(metrics)
+    new FilterMetricsUpdater(metrics, extraMetrics)
 
   override def genProjectTransformerMetrics(sparkContext: SparkContext): Map[String, SQLMetric] =
     Map(
@@ -206,7 +207,7 @@ class CHMetricsApi extends MetricsApi with Logging with LogLevelUtil {
   override def genProjectTransformerMetricsUpdater(
       metrics: Map[String, SQLMetric],
       extraMetrics: Seq[(String, SQLMetric)] = Seq.empty): MetricsUpdater =
-    new ProjectMetricsUpdater(metrics)
+    new ProjectMetricsUpdater(metrics, extraMetrics)
 
   override def genHashAggregateTransformerMetrics(
       sparkContext: SparkContext): Map[String, SQLMetric] =
@@ -251,7 +252,7 @@ class CHMetricsApi extends MetricsApi with Logging with LogLevelUtil {
 
   override def genColumnarShuffleExchangeMetrics(
       sparkContext: SparkContext,
-      isSort: Boolean): Map[String, SQLMetric] =
+      shuffleWriterType: ShuffleWriterType): Map[String, SQLMetric] =
     Map(
       "dataSize" -> SQLMetrics.createSizeMetric(sparkContext, "data size"),
       "bytesSpilled" -> SQLMetrics.createSizeMetric(sparkContext, "shuffle bytes spilled"),
