@@ -17,7 +17,6 @@
 package org.apache.gluten.backendsapi.clickhouse
 
 import org.apache.gluten.backendsapi.RuleApi
-import org.apache.gluten.columnarbatch.CHBatch
 import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.extension._
 import org.apache.gluten.extension.columnar._
@@ -52,6 +51,11 @@ class CHRuleApi extends RuleApi {
 }
 
 object CHRuleApi {
+
+  /**
+   * Registers Spark rules or extensions, except for Gluten's columnar rules that are supposed to be
+   * injected through [[injectLegacy]] / [[injectRas]].
+   */
   private def injectSpark(injector: SparkInjector): Unit = {
     // Inject the regular Spark rules directly.
     injector.injectQueryStagePrepRule(FallbackBroadcastHashJoinPrepQueryStage.apply)
@@ -60,29 +64,38 @@ object CHRuleApi {
       (spark, parserInterface) => new GlutenCacheFilesSqlParser(spark, parserInterface))
     injector.injectParser(
       (spark, parserInterface) => new GlutenClickhouseSqlParser(spark, parserInterface))
-    injector.injectResolutionRule(spark => new CoalesceAggregationUnion(spark))
-    injector.injectResolutionRule(spark => new CoalesceProjectionUnion(spark))
-    injector.injectResolutionRule(spark => new RewriteToDateExpresstionRule(spark))
-    injector.injectResolutionRule(spark => new RewriteDateTimestampComparisonRule(spark))
-    injector.injectResolutionRule(spark => new CollapseGetJsonObjectExpressionRule(spark))
-    injector.injectResolutionRule(spark => new RepalceFromJsonWithGetJsonObject(spark))
-    injector.injectOptimizerRule(spark => new CommonSubexpressionEliminateRule(spark))
-    injector.injectOptimizerRule(spark => new ExtendedColumnPruning(spark))
+    injector.injectResolutionRule(spark => new JoinAggregateToAggregateUnion(spark))
+    // CoalesceAggregationUnion and CoalesceProjectionUnion should follows
+    // JoinAggregateToAggregateUnion
+    injector.injectResolutionRule(spark => CoalesceAggregationUnion(spark))
+    injector.injectResolutionRule(spark => CoalesceProjectionUnion(spark))
+    injector.injectResolutionRule(spark => RewriteToDateExpresstionRule(spark))
+    injector.injectResolutionRule(spark => RewriteDateTimestampComparisonRule(spark))
+    injector.injectResolutionRule(spark => CollapseGetJsonObjectExpressionRule(spark))
+    injector.injectResolutionRule(spark => RepalceFromJsonWithGetJsonObject(spark))
+    injector.injectOptimizerRule(spark => CommonSubexpressionEliminateRule(spark))
+    injector.injectOptimizerRule(spark => AggregateIfToFilterRule(spark))
+    injector.injectOptimizerRule(spark => SimplifySumRule(spark))
+    injector.injectOptimizerRule(spark => ExtendedColumnPruning(spark))
     injector.injectOptimizerRule(spark => CHAggregateFunctionRewriteRule(spark))
     injector.injectOptimizerRule(_ => CountDistinctWithoutExpand)
     injector.injectOptimizerRule(_ => EqualToRewrite)
     injector.injectPreCBORule(spark => new CHOptimizeMetadataOnlyDeltaQuery(spark))
   }
 
+  /**
+   * Registers Gluten's columnar rules. These rules will be executed by default in Gluten for
+   * columnar query planning.
+   */
   private def injectLegacy(injector: LegacyInjector): Unit = {
     // Legacy: Pre-transform rules.
     injector.injectPreTransform(_ => RemoveTransitions)
     injector.injectPreTransform(_ => PushDownInputFileExpression.PreOffload)
-    injector.injectPreTransform(c => FallbackOnANSIMode.apply(c.session))
-    injector.injectPreTransform(c => FallbackMultiCodegens.apply(c.session))
+    injector.injectPreTransform(c => FallbackOnANSIMode(c.session))
+    injector.injectPreTransform(c => FallbackMultiCodegens(c.session))
     injector.injectPreTransform(_ => RewriteSubqueryBroadcast())
-    injector.injectPreTransform(c => FallbackBroadcastHashJoin.apply(c.session))
-    injector.injectPreTransform(c => MergeTwoPhasesHashBaseAggregate.apply(c.session))
+    injector.injectPreTransform(c => FallbackBroadcastHashJoin(c.session))
+    injector.injectPreTransform(c => MergeTwoPhasesHashBaseAggregate(c.session))
     injector.injectPreTransform(_ => WriteFilesWithBucketValue)
 
     // Legacy: The legacy transform rule.
@@ -100,7 +113,8 @@ object CHRuleApi {
     injector.injectTransform(
       c =>
         intercept(
-          HeuristicTransform.WithRewrites(validatorBuilder(c.glutenConf), rewrites, offloads)))
+          HeuristicTransform
+            .WithRewrites(validatorBuilder(new GlutenConfig(c.sqlConf)), rewrites, offloads)))
 
     // Legacy: Post-transform rules.
     injector.injectPostTransform(_ => PruneNestedColumnsInHiveTableScan)
@@ -109,42 +123,53 @@ object CHRuleApi {
     injector.injectPostTransform(_ => EnsureLocalSortRequirements)
     injector.injectPostTransform(_ => EliminateLocalSort)
     injector.injectPostTransform(_ => CollapseProjectExecTransformer)
-    injector.injectPostTransform(c => RewriteSortMergeJoinToHashJoinRule.apply(c.session))
-    injector.injectPostTransform(c => PushdownAggregatePreProjectionAheadExpand.apply(c.session))
-    injector.injectPostTransform(c => LazyAggregateExpandRule.apply(c.session))
+    injector.injectPostTransform(c => RewriteSortMergeJoinToHashJoinRule(c.session))
+    injector.injectPostTransform(c => PushdownAggregatePreProjectionAheadExpand(c.session))
+    injector.injectPostTransform(c => LazyAggregateExpandRule(c.session))
     injector.injectPostTransform(c => ConverRowNumbertWindowToAggregateRule(c.session))
     injector.injectPostTransform(
       c =>
         intercept(
-          SparkPlanRules.extendedColumnarRule(c.glutenConf.extendedColumnarTransformRules)(
-            c.session)))
-    injector.injectPostTransform(c => InsertTransitions.create(c.outputsColumnar, CHBatch))
-    injector.injectPostTransform(c => RemoveDuplicatedColumns.apply(c.session))
-    injector.injectPostTransform(c => AddPreProjectionForHashJoin.apply(c.session))
-    injector.injectPostTransform(c => ReplaceSubStringComparison.apply(c.session))
+          SparkPlanRules.extendedColumnarRule(
+            new GlutenConfig(c.sqlConf).extendedColumnarTransformRules)(c.session)))
+    injector.injectPostTransform(_ => CollectLimitTransformerRule())
+    injector.injectPostTransform(_ => CollectTailTransformerRule())
+    injector.injectPostTransform(c => InsertTransitions.create(c.outputsColumnar, CHBatchType))
+    injector.injectPostTransform(c => RemoveDuplicatedColumns(c.session))
+    injector.injectPostTransform(c => AddPreProjectionForHashJoin(c.session))
+    injector.injectPostTransform(c => ReplaceSubStringComparison(c.session))
     injector.injectPostTransform(c => EliminateDeduplicateAggregateWithAnyJoin(c.session))
+    injector.injectPostTransform(c => FlattenNestedExpressions(c.session))
 
     // Gluten columnar: Fallback policies.
     injector.injectFallbackPolicy(c => p => ExpandFallbackPolicy(c.caller.isAqe(), p))
 
     // Gluten columnar: Post rules.
     injector.injectPost(c => RemoveTopmostColumnarToRow(c.session, c.caller.isAqe()))
+    injector.injectPost(c => CHRemoveTopmostColumnarToRow(c.session, c.caller.isAqe()))
     SparkShimLoader.getSparkShims
       .getExtendedColumnarPostRules()
       .foreach(each => injector.injectPost(c => intercept(each(c.session))))
-    injector.injectPost(c => ColumnarCollapseTransformStages(c.glutenConf))
+    injector.injectPost(c => ColumnarCollapseTransformStages(new GlutenConfig(c.sqlConf)))
     injector.injectPost(
       c =>
         intercept(
-          SparkPlanRules.extendedColumnarRule(c.glutenConf.extendedColumnarPostRules)(c.session)))
+          SparkPlanRules.extendedColumnarRule(
+            new GlutenConfig(c.sqlConf).extendedColumnarPostRules)(c.session)))
     injector.injectPost(c => GlutenNoopWriterRule.apply(c.session))
 
     // Gluten columnar: Final rules.
     injector.injectFinal(c => RemoveGlutenTableCacheColumnarToRow(c.session))
-    injector.injectFinal(c => GlutenFallbackReporter(c.glutenConf, c.session))
+    injector.injectFinal(c => GlutenFallbackReporter(new GlutenConfig(c.sqlConf), c.session))
     injector.injectFinal(_ => RemoveFallbackTagRule())
   }
 
+  /**
+   * Registers Gluten's columnar rules. These rules will be executed only when RAS (relational
+   * algebra selector) is enabled by spark.gluten.ras.enabled=true.
+   *
+   * These rules are covered by CI test job spark-test-spark35-ras.
+   */
   private def injectRas(injector: RasInjector): Unit = {
     // CH backend doesn't work with RAS at the moment. Inject a rule that aborts any
     // execution calls.
@@ -189,7 +214,8 @@ object CHRuleApi {
         // case s: SerializeFromObjectExec => true
         // case d: DeserializeToObjectExec => true
         // case o: ObjectHashAggregateExec => true
-        case rddScanExec: RDDScanExec if rddScanExec.nodeName.contains("Delta Table State") => true
+//        case rddScanExec: RDDScanExec if rddScanExec.no
+        //        deName.contains("Delta Table State") => true
         case f: FileSourceScanExec if includedDeltaOperator(f) => true
         case v2CommandExec: V2CommandExec => true
         case commandResultExec: CommandResultExec => true
