@@ -17,7 +17,6 @@
 package org.apache.gluten.extension
 
 import org.apache.gluten.backendsapi.clickhouse.CHBackendSettings
-import org.apache.gluten.exception.GlutenNotSupportException
 
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.SparkSession
@@ -107,9 +106,9 @@ object CoalesceUnionUtil extends Logging {
 
   def buildAttributesMap(
       attributes: Seq[Attribute],
-      expressions: Seq[Expression]): Map[ExprId, Expression] = {
+      expressions: Seq[NamedExpression]): Map[ExprId, NamedExpression] = {
     assert(attributes.length == expressions.length)
-    val map = new mutable.HashMap[ExprId, Expression]()
+    val map = new mutable.HashMap[ExprId, NamedExpression]()
     attributes.zip(expressions).foreach {
       case (attr, expr) =>
         map.put(attr.exprId, expr)
@@ -117,19 +116,38 @@ object CoalesceUnionUtil extends Logging {
     map.toMap
   }
 
-  def replaceAttributes(e: Expression, replaceMap: Map[ExprId, Expression]): Expression = {
+  def replaceAttributes(e: Expression, replaceMap: Map[ExprId, NamedExpression]): Expression = {
     e match {
       case attr: Attribute =>
+        replaceMap.getOrElse(attr.exprId, attr)
+      case attr: OuterReference =>
         replaceMap.get(attr.exprId) match {
-          case Some(replaceAttr) => replaceAttr
-          case None =>
-            throw new GlutenNotSupportException(s"Not found attribute: $attr ${attr.qualifiedName}")
+          case Some(replaceAttr) =>
+            OuterReference.apply(replaceAttr)
+          case _ => attr
         }
+      case subquery: ScalarSubquery =>
+        val plan = replaceSubqueryAttributes(subquery.plan, replaceMap)
+        val outerAttrs = subquery.outerAttrs.map(replaceAttributes(_, replaceMap))
+        subquery.copy(plan, outerAttrs)
       case _ =>
         e.withNewChildren(e.children.map(replaceAttributes(_, replaceMap)))
     }
   }
 
+  def replaceSubqueryAttributes(
+      plan: LogicalPlan,
+      replaceMap: Map[ExprId, NamedExpression]): LogicalPlan = {
+    plan match {
+      case filter: Filter =>
+        filter.copy(
+          replaceAttributes(filter.condition, replaceMap),
+          replaceSubqueryAttributes(filter.child, replaceMap))
+      case _ =>
+        plan.withNewChildren(plan.children.map(replaceSubqueryAttributes(_, replaceMap)))
+    }
+  }
+
   // Plans and expressions are the same.
   def areStrictMatchedRelation(leftRelation: LogicalPlan, rightRelation: LogicalPlan): Boolean = {
     (leftRelation, rightRelation) match {
@@ -312,6 +330,13 @@ object CoalesceUnionUtil extends Logging {
       Project(reprojectOutputs, coalescePlan)
     }
   }
+
+  def hasListQueryInside(expression: Expression): Boolean = {
+    expression match {
+      case list: ListQuery => true
+      case _ => expression.children.exists(hasListQueryInside)
+    }
+  }
 }
 
 /*
@@ -336,12 +361,12 @@ object CoalesceUnionUtil extends Logging {
  * really slow. The rewritten query only scan `t` once, and the performance is much better.
  */
 
-class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] with Logging {
+case class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] with Logging {
 
   case class PlanAnalyzer(originalAggregate: Aggregate) extends AbstractPlanAnalyzer {
 
     protected def extractFilter(): Option[Filter] = {
-      originalAggregate.child match {
+      val filter = originalAggregate.child match {
         case filter: Filter => Some(filter)
         case project @ Project(_, filter: Filter) => Some(filter)
         case subquery: SubqueryAlias =>
@@ -357,6 +382,10 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
           }
         case _ => None
       }
+      filter match {
+        case Some(f) if CoalesceUnionUtil.hasListQueryInside(f.condition) => None
+        case _ => filter
+      }
     }
 
     // Try to make the plan simple, contain only three steps, source, filter, aggregate.
@@ -385,9 +414,8 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
 
           val newFilter = innerProject match {
             case Some(project) =>
-              val replaceMap = CoalesceUnionUtil.buildAttributesMap(
-                project.output,
-                project.projectList.map(_.asInstanceOf[Expression]))
+              val replaceMap =
+                CoalesceUnionUtil.buildAttributesMap(project.output, project.projectList)
               val newCondition = CoalesceUnionUtil.replaceAttributes(filter.condition, replaceMap)
               Filter(newCondition, sourcePlan)
             case None => filter.withNewChildren(Seq(sourcePlan))
@@ -411,9 +439,8 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
 
         val newAggregate = innerProject match {
           case Some(project) =>
-            val replaceMap = CoalesceUnionUtil.buildAttributesMap(
-              project.output,
-              project.projectList.map(_.asInstanceOf[Expression]))
+            val replaceMap =
+              CoalesceUnionUtil.buildAttributesMap(project.output, project.projectList)
             val newGroupExpressions = originalAggregate.groupingExpressions.map {
               e => CoalesceUnionUtil.replaceAttributes(e, replaceMap)
             }
@@ -479,6 +506,7 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
     }
 
     override def doValidate(): Boolean = {
+      originalAggregate.groupingExpressions.nonEmpty &&
       !hasAggregateWithFilter &&
       constructedAggregatePlan.isDefined &&
       aggregateResultMatchedGroupingKeysPositions.forall(_ >= 0) &&
@@ -506,7 +534,7 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
   override def apply(plan: LogicalPlan): LogicalPlan = {
     if (
       spark.conf
-        .get(CHBackendSettings.GLUTEN_ENABLE_COALESCE_AGGREGATION_UNION, "true")
+        .get(CHBackendSettings.GLUTEN_ENABLE_COALESCE_AGGREGATION_UNION, "false")
         .toBoolean && CoalesceUnionUtil.isResolvedPlan(plan)
     ) {
       Try {
@@ -677,7 +705,7 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
   def collectClauseStructFields(
       analyzedPlan: AnalyzedPlan,
       clauseIndex: Int,
-      attributeReplaceMap: Map[ExprId, Expression]): Seq[Expression] = {
+      attributeReplaceMap: Map[ExprId, NamedExpression]): Seq[Expression] = {
 
     val planAnalyzer = analyzedPlan.planAnalyzer.get.asInstanceOf[PlanAnalyzer]
     val aggregate = planAnalyzer.constructedAggregatePlan.get.asInstanceOf[Aggregate]
@@ -825,14 +853,18 @@ class CoalesceAggregationUnion(spark: SparkSession) extends Rule[LogicalPlan] wi
  * select array(if(d=1, named_struct('f0', a, 'f1', b, 'f2', 1), null), if(d=2, named_struct('f0',
  * a, 'f1', b, 'f2', 2), null)) as s from t where d = 1 or d = 2 ) ) where s is not null
  */
-class CoalesceProjectionUnion(spark: SparkSession) extends Rule[LogicalPlan] with Logging {
+case class CoalesceProjectionUnion(spark: SparkSession) extends Rule[LogicalPlan] with Logging {
 
   case class PlanAnalyzer(originalPlan: LogicalPlan) extends AbstractPlanAnalyzer {
     def extractFilter(): Option[Filter] = {
-      originalPlan match {
+      val filter = originalPlan match {
         case project @ Project(_, filter: Filter) => Some(filter)
         case _ => None
       }
+      filter match {
+        case Some(f) if CoalesceUnionUtil.hasListQueryInside(f.condition) => None
+        case _ => filter
+      }
     }
 
     lazy val extractedSourcePlan = {
@@ -861,14 +893,14 @@ class CoalesceProjectionUnion(spark: SparkSession) extends Rule[LogicalPlan] wit
               val replaceMap =
                 CoalesceUnionUtil.buildAttributesMap(
                   project.output,
-                  project.projectList.map(_.asInstanceOf[Expression]))
+                  project.projectList.map(_.asInstanceOf[NamedExpression]))
               val newCondition = CoalesceUnionUtil.replaceAttributes(filter.condition, replaceMap)
               Some(Filter(newCondition, source))
             case subquery @ SubqueryAlias(_, project: Project) =>
               val replaceMap =
                 CoalesceUnionUtil.buildAttributesMap(
                   project.output,
-                  project.projectList.map(_.asInstanceOf[Expression]))
+                  project.projectList.map(_.asInstanceOf[NamedExpression]))
               val newCondition = CoalesceUnionUtil.replaceAttributes(filter.condition, replaceMap)
               Some(Filter(newCondition, source))
             case _ => Some(filter)
@@ -883,11 +915,8 @@ class CoalesceProjectionUnion(spark: SparkSession) extends Rule[LogicalPlan] wit
           val originalFilter = extractFilter().get
           originalFilter.child match {
             case project: Project =>
-              None
               val replaceMap =
-                CoalesceUnionUtil.buildAttributesMap(
-                  project.output,
-                  project.projectList.map(_.asInstanceOf[Expression]))
+                CoalesceUnionUtil.buildAttributesMap(project.output, project.projectList)
               val originalProject = originalPlan.asInstanceOf[Project]
               val newProjectList =
                 originalProject.projectList
@@ -897,9 +926,7 @@ class CoalesceProjectionUnion(spark: SparkSession) extends Rule[LogicalPlan] wit
               Some(newProject)
             case subquery @ SubqueryAlias(_, project: Project) =>
               val replaceMap =
-                CoalesceUnionUtil.buildAttributesMap(
-                  project.output,
-                  project.projectList.map(_.asInstanceOf[Expression]))
+                CoalesceUnionUtil.buildAttributesMap(project.output, project.projectList)
               val originalProject = originalPlan.asInstanceOf[Project]
               val newProjectList =
                 originalProject.projectList
@@ -927,7 +954,7 @@ class CoalesceProjectionUnion(spark: SparkSession) extends Rule[LogicalPlan] wit
   override def apply(plan: LogicalPlan): LogicalPlan = {
     if (
       spark.conf
-        .get(CHBackendSettings.GLUTEN_ENABLE_COALESCE_PROJECT_UNION, "true")
+        .get(CHBackendSettings.GLUTEN_ENABLE_COALESCE_PROJECT_UNION, "false")
         .toBoolean && CoalesceUnionUtil.isResolvedPlan(plan)
     ) {
       Try {
