@@ -64,6 +64,7 @@ class BatchScanMetricsUpdater(@transient val metrics: Map[String, SQLMetric])
 
 object BatchScanMetricsUpdater {
   // in mergetree format, the processor name is `MergeTreeSelect(pool: XXX, algorithm: XXX)`
-  val INCLUDING_PROCESSORS = Array("MergeTreeSelect(pool", "SubstraitFileSource")
-  val CH_PLAN_NODE_NAME = Array("MergeTreeSelect(pool", "SubstraitFileSource")
+  val INCLUDING_PROCESSORS =
+    Array("MergeTreeSelect(pool", "SubstraitFileSource", "GlutenKafkaSource")
+  val CH_PLAN_NODE_NAME = Array("MergeTreeSelect(pool", "SubstraitFileSource", "GlutenKafkaSource")
 }
