@@ -106,6 +106,8 @@ object MetricsUtil extends Logging {
         val numNativeMetrics = metrics.metricsDataList.size()
         val relSize = relMap.values().asScala.flatMap(l => l.asScala).size
         if (numNativeMetrics == 0 || numNativeMetrics != relSize) {
+          // TODO: if `RuntimeSettings.COLLECT_METRICS` set to false, we should not log the warning
+          //       otherwise, we should raise an exception
           logWarning(
             s"Updating native metrics failed due to the wrong size of metrics data: " +
               s"$numNativeMetrics")
@@ -194,7 +196,7 @@ object MetricsUtil extends Logging {
       extraTime: SQLMetric,
       outputRows: SQLMetric,
       outputBytes: SQLMetric,
-      inputRows: SQLMetric,
+      inputRows: Seq[SQLMetric],
       inputBytes: SQLMetric,
       includingMetrics: Array[String],
       planNodeNames: Array[String]): Unit = {
@@ -207,9 +209,29 @@ object MetricsUtil extends Logging {
         if (planNodeNames.exists(processor.name.startsWith(_))) {
           outputRows += processor.outputRows
           outputBytes += processor.outputBytes
-          inputRows += processor.inputRows
+          inputRows.foreach(inputRow => inputRow += processor.inputRows)
           inputBytes += processor.inputBytes
         }
       })
   }
+
+  def updateExtraTimeMetric(
+      metricData: MetricsData,
+      extraTime: SQLMetric,
+      outputRows: SQLMetric,
+      outputBytes: SQLMetric,
+      inputRows: SQLMetric,
+      inputBytes: SQLMetric,
+      includingMetrics: Array[String],
+      planNodeNames: Array[String]): Unit = {
+    updateExtraTimeMetric(
+      metricData,
+      extraTime,
+      outputRows,
+      outputBytes,
+      Seq(inputRows),
+      inputBytes,
+      includingMetrics,
+      planNodeNames)
+  }
 }
