@@ -17,7 +17,7 @@
 package org.apache.spark.sql.execution
 
 import org.apache.gluten.execution.ColumnarToRowExecBase
-import org.apache.gluten.extension.ValidationResult
+import org.apache.gluten.execution.ValidationResult
 import org.apache.gluten.metrics.GlutenTimeMetric
 
 import org.apache.spark.{OneToOneDependency, Partition, SparkContext, TaskContext}
