@@ -18,11 +18,7 @@ package org.apache.gluten.execution
 
 import org.apache.spark.SparkConf
 
-class GlutenClickHouseDSV2Suite extends GlutenClickHouseTPCHAbstractSuite {
-
-  override protected val tablesPath: String = basePath + "/tpch-data-ch"
-  override protected val tpchQueries: String = rootPath + "queries/tpch-queries-ch"
-  override protected val queriesResults: String = rootPath + "mergetree-queries-output"
+class GlutenClickHouseDSV2Suite extends MergeTreeSuite {
 
   /** Run Gluten + ClickHouse Backend with ColumnarShuffleManager */
   override protected def sparkConf: SparkConf = {
@@ -33,8 +29,13 @@ class GlutenClickHouseDSV2Suite extends GlutenClickHouseTPCHAbstractSuite {
       .set("spark.sql.autoBroadcastJoinThreshold", "-1")
   }
 
+  final override val testCases: Seq[Int] = Seq(
+    3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22
+  )
+  setupTestCase()
+
   test("TPCH Q1") {
-    runTPCHQuery(1) {
+    customCheck(1) {
       df =>
         val scanExec = df.queryExecution.executedPlan.collect {
           case scanExec: BasicScanExecTransformer => scanExec
@@ -44,7 +45,7 @@ class GlutenClickHouseDSV2Suite extends GlutenClickHouseTPCHAbstractSuite {
   }
 
   test("TPCH Q2") {
-    runTPCHQuery(2) {
+    customCheck(2) {
       df =>
         val scanExec = df.queryExecution.executedPlan.collect {
           case scanExec: BasicScanExecTransformer => scanExec
@@ -53,105 +54,25 @@ class GlutenClickHouseDSV2Suite extends GlutenClickHouseTPCHAbstractSuite {
     }
   }
 
-  test("TPCH Q3") {
-    runTPCHQuery(3) { df => }
-  }
-
-  test("TPCH Q4") {
-    runTPCHQuery(4) { df => }
-  }
-
-  test("TPCH Q5") {
-    runTPCHQuery(5) { df => }
-  }
-
-  test("TPCH Q6") {
-    runTPCHQuery(6) { df => }
-  }
-
-  test("TPCH Q7") {
-    runTPCHQuery(7) { df => }
-  }
-
-  test("TPCH Q8") {
-    runTPCHQuery(8) { df => }
-  }
-
-  test("TPCH Q9") {
-    runTPCHQuery(9) { df => }
-  }
-
-  test("TPCH Q10") {
-    runTPCHQuery(10) { df => }
-  }
-
-  test("TPCH Q11") {
-    runTPCHQuery(11) { df => }
-  }
-
-  test("TPCH Q12") {
-    runTPCHQuery(12) { df => }
-  }
-
-  test("TPCH Q13") {
-    runTPCHQuery(13) { df => }
-  }
-
-  test("TPCH Q14") {
-    runTPCHQuery(14) { df => }
-  }
-
-  test("TPCH Q15") {
-    runTPCHQuery(15) { df => }
-  }
-
-  test("TPCH Q16") {
-    runTPCHQuery(16) { df => }
-  }
-
-  test("TPCH Q17") {
-    runTPCHQuery(17) { df => }
-  }
-
-  test("TPCH Q18") {
-    runTPCHQuery(18) { df => }
-  }
-
-  test("TPCH Q19") {
-    runTPCHQuery(19) { df => }
-  }
-
-  test("TPCH Q20") {
-    runTPCHQuery(20) { df => }
-  }
-
-  test("TPCH Q21") {
-    runTPCHQuery(21) { df => }
-  }
-
-  test("TPCH Q22") {
-    runTPCHQuery(22) { df => }
-  }
-
   test("test 'select count(1)' with empty columns to read") {
     val result = runSql("""
                           |select count(1) from lineitem
                           |""".stripMargin) { _ => }
-    assert(result(0).getLong(0) == 600572L)
+    assert(result.head.getLong(0) == 600572L)
   }
 
   test("test 'select count(*)' with empty columns to read") {
     val result = runSql("""
                           |select count(*) from lineitem
                           |""".stripMargin) { _ => }
-    assert(result(0).getLong(0) == 600572L)
+    assert(result.head.getLong(0) == 600572L)
   }
 
   test("test 'select sum(2)' with empty columns to read") {
     val result = runSql("""
                           |select sum(2) from lineitem
                           |""".stripMargin) { _ => }
-    assert(result(0).getLong(0) == 1201144L)
+    assert(result.head.getLong(0) == 1201144L)
   }
 
   test("test 'select 1' with empty columns to read") {
@@ -159,6 +80,6 @@ class GlutenClickHouseDSV2Suite extends GlutenClickHouseTPCHAbstractSuite {
                           |select 1 from lineitem limit 2
                           |""".stripMargin) { _ => }
     assert(result.size == 2)
-    assert(result(0).getInt(0) == 1 && result(1).getInt(0) == 1)
+    assert(result.head.getInt(0) == 1 && result(1).getInt(0) == 1)
   }
 }
