@@ -20,11 +20,7 @@ import org.apache.spark.SparkConf
 
 import java.io.File
 
-class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbstractSuite {
-
-  override protected val tablesPath: String = basePath + "/tpch-data-ch"
-  override protected val tpchQueries: String = rootPath + "queries/tpch-queries-ch"
-  override protected val queriesResults: String = rootPath + "mergetree-queries-output"
+class GlutenClickHouseTPCHNotNullSkipIndexSuite extends MergeTreeSuite {
 
   override protected def sparkConf: SparkConf = {
     super.sparkConf
@@ -60,7 +56,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
                  | l_comment       string not null
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_minmax'
+                 |LOCATION '$dataHome/lineitem_mergetree_minmax'
                  |TBLPROPERTIES('minmaxIndexKey'='l_receiptdate')
                  |""".stripMargin)
 
@@ -84,7 +80,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
     val marks = mergetreeScan.metrics("selectedMarks").value
     assert(marks == 1)
 
-    val directory = new File(s"$basePath/lineitem_mergetree_minmax")
+    val directory = new File(s"$dataHome/lineitem_mergetree_minmax")
     // find a folder whose name is like 48b70783-b3b8-4bf8-9c52-5261aead8e3e_0_006
     val partDir = directory.listFiles().filter(f => f.getName.length > 20).head
     assert(!partDir.listFiles().exists(p => p.getName.contains("null")))
@@ -118,7 +114,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
                  | l_comment       string not null
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_bf'
+                 |LOCATION '$dataHome/lineitem_mergetree_bf'
                  |TBLPROPERTIES('bloomfilterIndexKey'='l_orderkey')
                  |""".stripMargin)
 
@@ -142,7 +138,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
     val marks = mergetreeScan.metrics("selectedMarks").value
     assert(marks == 1)
 
-    val directory = new File(s"$basePath/lineitem_mergetree_bf")
+    val directory = new File(s"$dataHome/lineitem_mergetree_bf")
     // find a folder whose name is like 48b70783-b3b8-4bf8-9c52-5261aead8e3e_0_006
     val partDir = directory.listFiles().filter(f => f.getName.length > 20).head
     assert(!partDir.listFiles().exists(p => p.getName.contains("null")))
@@ -176,7 +172,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
                  | l_comment       string not null
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_set'
+                 |LOCATION '$dataHome/lineitem_mergetree_set'
                  |TBLPROPERTIES('setIndexKey'='l_orderkey')
                  |""".stripMargin)
 
@@ -200,7 +196,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
     val marks = mergetreeScan.metrics("selectedMarks").value
     assert(marks == 1)
 
-    val directory = new File(s"$basePath/lineitem_mergetree_set")
+    val directory = new File(s"$dataHome/lineitem_mergetree_set")
     // find a folder whose name is like 48b70783-b3b8-4bf8-9c52-5261aead8e3e_0_006
     val partDir = directory.listFiles().filter(f => f.getName.length > 20).head
     assert(!partDir.listFiles().exists(p => p.getName.contains("null")))
@@ -234,7 +230,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_minmax2'
+                 |LOCATION '$dataHome/lineitem_mergetree_minmax2'
                  |TBLPROPERTIES('minmaxIndexKey'='l_receiptdate')
                  |""".stripMargin)
 
@@ -258,7 +254,7 @@ class GlutenClickHouseTPCHNotNullSkipIndexSuite extends GlutenClickHouseTPCHAbst
     val marks = mergetreeScan.metrics("selectedMarks").value
     assert(marks == 1)
 
-    val directory = new File(s"$basePath/lineitem_mergetree_minmax2")
+    val directory = new File(s"$dataHome/lineitem_mergetree_minmax2")
     // find a folder whose name is like 48b70783-b3b8-4bf8-9c52-5261aead8e3e_0_006
     val partDir = directory.listFiles().filter(f => f.getName.length > 20).head
     assert(partDir.listFiles().exists(p => p.getName.contains("null")))
