@@ -16,11 +16,16 @@
  */
 package org.apache.gluten.execution
 
+import org.apache.gluten.backendsapi.clickhouse.CHConfig
+import org.apache.gluten.config.GlutenConfig
+import org.apache.gluten.expression.{FlattenedAnd, FlattenedOr}
+
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.{DataFrame, GlutenTestUtils, Row}
 import org.apache.spark.sql.catalyst.expressions._
+import org.apache.spark.sql.catalyst.expressions.aggregate._
 import org.apache.spark.sql.catalyst.optimizer.{ConstantFolding, NullPropagation}
-import org.apache.spark.sql.catalyst.plans.logical.{Filter, LogicalPlan, Project}
+import org.apache.spark.sql.catalyst.plans.logical.{Aggregate, Filter, LogicalPlan, Project}
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.datasources.v2.clickhouse.ClickHouseConfig
 import org.apache.spark.sql.internal.SQLConf
@@ -32,12 +37,6 @@ import java.sql.Date
 import scala.reflect.ClassTag
 
 class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerSuite {
-
-  protected val tablesPath: String = basePath + "/tpch-data"
-  protected val tpchQueries: String =
-    rootPath + "../../../../tools/gluten-it/common/src/main/resources/tpch-queries"
-  protected val queriesResults: String = rootPath + "queries-output"
-
   private var parquetPath: String = _
 
   override protected def sparkConf: SparkConf = {
@@ -57,13 +56,13 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
       .set(ClickHouseConfig.CLICKHOUSE_WORKER_ID, "1")
       .set("spark.gluten.sql.columnar.iterator", "true")
       .set("spark.gluten.sql.columnar.hashagg.enablefinal", "true")
-      .set("spark.gluten.sql.enable.native.validation", "false")
+      .set(GlutenConfig.NATIVE_VALIDATION_ENABLED.key, "false")
       .set("spark.sql.warehouse.dir", warehouse)
       .set("spark.shuffle.manager", "sort")
       .set("spark.io.compression.codec", "snappy")
       .set("spark.sql.shuffle.partitions", "5")
       .set("spark.sql.autoBroadcastJoinThreshold", "10MB")
-      .set("spark.gluten.supported.scala.udfs", "compare_substrings:compare_substrings")
+      .set(GlutenConfig.GLUTEN_SUPPORTED_SCALA_UDFS.key, "compare_substrings:compare_substrings")
   }
 
   override def beforeAll(): Unit = {
@@ -284,6 +283,7 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
           }
       }
     }
+
     def checkPlan(plan: LogicalPlan, path: String): Boolean = plan match {
       case p: Project =>
         p.projectList.exists(x => checkExpression(x, path)) || checkPlan(p.child, path)
@@ -296,10 +296,12 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
           plan.children.exists(c => checkPlan(c, path))
         }
     }
+
     def checkGetJsonObjectPath(df: DataFrame, path: String): Boolean = {
       checkPlan(df.queryExecution.analyzed, path)
     }
-    withSQLConf(("spark.gluten.sql.collapseGetJsonObject.enabled", "true")) {
+
+    withSQLConf((GlutenConfig.ENABLE_COLLAPSE_GET_JSON_OBJECT.key, "true")) {
       runQueryAndCompare(
         "select get_json_object(get_json_object(string_field1, '$.a'), '$.y') " +
           " from json_test where int_field1 = 6") {
@@ -379,6 +381,45 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
     }
   }
 
+  test("GLUTEN-8557: Optimize nested and/or") {
+    def checkFlattenedFunctions(plan: SparkPlan, functionName: String, argNum: Int): Boolean = {
+
+      def checkExpression(expr: Expression, functionName: String, argNum: Int): Boolean =
+        expr match {
+          case s: FlattenedAnd if s.name.equals(functionName) && s.children.size == argNum =>
+            true
+          case o: FlattenedOr if o.name.equals(functionName) && o.children.size == argNum =>
+            true
+          case _ => expr.children.exists(c => checkExpression(c, functionName, argNum))
+        }
+      plan match {
+        case f: FilterExecTransformer => return checkExpression(f.condition, functionName, argNum)
+        case _ => return plan.children.exists(c => checkFlattenedFunctions(c, functionName, argNum))
+      }
+      false
+    }
+    runQueryAndCompare(
+      "SELECT count(1) from json_test where int_field1 = 5 and double_field1 > 1.0" +
+        " and string_field1 is not null") {
+      x => assert(checkFlattenedFunctions(x.queryExecution.executedPlan, "and", 5))
+    }
+    runQueryAndCompare(
+      "SELECT count(1) from json_test where int_field1 = 5 or double_field1 > 1.0" +
+        " or string_field1 is not null") {
+      x => assert(checkFlattenedFunctions(x.queryExecution.executedPlan, "or", 3))
+    }
+    runQueryAndCompare(
+      "SELECT count(1) from json_test where int_field1 = 5 and double_field1 > 1.0" +
+        " and double_field1 < 10 or int_field1 = 12 or string_field1 is not null") {
+      x =>
+        assert(
+          checkFlattenedFunctions(
+            x.queryExecution.executedPlan,
+            "and",
+            3) && checkFlattenedFunctions(x.queryExecution.executedPlan, "or", 3))
+    }
+  }
+
   test("Test covar_samp") {
     runQueryAndCompare("SELECT covar_samp(double_field1, int_field1) from json_test") { _ => }
   }
@@ -517,7 +558,7 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
         | str_to_map('a,b', ',', ''),
         | str_to_map('a:c|b:c', '\\|', ':')
         |""".stripMargin
-    runQueryAndCompare(sql1, true, false)(checkGlutenOperatorMatch[ProjectExecTransformer])
+    runQueryAndCompare(sql1)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
   test("test parse_url") {
@@ -576,14 +617,12 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
         (ConstantFolding.ruleName + "," + NullPropagation.ruleName)) {
       // Test codec with 'US-ASCII'
       runQueryAndCompare(
-        "SELECT decode(encode('Spark SQL', 'US-ASCII'), 'US-ASCII')",
-        noFallBack = false
+        "SELECT decode(encode('Spark SQL', 'US-ASCII'), 'US-ASCII')"
       )(checkGlutenOperatorMatch[ProjectExecTransformer])
 
       // Test codec with 'UTF-16'
       runQueryAndCompare(
-        "SELECT decode(encode('Spark SQL', 'UTF-16'), 'UTF-16')",
-        noFallBack = false
+        "SELECT decode(encode('Spark SQL', 'UTF-16'), 'UTF-16')"
       )(checkGlutenOperatorMatch[ProjectExecTransformer])
     }
   }
@@ -599,8 +638,7 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
       SQLConf.OPTIMIZER_EXCLUDED_RULES.key ->
         (ConstantFolding.ruleName + "," + NullPropagation.ruleName)) {
       runQueryAndCompare(
-        "select cast('7.921901' as float), cast('7.921901' as double)",
-        noFallBack = false
+        "select cast('7.921901' as float), cast('7.921901' as double)"
       )(checkGlutenOperatorMatch[ProjectExecTransformer])
     }
   }
@@ -657,28 +695,26 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
   }
 
   test("test date comparision expression override") {
+    runQueryAndCompare("select * from date_table where to_date(from_unixtime(ts)) < '2019-01-02'") {
+      _ =>
+    }
     runQueryAndCompare(
-      "select * from date_table where to_date(from_unixtime(ts)) < '2019-01-02'",
-      noFallBack = true) { _ => }
+      "select * from date_table where to_date(from_unixtime(ts)) <= '2019-01-02'") { _ => }
+    runQueryAndCompare("select * from date_table where to_date(from_unixtime(ts)) > '2019-01-02'") {
+      _ =>
+    }
     runQueryAndCompare(
-      "select * from date_table where to_date(from_unixtime(ts)) <= '2019-01-02'",
-      noFallBack = true) { _ => }
+      "select * from date_table where to_date(from_unixtime(ts)) >= '2019-01-02'") { _ => }
+    runQueryAndCompare("select * from date_table where to_date(from_unixtime(ts)) = '2019-01-01'") {
+      _ =>
+    }
     runQueryAndCompare(
-      "select * from date_table where to_date(from_unixtime(ts)) > '2019-01-02'",
-      noFallBack = true) { _ => }
-    runQueryAndCompare(
-      "select * from date_table where to_date(from_unixtime(ts)) >= '2019-01-02'",
-      noFallBack = true) { _ => }
-    runQueryAndCompare(
-      "select * from date_table where to_date(from_unixtime(ts)) = '2019-01-01'",
-      noFallBack = true) { _ => }
-    runQueryAndCompare(
-      "select * from date_table where from_unixtime(ts) between '2019-01-01' and '2019-01-02'",
-      noFallBack = true) { _ => }
+      "select * from date_table where from_unixtime(ts) between '2019-01-01' and '2019-01-02'") {
+      _ =>
+    }
     runQueryAndCompare(
       "select * from date_table where from_unixtime(ts, 'yyyy-MM-dd') between" +
-        " '2019-01-01' and '2019-01-02'",
-      noFallBack = true) { _ => }
+        " '2019-01-01' and '2019-01-02'") { _ => }
   }
 
   test("test element_at function") {
@@ -688,24 +724,19 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
       // input type is array<array<int>>
       runQueryAndCompare(
         "SELECT array(array(1,2,3), array(4,5,6))[1], " +
-          "array(array(id,id+1,id+2), array(id+3,id+4,id+5)) from range(100)",
-        noFallBack = true
-      )(checkGlutenOperatorMatch[ProjectExecTransformer])
+          "array(array(id,id+1,id+2), array(id+3,id+4,id+5)) from range(100)")(
+        checkGlutenOperatorMatch[ProjectExecTransformer])
 
       // input type is array<array<string>>
       runQueryAndCompare(
         "SELECT array(array('1','2','3'), array('4','5','6'))[1], " +
           "array(array('1','2',cast(id as string)), array('4','5',cast(id as string)))[1] " +
-          "from range(100)",
-        noFallBack = true
-      )(checkGlutenOperatorMatch[ProjectExecTransformer])
+          "from range(100)")(checkGlutenOperatorMatch[ProjectExecTransformer])
 
       // input type is array<map<string, int>>
       runQueryAndCompare(
         "SELECT array(map(cast(id as string), id), map(cast(id+1 as string), id+1))[1] " +
-          "from range(100)",
-        noFallBack = true
-      )(checkGlutenOperatorMatch[ProjectExecTransformer])
+          "from range(100)")(checkGlutenOperatorMatch[ProjectExecTransformer])
     }
   }
 
@@ -722,7 +753,7 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
       }
     }
 
-    withSQLConf(("spark.gluten.sql.commonSubexpressionEliminate", "true")) {
+    withSQLConf((GlutenConfig.ENABLE_COMMON_SUBEXPRESSION_ELIMINATE.key, "true")) {
       // CSE in project
       runQueryAndCompare("select hash(id), hash(id)+1, hash(id)-1 from range(10)") {
         df => checkOperatorCount[ProjectExecTransformer](2)(df)
@@ -779,8 +810,28 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
                             |FROM (select id, cast(id as string) name from range(10))
                             |GROUP BY name
                             |""".stripMargin) {
-        df => checkOperatorCount[ProjectExecTransformer](3)(df)
+        df => checkOperatorCount[ProjectExecTransformer](4)(df)
       }
+
+      runQueryAndCompare(
+        s"""
+           |select id % 2, max(hash(id)), min(hash(id)) from range(10) group by id % 2
+           |""".stripMargin)(
+        df => {
+          df.queryExecution.optimizedPlan.collect {
+            case Aggregate(_, aggregateExpressions, _) =>
+              val result =
+                aggregateExpressions
+                  .map(a => a.asInstanceOf[Alias].child)
+                  .filter(_.isInstanceOf[AggregateExpression])
+                  .map(expr => expr.asInstanceOf[AggregateExpression].aggregateFunction)
+                  .filter(aggFunc => aggFunc.children.head.isInstanceOf[AttributeReference])
+                  .map(aggFunc => aggFunc.children.head.asInstanceOf[AttributeReference].name)
+                  .distinct
+              assertResult(1)(result.size)
+          }
+          checkOperatorCount[ProjectExecTransformer](1)(df)
+        })
     }
   }
 
@@ -807,7 +858,7 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
 
   test("avg(bigint) overflow") {
     withSQLConf(
-      "spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false",
+      GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false",
       "spark.sql.autoBroadcastJoinThreshold" -> "-1") {
       withTable("myitem") {
         sql("create table big_int(id bigint) using parquet")
@@ -855,27 +906,29 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
   }
 
   test("test issue: https://github.com/apache/incubator-gluten/issues/6561") {
-    val sql = """
-                |select
-                | map_from_arrays(
-                |   transform(map_keys(map('t1',id,'t2',id+1)), v->v),
-                |   array('a','b')) as b from range(10)
-                |""".stripMargin
+    val sql =
+      """
+        |select
+        | map_from_arrays(
+        |   transform(map_keys(map('t1',id,'t2',id+1)), v->v),
+        |   array('a','b')) as b from range(10)
+        |""".stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
   test("test function format_string") {
-    val sql = """
-                | SELECT
-                |  format_string(
-                |    'hello world %d %d %s %f',
-                |    id,
-                |    id,
-                |    CAST(id AS STRING),
-                |    CAST(id AS float)
-                |  )
-                |FROM range(10)
-                |""".stripMargin
+    val sql =
+      """
+        | SELECT
+        |  format_string(
+        |    'hello world %d %d %s %f',
+        |    id,
+        |    id,
+        |    CAST(id AS STRING),
+        |    CAST(id AS float)
+        |  )
+        |FROM range(10)
+        |""".stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
@@ -889,64 +942,69 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
   }
 
   test("test functions unix_seconds/unix_date/unix_millis/unix_micros") {
-    val sql = """
-                |SELECT
-                |  id,
-                |  unix_seconds(cast(concat('2024-09-03 17:23:1',
-                |     cast(id as string)) as timestamp)),
-                |  unix_date(cast(concat('2024-09-1', cast(id as string)) as date)),
-                |  unix_millis(cast(concat('2024-09-03 17:23:10.11',
-                |     cast(id as string)) as timestamp)),
-                |  unix_micros(cast(concat('2024-09-03 17:23:10.12345',
-                |     cast(id as string)) as timestamp))
-                |FROM range(10)
-                |""".stripMargin
+    val sql =
+      """
+        |SELECT
+        |  id,
+        |  unix_seconds(cast(concat('2024-09-03 17:23:1',
+        |     cast(id as string)) as timestamp)),
+        |  unix_date(cast(concat('2024-09-1', cast(id as string)) as date)),
+        |  unix_millis(cast(concat('2024-09-03 17:23:10.11',
+        |     cast(id as string)) as timestamp)),
+        |  unix_micros(cast(concat('2024-09-03 17:23:10.12345',
+        |     cast(id as string)) as timestamp))
+        |FROM range(10)
+        |""".stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
   test("test function arrays_zip") {
-    val sql = """
-                |SELECT arrays_zip(array(id, id+1, id+2), array(id, id-1, id-2))
-                |FROM range(10)
-                |""".stripMargin
+    val sql =
+      """
+        |SELECT arrays_zip(array(id, id+1, id+2), array(id, id-1, id-2))
+        |FROM range(10)
+        |""".stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
   test("test function timestamp_seconds/timestamp_millis/timestamp_micros") {
-    val sql = """
-                |SELECT
-                |  id,
-                |  timestamp_seconds(1725453790 + id) as ts_seconds,
-                |  timestamp_millis(1725453790123 + id) as ts_millis,
-                |  timestamp_micros(1725453790123456 + id) as ts_micros
-                |from range(10);
-                |""".stripMargin
+    val sql =
+      """
+        |SELECT
+        |  id,
+        |  timestamp_seconds(1725453790 + id) as ts_seconds,
+        |  timestamp_millis(1725453790123 + id) as ts_millis,
+        |  timestamp_micros(1725453790123456 + id) as ts_micros
+        |from range(10);
+        |""".stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
   test("GLUTEN-7426 get_json_object") {
-    val sql = """
-                |select
-                |get_json_object(a, '$.a b'),
-                |get_json_object(a, '$.a b '),
-                |get_json_object(a, '$.a b c'),
-                |get_json_object(a, '$.a 1 c'),
-                |get_json_object(a, '$.1 '),
-                |get_json_object(a, '$.1 2'),
-                |get_json_object(a, '$.1 2 c')
-                |from values('{"a b":1}'), ('{"a b ":1}'), ('{"a b c":1}')
-                |, ('{"a 1 c":1}'), ('{"1 ":1}'), ('{"1 2":1}'), ('{"1 2 c":1}')
-                |as data(a)
+    val sql =
+      """
+        |select
+        |get_json_object(a, '$.a b'),
+        |get_json_object(a, '$.a b '),
+        |get_json_object(a, '$.a b c'),
+        |get_json_object(a, '$.a 1 c'),
+        |get_json_object(a, '$.1 '),
+        |get_json_object(a, '$.1 2'),
+        |get_json_object(a, '$.1 2 c')
+        |from values('{"a b":1}'), ('{"a b ":1}'), ('{"a b c":1}')
+        |, ('{"a 1 c":1}'), ('{"1 ":1}'), ('{"1 2":1}'), ('{"1 2 c":1}')
+        |as data(a)
     """.stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
 
   test("GLUTEN-7432 get_json_object returns array") {
-    val sql = """
-                |select
-                |get_json_object(a, '$.a[*].x')
-                |from values('{"a":[{"x":1}, {"x":5}]}'), ('{"a":[{"x":1}]}')
-                |as data(a)
+    val sql =
+      """
+        |select
+        |get_json_object(a, '$.a[*].x')
+        |from values('{"a":[{"x":1}, {"x":5}]}'), ('{"a":[{"x":1}]}')
+        |as data(a)
     """.stripMargin
     runQueryAndCompare(sql)(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
@@ -988,34 +1046,45 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
       SQLConf.OPTIMIZER_EXCLUDED_RULES.key ->
         (ConstantFolding.ruleName + "," + NullPropagation.ruleName)) {
       runQueryAndCompare(
-        "select cast(' \t2570852431\n' as long), cast('25708\t52431\n' as long)",
-        noFallBack = false
+        "select cast(' \t2570852431\n' as long), cast('25708\t52431\n' as long)"
+      )(checkGlutenOperatorMatch[ProjectExecTransformer])
+    }
+  }
+
+  test("Test map_concat") {
+    withSQLConf(
+      SQLConf.OPTIMIZER_EXCLUDED_RULES.key ->
+        (ConstantFolding.ruleName + "," + NullPropagation.ruleName)) {
+      runQueryAndCompare(
+        "select map_concat(map(1, 'a', 2, 'b'), map(3, null)), map_concat()"
       )(checkGlutenOperatorMatch[ProjectExecTransformer])
     }
   }
 
   test("Test transform_keys/transform_values") {
-    val sql = """
-                |select id, sort_array(map_entries(m1)), sort_array(map_entries(m2)) from(
-                |select id, first(m1) as m1, first(m2) as m2 from(
-                |select
-                |  id,
-                |  transform_keys(map_from_arrays(array(id+1, id+2, id+3),
-                |    array(1, id+2, 3)), (k, v) -> k + 1) as m1,
-                |  transform_values(map_from_arrays(array(id+1, id+2, id+3),
-                |    array(1, id+2, 3)), (k, v) -> v + 1) as m2
-                |from range(10)
-                |) group by id
-                |) order by id
-                |""".stripMargin
+    val sql =
+      """
+        |select id, sort_array(map_entries(m1)), sort_array(map_entries(m2)) from(
+        |select id, first(m1) as m1, first(m2) as m2 from(
+        |select
+        |  id,
+        |  transform_keys(map_from_arrays(array(id+1, id+2, id+3),
+        |    array(1, id+2, 3)), (k, v) -> k + 1) as m1,
+        |  transform_values(map_from_arrays(array(id+1, id+2, id+3),
+        |    array(1, id+2, 3)), (k, v) -> v + 1) as m2
+        |from range(10)
+        |) group by id
+        |) order by id
+        |""".stripMargin
 
     def checkProjects(df: DataFrame): Unit = {
       val projects = collectWithSubqueries(df.queryExecution.executedPlan) {
         case e: ProjectExecTransformer => e
       }
-      assert(projects.size >= 1)
+      assert(projects.nonEmpty)
     }
-    compareResultsAgainstVanillaSpark(sql, true, checkProjects, false)
+
+    compareResultsAgainstVanillaSpark(sql, compareResult = true, checkProjects, noFallBack = false)
   }
 
   test("GLUTEN-8406 replace from_json with get_json_object") {
@@ -1225,7 +1294,7 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
               case project: ProjectExecTransformer if isRewriteSubstringCompareProject(project) =>
                 project
             }
-            assert(projects.length == 0)
+            assert(projects.isEmpty)
         }
       )
     }
@@ -1259,4 +1328,56 @@ class GlutenFunctionValidateSuite extends GlutenClickHouseWholeStageTransformerS
     val sql2 = "select array_sort(array(id+1, null, id+2)) from range(10)"
     compareResultsAgainstVanillaSpark(sql2, true, { _ => })
   }
+
+  test("Test SimplifySumRule for sum simplification") {
+    val sql = "select sum(id / 3), sum(id * 7), sum(7 * id) from range(10)"
+
+    def checkSimplifiedSum(df: DataFrame): Unit = {
+      val projects = collectWithSubqueries(df.queryExecution.executedPlan) {
+        case project: ProjectExecTransformer
+            if project.child.isInstanceOf[CHHashAggregateExecTransformer] =>
+          project
+      }
+
+      assert(projects.size == 1)
+      assert(projects.head.projectList.size == 3)
+      assert(projects.head.projectList.head.asInstanceOf[Alias].child.isInstanceOf[Divide])
+      assert(projects.head.projectList(1).asInstanceOf[Alias].child.isInstanceOf[Multiply])
+      assert(projects.head.projectList(2).asInstanceOf[Alias].child.isInstanceOf[Multiply])
+    }
+
+    withSQLConf((CHConfig.runtimeConfig("enable_simplify_sum"), "true")) {
+      compareResultsAgainstVanillaSpark(sql, compareResult = true, checkSimplifiedSum)
+    }
+  }
+
+  ignore("Test rewrite aggregate if to aggregate with filter") {
+    val sql = "select sum(if(id % 2=0, id, null)), count(if(id % 2 = 0, 1, null)), " +
+      "avg(if(id % 4 = 0, id, null)), sum(if(id % 3 = 0, id, 0)) from range(10)"
+
+    def checkAggregateWithFilter(df: DataFrame): Unit = {
+      val aggregates = collectWithSubqueries(df.queryExecution.executedPlan) {
+        case agg: CHHashAggregateExecTransformer if agg.modes.contains(Partial) => agg
+      }
+
+      assert(aggregates.nonEmpty, "No aggregate operations found in the execution plan")
+      aggregates.foreach {
+        agg =>
+          agg.aggregateExpressions.foreach {
+            expr =>
+              assert(expr.isInstanceOf[AggregateExpression], "AggregateExpression should be used")
+              assert(expr.filter.isDefined, "AggregateExpression filter should not be None")
+          }
+      }
+    }
+
+    withSQLConf((CHConfig.runtimeConfig("enable_aggregate_if_to_filter"), "true")) {
+      compareResultsAgainstVanillaSpark(sql, compareResult = true, checkAggregateWithFilter)
+    }
+  }
+
+  test("Test map with nullable key") {
+    val sql = "select map(string_field1, int_field1) from json_test where string_field1 is not null"
+    compareResultsAgainstVanillaSpark(sql, true, { _ => })
+  }
 }
