@@ -17,7 +17,7 @@
 package org.apache.gluten.execution.compatibility
 
 import org.apache.gluten.config.GlutenConfig
-import org.apache.gluten.execution.{GlutenClickHouseTPCHAbstractSuite, ProjectExecTransformer}
+import org.apache.gluten.execution.{ParquetSuite, ProjectExecTransformer}
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.catalyst.optimizer.{ConstantFolding, NullPropagation}
@@ -27,17 +27,7 @@ import org.apache.spark.sql.internal.SQLConf
 // Some sqls' line length exceeds 100
 // scalastyle:off line.size.limit
 
-class GlutenClickhouseFunctionSuite extends GlutenClickHouseTPCHAbstractSuite {
-  override protected val needCopyParquetToTablePath = true
-
-  override protected val tablesPath: String = basePath + "/tpch-data"
-  override protected val tpchQueries: String =
-    rootPath + "../../../../tools/gluten-it/common/src/main/resources/tpch-queries"
-  override protected val queriesResults: String = rootPath + "queries-output"
-
-  override protected def createTPCHNotNullTables(): Unit = {
-    createNotNullTPCHTablesInParquet(tablesPath)
-  }
+class GlutenClickhouseFunctionSuite extends ParquetSuite {
 
   override protected def sparkConf: SparkConf = {
     new SparkConf()
@@ -57,7 +47,7 @@ class GlutenClickhouseFunctionSuite extends GlutenClickHouseTPCHAbstractSuite {
       .set(ClickHouseConfig.CLICKHOUSE_WORKER_ID, "1")
       .set("spark.gluten.sql.columnar.iterator", "true")
       .set("spark.gluten.sql.columnar.hashagg.enablefinal", "true")
-      .set("spark.gluten.sql.enable.native.validation", "false")
+      .set(GlutenConfig.NATIVE_VALIDATION_ENABLED.key, "false")
       // TODO: support default ANSI policy
       .set("spark.sql.storeAssignmentPolicy", "legacy")
       .set("spark.sql.warehouse.dir", warehouse)
@@ -187,8 +177,8 @@ class GlutenClickhouseFunctionSuite extends GlutenClickHouseTPCHAbstractSuite {
   }
 
   test("array decimal32 CH column to row") {
-    compareResultsAgainstVanillaSpark("SELECT array(1.0, 2.0)", true, { _ => }, false)
-    compareResultsAgainstVanillaSpark("SELECT map(1.0, '2', 3.0, '4')", true, { _ => }, false)
+    compareResultsAgainstVanillaSpark("SELECT array(1.0, 2.0)", true, { _ => })
+    compareResultsAgainstVanillaSpark("SELECT map(1.0, '2', 3.0, '4')", true, { _ => })
   }
 
   test("array decimal32 spark row to CH column") {
@@ -277,13 +267,8 @@ class GlutenClickhouseFunctionSuite extends GlutenClickHouseTPCHAbstractSuite {
         "spark.sql.optimizer.excludedRules",
         "org.apache.spark.sql.catalyst.optimizer.ConstantFolding," +
           "org.apache.spark.sql.catalyst.optimizer.NullPropagation")) {
-      runQueryAndCompare(
-        """
-          |select cast(map(1,'2') as string)
-          |""".stripMargin,
-        true,
-        false
-      )(checkGlutenOperatorMatch[ProjectExecTransformer])
+      runQueryAndCompare("select cast(map(1,'2') as string)")(
+        checkGlutenOperatorMatch[ProjectExecTransformer])
     }
   }
 
@@ -443,7 +428,7 @@ class GlutenClickhouseFunctionSuite extends GlutenClickHouseTPCHAbstractSuite {
         { _ => }
       )
       val q = "select cast(a as string) from (select array('123',NULL) as a)"
-      compareResultsAgainstVanillaSpark(q, true, { _ => }, false)
+      compareResultsAgainstVanillaSpark(q, true, { _ => })
     }
   }
 
@@ -505,8 +490,7 @@ class GlutenClickhouseFunctionSuite extends GlutenClickHouseTPCHAbstractSuite {
             |cast(map(array(1), map("aa", "123\'")) as string),
             |cast(named_struct("a", "test\'", "b", 1) as string),
             |cast(named_struct("a", "test\'", "b", 1, "c", struct("\'test"), "d", array('123\'')) as string)
-            |""".stripMargin,
-          noFallBack = false
+            |""".stripMargin
         )(checkGlutenOperatorMatch[ProjectExecTransformer])
       }
     }
