@@ -16,13 +16,13 @@
  */
 package org.apache.gluten.execution.hive
 
+import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.execution.{FileSourceScanExecTransformer, GlutenClickHouseWholeStageTransformerSuite, ProjectExecTransformer, TransformSupport}
 import org.apache.gluten.test.AllDataTypesWithComplexType
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.{DataFrame, SaveMode}
 import org.apache.spark.sql.delta.DeltaLog
-import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.execution.datasources.v2.clickhouse.ClickHouseConfig
 import org.apache.spark.sql.hive.HiveTableScanExecTransformer
 import org.apache.spark.sql.internal.SQLConf
@@ -36,8 +36,7 @@ import scala.reflect.ClassTag
 
 class GlutenClickHouseHiveTableSuite
   extends GlutenClickHouseWholeStageTransformerSuite
-  with ReCreateHiveSession
-  with AdaptiveSparkPlanHelper {
+  with ReCreateHiveSession {
 
   override protected def sparkConf: SparkConf = {
     import org.apache.gluten.backendsapi.clickhouse.CHConfig._
@@ -56,13 +55,13 @@ class GlutenClickHouseHiveTableSuite
       .set(ClickHouseConfig.CLICKHOUSE_WORKER_ID, "1")
       .set("spark.gluten.sql.columnar.iterator", "true")
       .set("spark.gluten.sql.columnar.hashagg.enablefinal", "true")
-      .set("spark.gluten.sql.enable.native.validation", "false")
-      .set("spark.gluten.sql.parquet.maxmin.index", "true")
+      .set(GlutenConfig.NATIVE_VALIDATION_ENABLED.key, "false")
+      .set(GlutenConfig.ENABLE_PARQUET_ROW_GROUP_MAX_MIN_INDEX.key, "true")
       .set(
         "spark.sql.warehouse.dir",
         this.getClass.getResource("/").getPath + "tests-working-home/spark-warehouse")
       .set("spark.hive.exec.dynamic.partition.mode", "nonstrict")
-      .set("spark.gluten.supported.hive.udfs", "my_add")
+      .set(GlutenConfig.GLUTEN_SUPPORTED_HIVE_UDFS.key, "my_add")
       .setCHConfig("use_local_format", true)
       .set("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension")
       .set(
@@ -849,7 +848,7 @@ class GlutenClickHouseHiveTableSuite
   }
 
   test("fix reading string from number bug: https://github.com/oap-project/gluten/issues/3023") {
-    val data_path = rootPath + "/text-data/json-settings"
+    val data_path = resPath + "/text-data/json-settings"
     spark.sql(s"""
                  |CREATE TABLE json_settings (
                  |  a string,
@@ -915,7 +914,7 @@ class GlutenClickHouseHiveTableSuite
   }
 
   test("GLUTEN-3337: fix get_json_object for abnormal json") {
-    val data_path = rootPath + "/text-data/abnormal-json"
+    val data_path = resPath + "/text-data/abnormal-json"
     spark.sql(s"""
                  |CREATE TABLE test_tbl_3337 (
                  |  id bigint,
@@ -979,7 +978,7 @@ class GlutenClickHouseHiveTableSuite
   }
 
   test("GLUTEN-3552: Bug fix csv field whitespaces") {
-    val data_path = rootPath + "/text-data/field_whitespaces"
+    val data_path = resPath + "/text-data/field_whitespaces"
     spark.sql(s"""
                  | CREATE TABLE test_tbl_3552(
                  | a string,
@@ -1002,7 +1001,7 @@ class GlutenClickHouseHiveTableSuite
   }
 
   test("GLUTEN-3548: Bug fix csv allow cr end of line") {
-    val data_path = rootPath + "/text-data/cr_end_of_line"
+    val data_path = resPath + "/text-data/cr_end_of_line"
     spark.sql(s"""
                  | CREATE TABLE test_tbl_3548(
                  | a string,
@@ -1026,7 +1025,7 @@ class GlutenClickHouseHiveTableSuite
 
   test("test 'hive udf'") {
     val jarPath = "udfs/hive-test-udfs.jar"
-    val jarUrl = s"file://$rootPath/$jarPath"
+    val jarUrl = s"file://$resPath/$jarPath"
     spark.sql(
       s"CREATE FUNCTION my_add as " +
         s"'org.apache.hadoop.hive.contrib.udf.example.UDFExampleAdd2' USING JAR '$jarUrl'")
@@ -1262,7 +1261,7 @@ class GlutenClickHouseHiveTableSuite
   }
 
   test("test mergetree write with column case sensitive on hive") {
-    val dataPath = s"$basePath/lineitem_mergetree_bucket"
+    val dataPath = s"$dataHome/lineitem_mergetree_bucket"
     val sourceDF = spark.sql(s"""
                                 |select
                                 |  string_field,
@@ -1281,7 +1280,7 @@ class GlutenClickHouseHiveTableSuite
 
     assert(new File(dataPath).listFiles().nonEmpty)
 
-    val dataPath2 = s"$basePath/lineitem_mergetree_bucket2"
+    val dataPath2 = s"$dataHome/lineitem_mergetree_bucket2"
     val df2 = spark.sql(s"""
                            |select
                            |  string_field STRING_FIELD,
@@ -1302,7 +1301,7 @@ class GlutenClickHouseHiveTableSuite
       .save(dataPath2)
     assert(new File(dataPath2).listFiles().nonEmpty)
 
-    val dataPath3 = s"$basePath/lineitem_mergetree_bucket3"
+    val dataPath3 = s"$dataHome/lineitem_mergetree_bucket3"
     val df3 = spark.sql(s"""
                            |select
                            |  string_field,
@@ -1323,7 +1322,7 @@ class GlutenClickHouseHiveTableSuite
       .save(dataPath3)
     assert(new File(dataPath3).listFiles().nonEmpty)
 
-    val dataPath4 = s"$basePath/lineitem_mergetree_bucket2"
+    val dataPath4 = s"$dataHome/lineitem_mergetree_bucket2"
     val df4 = spark
       .sql(s"""
               |select
@@ -1349,7 +1348,7 @@ class GlutenClickHouseHiveTableSuite
   }
 
   test("GLUTEN-6506: Orc read time zone") {
-    val dataPath = s"$basePath/orc-data/test_reader_time_zone.snappy.orc"
+    val dataPath = s"$dataHome/orc-data/test_reader_time_zone.snappy.orc"
     val create_table_sql = ("create table test_tbl_6506(" +
       "id bigint, t timestamp) stored as orc location '%s'")
       .format(dataPath)
@@ -1702,4 +1701,27 @@ class GlutenClickHouseHiveTableSuite
     }
   }
 
+  test("GLUTEN-9647: Fix SimplifySumRule on different types") {
+    val sql =
+      s"""
+         | select sum(int_field * 2L),
+         |        min(float_field / 2),
+         |        max(double_field * 0.03),
+         |        sum(short_field * 2),
+         |        sum(decimal_field * 3L)
+         | from $json_table_name
+         | where day = '2023-06-06'
+         |""".stripMargin
+    compareResultsAgainstVanillaSpark(
+      sql,
+      compareResult = true,
+      df => {
+        val jsonFileScan = collect(df.queryExecution.executedPlan) {
+          case l: HiveTableScanExecTransformer => l
+        }
+        assert(jsonFileScan.size == 1)
+      }
+    )
+  }
+
 }
