@@ -16,7 +16,6 @@
  */
 package org.apache.gluten.execution.hive
 
-import org.apache.gluten.backendsapi.clickhouse.RuntimeConfig
 import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.execution.GlutenClickHouseWholeStageTransformerSuite
 import org.apache.gluten.test.AllDataTypesWithComplexType.genTestData
@@ -60,12 +59,11 @@ class GlutenClickHouseNativeWriteTableSuite
       .set(ClickHouseConfig.CLICKHOUSE_WORKER_ID, "1")
       .set("spark.gluten.sql.columnar.iterator", "true")
       .set("spark.gluten.sql.columnar.hashagg.enablefinal", "true")
-      .set("spark.gluten.sql.enable.native.validation", "false")
+      .set(GlutenConfig.NATIVE_VALIDATION_ENABLED.key, "false")
       // TODO: support default ANSI policy
       .set("spark.sql.storeAssignmentPolicy", "legacy")
       .set("spark.sql.warehouse.dir", getWarehouseDir)
       .set("spark.sql.session.timeZone", sessionTimeZone)
-      .set(RuntimeConfig.LOGGER_LEVEL.key, "error")
       .setMaster("local[1]")
   }
 
@@ -107,7 +105,7 @@ class GlutenClickHouseNativeWriteTableSuite
       .option("delimiter", "|")
       .option("header", "false")
       .schema(supplierSchema)
-      .csv(s"$rootPath/csv-data/supplier.csv")
+      .csv(s"${resPath}csv-data/supplier.csv")
       .toDF()
   }
 
@@ -118,19 +116,19 @@ class GlutenClickHouseNativeWriteTableSuite
         format => {
           val sql =
             s"""insert overwrite local directory
-               |'$basePath/test_insert_into_${format}_supplier'
+               |'$dataHome/test_insert_into_${format}_supplier'
                |stored as $format
                |select /*+ REPARTITION($partitionNumber) */ * from supplier""".stripMargin
           (s"test_insert_into_${format}_supplier", null, sql)
         },
         (table_name, format) => {
           // spark 3.2 without orc or parquet suffix
-          val files = recursiveListFiles(new File(s"$basePath/$table_name"))
+          val files = recursiveListFiles(new File(s"$dataHome/$table_name"))
             .map(_.getName)
             .filterNot(s => s.endsWith(s".crc") || s.equals("_SUCCESS"))
 
           lazy val fileNames = {
-            val dir = s"$basePath/$table_name"
+            val dir = s"$dataHome/$table_name"
             recursiveListFiles(new File(dir))
               .map(f => f.getAbsolutePath.stripPrefix(dir))
               .sorted
@@ -138,7 +136,7 @@ class GlutenClickHouseNativeWriteTableSuite
           }
 
           lazy val errorMessage =
-            s"Search $basePath/$table_name with suffix .$format, all files: \n $fileNames"
+            s"Search $dataHome/$table_name with suffix .$format, all files: \n $fileNames"
           assert(files.length === partitionNumber, errorMessage)
         }
       )
@@ -176,10 +174,10 @@ class GlutenClickHouseNativeWriteTableSuite
       nativeWrite {
         format =>
           Seq(
-            s"""insert overwrite local directory '$basePath/test_insert_into_${format}_dir1'
+            s"""insert overwrite local directory '$dataHome/test_insert_into_${format}_dir1'
                |stored as $format select ${fields_.keys.mkString(",")}
                |from origin_table""".stripMargin,
-            s"""insert overwrite local directory '$basePath/test_insert_into_${format}_dir2'
+            s"""insert overwrite local directory '$dataHome/test_insert_into_${format}_dir2'
                |stored as $format select string_field, sum(int_field) as x
                |from origin_table group by string_field""".stripMargin
           ).foreach(checkInsertQuery(_, checkNative = true))
