@@ -16,39 +16,21 @@
  */
 package org.apache.gluten.execution.mergetree
 
-import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeConfig}
+import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeSettings}
 import org.apache.gluten.config.GlutenConfig
-import org.apache.gluten.execution.{FileSourceScanExecTransformer, GlutenClickHouseTPCHAbstractSuite}
+import org.apache.gluten.execution.{CreateMergeTreeSuite, FileSourceScanExecTransformer}
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.delta.files.TahoeFileIndex
-import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.execution.datasources.v2.clickhouse.metadata.AddMergeTreeParts
 
-import org.apache.commons.io.FileUtils
-import org.apache.hadoop.conf.Configuration
-import org.apache.hadoop.fs.FileSystem
-
 import java.io.File
 
 import scala.concurrent.duration.DurationInt
 
-class GlutenClickHouseMergeTreeCacheDataSuite
-  extends GlutenClickHouseTPCHAbstractSuite
-  with AdaptiveSparkPlanHelper {
-
-  override protected val needCopyParquetToTablePath = true
-
-  override protected val tablesPath: String = basePath + "/tpch-data"
-  override protected val tpchQueries: String = rootPath + "queries/tpch-queries-ch"
-  override protected val queriesResults: String = rootPath + "mergetree-queries-output"
-
-  override protected def createTPCHNotNullTables(): Unit = {
-    createNotNullTPCHTablesInParquet(tablesPath)
-  }
+class GlutenClickHouseMergeTreeCacheDataSuite extends CreateMergeTreeSuite {
 
   override protected def sparkConf: SparkConf = {
-    import org.apache.gluten.backendsapi.clickhouse.CHConfig._
 
     super.sparkConf
       .set("spark.shuffle.manager", "org.apache.spark.shuffle.sort.ColumnarShuffleManager")
@@ -56,23 +38,18 @@ class GlutenClickHouseMergeTreeCacheDataSuite
       .set("spark.sql.shuffle.partitions", "5")
       .set("spark.sql.autoBroadcastJoinThreshold", "10MB")
       .set("spark.sql.adaptive.enabled", "true")
-      .set(RuntimeConfig.LOGGER_LEVEL.key, "error")
       .set("spark.gluten.soft-affinity.enabled", "true")
       .set(GlutenConfig.NATIVE_WRITER_ENABLED.key, "true")
       .set(CHConfig.ENABLE_ONEPIPELINE_MERGETREE_WRITE.key, spark35.toString)
-      .setCHSettings("mergetree.merge_after_insert", false)
+      .set(RuntimeSettings.MERGE_AFTER_INSERT.key, "false")
   }
 
+  private val remotePath: String = hdfsHelper.independentHdfsURL("test")
   override protected def beforeEach(): Unit = {
     super.beforeEach()
-    val conf = new Configuration
-    conf.set("fs.defaultFS", HDFS_URL)
-    val fs = FileSystem.get(conf)
-    fs.delete(new org.apache.hadoop.fs.Path(HDFS_URL), true)
-    FileUtils.deleteDirectory(new File(HDFS_METADATA_PATH))
-    FileUtils.forceMkdir(new File(HDFS_METADATA_PATH))
-    FileUtils.deleteDirectory(new File(HDFS_CACHE_PATH))
-    FileUtils.forceMkdir(new File(HDFS_CACHE_PATH))
+    hdfsHelper.deleteDir(remotePath)
+    hdfsHelper.resetMeta()
+    hdfsHelper.resetCache()
   }
 
   def countFiles(directory: File): Int = {
@@ -113,7 +90,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  |)
                  |USING clickhouse
                  |PARTITIONED BY (l_shipdate)
-                 |LOCATION '$HDFS_URL/test/lineitem_mergetree_hdfs'
+                 |LOCATION '$remotePath/lineitem_mergetree_hdfs'
                  |TBLPROPERTIES (storage_policy='__hdfs_main',
                  |               orderByKey='l_linenumber,l_orderkey')
                  |""".stripMargin)
@@ -123,9 +100,8 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  | select * from lineitem a
                  | where a.l_shipdate between date'1995-01-01' and date'1995-01-31'
                  |""".stripMargin)
-    FileUtils.deleteDirectory(new File(HDFS_METADATA_PATH))
-    FileUtils.forceMkdir(new File(HDFS_METADATA_PATH))
-    val dataPath = new File(HDFS_CACHE_PATH)
+    hdfsHelper.resetMeta()
+    val dataPath = new File(hdfsHelper.HDFS_CACHE_PATH)
     val initial_cache_files = countFiles(dataPath)
 
     val res = spark
@@ -137,7 +113,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
               |                aaa='ccc')""".stripMargin)
       .collect()
     assertResult(true)(res(0).getBoolean(0))
-    val metaPath = new File(s"$HDFS_METADATA_PATH/$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs")
+    val metaPath = new File(hdfsHelper.metaPath(s"$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs"))
     assertResult(true)(metaPath.exists() && metaPath.isDirectory)
     assertResult(22)(metaPath.list().length)
     assert(countFiles(dataPath) > initial_cache_files)
@@ -217,7 +193,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  |)
                  |USING clickhouse
                  |PARTITIONED BY (l_shipdate)
-                 |LOCATION '$HDFS_URL/test/lineitem_mergetree_hdfs'
+                 |LOCATION '$remotePath/lineitem_mergetree_hdfs'
                  |TBLPROPERTIES (storage_policy='__hdfs_main',
                  |               orderByKey='l_linenumber,l_orderkey')
                  |""".stripMargin)
@@ -227,9 +203,8 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  | select * from lineitem a
                  | where a.l_shipdate between date'1995-01-01' and date'1995-01-31'
                  |""".stripMargin)
-    FileUtils.deleteDirectory(new File(HDFS_METADATA_PATH))
-    FileUtils.forceMkdir(new File(HDFS_METADATA_PATH))
-    val dataPath = new File(HDFS_CACHE_PATH)
+    hdfsHelper.resetMeta()
+    val dataPath = new File(hdfsHelper.HDFS_CACHE_PATH)
     val initial_cache_files = countFiles(dataPath)
 
     val res = spark
@@ -241,7 +216,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
               |                aaa='ccc')""".stripMargin)
       .collect()
     assertResult(true)(res(0).getBoolean(0))
-    val metaPath = new File(s"$HDFS_METADATA_PATH/$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs")
+    val metaPath = new File(hdfsHelper.metaPath(s"$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs"))
     assertResult(true)(metaPath.exists() && metaPath.isDirectory)
     eventually(timeout(60.seconds), interval(2.seconds)) {
       assertResult(22)(metaPath.list().length)
@@ -325,7 +300,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  |)
                  |USING clickhouse
                  |PARTITIONED BY (l_shipdate)
-                 |LOCATION '$HDFS_URL/test/lineitem_mergetree_hdfs'
+                 |LOCATION '$remotePath/lineitem_mergetree_hdfs'
                  |TBLPROPERTIES (storage_policy='__hdfs_main',
                  |               orderByKey='l_linenumber,l_orderkey')
                  |""".stripMargin)
@@ -335,21 +310,20 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  | select * from lineitem a
                  | where a.l_shipdate between date'1995-01-01' and date'1995-01-31'
                  |""".stripMargin)
-    FileUtils.deleteDirectory(new File(HDFS_METADATA_PATH))
-    FileUtils.forceMkdir(new File(HDFS_METADATA_PATH))
-    val dataPath = new File(HDFS_CACHE_PATH)
+    hdfsHelper.resetMeta()
+    val dataPath = new File(hdfsHelper.HDFS_CACHE_PATH)
     val initial_cache_files = countFiles(dataPath)
 
     val res = spark
       .sql(s"""
               |cache data
-              |  select * from '$HDFS_URL/test/lineitem_mergetree_hdfs'
+              |  select * from '$remotePath/lineitem_mergetree_hdfs'
               |  after l_shipdate AS OF '1995-01-10'
               |  CACHEPROPERTIES(storage_policy='__hdfs_main',
               |                aaa='ccc')""".stripMargin)
       .collect()
     assertResult(true)(res(0).getBoolean(0))
-    val metaPath = new File(s"$HDFS_METADATA_PATH/$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs")
+    val metaPath = new File(hdfsHelper.metaPath(s"$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs"))
     assertResult(true)(metaPath.exists() && metaPath.isDirectory)
     assertResult(22)(metaPath.list().length)
     assert(countFiles(dataPath) > initial_cache_files)
@@ -428,7 +402,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  | L_COMMENT       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$HDFS_URL/test/lineitem_mergetree_hdfs'
+                 |LOCATION '$remotePath/lineitem_mergetree_hdfs'
                  |TBLPROPERTIES (storage_policy='__hdfs_main')
                  |""".stripMargin)
 
@@ -437,12 +411,11 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  | select * from lineitem a
                  | where a.l_shipdate between date'1995-01-01' and date'1995-01-31'
                  |""".stripMargin)
-    FileUtils.deleteDirectory(new File(HDFS_METADATA_PATH))
-    FileUtils.forceMkdir(new File(HDFS_METADATA_PATH))
-    val dataPath = new File(HDFS_CACHE_PATH)
+    hdfsHelper.resetMeta()
+    val dataPath = new File(hdfsHelper.HDFS_CACHE_PATH)
     val initial_cache_files = countFiles(dataPath)
 
-    val metaPath = new File(s"$HDFS_METADATA_PATH/$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs")
+    val metaPath = new File(hdfsHelper.metaPath(s"$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs"))
     val res1 = spark.sql(s"cache data select * from lineitem_mergetree_hdfs").collect()
     assertResult(true)(res1(0).getBoolean(0))
     assertResult(1)(metaPath.list().length)
@@ -518,7 +491,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  |)
                  |USING clickhouse
                  |PARTITIONED BY (L_SHIPDATE)
-                 |LOCATION '$HDFS_URL/test/lineitem_mergetree_hdfs'
+                 |LOCATION '$remotePath/lineitem_mergetree_hdfs'
                  |TBLPROPERTIES (storage_policy='__hdfs_main',
                  |               orderByKey='L_LINENUMBER,L_ORDERKEY')
                  |""".stripMargin)
@@ -528,21 +501,20 @@ class GlutenClickHouseMergeTreeCacheDataSuite
                  | select * from lineitem a
                  | where a.l_shipdate between date'1995-01-01' and date'1995-01-31'
                  |""".stripMargin)
-    FileUtils.deleteDirectory(new File(HDFS_METADATA_PATH))
-    FileUtils.forceMkdir(new File(HDFS_METADATA_PATH))
-    val dataPath = new File(HDFS_CACHE_PATH)
+    hdfsHelper.resetMeta()
+    val dataPath = new File(hdfsHelper.HDFS_CACHE_PATH)
     val initial_cache_files = countFiles(dataPath)
 
     val res = spark
       .sql(s"""
               |cache data
-              |  select * from '$HDFS_URL/test/lineitem_mergetree_hdfs'
+              |  select * from '$remotePath/lineitem_mergetree_hdfs'
               |  after L_SHIPDATE AS OF '1995-01-10'
               |  CACHEPROPERTIES(storage_policy='__hdfs_main',
               |                aaa='ccc')""".stripMargin)
       .collect()
     assertResult(true)(res(0).getBoolean(0))
-    val metaPath = new File(s"$HDFS_METADATA_PATH/$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs")
+    val metaPath = new File(hdfsHelper.metaPath(s"$SPARK_DIR_NAME/test/lineitem_mergetree_hdfs"))
     assertResult(true)(metaPath.exists() && metaPath.isDirectory)
     assertResult(22)(metaPath.list().length)
     assert(countFiles(dataPath) > initial_cache_files)
@@ -597,7 +569,7 @@ class GlutenClickHouseMergeTreeCacheDataSuite
   test("test disable cache files return") {
     withSQLConf(CHConfig.ENABLE_GLUTEN_LOCAL_FILE_CACHE.key -> "false") {
       runSql(
-        s"CACHE FILES select * from '$HDFS_URL_ENDPOINT/tpch-data/lineitem'",
+        s"CACHE FILES select * from '${hdfsHelper.hdfsURL("tpch-data/lineitem")}'",
         noFallBack = false) {
         df =>
           val res = df.collect()
