@@ -16,30 +16,25 @@
  */
 package org.apache.gluten.execution.mergetree
 
-import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeConfig, RuntimeSettings}
+import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeSettings}
 import org.apache.gluten.config.GlutenConfig
-import org.apache.gluten.execution.{FileSourceScanExecTransformer, GlutenClickHouseTPCHAbstractSuite}
+import org.apache.gluten.execution.{CreateMergeTreeSuite, FileSourceScanExecTransformer}
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.SaveMode
 import org.apache.spark.sql.delta.MergeTreeConf
-import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
+import org.apache.spark.sql.delta.sources.DeltaSQLConf
 
 import io.delta.tables.ClickhouseTable
+import org.apache.commons.io.FileUtils
+import org.apache.commons.io.filefilter._
 
 import java.io.File
 
+import scala.collection.JavaConverters.collectionAsScalaIterableConverter
 import scala.concurrent.duration.DurationInt
 
-class GlutenClickHouseMergeTreeOptimizeSuite
-  extends GlutenClickHouseTPCHAbstractSuite
-  with AdaptiveSparkPlanHelper {
-
-  override protected val needCopyParquetToTablePath = true
-
-  override protected val tablesPath: String = basePath + "/tpch-data"
-  override protected val tpchQueries: String = rootPath + "queries/tpch-queries-ch"
-  override protected val queriesResults: String = rootPath + "mergetree-queries-output"
+class GlutenClickHouseMergeTreeOptimizeSuite extends CreateMergeTreeSuite {
 
   /** Run Gluten + ClickHouse Backend with SortShuffleManager */
   override protected def sparkConf: SparkConf = {
@@ -51,7 +46,6 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       .set("spark.sql.shuffle.partitions", "5")
       .set("spark.sql.autoBroadcastJoinThreshold", "10MB")
       .set("spark.sql.adaptive.enabled", "true")
-      .set(RuntimeConfig.LOGGER_LEVEL.key, "error")
       .set(GlutenConfig.NATIVE_WRITER_ENABLED.key, "true")
       .set(CHConfig.ENABLE_ONEPIPELINE_MERGETREE_WRITE.key, spark35.toString)
       .set(RuntimeSettings.MIN_INSERT_BLOCK_SIZE_ROWS.key, "10000")
@@ -59,7 +53,8 @@ class GlutenClickHouseMergeTreeOptimizeSuite
         "spark.databricks.delta.retentionDurationCheck.enabled",
         "false"
       ) // otherwise, RETAIN 0 HOURS will fail
-      .setCHSettings("mergetree.merge_after_insert", false)
+      .set(RuntimeSettings.MERGE_AFTER_INSERT.key, "false")
+      .set(MergeTreeConf.WRITE_MARKS_FOR_SUBSTREAMS_IN_COMPACT_PARTS.key, "true")
       .setCHSettings("input_format_parquet_max_block_size", 8192)
   }
 
@@ -80,12 +75,8 @@ class GlutenClickHouseMergeTreeOptimizeSuite
     withSQLConf(defaultBlockSize, assign_part_uuids, optimized, min_rows_for_wide_part)(f)
   }
 
-  override protected def createTPCHNotNullTables(): Unit = {
-    createNotNullTPCHTablesInParquet(tablesPath)
-  }
-
   test("test mergetree optimize basic") {
-    withSQLConf("spark.databricks.delta.optimize.maxFileSize" -> "2000000") {
+    withSQLConf(DeltaSQLConf.DELTA_OPTIMIZE_MAX_FILE_SIZE.key -> "2000000") {
       spark.sql(s"""
                    |DROP TABLE IF EXISTS lineitem_mergetree_optimize;
                    |""".stripMargin)
@@ -93,7 +84,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       spark.sql(s"""
                    |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize
                    |USING clickhouse
-                   |LOCATION '$basePath/lineitem_mergetree_optimize'
+                   |LOCATION '$dataHome/lineitem_mergetree_optimize'
                    | as select * from lineitem
                    |""".stripMargin)
 
@@ -102,21 +93,32 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       assertResult(600572)(ret.apply(0).get(0))
 
       assertResult(462)(
-        countFiles(new File(s"$basePath/lineitem_mergetree_optimize"))
+        countFiles(new File(s"$dataHome/lineitem_mergetree_optimize"))
       ) // many merged parts
     }
   }
 
   def countFiles(directory: File): Int = {
-    if (directory.exists && directory.isDirectory && !directory.getName.equals("_commits")) {
-      val files = directory.listFiles
-      val count = files
-        .filter(!_.getName.endsWith(".crc"))
-        .count(_.isFile) + files.filter(_.isDirectory).map(countFiles).sum
-      count + 1
-    } else {
-      0
-    }
+    val NO_COMMIT_DIR = new AndFileFilter(
+      DirectoryFileFilter.DIRECTORY,
+      new NotFileFilter(new NameFileFilter("_commits")))
+
+    val CRC_FILES = new SuffixFileFilter(".crc")
+    val VACUUM_INFO = new SuffixFileFilter("vacuum_info")
+    // https://github.com/ClickHouse/ClickHouse/pull/77940 introduce "columns_substreams.txt"
+    val COLUMNS_SUBSTREAMS = new NameFileFilter("columns_substreams.txt")
+
+    val EXClUDE_FILES = new NotFileFilter(
+      new OrFileFilter(
+        CRC_FILES,
+        VACUUM_INFO,
+        COLUMNS_SUBSTREAMS
+      )
+    )
+    FileUtils
+      .listFilesAndDirs(directory, EXClUDE_FILES, NO_COMMIT_DIR)
+      .asScala
+      .count(_ => true)
   }
 
   test("test mergetree optimize partitioned, each partition too small to trigger optimize") {
@@ -128,30 +130,26 @@ class GlutenClickHouseMergeTreeOptimizeSuite
                  |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize_p
                  |USING clickhouse
                  |PARTITIONED BY (l_shipdate)
-                 |LOCATION '$basePath/lineitem_mergetree_optimize_p'
+                 |LOCATION '$dataHome/lineitem_mergetree_optimize_p'
                  | as select * from lineitem
                  |""".stripMargin)
 
     spark.sparkContext.setJobGroup("test", "test")
     with_ut_conf(spark.sql("optimize lineitem_mergetree_optimize_p"))
     val job_ids = spark.sparkContext.statusTracker.getJobIdsForGroup("test")
-    if (spark35) {
-      assertResult(4)(job_ids.length)
-    } else {
-      assertResult(1)(job_ids.length) // will not trigger actual merge job
-    }
+    assertResult(1)(job_ids.length)
     spark.sparkContext.clearJobGroup()
 
     val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p").collect()
     assertResult(600572)(ret.apply(0).get(0))
 
-    assertResult(22728)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p")))
+    assertResult(22728)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p")))
     spark.sql("VACUUM lineitem_mergetree_optimize_p RETAIN 0 HOURS")
     if (spark32) {
-      assertResult(22728)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p")))
+      assertResult(22728)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p")))
     } else {
       // For Spark 3.3 + Delta 2.3, vacuum command will create two commit files in deltalog dir.
-      assertResult(22730)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p")))
+      assertResult(22730)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p")))
     }
 
     val ret2 = spark.sql("select count(*) from lineitem_mergetree_optimize_p").collect()
@@ -167,7 +165,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
                  |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize_p2
                  |USING clickhouse
                  |PARTITIONED BY (l_returnflag)
-                 |LOCATION '$basePath/lineitem_mergetree_optimize_p2'
+                 |LOCATION '$dataHome/lineitem_mergetree_optimize_p2'
                  | as select * from lineitem
                  |""".stripMargin)
 
@@ -185,19 +183,19 @@ class GlutenClickHouseMergeTreeOptimizeSuite
     val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p2").collect()
     assertResult(600572)(ret.apply(0).get(0))
 
-    assertResult(372)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p2")))
+    assertResult(372)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p2")))
     spark.sql("VACUUM lineitem_mergetree_optimize_p2 RETAIN 0 HOURS")
     if (spark32) {
-      assertResult(239)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p2")))
+      assertResult(239)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p2")))
     } else {
-      assertResult(241)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p2")))
+      assertResult(241)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p2")))
     }
     spark.sql("VACUUM lineitem_mergetree_optimize_p2 RETAIN 0 HOURS")
     // the second VACUUM will remove some empty folders
     if (spark32) {
-      assertResult(220)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p2")))
+      assertResult(220)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p2")))
     } else {
-      assertResult(226)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p2")))
+      assertResult(226)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p2")))
     }
 
     val ret2 = spark.sql("select count(*) from lineitem_mergetree_optimize_p2").collect()
@@ -214,7 +212,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
                    |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize_p3
                    |USING clickhouse
                    |PARTITIONED BY (l_linenumber,l_returnflag)
-                   |LOCATION '$basePath/lineitem_mergetree_optimize_p3'
+                   |LOCATION '$dataHome/lineitem_mergetree_optimize_p3'
                    | as select * from lineitem
                    |""".stripMargin)
 
@@ -222,18 +220,18 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p3").collect()
       assertResult(600572)(ret.apply(0).get(0))
 
-      assertResult(516)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p3")))
+      assertResult(516)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p3")))
       spark.sql("VACUUM lineitem_mergetree_optimize_p3 RETAIN 0 HOURS")
       if (spark32) {
-        assertResult(306)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p3")))
+        assertResult(306)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p3")))
       } else {
-        assertResult(308)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p3")))
+        assertResult(308)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p3")))
       }
       spark.sql("VACUUM lineitem_mergetree_optimize_p3 RETAIN 0 HOURS")
       if (spark32) {
-        assertResult(276)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p3")))
+        assertResult(276)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p3")))
       } else {
-        assertResult(282)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p3")))
+        assertResult(282)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p3")))
       }
 
       val ret2 = spark.sql("select count(*) from lineitem_mergetree_optimize_p3").collect()
@@ -251,7 +249,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
                    |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize_p4
                    |USING clickhouse
                    |PARTITIONED BY (l_linenumber,l_returnflag)
-                   |LOCATION '$basePath/lineitem_mergetree_optimize_p4'
+                   |LOCATION '$dataHome/lineitem_mergetree_optimize_p4'
                    | as select * from lineitem
                    |""".stripMargin)
 
@@ -259,18 +257,18 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p4").collect()
       assertResult(600572)(ret.apply(0).get(0))
 
-      assertResult(516)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p4")))
+      assertResult(516)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p4")))
       spark.sql("VACUUM lineitem_mergetree_optimize_p4 RETAIN 0 HOURS")
       if (spark32) {
-        assertResult(306)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p4")))
+        assertResult(306)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p4")))
       } else {
-        assertResult(308)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p4")))
+        assertResult(308)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p4")))
       }
       spark.sql("VACUUM lineitem_mergetree_optimize_p4 RETAIN 0 HOURS")
       if (spark32) {
-        assertResult(276)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p4")))
+        assertResult(276)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p4")))
       } else {
-        assertResult(282)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p4")))
+        assertResult(282)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p4")))
       }
 
       val ret2 = spark.sql("select count(*) from lineitem_mergetree_optimize_p4").collect()
@@ -279,8 +277,8 @@ class GlutenClickHouseMergeTreeOptimizeSuite
   }
 
   test("test mergetree optimize with optimize.minFileSize and optimize.maxFileSize") {
-    withSQLConf("spark.databricks.delta.optimize.minFileSize" -> "838000") {
-      // 3 from 37 parts are larger than this, so after optimize there should be 4 parts:
+    withSQLConf(DeltaSQLConf.DELTA_OPTIMIZE_MIN_FILE_SIZE.key -> "838018") {
+      // 3 of 37 parts are >= 838,018, so after optimizing there should be 4 parts:
       // 3 original parts and 1 merged part
       spark.sql(s"""
                    |DROP TABLE IF EXISTS lineitem_mergetree_optimize_p5;
@@ -289,7 +287,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       spark.sql(s"""
                    |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize_p5
                    |USING clickhouse
-                   |LOCATION '$basePath/lineitem_mergetree_optimize_p5'
+                   |LOCATION '$dataHome/lineitem_mergetree_optimize_p5'
                    | as select * from lineitem
                    |""".stripMargin)
 
@@ -298,11 +296,11 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       spark.sql("VACUUM lineitem_mergetree_optimize_p5 RETAIN 0 HOURS")
       spark.sql("VACUUM lineitem_mergetree_optimize_p5 RETAIN 0 HOURS")
       if (spark32) {
-        assertResult(99)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p5")))
+        assertResult(99)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p5")))
       } else {
         // For Spark 3.3 + Delta 2.3, vacuum command will create two commit files in deltalog dir.
         // this case will create a checkpoint
-        assertResult(105)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p5")))
+        assertResult(105)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p5")))
       }
 
       val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p5").collect()
@@ -310,22 +308,22 @@ class GlutenClickHouseMergeTreeOptimizeSuite
     }
 
     withSQLConf(
-      "spark.databricks.delta.optimize.maxFileSize" -> "10000000",
-      "spark.databricks.delta.optimize.minFileSize" -> "838250") {
-      // of the remaing 3 original parts, 2 are less than 838250, 1 is larger (size 838255)
-      // the merged part is ~27MB, so after optimize there should be 3 parts:
+      DeltaSQLConf.DELTA_OPTIMIZE_MAX_FILE_SIZE.key -> "10000000",
+      DeltaSQLConf.DELTA_OPTIMIZE_MIN_FILE_SIZE.key -> "838291") {
+      // of the remaining 3 original parts, 2 are < 838,291, 1 is larger (size 838,306)
+      // the merged part is ~27MB, so after optimizing there should be 3 parts:
       // 1 merged part from 2 original parts, 1 merged part from 34 original parts
-      // and 1 original part (size 838255)
+      // and 1 original part (size 838,306)
 
       with_ut_conf(spark.sql("optimize lineitem_mergetree_optimize_p5"))
 
       spark.sql("VACUUM lineitem_mergetree_optimize_p5 RETAIN 0 HOURS")
       spark.sql("VACUUM lineitem_mergetree_optimize_p5 RETAIN 0 HOURS")
       if (spark32) {
-        assertResult(93)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p5")))
+        assertResult(93)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p5")))
       } else {
         // For Spark 3.3 + Delta 2.3, vacuum command will create two commit files in deltalog dir.
-        assertResult(104)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p5")))
+        assertResult(104)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p5")))
       }
 
       val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p5").collect()
@@ -338,10 +336,10 @@ class GlutenClickHouseMergeTreeOptimizeSuite
     spark.sql("VACUUM lineitem_mergetree_optimize_p5 RETAIN 0 HOURS")
     spark.sql("VACUUM lineitem_mergetree_optimize_p5 RETAIN 0 HOURS")
     if (spark32) {
-      assertResult(77)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p5")))
+      assertResult(77)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p5")))
     } else {
       // For Spark 3.3 + Delta 2.3, vacuum command will create two commit files in deltalog dir.
-      assertResult(93)(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p5")))
+      assertResult(93)(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p5")))
     }
 
     val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p5").collect()
@@ -357,7 +355,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
                  |CREATE TABLE IF NOT EXISTS lineitem_mergetree_optimize_p6
                  |USING clickhouse
                  |PARTITIONED BY (l_returnflag)
-                 |LOCATION '$basePath/lineitem_mergetree_optimize_p6'
+                 |LOCATION '$dataHome/lineitem_mergetree_optimize_p6'
                  | CLUSTERED BY (l_partkey)
                  | ${if (spark32) "" else "SORTED BY (l_partkey)"} INTO 2 BUCKETS
                  | as select * from lineitem
@@ -368,11 +366,11 @@ class GlutenClickHouseMergeTreeOptimizeSuite
     val ret = spark.sql("select count(*) from lineitem_mergetree_optimize_p6").collect()
     assertResult(600572)(ret.apply(0).get(0))
 
-    assertResult(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p6")))(
+    assertResult(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p6")))(
       if (spark32) 499 else 528)
     spark.sql("VACUUM lineitem_mergetree_optimize_p6 RETAIN 0 HOURS")
     spark.sql("VACUUM lineitem_mergetree_optimize_p6 RETAIN 0 HOURS")
-    assertResult(countFiles(new File(s"$basePath/lineitem_mergetree_optimize_p6")))(
+    assertResult(countFiles(new File(s"$dataHome/lineitem_mergetree_optimize_p6")))(
       if (spark32) 315 else 327)
 
     val ret2 = spark.sql("select count(*) from lineitem_mergetree_optimize_p6").collect()
@@ -381,7 +379,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
 
   test("test skip index after optimize") {
     withSQLConf(
-      "spark.databricks.delta.optimize.maxFileSize" -> "2000000",
+      DeltaSQLConf.DELTA_OPTIMIZE_MAX_FILE_SIZE.key -> "2000000",
       "spark.sql.adaptive.enabled" -> "false") {
       spark.sql(s"""
                    |DROP TABLE IF EXISTS lineitem_mergetree_index;
@@ -390,7 +388,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       spark.sql(s"""
                    |CREATE TABLE IF NOT EXISTS lineitem_mergetree_index
                    |USING clickhouse
-                   |LOCATION '$basePath/lineitem_mergetree_index'
+                   |LOCATION '$dataHome/lineitem_mergetree_index'
                    |TBLPROPERTIES('bloomfilterIndexKey'='l_orderkey')
                    | as select * from lineitem
                    |""".stripMargin)
@@ -413,7 +411,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       val marks = mergetreeScan.metrics("selectedMarks").value
       assertResult(1)(marks)
 
-      val directory = new File(s"$basePath/lineitem_mergetree_index")
+      val directory = new File(s"$dataHome/lineitem_mergetree_index")
       val partDir = directory.listFiles().filter(f => f.getName.endsWith("merged")).head
       assert(
         partDir.listFiles().exists(p => p.getName.contains("skp_idx__bloomfilter_l_orderkey.idx")))
@@ -422,10 +420,10 @@ class GlutenClickHouseMergeTreeOptimizeSuite
   }
 
   test("test mergetree optimize with the path based table") {
-    val dataPath = s"$basePath/lineitem_mergetree_optimize_path_based"
+    val dataPath = s"$dataHome/lineitem_mergetree_optimize_path_based"
     clearDataPath(dataPath)
-    withSQLConf("spark.databricks.delta.optimize.minFileSize" -> "838000") {
-      // 3 from 37 parts are larger than this, so after optimize there should be 4 parts:
+    withSQLConf(DeltaSQLConf.DELTA_OPTIMIZE_MIN_FILE_SIZE.key -> "838018") {
+      // 3 of 37 parts are >= 838,018, so after optimizing there should be 4 parts:
       // 3 original parts and 1 merged part
 
       val sourceDF = spark.sql(s"""
@@ -453,12 +451,12 @@ class GlutenClickHouseMergeTreeOptimizeSuite
     }
 
     withSQLConf(
-      "spark.databricks.delta.optimize.maxFileSize" -> "10000000",
-      "spark.databricks.delta.optimize.minFileSize" -> "838250") {
-      // of the remaing 3 original parts, 2 are less than 838250, 1 is larger (size 838255)
-      // the merged part is ~27MB, so after optimize there should be 3 parts:
+      DeltaSQLConf.DELTA_OPTIMIZE_MAX_FILE_SIZE.key -> "10000000",
+      DeltaSQLConf.DELTA_OPTIMIZE_MIN_FILE_SIZE.key -> "838291") {
+      // of the remaining 3 original parts, 2 are < 838,291, 1 is larger (size 838,306)
+      // the merged part is ~27MB, so after optimizing there should be 3 parts:
       // 1 merged part from 2 original parts, 1 merged part from 34 original parts
-      // and 1 original part (size 838255)
+      // and 1 original part (size 838,306)
 
       val clickhouseTable = ClickhouseTable.forPath(spark, dataPath)
       with_ut_conf(clickhouseTable.optimize().executeCompaction())
@@ -493,8 +491,8 @@ class GlutenClickHouseMergeTreeOptimizeSuite
 
   test("test mergetree insert with optimize basic") {
     withSQLConf(
-      "spark.databricks.delta.optimize.minFileSize" -> "200000000",
-      CHConfig.runtimeSettings("mergetree.merge_after_insert") -> "true"
+      DeltaSQLConf.DELTA_OPTIMIZE_MIN_FILE_SIZE.key -> "200000000",
+      RuntimeSettings.MERGE_AFTER_INSERT.key -> "true"
     ) {
       spark.sql(s"""
                    |DROP TABLE IF EXISTS lineitem_mergetree_insert_optimize_basic;
@@ -503,7 +501,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       spark.sql(s"""
                    |CREATE TABLE IF NOT EXISTS lineitem_mergetree_insert_optimize_basic
                    |USING clickhouse
-                   |LOCATION '$basePath/lineitem_mergetree_insert_optimize_basic'
+                   |LOCATION '$dataHome/lineitem_mergetree_insert_optimize_basic'
                    | as select * from lineitem
                    |""".stripMargin)
 
@@ -511,7 +509,7 @@ class GlutenClickHouseMergeTreeOptimizeSuite
       assertResult(600572)(ret.apply(0).get(0))
       eventually(timeout(60.seconds), interval(3.seconds)) {
         assertResult(2)(
-          new File(s"$basePath/lineitem_mergetree_insert_optimize_basic").listFiles().length
+          new File(s"$dataHome/lineitem_mergetree_insert_optimize_basic").listFiles().length
         )
       }
     }
