@@ -16,46 +16,22 @@
  */
 package org.apache.gluten.execution.mergetree
 
-import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeConfig}
+import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeSettings}
 import org.apache.gluten.config.GlutenConfig
-import org.apache.gluten.execution.{BasicScanExecTransformer, FileSourceScanExecTransformer, GlutenClickHouseTPCHAbstractSuite}
+import org.apache.gluten.execution.{BasicScanExecTransformer, CreateMergeTreeSuite, FileSourceScanExecTransformer}
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.SaveMode
 import org.apache.spark.sql.delta.catalog.ClickHouseTableV2
 import org.apache.spark.sql.delta.files.TahoeFileIndex
-import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.execution.datasources.mergetree.StorageMeta
 import org.apache.spark.sql.execution.datasources.v2.clickhouse.metadata.AddMergeTreeParts
 
-import io.minio.{BucketExistsArgs, ListObjectsArgs, MakeBucketArgs, MinioClient, RemoveBucketArgs, RemoveObjectsArgs}
-import io.minio.messages.DeleteObject
-import org.apache.commons.io.FileUtils
-
 import java.io.File
-import java.util
 
 import scala.concurrent.duration.DurationInt
 
-class GlutenClickHouseMergeTreeWriteOnS3Suite
-  extends GlutenClickHouseTPCHAbstractSuite
-  with AdaptiveSparkPlanHelper {
-
-  override protected val needCopyParquetToTablePath = true
-
-  override protected val tablesPath: String = basePath + "/tpch-data"
-  override protected val tpchQueries: String = rootPath + "queries/tpch-queries-ch"
-  override protected val queriesResults: String = rootPath + "mergetree-queries-output"
-
-  private val client = MinioClient
-    .builder()
-    .endpoint(MINIO_ENDPOINT)
-    .credentials(S3_ACCESS_KEY, S3_SECRET_KEY)
-    .build()
-
-  override protected def createTPCHNotNullTables(): Unit = {
-    createNotNullTPCHTablesInParquet(tablesPath)
-  }
+class GlutenClickHouseMergeTreeWriteOnS3Suite extends CreateMergeTreeSuite {
 
   override protected def sparkConf: SparkConf = {
     super.sparkConf
@@ -64,34 +40,22 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
       .set("spark.sql.shuffle.partitions", "5")
       .set("spark.sql.autoBroadcastJoinThreshold", "10MB")
       .set("spark.sql.adaptive.enabled", "true")
-      .set(RuntimeConfig.LOGGER_LEVEL.key, "error")
       .set(GlutenConfig.NATIVE_WRITER_ENABLED.key, "true")
       .set(CHConfig.ENABLE_ONEPIPELINE_MERGETREE_WRITE.key, spark35.toString)
   }
 
   override protected def beforeEach(): Unit = {
     super.beforeEach()
-    if (client.bucketExists(BucketExistsArgs.builder().bucket(BUCKET_NAME).build())) {
-      val results =
-        client.listObjects(ListObjectsArgs.builder().bucket(BUCKET_NAME).recursive(true).build())
-      val objects = new util.LinkedList[DeleteObject]()
-      results.forEach(
-        obj => {
-          objects.add(new DeleteObject(obj.get().objectName()))
-        })
-      val removeResults = client.removeObjects(
-        RemoveObjectsArgs.builder().bucket(BUCKET_NAME).objects(objects).build())
-      removeResults.forEach(result => result.get().message())
-      client.removeBucket(RemoveBucketArgs.builder().bucket(BUCKET_NAME).build())
+    if (minioHelper.bucketExists(BUCKET_NAME)) {
+      minioHelper.clearBucket(BUCKET_NAME)
     }
-    client.makeBucket(MakeBucketArgs.builder().bucket(BUCKET_NAME).build())
-    FileUtils.deleteDirectory(new File(S3_METADATA_PATH))
-    FileUtils.forceMkdir(new File(S3_METADATA_PATH))
+    minioHelper.createBucket(BUCKET_NAME)
+    minioHelper.resetMeta()
   }
 
   override protected def afterEach(): Unit = {
     super.afterEach()
-    FileUtils.deleteDirectory(new File(S3_METADATA_PATH))
+    minioHelper.resetMeta()
   }
 
   test("test mergetree table write") {
@@ -128,9 +92,9 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
                  | insert into table lineitem_mergetree_s3
                  | select * from lineitem
                  |""".stripMargin)
-    FileUtils.deleteDirectory(new File(S3_METADATA_PATH))
+    minioHelper.resetMeta()
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_s3")) {
+    customCheckQuery(q1("lineitem_mergetree_s3")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -160,41 +124,25 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
   }
 
   private def verifyS3CompactFileExist(table: String): Unit = {
-    val args = ListObjectsArgs
-      .builder()
-      .bucket(BUCKET_NAME)
-      .recursive(true)
-      .prefix(table)
-      .build()
-    var objectCount: Int = 0
-    var metadataGlutenExist: Boolean = false
-    var metadataBinExist: Boolean = false
-    var dataBinExist: Boolean = false
+    val objectNames = minioHelper.listObjects(BUCKET_NAME, table)
+    var metadataGlutenExist = false
+    var metadataBinExist = false
+    var dataBinExist = false
     var hasCommits = false
-    client
-      .listObjects(args)
-      .forEach(
-        obj => {
-          objectCount += 1
-          val objectName = obj.get().objectName()
-          if (objectName.contains("metadata.gluten")) {
-            metadataGlutenExist = true
-          } else if (objectName.contains("part_meta.gluten")) {
-            metadataBinExist = true
-          } else if (objectName.contains("part_data.gluten")) {
-            dataBinExist = true
-          } else if (objectName.contains("_commits")) {
-            // Spark 35 has _commits directory
-            // table/_delta_log/_commits/
-            hasCommits = true
-          }
-        })
+
+    objectNames.foreach {
+      objectName =>
+        if (objectName.contains("metadata.gluten")) metadataGlutenExist = true
+        else if (objectName.contains("part_meta.gluten")) metadataBinExist = true
+        else if (objectName.contains("part_data.gluten")) dataBinExist = true
+        else if (objectName.contains("_commits")) hasCommits = true
+    }
 
     if (isSparkVersionGE("3.5")) {
-      assertResult(6)(objectCount)
+      assertResult(6)(countObjects(objectNames.toSeq))
       assert(hasCommits)
     } else {
-      assertResult(5)(objectCount)
+      assertResult(5)(countObjects(objectNames.toSeq))
     }
 
     assert(metadataGlutenExist)
@@ -202,6 +150,14 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
     assert(dataBinExist)
   }
 
+  def countObjects(objs: Seq[String]): Int = {
+    val count = objs
+      .filter(!_.endsWith(".crc"))
+      .filter(!_.endsWith("vacuum_info"))
+      .count(!_.endsWith("_SUCCESS"))
+    count
+  }
+
   test("test mergetree write with orderby keys / primary keys") {
     spark.sql(s"""
                  |DROP TABLE IF EXISTS lineitem_mergetree_orderbykey_s3;
@@ -239,7 +195,7 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_orderbykey_s3")) {
+    customCheckQuery(q1("lineitem_mergetree_orderbykey_s3")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -372,7 +328,7 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
                  |  where l_returnflag = 'A'
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_partition_s3"), compareResult = false) {
+    customCheckQuery(q1("lineitem_mergetree_partition_s3"), compare = false) {
       df =>
         val result = df.collect()
         assertResult(4)(result.length)
@@ -457,7 +413,7 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_bucket_s3")) {
+    customCheckQuery(q1("lineitem_mergetree_bucket_s3")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -514,7 +470,7 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
       .option("clickhouse.storage_policy", "__s3_main")
       .save(dataPath)
 
-    runTPCHQueryBySQL(1, q1(s"clickhouse.`$dataPath`")) {
+    customCheckQuery(q1(s"clickhouse.`$dataPath`")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -557,8 +513,8 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
 
     withSQLConf(
       "spark.databricks.delta.optimize.minFileSize" -> "200000000",
-      CHConfig.runtimeSettings("mergetree.insert_without_local_storage") -> "true",
-      CHConfig.runtimeSettings("mergetree.merge_after_insert") -> "true"
+      RuntimeSettings.INSERT_WITHOUT_LOCAL_STORAGE.key -> "true",
+      RuntimeSettings.MERGE_AFTER_INSERT.key -> "true"
     ) {
       spark.sql(s"""
                    |DROP TABLE IF EXISTS $tableName;
@@ -615,10 +571,10 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
                  | select * from lineitem
                  |""".stripMargin)
 
-    FileUtils.forceDelete(new File(S3_METADATA_PATH))
+    minioHelper.resetMeta()
 
     withSQLConf(CHConfig.runtimeSettings("enabled_driver_filter_mergetree_index") -> "true") {
-      runTPCHQueryBySQL(6, q6(tableName)) {
+      customCheckQuery(q6(tableName)) {
         df =>
           val scanExec = collect(df.queryExecution.executedPlan) {
             case f: FileSourceScanExecTransformer => f
@@ -675,7 +631,7 @@ class GlutenClickHouseMergeTreeWriteOnS3Suite
                  | select /*+ REPARTITION(3) */ * from lineitem
                  |""".stripMargin)
 
-    FileUtils.deleteDirectory(new File(S3_METADATA_PATH))
+    minioHelper.resetMeta()
     spark.sql("optimize lineitem_mergetree_bucket_s3")
     spark.sql("drop table lineitem_mergetree_bucket_s3")
   }
