@@ -16,7 +16,7 @@
  */
 package org.apache.gluten.execution.mergetree
 
-import org.apache.gluten.backendsapi.clickhouse.{CHConfig, RuntimeSettings}
+import org.apache.gluten.backendsapi.clickhouse.{CHBackendSettings, CHConfig, RuntimeSettings}
 import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.execution._
 import org.apache.gluten.utils.Arm
@@ -26,7 +26,6 @@ import org.apache.spark.sql.{DataFrame, SaveMode}
 import org.apache.spark.sql.delta.catalog.ClickHouseTableV2
 import org.apache.spark.sql.delta.files.TahoeFileIndex
 import org.apache.spark.sql.execution.LocalTableScanExec
-import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.execution.datasources.mergetree.StorageMeta
 import org.apache.spark.sql.execution.datasources.v2.clickhouse.metadata.AddMergeTreeParts
 
@@ -36,15 +35,7 @@ import java.io.File
 
 import scala.io.Source
 
-class GlutenClickHouseMergeTreeWriteSuite
-  extends GlutenClickHouseTPCHAbstractSuite
-  with AdaptiveSparkPlanHelper {
-
-  override protected val needCopyParquetToTablePath = true
-
-  override protected val tablesPath: String = basePath + "/tpch-data"
-  override protected val tpchQueries: String = rootPath + "queries/tpch-queries-ch"
-  override protected val queriesResults: String = rootPath + "mergetree-queries-output"
+class GlutenClickHouseMergeTreeWriteSuite extends CreateMergeTreeSuite {
 
   import org.apache.gluten.backendsapi.clickhouse.CHConfig._
 
@@ -60,14 +51,10 @@ class GlutenClickHouseMergeTreeWriteSuite
       .set(GlutenConfig.NATIVE_WRITER_ENABLED.key, "true")
       .set(CHConfig.ENABLE_ONEPIPELINE_MERGETREE_WRITE.key, spark35.toString)
       .set(RuntimeSettings.MIN_INSERT_BLOCK_SIZE_ROWS.key, "100000")
-      .setCHSettings("mergetree.merge_after_insert", false)
+      .set(RuntimeSettings.MERGE_AFTER_INSERT.key, "false")
       .setCHSettings("input_format_parquet_max_block_size", 8192)
   }
 
-  override protected def createTPCHNotNullTables(): Unit = {
-    createNotNullTPCHTablesInParquet(tablesPath)
-  }
-
   test("test mergetree table write") {
     spark.sql(s"""
                  |DROP TABLE IF EXISTS lineitem_mergetree;
@@ -96,7 +83,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |)
                  |USING clickhouse
                  |TBLPROPERTIES (write.format.default = 'mergetree')
-                 |LOCATION '$basePath/lineitem_mergetree'
+                 |LOCATION '$dataHome/lineitem_mergetree'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -104,7 +91,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree")) {
+    customCheckQuery(q1("lineitem_mergetree")) {
       df =>
         val plans = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -166,7 +153,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_insertoverwrite'
+                 |LOCATION '$dataHome/lineitem_mergetree_insertoverwrite'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -216,7 +203,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |)
                  |USING clickhouse
                  |PARTITIONED BY (l_shipdate)
-                 |LOCATION '$basePath/lineitem_mergetree_insertoverwrite2'
+                 |LOCATION '$dataHome/lineitem_mergetree_insertoverwrite2'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -268,7 +255,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                    |)
                    |USING clickhouse
                    |PARTITIONED BY (l_shipdate)
-                   |LOCATION '$basePath/lineitem_mergetree_insertoverwrite3'
+                   |LOCATION '$dataHome/lineitem_mergetree_insertoverwrite3'
                    |""".stripMargin)
 
       spark.sql(s"""
@@ -319,7 +306,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_update'
+                 |LOCATION '$dataHome/lineitem_mergetree_update'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -382,7 +369,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_delete'
+                 |LOCATION '$dataHome/lineitem_mergetree_delete'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -439,7 +426,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_upsert'
+                 |LOCATION '$dataHome/lineitem_mergetree_upsert'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -540,7 +527,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_shipdate,l_orderkey',
                  |               primaryKey='l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_orderbykey'
+                 |LOCATION '$dataHome/lineitem_mergetree_orderbykey'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -548,7 +535,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_orderbykey")) {
+    customCheckQuery(q1("lineitem_mergetree_orderbykey")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -606,7 +593,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |PARTITIONED BY (l_shipdate, l_returnflag)
                  |TBLPROPERTIES (orderByKey='l_orderkey',
                  |               primaryKey='l_orderkey')
-                 |LOCATION '$basePath/lineitem_mergetree_partition'
+                 |LOCATION '$dataHome/lineitem_mergetree_partition'
                  |""".stripMargin)
 
     // dynamic partitions
@@ -678,7 +665,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |  where l_shipdate BETWEEN date'1993-02-01' AND date'1993-02-10'
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_partition"), compareResult = false) {
+    customCheckQuery(q1("lineitem_mergetree_partition"), compare = false) {
       df =>
         val result = df.collect()
         assertResult(4)(result.length)
@@ -760,7 +747,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |PARTITIONED BY (l_returnflag)
                  |CLUSTERED BY (l_partkey)
                  |${if (spark32) "" else "SORTED BY (l_orderkey)"} INTO 4 BUCKETS
-                 |LOCATION '$basePath/lineitem_mergetree_bucket'
+                 |LOCATION '$dataHome/lineitem_mergetree_bucket'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -768,7 +755,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_bucket")) {
+    customCheckQuery(q1("lineitem_mergetree_bucket")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -918,22 +905,22 @@ class GlutenClickHouseMergeTreeWriteSuite
 
     // test table with the specified location
     tableName = "lineitem_mergetree_location_drop"
-    tableLocation = basePath + "/" + tableName
+    tableLocation = dataHome + "/" + tableName
     createAndDropTable(tableName, tableLocation)
     checkTableExists(tableName, tableLocation, exceptedExists = true)
 
     tableName = "lineitem_mergetree_external_location_drop"
-    tableLocation = basePath + "/" + tableName
+    tableLocation = dataHome + "/" + tableName
     createAndDropTable(tableName, tableLocation, isExternal = true)
     checkTableExists(tableName, tableLocation, exceptedExists = true)
 
     tableName = "lineitem_mergetree_location_purge"
-    tableLocation = basePath + "/" + tableName
+    tableLocation = dataHome + "/" + tableName
     createAndDropTable(tableName, tableLocation, purgeTable = true)
     checkTableExists(tableName, tableLocation, exceptedExists = false)
 
     tableName = "lineitem_mergetree_external_location_purge"
-    tableLocation = basePath + "/" + tableName
+    tableLocation = dataHome + "/" + tableName
     createAndDropTable(tableName, tableLocation, isExternal = true, purgeTable = true)
     checkTableExists(tableName, tableLocation, exceptedExists = false)
   }
@@ -946,11 +933,11 @@ class GlutenClickHouseMergeTreeWriteSuite
     spark.sql(s"""
                  |CREATE TABLE lineitem_mergetree_ctas1
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_ctas1'
+                 |LOCATION '$dataHome/lineitem_mergetree_ctas1'
                  | as select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_ctas1")) {
+    customCheckQuery(q1("lineitem_mergetree_ctas1")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -984,12 +971,11 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |CREATE TABLE IF NOT EXISTS lineitem_mergetree_ctas2
                  |USING clickhouse
                  |PARTITIONED BY (l_shipdate)
-                 |LOCATION '$basePath/lineitem_mergetree_ctas2'
+                 |LOCATION '$dataHome/lineitem_mergetree_ctas2'
                  | as select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_ctas2")) { _ => {} }
-
+    checkQuery(q1("lineitem_mergetree_ctas2"))
   }
 
   test("test mergetree table with low cardinality column") {
@@ -1018,7 +1004,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_lowcard'
+                 |LOCATION '$dataHome/lineitem_mergetree_lowcard'
                  |TBLPROPERTIES('lowCardKey'='l_returnflag,L_LINESTATUS,l_quantity')
                  |""".stripMargin)
 
@@ -1027,8 +1013,8 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(1, q1("lineitem_mergetree_lowcard")) { _ => {} }
-    val directory = new File(s"$basePath/lineitem_mergetree_lowcard")
+    checkQuery(q1("lineitem_mergetree_lowcard"))
+    val directory = new File(s"$dataHome/lineitem_mergetree_lowcard")
     // find a folder whose name is like 48b70783-b3b8-4bf8-9c52-5261aead8e3e_0_006
     val partDir = directory.listFiles().filter(f => f.getName.length > 20).head
     val columnsFile = new File(partDir, "columns.txt")
@@ -1094,7 +1080,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_shipdate,l_orderkey',
                  |               primaryKey='l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_orderbykey2'
+                 |LOCATION '$dataHome/lineitem_mergetree_orderbykey2'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1102,7 +1088,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(6, q6("lineitem_mergetree_orderbykey2")) {
+    customCheckQuery(q6("lineitem_mergetree_orderbykey2")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -1165,7 +1151,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |)
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_orderbykey3'
+                 |LOCATION '$dataHome/lineitem_mergetree_orderbykey3'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1173,7 +1159,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(6, q6("lineitem_mergetree_orderbykey3")) {
+    customCheckQuery(q6("lineitem_mergetree_orderbykey3")) {
       df =>
         val scanExec = collect(df.queryExecution.executedPlan) {
           case f: FileSourceScanExecTransformer => f
@@ -1237,7 +1223,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_5061'
+                 |LOCATION '$dataHome/lineitem_mergetree_5061'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1288,7 +1274,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_returnflag,l_shipdate',
                  |               primaryKey='l_returnflag,l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_5062'
+                 |LOCATION '$dataHome/lineitem_mergetree_5062'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1376,7 +1362,7 @@ class GlutenClickHouseMergeTreeWriteSuite
            |    l_linestatus;
            |
            |""".stripMargin
-      runTPCHQueryBySQL(1, q1(tableName)) {
+      customCheckQuery(q1(tableName)) {
         df =>
           val scanExec = collect(df.queryExecution.executedPlan) {
             case f: FileSourceScanExecTransformer => f
@@ -1402,7 +1388,7 @@ class GlutenClickHouseMergeTreeWriteSuite
     spark.sql(s"""
                  |CREATE TABLE lineitem_mergetree_ctas_5219
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_mergetree_ctas_5219'
+                 |LOCATION '$dataHome/lineitem_mergetree_ctas_5219'
                  | as select * from lineitem
                  |""".stripMargin)
 
@@ -1417,13 +1403,13 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_returnflag,l_shipdate',
                  |               primaryKey='l_returnflag,l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_ctas_5219_1'
+                 |LOCATION '$dataHome/lineitem_mergetree_ctas_5219_1'
                  | as select * from lineitem
                  |""".stripMargin)
 
     checkQueryResult("lineitem_mergetree_ctas_5219")
 
-    var dataPath = new File(s"$basePath/lineitem_mergetree_ctas_5219_1")
+    var dataPath = new File(s"$dataHome/lineitem_mergetree_ctas_5219_1")
     assert(dataPath.isDirectory && dataPath.isDirectory)
 
     val fileFilter = new WildcardFileFilter("*_0_*")
@@ -1458,7 +1444,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_returnflag,l_shipdate',
                  |               primaryKey='l_returnflag,l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_5219'
+                 |LOCATION '$dataHome/lineitem_mergetree_5219'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1495,7 +1481,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_shipdate',
                  |               primaryKey='l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_5219_1'
+                 |LOCATION '$dataHome/lineitem_mergetree_5219_1'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1505,7 +1491,7 @@ class GlutenClickHouseMergeTreeWriteSuite
 
     checkQueryResult("lineitem_mergetree_5219")
 
-    dataPath = new File(s"$basePath/lineitem_mergetree_5219_1")
+    dataPath = new File(s"$dataHome/lineitem_mergetree_5219_1")
     assert(dataPath.isDirectory && dataPath.isDirectory)
 
     dataFileList = dataPath.list(fileFilter)
@@ -1520,14 +1506,14 @@ class GlutenClickHouseMergeTreeWriteSuite
       spark.sql(s"""
                    |CREATE TABLE lineitem_mergetree_5219_s
                    |USING clickhouse
-                   |LOCATION '$basePath/lineitem_mergetree_5219_s'
+                   |LOCATION '$dataHome/lineitem_mergetree_5219_s'
                    | as select * from lineitem
                    |""".stripMargin)
 
       checkQueryResult("lineitem_mergetree_5219_s")
     }
 
-    dataPath = new File(s"$basePath/lineitem_mergetree_5219_s")
+    dataPath = new File(s"$dataHome/lineitem_mergetree_5219_s")
     assert(dataPath.isDirectory && dataPath.isDirectory)
 
     dataFileList = dataPath.list(fileFilter)
@@ -1561,7 +1547,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |)
                  |USING clickhouse
                  |TBLPROPERTIES (orderByKey='l_shipdate')
-                 |LOCATION '$basePath/lineitem_mergetree_pk_pruning_by_driver'
+                 |LOCATION '$dataHome/lineitem_mergetree_pk_pruning_by_driver'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1572,7 +1558,7 @@ class GlutenClickHouseMergeTreeWriteSuite
     Seq(("true", 2), ("false", 3)).foreach(
       conf => {
         withSQLConf(CHConfig.runtimeSettings("enabled_driver_filter_mergetree_index") -> conf._1) {
-          runTPCHQueryBySQL(6, q6("lineitem_mergetree_pk_pruning_by_driver")) {
+          customCheckQuery(q6("lineitem_mergetree_pk_pruning_by_driver")) {
             df =>
               val scanExec = collect(df.queryExecution.executedPlan) {
                 case f: FileSourceScanExecTransformer => f
@@ -1596,7 +1582,7 @@ class GlutenClickHouseMergeTreeWriteSuite
       CHConfig.runtimeSettings("enabled_driver_filter_mergetree_index") -> "true",
       CHConfig.prefixOf("files.per.partition.threshold") -> "10"
     ) {
-      runTPCHQueryBySQL(6, q6("lineitem_mergetree_pk_pruning_by_driver")) {
+      customCheckQuery(q6("lineitem_mergetree_pk_pruning_by_driver")) {
         df =>
           val scanExec = collect(df.queryExecution.executedPlan) {
             case f: FileSourceScanExecTransformer => f
@@ -1646,7 +1632,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |CLUSTERED by (l_orderkey)
                  |${if (spark32) "" else "SORTED BY (l_receiptdate)"} INTO 2 BUCKETS
-                 |LOCATION '$basePath/lineitem_mergetree_pk_pruning_by_driver_bucket'
+                 |LOCATION '$dataHome/lineitem_mergetree_pk_pruning_by_driver_bucket'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1663,7 +1649,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |CLUSTERED by (o_orderkey)
                  |${if (spark32) "" else "SORTED BY (o_orderdate)"} INTO 2 BUCKETS
-                 |LOCATION '$basePath/orders_mergetree_pk_pruning_by_driver_bucket'
+                 |LOCATION '$dataHome/orders_mergetree_pk_pruning_by_driver_bucket'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1712,7 +1698,7 @@ class GlutenClickHouseMergeTreeWriteSuite
     Seq(("true", 2), ("false", 2)).foreach(
       conf => {
         withSQLConf(CHConfig.runtimeSettings("enabled_driver_filter_mergetree_index") -> conf._1) {
-          runTPCHQueryBySQL(12, sqlStr) {
+          customCheckQuery(sqlStr -> 12) {
             df =>
               val scanExec = collect(df.queryExecution.executedPlan) {
                 case f: BasicScanExecTransformer => f
@@ -1753,7 +1739,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |PARTITIONED BY (l_shipdate, l_returnflag)
                  |TBLPROPERTIES (orderByKey='l_orderkey',
                  |               primaryKey='l_orderkey')
-                 |LOCATION '$basePath/lineitem_mergetree_count_opti'
+                 |LOCATION '$dataHome/lineitem_mergetree_count_opti'
                  |""".stripMargin)
 
     // dynamic partitions
@@ -1815,7 +1801,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |USING clickhouse
                  |PARTITIONED BY (L_SHIPDATE)
                  |TBLPROPERTIES (orderByKey='L_DISCOUNT')
-                 |LOCATION '$basePath/LINEITEM_MERGETREE_CASE_SENSITIVE'
+                 |LOCATION '$dataHome/LINEITEM_MERGETREE_CASE_SENSITIVE'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1823,7 +1809,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | select * from lineitem
                  |""".stripMargin)
 
-    runTPCHQueryBySQL(6, q6("lineitem_mergetree_case_sensitive")) { _ => }
+    checkQuery(q6("lineitem_mergetree_case_sensitive"))
   }
 
   test("test mergetree with partition with whitespace") {
@@ -1839,7 +1825,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |)
                  |USING clickhouse
                  |PARTITIONED BY (l_returnflag)
-                 |LOCATION '$basePath/lineitem_mergetree_partition_with_whitespace'
+                 |LOCATION '$dataHome/lineitem_mergetree_partition_with_whitespace'
                  |""".stripMargin)
 
     spark.sql(s"""
@@ -1879,7 +1865,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  | l_comment       string
                  |)
                  |USING clickhouse
-                 |LOCATION '$basePath/lineitem_split'
+                 |LOCATION '$dataHome/lineitem_split'
                  |""".stripMargin)
     spark.sql(s"""
                  | insert into table lineitem_split
@@ -1887,8 +1873,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                  |""".stripMargin)
     Seq(("-1", 3), ("3", 3), ("6", 1)).foreach(
       conf => {
-        withSQLConf(
-          "spark.gluten.sql.columnar.backend.ch.files.per.partition.threshold" -> conf._1) {
+        withSQLConf(CHBackendSettings.GLUTEN_CLICKHOUSE_FILES_PER_PARTITION_THRESHOLD -> conf._1) {
           val sql =
             s"""
                |select count(1), min(l_returnflag) from lineitem_split
@@ -1941,7 +1926,7 @@ class GlutenClickHouseMergeTreeWriteSuite
                      |PARTITIONED BY (l_returnflag)
                      |TBLPROPERTIES (orderByKey='l_orderkey',
                      |               primaryKey='l_orderkey')
-                     |LOCATION '$basePath/lineitem_mergetree_stats_skipping'
+                     |LOCATION '$dataHome/lineitem_mergetree_stats_skipping'
                      |""".stripMargin)
 
         // dynamic partitions
