@@ -17,31 +17,30 @@
 package org.apache.gluten.execution.datasource.v2
 
 import org.apache.spark.rdd.RDD
-import org.apache.spark.sql.catalyst.InternalRow
-import org.apache.spark.sql.catalyst.expressions.Attribute
-import org.apache.spark.sql.catalyst.plans.physical.Partitioning
-import org.apache.spark.sql.connector.read.{Batch, PartitionReaderFactory, Scan}
 import org.apache.spark.sql.execution.BaseArrowScanExec
 import org.apache.spark.sql.execution.datasources.v2.{ArrowBatchScanExecShim, BatchScanExec}
+import org.apache.spark.sql.execution.metric.SQLMetrics
+import org.apache.spark.sql.vectorized.ColumnarBatch
 
 case class ArrowBatchScanExec(original: BatchScanExec)
   extends ArrowBatchScanExecShim(original)
   with BaseArrowScanExec {
-
-  @transient lazy val batch: Batch = original.batch
-
-  override lazy val readerFactory: PartitionReaderFactory = original.readerFactory
-
-  override lazy val inputRDD: RDD[InternalRow] = original.inputRDD
-
-  override def outputPartitioning: Partitioning = original.outputPartitioning
-
-  override def scan: Scan = original.scan
-
   override def doCanonicalize(): ArrowBatchScanExec =
     this.copy(original = original.doCanonicalize())
 
   override def nodeName: String = "Arrow" + original.nodeName
 
-  override def output: Seq[Attribute] = original.output
+  override lazy val metrics = {
+    Map("numOutputRows" -> SQLMetrics.createMetric(sparkContext, "number of output rows")) ++
+      customMetrics
+  }
+
+  override def doExecuteColumnar(): RDD[ColumnarBatch] = {
+    val numOutputRows = longMetric("numOutputRows")
+    inputRDD.asInstanceOf[RDD[ColumnarBatch]].map {
+      b =>
+        numOutputRows += b.numRows()
+        b
+    }
+  }
 }
