@@ -17,8 +17,7 @@
 package org.apache.gluten.vectorized
 
 import org.apache.gluten.backendsapi.BackendsApiManager
-import org.apache.gluten.config.GlutenConfig
-import org.apache.gluten.config.ReservedKeys
+import org.apache.gluten.config.{GlutenConfig, ShuffleWriterType}
 import org.apache.gluten.iterator.ClosableIterator
 import org.apache.gluten.memory.arrow.alloc.ArrowBufferAllocators
 import org.apache.gluten.runtime.Runtimes
@@ -51,13 +50,10 @@ class ColumnarBatchSerializer(
     numOutputRows: SQLMetric,
     deserializeTime: SQLMetric,
     decompressTime: SQLMetric,
-    isSort: Boolean)
+    shuffleWriterType: ShuffleWriterType)
   extends Serializer
   with Serializable {
 
-  private val shuffleWriterType =
-    if (isSort) ReservedKeys.GLUTEN_SORT_SHUFFLE_WRITER else ReservedKeys.GLUTEN_HASH_SHUFFLE_WRITER
-
   /** Creates a new [[SerializerInstance]]. */
   override def newInstance(): SerializerInstance = {
     new ColumnarBatchSerializerInstance(
@@ -78,7 +74,7 @@ private class ColumnarBatchSerializerInstance(
     numOutputRows: SQLMetric,
     deserializeTime: SQLMetric,
     decompressTime: SQLMetric,
-    shuffleWriterType: String)
+    shuffleWriterType: ShuffleWriterType)
   extends SerializerInstance
   with Logging {
 
@@ -100,7 +96,8 @@ private class ColumnarBatchSerializerInstance(
     val compressionCodecBackend =
       GlutenConfig.get.columnarShuffleCodecBackend.orNull
     val batchSize = GlutenConfig.get.maxBatchSize
-    val bufferSize = GlutenConfig.get.columnarShuffleReaderBufferSize
+    val readerBufferSize = GlutenConfig.get.columnarShuffleReaderBufferSize
+    val deserializerBufferSize = GlutenConfig.get.columnarSortShuffleDeserializerBufferSize
     val runtime = Runtimes.contextInstance(BackendsApiManager.getBackendName, "ShuffleReader")
     val jniWrapper = ShuffleReaderJniWrapper.create(runtime)
     val shuffleReaderHandle = jniWrapper.make(
@@ -108,8 +105,9 @@ private class ColumnarBatchSerializerInstance(
       compressionCodec,
       compressionCodecBackend,
       batchSize,
-      bufferSize,
-      shuffleWriterType)
+      readerBufferSize,
+      deserializerBufferSize,
+      shuffleWriterType.name)
     // Close shuffle reader instance as lately as the end of task processing,
     // since the native reader could hold a reference to memory pool that
     // was used to create all buffers read from shuffle reader. The pool
