@@ -16,10 +16,9 @@
  */
 package org.apache.spark.api.python
 
-import org.apache.gluten.columnarbatch.ArrowBatches.ArrowJavaBatch
+import org.apache.gluten.backendsapi.arrow.ArrowBatchTypes.ArrowJavaBatchType
 import org.apache.gluten.columnarbatch.ColumnarBatches
-import org.apache.gluten.exception.GlutenException
-import org.apache.gluten.execution.GlutenPlan
+import org.apache.gluten.execution.{ValidatablePlan, ValidationResult}
 import org.apache.gluten.extension.columnar.transition.{Convention, ConventionReq}
 import org.apache.gluten.iterator.Iterators
 import org.apache.gluten.memory.arrow.alloc.ArrowBufferAllocators
@@ -28,11 +27,10 @@ import org.apache.gluten.vectorized.ArrowWritableColumnVector
 
 import org.apache.spark.{ContextAwareIterator, SparkEnv, TaskContext}
 import org.apache.spark.rdd.RDD
-import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions._
 import org.apache.spark.sql.execution.{ProjectExec, SparkPlan}
-import org.apache.spark.sql.execution.metric.SQLMetrics
-import org.apache.spark.sql.execution.python.{ArrowEvalPythonExec, BasePythonRunnerShim, EvalPythonExec, PythonUDFRunner}
+import org.apache.spark.sql.execution.metric.{SQLMetric, SQLMetrics}
+import org.apache.spark.sql.execution.python.{ArrowEvalPythonExec, BasePythonRunnerShim, EvalPythonExecBase}
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types.{DataType, StructField, StructType}
 import org.apache.spark.sql.utils.{SparkArrowUtil, SparkSchemaUtil, SparkVectorUtil}
@@ -43,20 +41,20 @@ import org.apache.arrow.vector.{VectorLoader, VectorSchemaRoot}
 import org.apache.arrow.vector.ipc.{ArrowStreamReader, ArrowStreamWriter}
 
 import java.io.{DataInputStream, DataOutputStream}
-import java.net.Socket
 import java.util.concurrent.atomic.AtomicBoolean
 
 import scala.collection.mutable
 import scala.collection.mutable.ArrayBuffer
 
 class ColumnarArrowPythonRunner(
-    funcs: Seq[ChainedPythonFunctions],
+    funcs: Seq[(ChainedPythonFunctions, Long)],
     evalType: Int,
     argOffsets: Array[Array[Int]],
     schema: StructType,
     timeZoneId: String,
-    conf: Map[String, String])
-  extends BasePythonRunnerShim(funcs.toSeq, evalType, argOffsets) {
+    conf: Map[String, String],
+    pythonMetrics: Map[String, SQLMetric])
+  extends BasePythonRunnerShim(funcs, evalType, argOffsets, pythonMetrics) {
 
   override val simplifiedTraceback: Boolean = SQLConf.get.pysparkSimplifiedTraceback
 
@@ -68,23 +66,15 @@ class ColumnarArrowPythonRunner(
 
   protected def newReaderIterator(
       stream: DataInputStream,
-      writerThread: WriterThread,
+      writer: Writer,
       startTime: Long,
       env: SparkEnv,
-      worker: Socket,
+      worker: PythonWorker,
       pid: scala.Option[scala.Int],
       releasedOrClosed: AtomicBoolean,
       context: TaskContext): Iterator[ColumnarBatch] = {
 
-    new ReaderIterator(
-      stream,
-      writerThread,
-      startTime,
-      env,
-      worker,
-      None,
-      releasedOrClosed,
-      context) {
+    new ReaderIterator(stream, writer, startTime, env, worker, pid, releasedOrClosed, context) {
       private val allocator = ArrowBufferAllocators.contextInstance()
 
       private var reader: ArrowStreamReader = _
@@ -105,8 +95,8 @@ class ColumnarArrowPythonRunner(
       private var batchLoaded = true
 
       override protected def read(): ColumnarBatch = {
-        if (writerThread.exception.isDefined) {
-          throw writerThread.exception.get
+        if (writer.exception.isDefined) {
+          throw writer.exception.get
         }
         try {
           if (reader != null && batchLoaded) {
@@ -145,13 +135,13 @@ class ColumnarArrowPythonRunner(
     }
   }
 
-  override protected def newWriterThread(
+  override def createNewWriter(
       env: SparkEnv,
-      worker: Socket,
+      worker: PythonWorker,
       inputIterator: Iterator[ColumnarBatch],
       partitionIndex: Int,
-      context: TaskContext): WriterThread = {
-    new WriterThread(env, worker, inputIterator, partitionIndex, context) {
+      context: TaskContext): Writer = {
+    new Writer(env, worker, inputIterator, partitionIndex, context) {
       override protected def writeCommand(dataOut: DataOutputStream): Unit = {
         // Write config for the worker as a number of key -> value pairs of strings
         dataOut.writeInt(conf.size)
@@ -159,10 +149,22 @@ class ColumnarArrowPythonRunner(
           PythonRDD.writeUTF(k, dataOut)
           PythonRDD.writeUTF(v, dataOut)
         }
-        PythonUDFRunner.writeUDFs(dataOut, funcs, argOffsets)
+        ColumnarArrowPythonRunner.this.writeUdf(dataOut, argOffsets)
       }
 
-      override protected def writeIteratorToStream(dataOut: DataOutputStream): Unit = {
+      // For Spark earlier than 4.0. It overrides the corresponding abstract method
+      // in Writer class. We omitted the override keyword for compatibility consideration.
+      def writeIteratorToStream(dataOut: DataOutputStream): Unit = {
+        writeToStreamHelper(dataOut)
+      }
+
+      // For Spark 4.0. It overrides the corresponding abstract method in Writer class.
+      // We omitted the override keyword for compatibility consideration.
+      def writeNextInputToStream(dataOut: DataOutputStream): Boolean = {
+        writeToStreamHelper(dataOut)
+      }
+
+      def writeToStreamHelper(dataOut: DataOutputStream): Boolean = {
         var numRows: Long = 0
         val arrowSchema = SparkSchemaUtil.toArrowSchema(schema, timeZoneId)
         val allocator = ArrowBufferAllocators.contextInstance()
@@ -195,6 +197,7 @@ class ColumnarArrowPythonRunner(
           // It could throw exception if the output stream is closed, so it should be
           // in the try block.
           writer.end()
+          true
         } {
           root.close()
           // allocator can't close now or the data will loss
@@ -203,6 +206,7 @@ class ColumnarArrowPythonRunner(
       }
     }
   }
+
 }
 
 case class ColumnarArrowEvalPythonExec(
@@ -210,15 +214,31 @@ case class ColumnarArrowEvalPythonExec(
     resultAttrs: Seq[Attribute],
     child: SparkPlan,
     evalType: Int)
-  extends EvalPythonExec
-  with GlutenPlan {
+  extends EvalPythonExecBase
+  with ValidatablePlan {
 
-  override def batchType(): Convention.BatchType = ArrowJavaBatch
+  override def batchType(): Convention.BatchType = ArrowJavaBatchType
 
   override def rowType0(): Convention.RowType = Convention.RowType.None
 
+  override protected def doValidateInternal(): ValidationResult = {
+    val (_, inputs) = udfs.map(ColumnarArrowEvalPythonExec.collectFunctions).unzip
+    inputs.foreach {
+      input =>
+        input.foreach {
+          case e: AttributeReference if child.output.exists(_.exprId == e.exprId) =>
+          // Valid case, continue validation
+          case _: AttributeReference =>
+            return ValidationResult.failed("Expression Id does not exist for AttributeReference")
+          case _ =>
+            return ValidationResult.failed("UDF input is not an instance of AttributeReference")
+        }
+    }
+    super.doValidateInternal()
+  }
+
   override def requiredChildConvention(): Seq[ConventionReq] = List(
-    ConventionReq.ofBatch(ConventionReq.BatchType.Is(ArrowJavaBatch)))
+    ConventionReq.ofBatch(ConventionReq.BatchType.Is(ArrowJavaBatchType)))
 
   override lazy val metrics = Map(
     "numOutputRows" -> SQLMetrics.createMetric(sparkContext, "number of output rows"),
@@ -227,16 +247,6 @@ case class ColumnarArrowEvalPythonExec(
     "processTime" -> SQLMetrics.createTimingMetric(sparkContext, "totaltime_arrow_udf")
   )
 
-  override protected def evaluate(
-      funcs: Seq[ChainedPythonFunctions],
-      argOffsets: Array[Array[Int]],
-      iter: Iterator[InternalRow],
-      schema: StructType,
-      context: TaskContext): Iterator[InternalRow] = {
-    throw new IllegalStateException(
-      "ColumnarArrowEvalPythonExec doesn't support evaluate InternalRow.")
-  }
-
   private val sessionLocalTimeZone = conf.sessionLocalTimeZone
 
   private def getPythonRunnerConfMap(conf: SQLConf): Map[String, String] = {
@@ -253,7 +263,7 @@ case class ColumnarArrowEvalPythonExec(
   private val pythonRunnerConf = getPythonRunnerConfMap(conf)
 
   protected def evaluateColumnar(
-      funcs: Seq[ChainedPythonFunctions],
+      funcs: Seq[(ChainedPythonFunctions, Long)],
       argOffsets: Array[Array[Int]],
       iter: Iterator[ColumnarBatch],
       schema: StructType,
@@ -267,7 +277,8 @@ case class ColumnarArrowEvalPythonExec(
       argOffsets,
       schema,
       sessionLocalTimeZone,
-      pythonRunnerConf).compute(iter, context.partitionId(), context)
+      pythonRunnerConf,
+      Map()).compute(iter, context.partitionId(), context)
 
     columnarBatchIter.map {
       batch =>
@@ -280,18 +291,6 @@ case class ColumnarArrowEvalPythonExec(
     }
   }
 
-  private def collectFunctions(udf: PythonUDF): (ChainedPythonFunctions, Seq[Expression]) = {
-    udf.children match {
-      case Seq(u: PythonUDF) =>
-        val (chained, children) = collectFunctions(u)
-        (ChainedPythonFunctions(chained.funcs ++ Seq(udf.func)), children)
-      case children =>
-        // There should not be any other UDFs, or the children can't be evaluated directly.
-        assert(children.forall(_.find(_.isInstanceOf[PythonUDF]).isEmpty))
-        (ChainedPythonFunctions(Seq(udf.func).toSeq), udf.children)
-    }
-  }
-
   override protected def doExecuteColumnar(): RDD[ColumnarBatch] = {
     val numOutputRows = longMetric("numOutputRows")
     val numOutputBatches = longMetric("numOutputBatches")
@@ -301,7 +300,7 @@ case class ColumnarArrowEvalPythonExec(
     inputRDD.mapPartitions {
       iter =>
         val context = TaskContext.get()
-        val (pyFuncs, inputs) = udfs.map(collectFunctions).unzip
+        val (pyFuncs, inputs) = udfs.map(ColumnarArrowEvalPythonExec.collectFunctions).unzip
         // We only write the referred cols by UDFs to python worker. So we need
         // get corresponding offsets
         val allInputs = new ArrayBuffer[Expression]
@@ -311,19 +310,11 @@ case class ColumnarArrowEvalPythonExec(
           input =>
             input.map {
               e =>
-                if (!e.isInstanceOf[AttributeReference]) {
-                  throw new GlutenException(
-                    "ColumnarArrowEvalPythonExec should only have [AttributeReference] inputs.")
-                } else if (allInputs.exists(_.semanticEquals(e))) {
+                if (allInputs.exists(_.semanticEquals(e))) {
                   allInputs.indexWhere(_.semanticEquals(e))
                 } else {
-                  var offset: Int = -1
-                  offset = child.output.indexWhere(
+                  val offset = child.output.indexWhere(
                     _.exprId.equals(e.asInstanceOf[AttributeReference].exprId))
-                  if (offset == -1) {
-                    throw new GlutenException(
-                      "ColumnarArrowEvalPythonExec can't find referred input col.")
-                  }
                   originalOffsets += offset
                   allInputs += e
                   dataTypes += e.dataType
@@ -409,16 +400,22 @@ case class ColumnarArrowEvalPythonExec(
     copy(udfs, resultAttrs, newChild)
 }
 
-object PullOutArrowEvalPythonPreProjectHelper extends PullOutProjectHelper {
-  private def collectFunctions(udf: PythonUDF): (ChainedPythonFunctions, Seq[Expression]) = {
+object ColumnarArrowEvalPythonExec {
+
+  def collectFunctions(udf: PythonUDF): ((ChainedPythonFunctions, Long), Seq[Expression]) = {
     udf.children match {
       case Seq(u: PythonUDF) =>
-        val (chained, children) = collectFunctions(u)
-        (ChainedPythonFunctions(chained.funcs ++ Seq(udf.func)), children)
+        val ((chained, _), children) = collectFunctions(u)
+        ((ChainedPythonFunctions(chained.funcs ++ Seq(udf.func)), udf.resultId.id), children)
       case children =>
-        (ChainedPythonFunctions(Seq(udf.func).toSeq), udf.children)
+        // There should be no PythonUDF, or the children can't be evaluated directly.
+        assert(!children.exists(_.isInstanceOf[PythonUDF]))
+        ((ChainedPythonFunctions(Seq(udf.func)), udf.resultId.id), udf.children)
     }
   }
+}
+
+object PullOutArrowEvalPythonPreProjectHelper extends PullOutProjectHelper {
 
   private def rewriteUDF(
       udf: PythonUDF,
@@ -441,12 +438,13 @@ object PullOutArrowEvalPythonPreProjectHelper extends PullOutProjectHelper {
 
   def pullOutPreProject(arrowEvalPythonExec: ArrowEvalPythonExec): SparkPlan = {
     // pull out preproject
-    val (_, inputs) = arrowEvalPythonExec.udfs.map(collectFunctions).unzip
+    val (_, inputs) =
+      arrowEvalPythonExec.udfs.map(ColumnarArrowEvalPythonExec.collectFunctions).unzip
     val expressionMap = new mutable.HashMap[Expression, NamedExpression]()
     // flatten all the arguments
     val allInputs = new ArrayBuffer[Expression]
     for (input <- inputs) {
-      input.map {
+      input.foreach {
         e =>
           if (!allInputs.exists(_.semanticEquals(e))) {
             allInputs += e
@@ -460,7 +458,9 @@ object PullOutArrowEvalPythonPreProjectHelper extends PullOutProjectHelper {
         eliminateProjectList(arrowEvalPythonExec.child.outputSet, expressionMap.values.toSeq),
         arrowEvalPythonExec.child)
       val newUDFs = arrowEvalPythonExec.udfs.map(f => rewriteUDF(f, expressionMap))
-      arrowEvalPythonExec.copy(udfs = newUDFs, child = preProject)
+      val newArrowEvalPythonExec = arrowEvalPythonExec.copy(udfs = newUDFs, child = preProject)
+      newArrowEvalPythonExec.copyTagsFrom(arrowEvalPythonExec)
+      newArrowEvalPythonExec
     } else {
       arrowEvalPythonExec
     }
