@@ -16,23 +16,25 @@
  */
 package org.apache.gluten.execution
 
-import org.apache.gluten.config.GlutenConfig
+import org.apache.gluten.config.{GlutenConfig, GlutenCoreConfig, VeloxConfig}
 import org.apache.gluten.expression.VeloxDummyExpression
 import org.apache.gluten.sql.shims.SparkShimLoader
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.{AnalysisException, DataFrame, Row}
 import org.apache.spark.sql.execution._
-import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
+import org.apache.spark.sql.execution.adaptive.{AdaptiveSparkPlanHelper, AQEShuffleReadExec, ShuffleQueryStageExec}
 import org.apache.spark.sql.execution.joins.BroadcastNestedLoopJoinExec
 import org.apache.spark.sql.execution.window.WindowExec
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types._
 
+import java.nio.file.{Files, Paths}
 import java.util.concurrent.TimeUnit
 
 import scala.collection.JavaConverters
+import scala.collection.JavaConverters._
 
 class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSparkPlanHelper {
   protected val rootPath: String = getClass.getResource("/").getPath
@@ -312,7 +314,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
     checkLengthAndPlan(df, 5)
   }
 
-  testWithSpecifiedSparkVersion("coalesce validation", Some("3.4")) {
+  testWithMinSparkVersion("coalesce validation", "3.4") {
     withTempPath {
       path =>
         val data = "2019-09-09 01:02:03.456789"
@@ -353,7 +355,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
   test("window expression") {
     Seq(("sort", 0), ("streaming", 1)).foreach {
       case (windowType, localSortSize) =>
-        withSQLConf("spark.gluten.sql.columnar.backend.velox.window.type" -> windowType) {
+        withSQLConf(VeloxConfig.COLUMNAR_VELOX_WINDOW_TYPE.key -> windowType) {
           runQueryAndCompare(
             "select max(l_partkey) over" +
               " (partition by l_suppkey order by l_commitdate" +
@@ -713,10 +715,9 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
   test("combine small batches before shuffle") {
     val minBatchSize = 15
     withSQLConf(
-      "spark.gluten.sql.columnar.backend.velox.resizeBatches.shuffleInput" -> "true",
-      "spark.gluten.sql.columnar.maxBatchSize" -> "2",
-      "spark.gluten.sql.columnar.backend.velox.resizeBatches.shuffleInput.minSize" ->
-        s"$minBatchSize"
+      VeloxConfig.COLUMNAR_VELOX_RESIZE_BATCHES_SHUFFLE_INPUT.key -> "true",
+      GlutenConfig.COLUMNAR_MAX_BATCH_SIZE.key -> "2",
+      VeloxConfig.COLUMNAR_VELOX_RESIZE_BATCHES_SHUFFLE_INPUT_MIN_SIZE.key -> s"$minBatchSize"
     ) {
       val df = runQueryAndCompare(
         "select l_orderkey, sum(l_partkey) as sum from lineitem " +
@@ -734,8 +735,8 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
     }
 
     withSQLConf(
-      "spark.gluten.sql.columnar.backend.velox.resizeBatches.shuffleInput" -> "true",
-      "spark.gluten.sql.columnar.maxBatchSize" -> "2"
+      VeloxConfig.COLUMNAR_VELOX_RESIZE_BATCHES_SHUFFLE_INPUT.key -> "true",
+      GlutenConfig.COLUMNAR_MAX_BATCH_SIZE.key -> "2"
     ) {
       val df = runQueryAndCompare(
         "select l_orderkey, sum(l_partkey) as sum from lineitem " +
@@ -781,7 +782,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
   test("Improve the local sort ensure requirements") {
     withSQLConf(
       "spark.sql.autoBroadcastJoinThreshold" -> "-1",
-      "spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
       withTable("t1", "t2") {
         sql("""
               |create table t1 using parquet as
@@ -928,81 +929,98 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
 
   test("test explode/posexplode function") {
     Seq("explode", "posexplode").foreach {
-      func =>
-        // Literal: func(literal)
-        runQueryAndCompare(s"""
-                              |SELECT $func(array(1, 2, 3));
-                              |""".stripMargin) {
-          checkGlutenOperatorMatch[GenerateExecTransformer]
-        }
-        runQueryAndCompare(s"""
-                              |SELECT $func(map(1, 'a', 2, 'b'));
-                              |""".stripMargin) {
-          checkGlutenOperatorMatch[GenerateExecTransformer]
-        }
-        runQueryAndCompare(
-          s"""
-             |SELECT $func(array(map(1, 'a', 2, 'b'), map(3, 'c', 4, 'd'), map(5, 'e', 6, 'f')));
-             |""".stripMargin) {
-          checkGlutenOperatorMatch[GenerateExecTransformer]
-        }
-        runQueryAndCompare(s"""
-                              |SELECT $func(map(1, array(1, 2), 2, array(3, 4)));
-                              |""".stripMargin) {
-          checkGlutenOperatorMatch[GenerateExecTransformer]
-        }
+      f =>
+        Seq(true, false).foreach {
+          isOuter =>
+            val func = if (isOuter) s"${f}_outer" else f
+            // Literal: func(literal)
+            runQueryAndCompare(s"""
+                                  |SELECT $func(array(1, 2, 3));
+                                  |""".stripMargin) {
+              checkGlutenOperatorMatch[GenerateExecTransformer]
+            }
+            runQueryAndCompare(s"""
+                                  |SELECT $func(map(1, 'a', 2, 'b'));
+                                  |""".stripMargin) {
+              checkGlutenOperatorMatch[GenerateExecTransformer]
+            }
+            runQueryAndCompare(
+              s"""
+                 |SELECT
+                 |  $func(array(map(1, 'a', 2, 'b'), map(3, 'c', 4, 'd'), map(5, '', 6, null)));
+                 |""".stripMargin) {
+              checkGlutenOperatorMatch[GenerateExecTransformer]
+            }
+            runQueryAndCompare(s"""
+                                  |SELECT $func(map(1, array(1, 2), 2, array(3, 4), 3, array()));
+                                  |""".stripMargin) {
+              checkGlutenOperatorMatch[GenerateExecTransformer]
+            }
 
-        // CreateArray/CreateMap: func(array(col)), func(map(k, v))
-        withTempView("t1") {
-          sql("""select * from values (1), (2), (3), (4)
-                |as tbl(a)
+            // CreateArray/CreateMap: func(array(col)), func(map(k, v))
+            withTempView("t1") {
+              sql("""select * from values (1), (2), (3), (4), (null)
+                    |as tbl(a)
          """.stripMargin).createOrReplaceTempView("t1")
-          runQueryAndCompare(s"""
-                                |SELECT $func(array(a)) from t1;
-                                |""".stripMargin) {
-            checkGlutenOperatorMatch[GenerateExecTransformer]
-          }
-          sql("""select * from values (1, 'a'), (2, 'b'), (3, null), (4, null)
-                |as tbl(a, b)
+              runQueryAndCompare(s"""
+                                    |SELECT $func(array(a)) from t1;
+                                    |""".stripMargin) {
+                checkGlutenOperatorMatch[GenerateExecTransformer]
+              }
+              sql("""select * from values (1, 'a'), (2, 'b'), (3, null), (4, null)
+                    |as tbl(a, b)
          """.stripMargin).createOrReplaceTempView("t1")
-          runQueryAndCompare(s"""
-                                |SELECT $func(map(a, b)) from t1;
-                                |""".stripMargin) {
-            checkGlutenOperatorMatch[GenerateExecTransformer]
-          }
-        }
+              runQueryAndCompare(s"""
+                                    |SELECT $func(map(a, b)) from t1;
+                                    |""".stripMargin) {
+                checkGlutenOperatorMatch[GenerateExecTransformer]
+              }
+            }
 
-        // AttributeReference: func(col)
-        withTempView("t2") {
-          sql("""select * from values
-                |  array(1, 2, 3),
-                |  array(4, null)
-                |as tbl(a)
+            // AttributeReference: func(col)
+            withTempView("t2") {
+              sql("""select * from values
+                    |  array(1, 2, 3),
+                    |  array(4, null),
+                    |  array(),
+                    |  null
+                    |as tbl(a)
          """.stripMargin).createOrReplaceTempView("t2")
-          runQueryAndCompare(s"""
-                                |SELECT $func(a) from t2;
-                                |""".stripMargin) {
-            // No ProjectExecTransformer is introduced.
-            checkSparkOperatorChainMatch[GenerateExecTransformer, FilterExecTransformer]
-          }
-          sql("""select * from values
-                |  map(1, 'a', 2, 'b', 3, null),
-                |  map(4, null)
-                |as tbl(a)
+              runQueryAndCompare(s"""
+                                    |SELECT $func(a) from t2;
+                                    |""".stripMargin) {
+                df =>
+                  if (!isOuter) {
+                    // No ProjectExecTransformer is introduced.
+                    checkSparkOperatorChainMatch[GenerateExecTransformer, FilterExecTransformer](df)
+                  }
+                  checkGlutenOperatorMatch[GenerateExecTransformer](df)
+              }
+              sql("""select * from values
+                    |  map(1, 'a', 2, 'b', 3, null),
+                    |  map(4, null),
+                    |  map(),
+                    |  null
+                    |as tbl(a)
          """.stripMargin).createOrReplaceTempView("t2")
-          runQueryAndCompare(s"""
-                                |SELECT $func(a) from t2;
-                                |""".stripMargin) {
-            // No ProjectExecTransformer is introduced.
-            checkSparkOperatorChainMatch[GenerateExecTransformer, FilterExecTransformer]
-          }
+              runQueryAndCompare(s"""
+                                    |SELECT $func(a) from t2;
+                                    |""".stripMargin) {
+                df =>
+                  if (!isOuter) {
+                    // No ProjectExecTransformer is introduced.
+                    checkSparkOperatorChainMatch[GenerateExecTransformer, FilterExecTransformer](df)
+                  }
+                  checkGlutenOperatorMatch[GenerateExecTransformer](df)
+              }
 
-          runQueryAndCompare(
-            s"""
-               |SELECT $func(${VeloxDummyExpression.VELOX_DUMMY_EXPRESSION}(a)) from t2;
-               |""".stripMargin) {
-            checkGlutenOperatorMatch[GenerateExecTransformer]
-          }
+              runQueryAndCompare(
+                s"""
+                   |SELECT $func(${VeloxDummyExpression.VELOX_DUMMY_EXPRESSION}(a)) from t2;
+                   |""".stripMargin) {
+                checkGlutenOperatorMatch[GenerateExecTransformer]
+              }
+            }
         }
     }
   }
@@ -1045,58 +1063,67 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
   }
 
   test("test inline function") {
-    // Literal: func(literal)
-    runQueryAndCompare(s"""
-                          |SELECT inline(array(
-                          |  named_struct('c1', 0, 'c2', 1),
-                          |  named_struct('c1', 2, 'c2', null)));
-                          |""".stripMargin) {
-      checkGlutenOperatorMatch[GenerateExecTransformer]
-    }
+    Seq(true, false).foreach {
+      isOuter =>
+        val func = if (isOuter) "inline_outer" else "inline"
 
-    // CreateArray: func(array(col))
-    withTempView("t1") {
-      sql("""SELECT * from values
-            |  (named_struct('c1', 0, 'c2', 1)),
-            |  (named_struct('c1', 2, 'c2', null)),
-            |  (null)
-            |as tbl(a)
+        // Literal: func(literal)
+        runQueryAndCompare(s"""
+                              |SELECT $func(array(
+                              |  named_struct('c1', 0, 'c2', 1),
+                              |  named_struct('c1', 2, 'c2', null)));
+                              |""".stripMargin) {
+          checkGlutenOperatorMatch[GenerateExecTransformer]
+        }
+
+        // CreateArray: func(array(col))
+        withTempView("t1") {
+          sql("""SELECT * from values
+                |  (named_struct('c1', 0, 'c2', 1)),
+                |  (named_struct('c1', 2, 'c2', null)),
+                |  (null)
+                |as tbl(a)
          """.stripMargin).createOrReplaceTempView("t1")
-      runQueryAndCompare(s"""
-                            |SELECT inline(array(a)) from t1;
-                            |""".stripMargin) {
-        checkGlutenOperatorMatch[GenerateExecTransformer]
-      }
-    }
+          runQueryAndCompare(s"""
+                                |SELECT $func(array(a)) from t1;
+                                |""".stripMargin) {
+            checkGlutenOperatorMatch[GenerateExecTransformer]
+          }
+        }
 
-    withTempView("t2") {
-      sql("""SELECT * from values
-            |  array(
-            |    named_struct('c1', 0, 'c2', 1),
-            |    null,
-            |    named_struct('c1', 2, 'c2', 3)
-            |  ),
-            |  array(
-            |    null,
-            |    named_struct('c1', 0, 'c2', 1),
-            |    named_struct('c1', 2, 'c2', 3)
-            |  )
-            |as tbl(a)
+        withTempView("t2") {
+          sql("""SELECT * from values
+                |  array(
+                |    named_struct('c1', 0, 'c2', 1),
+                |    null,
+                |    named_struct('c1', 2, 'c2', 3)
+                |  ),
+                |  array(
+                |    null,
+                |    named_struct('c1', 0, 'c2', 1),
+                |    named_struct('c1', 2, 'c2', 3)
+                |  ),
+                |  array(),
+                |  null
+                |as tbl(a)
          """.stripMargin).createOrReplaceTempView("t2")
-      runQueryAndCompare("""
-                           |SELECT inline(a) from t2;
-                           |""".stripMargin) {
-        checkGlutenOperatorMatch[GenerateExecTransformer]
-      }
-    }
+          runQueryAndCompare(s"""
+                                |SELECT $func(a) from t2;
+                                |""".stripMargin) {
+            checkGlutenOperatorMatch[GenerateExecTransformer]
+          }
+        }
 
-    // Fallback for array(struct(...), null) literal.
-    runQueryAndCompare(s"""
-                          |SELECT inline(array(
-                          |  named_struct('c1', 0, 'c2', 1),
-                          |  named_struct('c1', 2, 'c2', null),
-                          |  null));
-                          |""".stripMargin)(_)
+        // Fallback for array(struct(...), null) literal.
+        runQueryAndCompare(s"""
+                              |SELECT $func(array(
+                              |  named_struct('c1', 0, 'c2', 1),
+                              |  named_struct('c1', 2, 'c2', null),
+                              |  null));
+                              |""".stripMargin) {
+          checkSparkOperatorMatch[GenerateExec]
+        }
+    }
   }
 
   test("test multi-generate") {
@@ -1253,7 +1280,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
   }
 
   test("Test sample op") {
-    withSQLConf("spark.gluten.sql.columnarSampleEnabled" -> "true") {
+    withSQLConf(GlutenConfig.COLUMNAR_SAMPLE_ENABLED.key -> "true") {
       withTable("t") {
         sql("create table t (id int, b boolean) using parquet")
         sql("insert into t values (1, true), (2, false), (3, null), (4, true), (5, false)")
@@ -1275,7 +1302,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
             |select cast(id as int) as c1, cast(id as string) c2 from range(100) order by c1 desc;
             |""".stripMargin)
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "true") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "true") {
         runQueryAndCompare(
           """
             |select * from t1 cross join t2 on t1.c1 = t2.c1;
@@ -1295,7 +1322,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
         }
       }
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 cross join t2 on t1.c1 = t2.c1;
@@ -1305,7 +1332,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
         }
       }
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 left semi join t2 on t1.c1 = t2.c1 and t1.c1 > 50;
@@ -1353,7 +1380,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
             |create table t2 using parquet as
             |select cast(id as int) as c1, cast(id as string) c2 from range(100) order by c1 desc;
             |""".stripMargin)
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 inner join t2 on t1.c1 = t2.c1 and t1.c1 > 50;
@@ -1363,7 +1390,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
         }
       }
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 left join t2 on t1.c1 = t2.c1 and t1.c1 > 50;
@@ -1373,7 +1400,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
         }
       }
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 left semi join t2 on t1.c1 = t2.c1 and t1.c1 > 50;
@@ -1383,7 +1410,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
         }
       }
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 right join t2 on t1.c1 = t2.c1 and t1.c1 > 50;
@@ -1393,7 +1420,7 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
         }
       }
 
-      withSQLConf("spark.gluten.sql.columnar.forceShuffledHashJoin" -> "false") {
+      withSQLConf(GlutenConfig.COLUMNAR_FORCE_SHUFFLED_HASH_JOIN_ENABLED.key -> "false") {
         runQueryAndCompare(
           """
             |select * from t1 left anti join t2 on t1.c1 = t2.c1 and t1.c1 > 50;
@@ -1426,6 +1453,45 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
     }
   }
 
+  test("partitioned write column order") {
+    // Get a dataframe with a limited/known set of values for c_mktsegment and c_nationkey
+    // to test the directory structure created for partitioned writes
+    val df = sql(
+      "SELECT c_name, c_nationkey, c_mktsegment FROM customer " +
+        "WHERE c_mktsegment IN ('HOUSEHOLD', 'AUTOMOBILE') " +
+        "AND c_nationkey < 5")
+
+    withTempDir {
+      tempDir =>
+        val tempDirPath = tempDir.getPath
+        df.write
+          .format("parquet")
+          .partitionBy("c_mktsegment", "c_nationkey")
+          .mode("overwrite")
+          .save(tempDirPath)
+
+        // We expect the directory structure to look like:
+        // {tempDirPath}/c_mktsegment=AUTOMOBILE/c_nationkey=[0-4]/*.parquet
+        // {tempDirPath}/c_mktsegment=HOUSEHOLD/c_nationkey=[0-4]/*.parquet
+        val expectedDirs = for {
+          dir <- Seq("c_mktsegment=HOUSEHOLD", "c_mktsegment=AUTOMOBILE")
+          subDir <- 0 to 4
+        } yield Paths.get(tempDirPath, dir, "c_nationkey=" + subDir.toString).toString
+
+        // Each directory should have .parquet file(s)
+        expectedDirs.foreach {
+          dir =>
+            val path = Paths.get(dir)
+            assert(Files.exists(path) && Files.isDirectory(path))
+            val files = Files.list(path).iterator().asScala.toSeq
+            assert(files.nonEmpty)
+            val parquetFiles = files.filter(file => file.toString.contains(".parquet"))
+            assert(parquetFiles.nonEmpty && parquetFiles.size == files.size)
+        }
+
+    }
+  }
+
   test("timestamp cast fallback") {
     withTempPath {
       path =>
@@ -1971,11 +2037,11 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
   }
 
   test("test 'spark.gluten.enabled'") {
-    withSQLConf(GlutenConfig.GLUTEN_ENABLED.key -> "true") {
+    withSQLConf(GlutenCoreConfig.GLUTEN_ENABLED.key -> "true") {
       runQueryAndCompare("select * from lineitem limit 1") {
         checkGlutenOperatorMatch[FileSourceScanExecTransformer]
       }
-      withSQLConf(GlutenConfig.GLUTEN_ENABLED.key -> "false") {
+      withSQLConf(GlutenCoreConfig.GLUTEN_ENABLED.key -> "false") {
         runQueryAndCompare("select * from lineitem limit 1") {
           checkSparkOperatorMatch[FileSourceScanExec]
         }
@@ -2040,4 +2106,56 @@ class MiscOperatorSuite extends VeloxWholeStageTransformerSuite with AdaptiveSpa
       }
     }
   }
+
+  test("Blacklist expression can be handled by ColumnarPartialProject") {
+    withSQLConf(GlutenConfig.EXPRESSION_BLACK_LIST.key -> "regexp_replace") {
+      runQueryAndCompare(
+        "SELECT c_custkey, c_name, regexp_replace(c_comment, '\\w', 'something') FROM customer") {
+        df =>
+          val executedPlan = getExecutedPlan(df)
+          assert(executedPlan.count(_.isInstanceOf[ProjectExec]) == 0)
+          assert(executedPlan.count(_.isInstanceOf[ColumnarPartialProjectExec]) == 1)
+      }
+    }
+  }
+
+  test("Check VeloxResizeBatches is added in ShuffleRead") {
+    Seq(true, false).foreach(
+      coalesceEnabled => {
+        withSQLConf(
+          VeloxConfig.COLUMNAR_VELOX_RESIZE_BATCHES_SHUFFLE_OUTPUT.key -> "true",
+          SQLConf.SHUFFLE_PARTITIONS.key -> "10",
+          SQLConf.COALESCE_PARTITIONS_ENABLED.key -> coalesceEnabled.toString
+        ) {
+          runQueryAndCompare(
+            "SELECT l_orderkey, count(1) from lineitem group by l_orderkey".stripMargin) {
+            df =>
+              val executedPlan = getExecutedPlan(df)
+              if (coalesceEnabled) {
+                // VeloxResizeBatches(AQEShuffleRead(ShuffleQueryStage(ColumnarShuffleExchange)))
+                assert(executedPlan.sliding(4).exists {
+                  case Seq(
+                        _: ColumnarShuffleExchangeExec,
+                        _: ShuffleQueryStageExec,
+                        _: AQEShuffleReadExec,
+                        _: VeloxResizeBatchesExec
+                      ) =>
+                    true
+                  case _ => false
+                })
+              } else {
+                // VeloxResizeBatches(ShuffleQueryStage(ColumnarShuffleExchange))
+                assert(executedPlan.sliding(3).exists {
+                  case Seq(
+                        _: ColumnarShuffleExchangeExec,
+                        _: ShuffleQueryStageExec,
+                        _: VeloxResizeBatchesExec) =>
+                    true
+                  case _ => false
+                })
+              }
+          }
+        }
+      })
+  }
 }
