@@ -17,9 +17,9 @@
 package org.apache.gluten.execution
 
 import org.apache.spark.SparkConf
-import org.apache.spark.sql.Column
 import org.apache.spark.sql.catalyst.expressions.{Alias, Literal}
 import org.apache.spark.sql.catalyst.optimizer.{ConstantFolding, NullPropagation}
+import org.apache.spark.sql.classic.ClassicColumn
 import org.apache.spark.sql.functions.col
 import org.apache.spark.sql.types.StringType
 
@@ -38,7 +38,7 @@ class VeloxStringFunctionsSuite extends VeloxWholeStageTransformerSuite {
     createTPCHNotNullTables()
     spark
       .table("lineitem")
-      .select(col("*"), new Column(Alias(Literal(null, StringType), NULL_STR_COL)()))
+      .select(col("*"), ClassicColumn(Alias(Literal(null, StringType), NULL_STR_COL)()))
       .createOrReplaceTempView(LINEITEM_TABLE)
   }
 
@@ -418,6 +418,18 @@ class VeloxStringFunctionsSuite extends VeloxWholeStageTransformerSuite {
         s"from $LINEITEM_TABLE where l_comment rlike '%$$##@@#&&' limit $LENGTH") { _ => }
   }
 
+  testWithMinSparkVersion("ilike", "3.3") {
+    runQueryAndCompare(
+      s"select l_orderkey, l_comment, ilike(l_comment, 'a*') " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+    runQueryAndCompare(
+      s"select l_orderkey, ilike(l_comment, ' ') " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+    runQueryAndCompare(
+      s"select l_orderkey, ilike($NULL_STR_COL, '%a%') " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+  }
+
   test("regexp") {
     runQueryAndCompare(
       s"select l_orderkey, l_comment, regexp(l_comment, 'a*') " +
@@ -532,7 +544,7 @@ class VeloxStringFunctionsSuite extends VeloxWholeStageTransformerSuite {
         s"from $LINEITEM_TABLE limit 5") { _ => }
   }
 
-  testWithSpecifiedSparkVersion("split", Some("3.4")) {
+  testWithMinSparkVersion("split", "3.4") {
     runQueryAndCompare(
       s"select l_orderkey, l_comment, split(l_comment, '') " +
         s"from $LINEITEM_TABLE limit 5") {
@@ -639,4 +651,30 @@ class VeloxStringFunctionsSuite extends VeloxWholeStageTransformerSuite {
       s"select l_orderkey, left(l_comment, $NULL_STR_COL) " +
         s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
   }
+
+  test("right") {
+    runQueryAndCompare(
+      s"select l_orderkey, right(l_comment, 1) " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+
+    runQueryAndCompare(
+      s"select l_orderkey, right($NULL_STR_COL, 1) " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+
+    runQueryAndCompare(
+      s"select l_orderkey, right(l_comment, $NULL_STR_COL) " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+  }
+
+  testWithMinSparkVersion("luhn_check", "3.5") {
+    runQueryAndCompare(
+      s"select l_orderkey, luhn_check(l_comment) " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+  }
+
+  test("base64 and unbase64") {
+    runQueryAndCompare(
+      s"select l_orderkey, unbase64(base64(l_comment)) " +
+        s"from $LINEITEM_TABLE limit $LENGTH")(checkGlutenOperatorMatch[ProjectExecTransformer])
+  }
 }
