@@ -16,10 +16,12 @@
  */
 package org.apache.gluten.expression
 
+import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.execution.{ColumnarPartialProjectExec, WholeStageTransformerSuite}
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.catalyst.optimizer.{ConstantFolding, NullPropagation}
+import org.apache.spark.sql.execution.ProjectExec
 import org.apache.spark.sql.functions.udf
 
 import java.io.File
@@ -29,14 +31,14 @@ case class MyStruct(a: Long, b: Array[Long])
 class UDFPartialProjectSuiteRasOff extends UDFPartialProjectSuite {
   override protected def sparkConf: SparkConf = {
     super.sparkConf
-      .set("spark.gluten.ras.enabled", "false")
+      .set(GlutenConfig.RAS_ENABLED.key, "false")
   }
 }
 
 class UDFPartialProjectSuiteRasOn extends UDFPartialProjectSuite {
   override protected def sparkConf: SparkConf = {
     super.sparkConf
-      .set("spark.gluten.ras.enabled", "true")
+      .set(GlutenConfig.RAS_ENABLED.key, "true")
   }
 }
 
@@ -83,6 +85,15 @@ abstract class UDFPartialProjectSuite extends WholeStageTransformerSuite {
     }
   }
 
+  ignore("test subquery") {
+    runQueryAndCompare(
+      "select plus_one(" +
+        "(select plus_one(count(*)) from (values (1)) t0(inner_c))) as col " +
+        "from (values (2),(3)) t1(outer_c)") {
+      checkGlutenOperatorMatch[ColumnarPartialProjectExec]
+    }
+  }
+
   ignore("test plus_one with column used twice") {
     runQueryAndCompare(
       "SELECT sum(plus_one(cast(l_orderkey as long)) + hash(l_orderkey)) from lineitem") {
@@ -111,7 +122,7 @@ abstract class UDFPartialProjectSuite extends WholeStageTransformerSuite {
     }
   }
 
-  test("test function no argument") {
+  ignore("test function no argument") {
     runQueryAndCompare("""SELECT no_argument(), l_orderkey
                          | from lineitem limit 100""".stripMargin) {
       checkGlutenOperatorMatch[ColumnarPartialProjectExec]
@@ -204,4 +215,36 @@ abstract class UDFPartialProjectSuite extends WholeStageTransformerSuite {
       checkGlutenOperatorMatch[ColumnarPartialProjectExec]
     }
   }
+  // only SparkVersion >= 3.4 support columnar native writer
+  testWithSpecifiedSparkVersion(
+    "only the child and parent of the project both support Columnar," +
+      "just add ColumnarPartialProjectExec for the project",
+    "3.4",
+    "3.5") {
+    Seq("false", "true").foreach {
+      enableNativeScanAndWriter =>
+        withSQLConf(
+          GlutenConfig.NATIVE_WRITER_ENABLED.key -> enableNativeScanAndWriter,
+          GlutenConfig.COLUMNAR_BATCHSCAN_ENABLED.key -> enableNativeScanAndWriter
+        ) {
+          withTable("t1") {
+            spark.sql("""
+                        |create table if not exists t1 (revenue double) using parquet
+                        |""".stripMargin)
+            runQueryAndCompare(""" insert overwrite t1
+                                 |    select (plus_one(l_extendedprice) * l_discount
+                                 |      + hash(l_orderkey) + hash(l_comment)) as revenue
+                                 |    from   lineitem
+                                 |""".stripMargin) {
+
+              if (enableNativeScanAndWriter.toBoolean) {
+                checkGlutenOperatorMatch[ColumnarPartialProjectExec]
+              } else {
+                checkSparkOperatorMatch[ProjectExec]
+              }
+            }
+          }
+        }
+    }
+  }
 }
