@@ -33,11 +33,16 @@ object ColumnarTableCacheBenchmark extends SqlBasedBenchmark {
 
   private def doBenchmark(name: String, cardinality: Long)(f: => Unit): Unit = {
     val benchmark = new Benchmark(name, cardinality, output = output)
-    val flag = if (spark.sessionState.conf.getConf(GlutenConfig.COLUMNAR_TABLE_CACHE_ENABLED)) {
-      "enable"
-    } else {
-      "disable"
-    }
+    val flag =
+      if (
+        spark.sessionState.conf
+          .getConfString(GlutenConfig.COLUMNAR_TABLE_CACHE_ENABLED.key)
+          .toBoolean
+      ) {
+        "enable"
+      } else {
+        "disable"
+      }
     benchmark.addCase(s"$flag columnar table cache", 3)(_ => f)
     benchmark.run()
   }
