@@ -20,6 +20,7 @@
 #include <Columns/ColumnString.h>
 #include <Columns/ColumnsNumber.h>
 #include <Core/Block.h>
+#include <Core/Block_fwd.h>
 #include <DataTypes/DataTypeDate32.h>
 #include <DataTypes/DataTypeDateTime64.h>
 #include <DataTypes/DataTypeString.h>
@@ -28,6 +29,18 @@
 
 namespace local_engine
 {
+
+template <typename T>
+DB::SharedHeader toShared(T && block)
+{
+    return std::make_shared<const DB::Block>(std::forward<T>(block));
+}
+
+/**
+ * A simple alias for NamesAndTypesList
+ */
+using RowType = DB::NamesAndTypesList;
+
 inline DB::DataTypePtr BIGINT()
 {
     return std::make_shared<DB::DataTypeInt64>();
@@ -87,8 +100,8 @@ inline DB::ColumnWithTypeAndName toColumnType(const DB::NameAndTypePair & type)
     return DB::ColumnWithTypeAndName(type.type, type.name);
 }
 
-DB::Block toSampleBlock(const DB::NamesAndTypesList & type);
-DB::NamesAndTypesList blockToNameAndTypeList(const DB::Block & header);
+DB::Block toSampleBlock(const RowType & type);
+RowType blockToRowType(const DB::Block & header);
 DB::DataTypePtr wrapNullableType(bool nullable, DB::DataTypePtr nested_type);
 
 inline DB::DataTypePtr wrapNullableType(DB::DataTypePtr nested_type)
@@ -128,7 +141,7 @@ inline DB::NamesWithAliases buildNamesWithAliases(const DB::Block & input, const
     return aliases;
 }
 
-// begion of CppToDataType
+// begin of CppToDataType
 template <typename T>
 struct CppToDataType;
 
