@@ -35,8 +35,8 @@
 #include <Core/ServerSettings.h>
 #include <Core/Settings.h>
 #include <DataTypes/DataTypeArray.h>
-#include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypeMap.h>
+#include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypeString.h>
 #include <DataTypes/DataTypeTuple.h>
 #include <DataTypes/DataTypesDecimal.h>
@@ -188,7 +188,7 @@ BlockUtil::flattenBlock(const DB::Block & block, UInt64 flags, bool recursively,
         if (const DB::DataTypeArray * type_arr = typeid_cast<const DB::DataTypeArray *>(nested_type.get()))
         {
             const DB::DataTypeTuple * type_tuple = typeid_cast<const DB::DataTypeTuple *>(type_arr->getNestedType().get());
-            if (type_tuple && type_tuple->haveExplicitNames() && (flags & FLAT_NESTED_TABLE))
+            if (type_tuple && type_tuple->hasExplicitNames() && (flags & FLAT_NESTED_TABLE))
             {
                 const DB::DataTypes & element_types = type_tuple->getElements();
                 const DB::Strings & names = type_tuple->getElementNames();
@@ -239,7 +239,7 @@ BlockUtil::flattenBlock(const DB::Block & block, UInt64 flags, bool recursively,
         }
         else if (const DB::DataTypeTuple * type_tuple = typeid_cast<const DB::DataTypeTuple *>(nested_type.get()))
         {
-            if ((flags & FLAT_STRUCT_FORCE) || (type_tuple->haveExplicitNames() && (flags & FLAT_STRUCT)))
+            if ((flags & FLAT_STRUCT_FORCE) || (type_tuple->hasExplicitNames() && (flags & FLAT_STRUCT)))
             {
                 const DB::DataTypes & element_types = type_tuple->getElements();
                 DB::Strings element_names = type_tuple->getElementNames();
@@ -319,32 +319,6 @@ DB::Block BlockUtil::concatenateBlocksMemoryEfficiently(std::vector<DB::Block> &
     return out;
 }
 
-bool TypeUtil::hasNothingType(DB::DataTypePtr data_type)
-{
-    if (DB::isNothing(data_type))
-        return true;
-    else if (data_type->isNullable())
-        return hasNothingType(typeid_cast<const DB::DataTypeNullable *>(data_type.get())->getNestedType());
-    else if (DB::isArray(data_type))
-        return hasNothingType(typeid_cast<const DB::DataTypeArray *>(data_type.get())->getNestedType());
-    else if (DB::isMap(data_type))
-    {
-        const auto * type_map = typeid_cast<const DB::DataTypeMap *>(data_type.get());
-        return hasNothingType(type_map->getKeyType()) || hasNothingType(type_map->getValueType());
-    }
-    else if (DB::isTuple(data_type))
-    {
-        const auto * type_tuple = typeid_cast<const DB::DataTypeTuple *>(data_type.get());
-        for (size_t i = 0; i < type_tuple->getElements().size(); ++i)
-        {
-            if (hasNothingType(type_tuple->getElements()[i]))
-                return true;
-        }
-    }
-    return false;
-
-}
-
 size_t PODArrayUtil::adjustMemoryEfficientSize(size_t n)
 {
     /// According to definition of DEFUALT_BLOCK_SIZE
@@ -494,7 +468,7 @@ std::vector<String> BackendInitializerUtil::wrapDiskPathConfig(
     };
 
     Poco::Util::AbstractConfiguration::Keys disks;
-    std::unordered_set<String> disk_types = {"s3_gluten", "hdfs_gluten", "cache"};
+    std::unordered_set<String> disk_types = {GlutenObjectStorageConfig::S3_DISK_TYPE, GlutenObjectStorageConfig::HDFS_DISK_TYPE, "cache"};
     config.keys("storage_configuration.disks", disks);
 
     std::ranges::for_each(
@@ -507,7 +481,7 @@ std::vector<String> BackendInitializerUtil::wrapDiskPathConfig(
                 return;
             if (disk_type == "cache")
                 change_func(disk_prefix + ".path");
-            else if (disk_type == "s3_gluten" || disk_type == "hdfs_gluten")
+            else if (disk_type == GlutenObjectStorageConfig::S3_DISK_TYPE || disk_type == GlutenObjectStorageConfig::HDFS_DISK_TYPE)
                 change_func(disk_prefix + ".metadata_path");
         });
 
@@ -571,7 +545,8 @@ DB::Context::ConfigurationPtr BackendInitializerUtil::initConfig(const SparkConf
     // FIXMEX: workaround for https://github.com/ClickHouse/ClickHouse/pull/75452#pullrequestreview-2625467710
     // entry in DiskSelector::initialize
     // Bug in FileCacheSettings::loadFromConfig
-    auto updateCacheDiskType = [](Poco::Util::AbstractConfiguration & config) {
+    auto updateCacheDiskType = [](Poco::Util::AbstractConfiguration & config)
+    {
         const std::string config_prefix = "storage_configuration.disks";
         Poco::Util::AbstractConfiguration::Keys keys;
         config.keys(config_prefix, keys);
@@ -697,7 +672,7 @@ void BackendInitializerUtil::initSettings(const SparkConfigs::ConfigMap & spark_
     /// We currently do not support lazy materialization.
     /// "test 'order by' two keys" will failed if we enable it.
     settings[Setting::query_plan_optimize_lazy_materialization] = false;
-    
+
     for (const auto & [key, value] : spark_conf_map)
     {
         // Firstly apply spark.gluten.sql.columnar.backend.ch.runtime_config.local_engine.settings.* to settings
@@ -776,8 +751,6 @@ void BackendInitializerUtil::initSettings(const SparkConfigs::ConfigMap & spark_
     settings.set("enable_named_columns_in_function_tuple", false);
     settings.set("date_time_64_output_format_cut_trailing_zeros_align_to_groups_of_thousands", true);
     settings.set("input_format_orc_dictionary_as_low_cardinality", false); //after https://github.com/ClickHouse/ClickHouse/pull/69481
-    settings.set("input_format_parquet_bloom_filter_push_down", true);
-    settings.set("output_format_parquet_write_bloom_filter", true);
 
     if (spark_conf_map.contains(GLUTEN_TASK_OFFHEAP))
     {
@@ -790,7 +763,8 @@ void BackendInitializerUtil::initSettings(const SparkConfigs::ConfigMap & spark_
         {
             auto mem_gb = task_memory / static_cast<double>(1_GiB);
             // 2.8x+5, Heuristics calculate the block size of external sort, [8,16]
-            settings[Setting::prefer_external_sort_block_bytes] = std::max(std::min(static_cast<size_t>(2.8 * mem_gb + 5), 16ul), 8ul) * 1024 * 1024;
+            settings[Setting::prefer_external_sort_block_bytes]
+                = std::max(std::min(static_cast<size_t>(2.8 * mem_gb + 5), 16ul), 8ul) * 1024 * 1024;
         }
     }
 }
@@ -895,6 +869,7 @@ extern void registerAggregateFunctionCombinatorPartialMerge(AggregateFunctionCom
 extern void registerAggregateFunctionsBloomFilter(AggregateFunctionFactory &);
 extern void registerAggregateFunctionSparkAvg(AggregateFunctionFactory &);
 extern void registerAggregateFunctionRowNumGroup(AggregateFunctionFactory &);
+extern void registerAggregateFunctionDVRoaringBitmap(AggregateFunctionFactory &);
 
 
 extern void registerFunctions(FunctionFactory &);
@@ -909,6 +884,7 @@ void registerAllFunctions()
     registerAggregateFunctionSparkAvg(agg_factory);
     registerAggregateFunctionRowNumGroup(agg_factory);
     DB::registerAggregateFunctionUniqHyperLogLogPlusPlus(agg_factory);
+    registerAggregateFunctionDVRoaringBitmap(agg_factory);
 
     /// register aggregate function combinators from local_engine
     auto & combinator_factory = AggregateFunctionCombinatorFactory::instance();
@@ -1023,7 +999,7 @@ void BackendFinalizerUtil::finalizeGlobally()
 {
     // Make sure client caches release before ClientCacheRegistry
     ReadBufferBuilderFactory::instance().clean();
-    StorageMergeTreeFactory::clear();
+    StorageMergeTreeFactory::clear_cache_map();
     QueryContext::resetGlobal();
     std::lock_guard lock(paths_mutex);
     std::ranges::for_each(
@@ -1075,7 +1051,7 @@ UInt64 MemoryUtil::getMemoryRSS()
 
 void JoinUtil::reorderJoinOutput(DB::QueryPlan & plan, DB::Names cols)
 {
-    ActionsDAG project{plan.getCurrentHeader().getNamesAndTypesList()};
+    ActionsDAG project{plan.getCurrentHeader()->getNamesAndTypesList()};
     NamesWithAliases project_cols;
     for (const auto & col : cols)
     {
