@@ -31,7 +31,6 @@ struct ReadSettings;
 }
 namespace local_engine
 {
-
 struct SparkConfigs
 {
     using ConfigMap = google::protobuf::Map<std::string, std::string>;
@@ -196,4 +195,10 @@ struct GlutenCacheConfig
     inline static const String ENABLED = "enable.gluten_cache.local";
 };
 
+struct GlutenObjectStorageConfig
+{
+    inline static const String S3_DISK_TYPE = "s3_gluten";
+    inline static const String HDFS_DISK_TYPE = "hdfs_gluten";
+};
+
 }
