@@ -15,10 +15,14 @@
  * limitations under the License.
  */
 #include "config.h"
+
 #include <Core/Settings.h>
 #include <Disks/ObjectStorages/ObjectStorageFactory.h>
+#include <Interpreters/Context.h>
+#include <Common/GlutenConfig.h>
+#include <Common/Macros.h>
+
 #if USE_AWS_S3
-#include <Disks/ObjectStorages/S3/DiskS3Utils.h>
 #include <Disks/ObjectStorages/S3/S3ObjectStorage.h>
 #include <Disks/ObjectStorages/S3/diskSettings.h>
 #endif
@@ -27,10 +31,6 @@
 #include <Disks/ObjectStorages/GlutenHDFSObjectStorage.h>
 #endif
 
-#include <Interpreters/Context.h>
-#include <Common/Macros.h>
-
-
 namespace DB
 {
 namespace Setting
@@ -50,10 +50,7 @@ namespace local_engine
 using namespace DB;
 
 #if USE_AWS_S3
-static S3::URI getS3URI(
-    const Poco::Util::AbstractConfiguration & config,
-    const std::string & config_prefix,
-    const ContextPtr & context)
+static S3::URI getS3URI(const Poco::Util::AbstractConfiguration & config, const std::string & config_prefix, const ContextPtr & context)
 {
     String endpoint = context->getMacros()->expand(config.getString(config_prefix + ".endpoint"));
     S3::URI uri(endpoint);
@@ -65,41 +62,32 @@ static S3::URI getS3URI(
     return uri;
 }
 
-static std::string getEndpoint(
-        const Poco::Util::AbstractConfiguration & config,
-        const std::string & config_prefix,
-        const ContextPtr & context)
+static std::string
+getEndpoint(const Poco::Util::AbstractConfiguration & config, const std::string & config_prefix, const ContextPtr & context)
 {
     return context->getMacros()->expand(config.getString(config_prefix + ".endpoint"));
 }
 
 void registerGlutenS3ObjectStorage(ObjectStorageFactory & factory)
 {
-    static constexpr auto disk_type = "s3_gluten";
-
     factory.registerObjectStorageType(
-        disk_type,
-        [](
-        const std::string & name,
-        const Poco::Util::AbstractConfiguration & config,
-        const std::string & config_prefix,
-        const ContextPtr & context,
-        bool /*skip_access_check*/) -> ObjectStoragePtr
+        GlutenObjectStorageConfig::S3_DISK_TYPE,
+        [](const std::string & name,
+           const Poco::Util::AbstractConfiguration & config,
+           const std::string & config_prefix,
+           const ContextPtr & context,
+           bool /*skip_access_check*/) -> ObjectStoragePtr
         {
             auto uri = getS3URI(config, config_prefix, context);
             auto s3_capabilities = getCapabilitiesFromConfig(config, config_prefix);
             auto endpoint = getEndpoint(config, config_prefix, context);
-            auto settings = getSettings(config, config_prefix, context, endpoint, /* validate_settings */true);
-            auto client = getClient(endpoint, *settings, context, /* for_disk_s3 */true);
+            auto settings = std::make_unique<S3Settings>();
+            settings->loadFromConfigForObjectStorage(config, config_prefix, context->getSettingsRef(), uri.uri.getScheme(), true);
+            auto client = getClient(endpoint, *settings, context, /* for_disk_s3 */ true);
             auto key_generator = createObjectStorageKeysGeneratorAsIsWithPrefix(uri.key);
 
-            auto object_storage = std::make_shared<S3ObjectStorage>(
-                std::move(client),
-                std::move(settings),
-                uri,
-                s3_capabilities,
-                key_generator,
-                name);
+            auto object_storage
+                = std::make_shared<S3ObjectStorage>(std::move(client), std::move(settings), uri, s3_capabilities, key_generator, name);
             return object_storage;
         });
 }
@@ -110,13 +98,12 @@ void registerGlutenS3ObjectStorage(ObjectStorageFactory & factory)
 void registerGlutenHDFSObjectStorage(ObjectStorageFactory & factory)
 {
     factory.registerObjectStorageType(
-        "hdfs_gluten",
-        [](
-            const std::string & /* name */,
-            const Poco::Util::AbstractConfiguration & config,
-            const std::string & config_prefix,
-            const ContextPtr & context,
-            bool /* skip_access_check */) -> ObjectStoragePtr
+        GlutenObjectStorageConfig::HDFS_DISK_TYPE,
+        [](const std::string & /* name */,
+           const Poco::Util::AbstractConfiguration & config,
+           const std::string & config_prefix,
+           const ContextPtr & context,
+           bool /* skip_access_check */) -> ObjectStoragePtr
         {
             auto uri = context->getMacros()->expand(config.getString(config_prefix + ".endpoint"));
             checkHDFSURL(uri);
@@ -124,9 +111,7 @@ void registerGlutenHDFSObjectStorage(ObjectStorageFactory & factory)
                 throw Exception(ErrorCodes::BAD_ARGUMENTS, "HDFS path must ends with '/', but '{}' doesn't.", uri);
 
             std::unique_ptr<HDFSObjectStorageSettings> settings = std::make_unique<HDFSObjectStorageSettings>(
-                config.getUInt64(config_prefix + ".min_bytes_for_seek", 1024 * 1024),
-                context->getSettingsRef()[Setting::hdfs_replication]
-            );
+                config.getUInt64(config_prefix + ".min_bytes_for_seek", 1024 * 1024), context->getSettingsRef()[Setting::hdfs_replication]);
             return std::make_shared<GlutenHDFSObjectStorage>(uri, std::move(settings), config);
         });
 }
