@@ -27,7 +27,7 @@ REGISTER_FUNCTION(RoundSpark)
 Similar to function round,except that in case when given number has equal distance to surrounding numbers, the function rounds away from zero(towards +inf/-inf).
         )",
             .examples{{"roundHalfUp", "SELECT roundHalfUp(3.165,2)", "3.17"}},
-            .category{"Rounding"}
+            .category = DB::FunctionDocumentation::Category::Rounding
         },
         DB::FunctionFactory::Case::Insensitive);
 
