@@ -14,6 +14,9 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#include <bitset>
+#include <memory>
+#include <string>
 #include <Columns/ColumnString.h>
 #include <DataTypes/DataTypeString.h>
 #include <Functions/FunctionFactory.h>
@@ -21,9 +24,6 @@
 #include <Functions/IFunction.h>
 #include <IO/WriteHelpers.h>
 
-#include <memory>
-#include <string>
-
 using namespace DB;
 
 namespace DB
@@ -32,6 +32,7 @@ namespace ErrorCodes
 {
     extern const int ILLEGAL_COLUMN;
     extern const int NUMBER_OF_ARGUMENTS_DOESNT_MATCH;
+    extern const int ILLEGAL_TYPE_OF_ARGUMENT;
 }
 }
 
