@@ -21,6 +21,7 @@
 #include <Processors/Port.h>
 #include <Processors/Transforms/AggregatingTransform.h>
 #include <QueryPipeline/QueryPipelineBuilder.h>
+#include <Common/BlockTypeUtils.h>
 #include <Common/CHUtil.h>
 
 namespace DB::ErrorCodes
@@ -48,8 +49,8 @@ static DB::Block buildOutputHeader(const DB::Block & input_header_, const DB::Ag
 }
 
 GraceAggregatingStep::GraceAggregatingStep(
-    DB::ContextPtr context_, const DB::Block & input_header, DB::Aggregator::Params params_, bool no_pre_aggregated_)
-    : DB::ITransformingStep(input_header, buildOutputHeader(input_header, params_, false), getTraits())
+    DB::ContextPtr context_, const DB::SharedHeader & input_header, DB::Aggregator::Params params_, bool no_pre_aggregated_)
+    : DB::ITransformingStep(input_header, toShared(buildOutputHeader(*input_header, params_, false)), getTraits())
     , context(context_)
     , params(std::move(params_))
     , no_pre_aggregated(no_pre_aggregated_)
@@ -63,7 +64,7 @@ void GraceAggregatingStep::transformPipeline(DB::QueryPipelineBuilder & pipeline
         throw DB::Exception(DB::ErrorCodes::LOGICAL_ERROR, "max_bytes_before_external_group_by is not supported in GraceAggregatingStep");
     }
     auto num_streams = pipeline.getNumStreams();
-    auto transform_params = std::make_shared<DB::AggregatingTransformParams>(pipeline.getHeader(), params, false);
+    auto transform_params = std::make_shared<DB::AggregatingTransformParams>(pipeline.getSharedHeader(), params, false);
     pipeline.resize(1);
     auto build_transform = [&](DB::OutputPortRawPtrs outputs)
     {
@@ -71,7 +72,7 @@ void GraceAggregatingStep::transformPipeline(DB::QueryPipelineBuilder & pipeline
         for (auto & output : outputs)
         {
             auto op
-                = std::make_shared<GraceAggregatingTransform>(pipeline.getHeader(), transform_params, context, no_pre_aggregated, false);
+                = std::make_shared<GraceAggregatingTransform>(pipeline.getSharedHeader(), transform_params, context, no_pre_aggregated, false);
             new_processors.push_back(op);
             DB::connect(*output, op->getInputs().front());
         }
@@ -93,7 +94,7 @@ void GraceAggregatingStep::describeActions(DB::JSONBuilder::JSONMap & map) const
 
 void GraceAggregatingStep::updateOutputHeader()
 {
-    output_header = buildOutputHeader(input_headers.front(), params, false);
+    output_header = toShared(buildOutputHeader(*input_headers.front(), params, false));
 }
 
 
