@@ -199,13 +199,7 @@ std::pair<DB::DataTypePtr, DB::Field> LiteralParser::parse(const substrait::Expr
 
                 DB::DataTypePtr key_type;
                 std::tie(key_type, tuple[0]) = parse(key_value.key());
-                /// Each key should has the same type
-                if (!common_key_type->equals(*key_type))
-                    throw DB::Exception(
-                        DB::ErrorCodes::LOGICAL_ERROR,
-                        "Literal map key type mismatch:{} and {}",
-                        common_key_type->getName(),
-                        key_type->getName());
+                common_key_type = getLeastSupertype(DB::DataTypes{common_key_type, key_type});
 
                 DB::DataTypePtr value_type;
                 std::tie(value_type, tuple[1]) = parse(key_value.value());
@@ -247,7 +241,6 @@ std::pair<DB::DataTypePtr, DB::Field> LiteralParser::parse(const substrait::Expr
         }
         case substrait::Expression_Literal::kNull: {
             type = TypeParser::parseType(literal.null());
-            type = TypeParser::tryWrapNullable(substrait::Type_Nullability::Type_Nullability_NULLABILITY_NULLABLE, type);
             field = DB::Field{};
             break;
         }
@@ -374,12 +367,7 @@ ExpressionParser::NodeRawConstPtr ExpressionParser::parseExpression(ActionsDAG &
             {
                 /// Common process: CAST(input, type)
                 args.emplace_back(addConstColumn(actions_dag, std::make_shared<DataTypeString>(), output_type->getName()));
-                if (TypeUtil::hasNothingType(args[0]->result_type))
-                {
-                    result_node = toFunctionNode(actions_dag, "accurateCastOrNull", args);
-                }
-                else
-                    result_node = toFunctionNode(actions_dag, "CAST", args);
+                result_node = toFunctionNode(actions_dag, "CAST", args);
             }
 
             actions_dag.addOrReplaceInOutputs(*result_node);
