@@ -235,8 +235,8 @@ DB::QueryPlanPtr AggregateGroupLimitRelParser::parse(
     addSortStep(*window_plan);
     addWindowLimitStep(*window_plan);
     auto convert_actions_dag = DB::ActionsDAG::makeConvertingActions(
-        window_plan->getCurrentHeader().getColumnsWithTypeAndName(),
-        aggregation_plan->getCurrentHeader().getColumnsWithTypeAndName(),
+        window_plan->getCurrentHeader()->getColumnsWithTypeAndName(),
+        aggregation_plan->getCurrentHeader()->getColumnsWithTypeAndName(),
         DB::ActionsDAG::MatchColumnsMode::Position);
     auto convert_step = std::make_unique<DB::ExpressionStep>(window_plan->getCurrentHeader(), std::move(convert_actions_dag));
     convert_step->setStepDescription("Rename rank column name");
@@ -265,7 +265,7 @@ String AggregateGroupLimitRelParser::getAggregateFunctionName(const String & win
 // Build one tuple column as the aggregate function's arguments
 void AggregateGroupLimitRelParser::prePrejectionForAggregateArguments(DB::QueryPlan & plan)
 {
-    auto projection_actions = std::make_shared<DB::ActionsDAG>(input_header.getColumnsWithTypeAndName());
+    auto projection_actions = std::make_shared<DB::ActionsDAG>(input_header->getColumnsWithTypeAndName());
 
     auto partition_fields = parsePartitionFields(win_rel_def->partition_expressions());
     auto sort_fields = parseSortFields(win_rel_def->sorts());
@@ -293,9 +293,9 @@ void AggregateGroupLimitRelParser::prePrejectionForAggregateArguments(DB::QueryP
     DB::DataTypes aggregate_data_tuple_types;
     Strings aggregate_data_tuple_names;
     DB::ActionsDAG::NodeRawConstPtrs aggregate_data_tuple_nodes;
-    for (size_t i = 0; i < input_header.columns(); ++i)
+    for (size_t i = 0; i < input_header->columns(); ++i)
     {
-        const auto & col = input_header.getByPosition(i);
+        const auto & col = input_header->getByPosition(i);
         if (unique_partition_fields.count(i) && !unique_sort_fields.count(i))
         {
             required_column_names.insert(col.name);
@@ -331,12 +331,12 @@ DB::AggregateDescription AggregateGroupLimitRelParser::buildAggregateDescription
     DB::AggregateDescription agg_desc;
     agg_desc.column_name = aggregate_tuple_column_name;
     agg_desc.argument_names = {aggregate_tuple_column_name};
-    DB::Array parameters;
+    auto & parameters = agg_desc.parameters;
     parameters.push_back(static_cast<UInt32>(limit));
-    auto sort_directions = buildSQLLikeSortDescription(input_header, win_rel_def->sorts());
+    auto sort_directions = buildSQLLikeSortDescription(*input_header, win_rel_def->sorts());
     parameters.push_back(sort_directions);
 
-    auto header = plan.getCurrentHeader();
+    const auto & header = *plan.getCurrentHeader();
     DB::DataTypes arg_types;
     arg_types.push_back(header.getByName(aggregate_tuple_column_name).type);
 
@@ -358,14 +358,14 @@ void AggregateGroupLimitRelParser::addGroupLmitAggregationStep(DB::QueryPlan & p
 
 void AggregateGroupLimitRelParser::postProjectionForExplodingArrays(DB::QueryPlan & plan)
 {
-    auto header = plan.getCurrentHeader();
+    const auto & header = *plan.getCurrentHeader();
 
     /// flatten the array column.
     auto agg_result_index = header.columns() - 1;
     auto array_join_actions_dag = ArrayJoinHelper::applyArrayJoinOnOneColumn(header, agg_result_index);
     auto new_steps = ArrayJoinHelper::addArrayJoinStep(getContext(), plan, array_join_actions_dag, false);
 
-    auto array_join_output_header = plan.getCurrentHeader();
+    const auto & array_join_output_header = *plan.getCurrentHeader();
     DB::ActionsDAG flatten_actions_dag(array_join_output_header.getColumnsWithTypeAndName());
     DB::Names flatten_output_column_names;
     for (size_t i = 0; i < array_join_output_header.columns() - 1; ++i)
@@ -394,13 +394,13 @@ void AggregateGroupLimitRelParser::postProjectionForExplodingArrays(DB::QueryPla
     flatten_expression_step->setStepDescription("Untuple the aggregation result");
     plan.addStep(std::move(flatten_expression_step));
 
-    auto flatten_tuple_output_header = plan.getCurrentHeader();
-    auto window_result_column = flatten_tuple_output_header.getByPosition(flatten_tuple_output_header.columns() - 1);
+    const auto & flatten_tuple_output_header = plan.getCurrentHeader();
+    auto window_result_column = flatten_tuple_output_header->getByPosition(flatten_tuple_output_header->columns() - 1);
     /// The result column is put at the end of the header.
-    auto output_header = input_header;
+    auto output_header{*input_header};
     output_header.insert(window_result_column);
     auto adjust_pos_actions_dag = DB::ActionsDAG::makeConvertingActions(
-        flatten_tuple_output_header.getColumnsWithTypeAndName(),
+        flatten_tuple_output_header->getColumnsWithTypeAndName(),
         output_header.getColumnsWithTypeAndName(),
         DB::ActionsDAG::MatchColumnsMode::Name);
     LOG_DEBUG(getLogger("AggregateGroupLimitRelParser"), "Actions dag for replacing columns:\n{}", adjust_pos_actions_dag.dumpDAG());
@@ -411,7 +411,7 @@ void AggregateGroupLimitRelParser::postProjectionForExplodingArrays(DB::QueryPla
 
 void AggregateGroupLimitRelParser::addSortStep(DB::QueryPlan & plan)
 {
-    auto header = plan.getCurrentHeader();
+    const auto & header = *plan.getCurrentHeader();
     auto full_sort_descr = parseSortFields(header, win_rel_def->partition_expressions());
     auto sort_descr = parseSortFields(header, win_rel_def->sorts());
     full_sort_descr.insert(full_sort_descr.end(), sort_descr.begin(), sort_descr.end());
@@ -465,11 +465,11 @@ void AggregateGroupLimitRelParser::addWindowLimitStep(DB::QueryPlan & plan)
     auto optimization_info = WindowGroupOptimizationInfo::parse(optimize_info_str.value());
     auto window_function_name = optimization_info.window_function;
 
-    auto in_header = plan.getCurrentHeader();
+    const auto & in_header = plan.getCurrentHeader();
     DB::WindowDescription win_descr;
     win_descr.frame = buildWindowFrame(window_function_name);
-    win_descr.partition_by = parseSortFields(in_header, win_rel_def->partition_expressions());
-    win_descr.order_by = parseSortFields(in_header, win_rel_def->sorts());
+    win_descr.partition_by = parseSortFields(*in_header, win_rel_def->partition_expressions());
+    win_descr.order_by = parseSortFields(*in_header, win_rel_def->sorts());
     win_descr.full_sort_description = win_descr.partition_by;
     win_descr.full_sort_description.insert(win_descr.full_sort_description.end(), win_descr.order_by.begin(), win_descr.order_by.end());
     DB::WriteBufferFromOwnString ss;
@@ -485,8 +485,8 @@ void AggregateGroupLimitRelParser::addWindowLimitStep(DB::QueryPlan & plan)
     win_step->setStepDescription("Window (" + win_descr.window_name + ")");
     plan.addStep(std::move(win_step));
 
-    auto win_result_header = plan.getCurrentHeader();
-    DB::ActionsDAG limit_actions_dag(win_result_header.getColumnsWithTypeAndName());
+    const auto & win_result_header = plan.getCurrentHeader();
+    DB::ActionsDAG limit_actions_dag(win_result_header->getColumnsWithTypeAndName());
     const auto * rank_value_node = limit_actions_dag.getInputs().back();
     const auto * limit_value_node = expression_parser->addConstColumn(limit_actions_dag, std::make_shared<DB::DataTypeInt32>(), limit);
     const auto * cmp_node = expression_parser->toFunctionNode(limit_actions_dag, "lessOrEquals", {rank_value_node, limit_value_node});
