@@ -28,13 +28,13 @@
 #include <Parser/TypeParser.h>
 #include <Processors/QueryPlan/ReadFromPreparedSource.h>
 #include <Storages/SourceFromJavaIter.h>
+#include <Storages/SourceFromRange.h>
 #include <Storages/SubstraitSource/SubstraitFileSource.h>
 #include <Storages/SubstraitSource/SubstraitFileSourceStep.h>
 #include <google/protobuf/wrappers.pb.h>
 #include <rapidjson/document.h>
 #include <Common/BlockTypeUtils.h>
 #include <Common/DebugUtils.h>
-#include <Storages/SourceFromRange.h>
 
 namespace DB
 {
@@ -198,7 +198,7 @@ QueryPlanStepPtr ReadRelParser::parseReadRelWithLocalFile(const substrait::ReadR
 
     if (rel.has_filter())
     {
-        DB::ActionsDAG actions_dag{blockToNameAndTypeList(header)};
+        DB::ActionsDAG actions_dag{blockToRowType(header)};
         const DB::ActionsDAG::Node * filter_node = parseExpression(actions_dag, rel.filter());
         actions_dag.addOrReplaceInOutputs(*filter_node);
         assert(filter_node == &(actions_dag.findInOutputs(filter_node->result_name)));
@@ -234,7 +234,7 @@ QueryPlanStepPtr ReadRelParser::parseReadRelWithRange(const substrait::ReadRel &
     Int32 num_slices = document["numSlices"].GetInt();
     Int32 slice_index = document["sliceIndex"].GetInt();
 
-    auto header = TypeParser::buildBlockFromNamedStruct(rel.base_schema());
+    auto header = toShared(TypeParser::buildBlockFromNamedStruct(rel.base_schema()));
     size_t max_block_size = getContext()->getSettingsRef()[Setting::max_block_size];
     auto source = std::make_shared<SourceFromRange>(header, start, end, step, num_slices, slice_index, max_block_size);
     QueryPlanStepPtr source_step = std::make_unique<ReadFromPreparedSource>(Pipe(source));
