@@ -101,7 +101,7 @@ void SerializedPlanParser::adjustOutput(const DB::QueryPlanPtr & query_plan, con
     const substrait::PlanRel & root_rel = plan.relations().at(0);
     if (root_rel.root().names_size())
     {
-        auto columns = query_plan->getCurrentHeader().columns();
+        auto columns = query_plan->getCurrentHeader()->columns();
         if (columns != static_cast<size_t>(root_rel.root().names_size()))
         {
             debug::dumpPlan(*query_plan, "clickhouse plan", true);
@@ -126,7 +126,7 @@ void SerializedPlanParser::adjustOutput(const DB::QueryPlanPtr & query_plan, con
     const auto & output_schema = root_rel.root().output_schema();
     if (output_schema.types_size())
     {
-        auto origin_header = query_plan->getCurrentHeader();
+        const auto & origin_header = *query_plan->getCurrentHeader();
         const auto & origin_columns = origin_header.getColumnsWithTypeAndName();
 
         if (static_cast<size_t>(output_schema.types_size()) != origin_columns.size())
@@ -179,7 +179,8 @@ void SerializedPlanParser::adjustOutput(const DB::QueryPlanPtr & query_plan, con
 
         if (need_final_project)
         {
-            ActionsDAG final_project = ActionsDAG::makeConvertingActions(origin_columns, final_columns, ActionsDAG::MatchColumnsMode::Position, true);
+            ActionsDAG final_project
+                = ActionsDAG::makeConvertingActions(origin_columns, final_columns, ActionsDAG::MatchColumnsMode::Position, true);
             QueryPlanStepPtr final_project_step
                 = std::make_unique<ExpressionStep>(query_plan->getCurrentHeader(), std::move(final_project));
             final_project_step->setStepDescription("Project for output schema");
@@ -300,6 +301,7 @@ DB::QueryPipelineBuilderPtr SerializedPlanParser::buildQueryPipeline(DB::QueryPl
 {
     const Settings & settings = parser_context->queryContext()->getSettingsRef();
     QueryPriorities priorities;
+    QuerySlotPtr query_slot; // unlimited query
     const auto query_status = std::make_shared<QueryStatus>(
         parser_context->queryContext(),
         "",
@@ -307,6 +309,7 @@ DB::QueryPipelineBuilderPtr SerializedPlanParser::buildQueryPipeline(DB::QueryPl
         parser_context->queryContext()->getClientInfo(),
         priorities.insert(
             settings[Setting::priority], std::chrono::milliseconds(settings[Setting::low_priority_query_wait_time_ms].totalMilliseconds())),
+        std::move(query_slot),
         CurrentThread::getGroup(),
         IAST::QueryKind::Select,
         settings,
