@@ -23,8 +23,8 @@ namespace local_engine
 class FunctionParserToUtcTimestamp : public FunctionParserUtcTimestampTransform
 {
 public:
-    explicit FunctionParserToUtcTimestamp(SerializedPlanParser * plan_parser_) : FunctionParserUtcTimestampTransform(plan_parser_) { }
-    ~FunctionParserToUtcTimestamp() = default;
+    explicit FunctionParserToUtcTimestamp(ParserContextPtr parser_context_) : FunctionParserUtcTimestampTransform(parser_context_) { }
+    ~FunctionParserToUtcTimestamp() override = default;
 
     static constexpr auto name = "to_utc_timestamp";
     String getCHFunctionName(const substrait::Expression_ScalarFunction &) const override { return "to_utc_timestamp"; }
