@@ -17,19 +17,22 @@
 #include "SelectorBuilder.h"
 #include <limits>
 #include <memory>
-#include <Columns/ColumnArray.h>
+#include <Columns/ColumnConst.h>
 #include <Columns/ColumnMap.h>
 #include <Columns/ColumnNullable.h>
 #include <DataTypes/DataTypeArray.h>
 #include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypesDecimal.h>
 #include <Functions/FunctionFactory.h>
+#include <Parser/ExpressionParser.h>
+#include <Parser/ParserContext.h>
 #include <Parser/SerializedPlanParser.h>
 #include <Parser/TypeParser.h>
 #include <Poco/JSON/Parser.h>
 #include <Poco/MemoryStream.h>
 #include <Common/CHUtil.h>
 #include <Common/Exception.h>
+#include <Common/QueryContext.h>
 
 namespace DB
 {
@@ -40,13 +43,12 @@ extern const int LOGICAL_ERROR;
 }
 namespace local_engine
 {
+using namespace DB;
 PartitionInfo PartitionInfo::fromSelector(DB::IColumn::Selector selector, size_t partition_num, bool use_external_sort_shuffle)
 {
     if (use_external_sort_shuffle)
     {
-        return PartitionInfo{
-            .src_partition_num = std::move(selector),
-            .partition_num = partition_num};
+        return PartitionInfo{.src_partition_num = std::move(selector), .partition_num = partition_num};
     }
     else
     {
@@ -69,7 +71,6 @@ PartitionInfo PartitionInfo::fromSelector(DB::IColumn::Selector selector, size_t
             .src_partition_num = std::move(selector),
             .partition_num = partition_num};
     }
-
 }
 
 PartitionInfo RoundRobinSelectorBuilder::build(DB::Block & block)
@@ -101,7 +102,7 @@ PartitionInfo HashSelectorBuilder::build(DB::Block & block)
     if (!hash_function) [[unlikely]]
     {
         auto & factory = DB::FunctionFactory::instance();
-        auto function = factory.get(hash_function_name, local_engine::SerializedPlanParser::global_context);
+        auto function = factory.get(hash_function_name, QueryContext::globalContext());
 
         hash_function = function->build(args);
     }
@@ -328,8 +329,9 @@ void RangeSelectorBuilder::initActionsDAG(const DB::Block & block)
     std::lock_guard lock(actions_dag_mutex);
     if (has_init_actions_dag)
         return;
-    SerializedPlanParser plan_parser(local_engine::SerializedPlanParser::global_context);
-    plan_parser.parseExtensions(projection_plan_pb->extensions());
+
+    auto parser_context = ParserContext::build(QueryContext::globalContext(), *projection_plan_pb);
+    ExpressionParser expression_parser(parser_context);
 
     const auto & expressions = projection_plan_pb->relations().at(0).root().input().project().expressions();
     std::vector<substrait::Expression> exprs;
@@ -337,8 +339,8 @@ void RangeSelectorBuilder::initActionsDAG(const DB::Block & block)
     for (const auto & expression : expressions)
         exprs.emplace_back(expression);
 
-    auto projection_actions_dag = plan_parser.expressionsToActionsDAG(exprs, block, block);
-    projection_expression_actions = std::make_unique<DB::ExpressionActions>(projection_actions_dag);
+    auto projection_actions_dag = expression_parser.expressionsToActionsDAG(exprs, block);
+    projection_expression_actions = std::make_unique<DB::ExpressionActions>(std::move(projection_actions_dag));
     has_init_actions_dag = true;
 }
 
@@ -367,19 +369,32 @@ void RangeSelectorBuilder::computePartitionIdByBinarySearch(DB::Block & block, D
         selector.emplace_back(selected_partition);
     }
 }
+namespace
+{
+int doCompareAt(const ColumnPtr & lhs, size_t n, size_t m, const IColumn & rhs, int nan_direction_hint)
+{
+    if (const auto * l_const = typeid_cast<const ColumnConst *>(lhs.get()))
+    {
+        // we know rhs never be Const
+        chassert(l_const->getDataType() == rhs.getDataType());
+        return l_const->getDataColumn().compareAt(0, m, rhs, nan_direction_hint);
+    }
+    return lhs->compareAt(n, m, rhs, nan_direction_hint);
+}
+}
 
 int RangeSelectorBuilder::compareRow(
     const DB::Columns & columns,
     const std::vector<size_t> & required_columns,
     size_t row,
     const DB::Columns & bound_columns,
-    size_t bound_row)
+    size_t bound_row) const
 {
     for (size_t i = 0, n = required_columns.size(); i < n; ++i)
     {
         auto lpos = required_columns[i];
         auto rpos = i;
-        auto res = columns[lpos]->compareAt(row, bound_row, *bound_columns[rpos], sort_descriptions[i].nulls_direction)
+        auto res = doCompareAt(columns[lpos], row, bound_row, *bound_columns[rpos], sort_descriptions[i].nulls_direction)
             * sort_descriptions[i].direction;
         if (res != 0)
             return res;
