@@ -115,7 +115,7 @@ doBuildMetadata(const DB::NamesAndTypesList & columns, const ContextPtr & contex
 
     setSecondaryIndex(columns, context, table, metadata);
 
-    metadata->partition_key.expression_list_ast = std::make_shared<ASTExpressionList>();
+    metadata->partition_key = KeyDescription::buildEmptyKey();
     metadata->sorting_key = KeyDescription::parse(table.order_by_key, metadata->getColumns(), context, true);
     if (table.primary_key.empty())
         if (table.order_by_key != MergeTreeTable::TUPLE)
@@ -218,6 +218,10 @@ MergeTreeTableInstance::MergeTreeTableInstance(const std::string & info) : Merge
         assertChar('\n', in);
         readIntText(part.end, in);
         assertChar('\n', in);
+        readString(part.row_index_filter_type, in);
+        assertChar('\n', in);
+        readString(part.row_index_filter_id_encoded, in);
+        assertChar('\n', in);
         parts.emplace_back(part);
     }
 }
@@ -292,10 +296,7 @@ RangesInDataParts MergeTreeTableInstance::extractRange(DataPartsVector parts_vec
         std::inserter(ranges_in_data_parts, ranges_in_data_parts.end()),
         [&](const MergeTreePart & part)
         {
-            RangesInDataPart ranges_in_data_part;
-            ranges_in_data_part.data_part = name_index.at(part.name);
-            ranges_in_data_part.part_index_in_query = 0;
-            ranges_in_data_part.ranges.emplace_back(MarkRange(part.begin, part.end));
+            RangesInDataPart ranges_in_data_part{name_index.at(part.name),nullptr, 0, 0, {MarkRange(part.begin, part.end)}};
             return ranges_in_data_part;
         });
     return ranges_in_data_parts;
