@@ -383,13 +383,13 @@ std::map<std::string, MutationCommands> SparkStorageMergeTree::getUnfinishedMuta
     throw std::runtime_error("not implement");
 }
 
-MergeTreeDataWriter::TemporaryPart SparkMergeTreeDataWriter::writeTempPart(
+MergeTreeTemporaryPartPtr SparkMergeTreeDataWriter::writeTempPart(
     BlockWithPartition & block_with_partition,
     const StorageMetadataPtr & metadata_snapshot,
     const ContextPtr & context,
     const std::string & part_dir) const
 {
-    MergeTreeDataWriter::TemporaryPart temp_part;
+    auto temp_part = std::make_unique<MergeTreeTemporaryPart>();
 
     Block & block = block_with_partition.block;
 
@@ -406,7 +406,7 @@ MergeTreeDataWriter::TemporaryPart SparkMergeTreeDataWriter::writeTempPart(
 
     MergeTreePartInfo new_part_info(partition.getID(metadata_snapshot->getPartitionKey().sample_block), 1, 1, 0);
 
-    temp_part.temporary_directory_lock = data.getTemporaryPartDirectoryHolder(part_dir);
+    temp_part->temporary_directory_lock = data.getTemporaryPartDirectoryHolder(part_dir);
 
     auto indices = MergeTreeIndexFactory::instance().getMany(metadata_snapshot->getSecondaryIndices());
 
@@ -516,9 +516,9 @@ MergeTreeDataWriter::TemporaryPart SparkMergeTreeDataWriter::writeTempPart(
     out->writeWithPermutation(block, perm_ptr);
     auto finalizer = out->finalizePartAsync(new_data_part, data_settings[MergeTreeSetting::fsync_after_insert], nullptr, nullptr);
 
-    temp_part.part = new_data_part;
-    temp_part.streams.emplace_back(MergeTreeDataWriter::TemporaryPart::Stream{.stream = std::move(out), .finalizer = std::move(finalizer)});
-    temp_part.finalize();
+    temp_part->part = new_data_part;
+    temp_part->streams.emplace_back(MergeTreeTemporaryPart::Stream{.stream = std::move(out), .finalizer = std::move(finalizer)});
+    temp_part->finalize();
     data_part_storage->commitTransaction();
     return temp_part;
 }
@@ -539,6 +539,9 @@ SparkWriteStorageMergeTree::buildMergeTreeSettings(const ContextMutablePtr & con
     if (String min_rows_for_wide_part; tryGetString(context->getSettingsRef(), "merge_tree.min_rows_for_wide_part", min_rows_for_wide_part))
         settings->set("min_rows_for_wide_part", Field(std::strtoll(min_rows_for_wide_part.c_str(), nullptr, 10)));
 
+    if (settingsEqual(context->getSettingsRef(), "merge_tree.write_marks_for_substreams_in_compact_parts", "true"))
+        settings->set("write_marks_for_substreams_in_compact_parts", Field(true));
+
     return settings;
 }
 
