@@ -35,7 +35,7 @@ class SparkMergeTreeDataWriter
 {
 public:
     explicit SparkMergeTreeDataWriter(DB::MergeTreeData & data_) : data(data_), log(getLogger(data.getLogName() + " (Writer)")) { }
-    DB::MergeTreeDataWriter::TemporaryPart writeTempPart(
+    DB::MergeTreeTemporaryPartPtr writeTempPart(
         DB::BlockWithPartition & block_with_partition,
         const DB::StorageMetadataPtr & metadata_snapshot,
         const DB::ContextPtr & context,
@@ -50,11 +50,11 @@ class SparkStorageMergeTree : public DB::MergeTreeData
 {
     friend class MergeSparkMergeTreeTask;
 
-    struct SparkMutationsSnapshot : public IMutationsSnapshot
+    struct SparkMutationsSnapshot : public MutationsSnapshotBase
     {
         SparkMutationsSnapshot() = default;
 
-        DB::MutationCommands getAlterMutationCommandsForPart(const MergeTreeData::DataPartPtr & part) const override { return {}; }
+        DB::MutationCommands getOnFlyMutationCommandsForPart(const MergeTreeData::DataPartPtr & part) const override { return {}; }
         std::shared_ptr<MergeTreeData::IMutationsSnapshot> cloneEmpty() const override
         {
             return std::make_shared<SparkMutationsSnapshot>();
@@ -110,8 +110,7 @@ protected:
     void movePartitionToTable(const DB::StoragePtr & dest_table, const DB::ASTPtr & partition, DB::ContextPtr context) override;
     bool partIsAssignedToBackgroundOperation(const DataPartPtr & part) const override;
     void attachRestoredParts(MutableDataPartsVector && /*parts*/) override { throw std::runtime_error("not implement"); }
-    UInt64 getNumberOnFlyDataMutations() const override { throw std::runtime_error("not implement"); }
-    UInt64 getNumberOnFlyMetadataMutations() const override { throw std::runtime_error("not implement"); }
+    DB::MutationCounters getMutationCounters() const override { throw std::runtime_error("not implement"); }
 
 public:
     MutationsSnapshotPtr getMutationsSnapshot(const IMutationsSnapshot::Params & /*params*/) const override
