@@ -12,6 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 mkdir -p /build && cd /build || exit
 export CCACHE_DIR=/ccache
 export CCACHE_BASEDIR=/build
@@ -19,7 +20,22 @@ export CCACHE_NOHASHDIR=true
 export CCACHE_COMPILERCHECK=content
 export CCACHE_MAXSIZE=15G
 
-cmake -G Ninja  "-DCMAKE_C_COMPILER=$CC" "-DCMAKE_CXX_COMPILER=$CXX" "-DCMAKE_BUILD_TYPE=Release" "-DENABLE_PROTOBUF=1" "-DENABLE_EMBEDDED_COMPILER=$ENABLE_EMBEDDED_COMPILER" "-DENABLE_TESTS=OFF" "-DWERROR=OFF" "-DENABLE_JEMALLOC=1" "-DENABLE_MULTITARGET_CODE=ON" /clickhouse
-ninja ch
+# link the local-engine lib to a subdir of clickhouse
+rm -f /clickhosue/util/extern-local-engine
+ln -s /gluten/cpp-ch/local-engine /clickhouse/utils/extern-local-engine
 
-cp /build/utils/local-engine/libch.so "/output/libch_$(date +%Y%m%d).so"
\ No newline at end of file
+cmake -G Ninja  "-DCMAKE_C_COMPILER=$CC" "-DCMAKE_CXX_COMPILER=$CXX" \
+          "-DCMAKE_BUILD_TYPE=Release" \
+          "-DENABLE_PROTOBUF=1" \
+          "-DENABLE_EMBEDDED_COMPILER=$ENABLE_EMBEDDED_COMPILER" \
+          "-DENABLE_TESTS=OFF" \
+          "-DWERROR=OFF" \
+          "-DENABLE_JEMALLOC=1" \
+          "-DENABLE_MULTITARGET_CODE=ON" \
+          "-DENABLE_GWP_ASAN=OFF" \
+          "-DENABLE_EXTERN_LOCAL_ENGINE=ON" \
+          "-DENABLE_THINLTO=false" \
+          /clickhouse
+ninja
+
+cp /build/utils/extern-local-engine/libch.so "/output/libch_$(date +%Y%m%d).so"
