@@ -803,20 +803,20 @@ QueryPlanPtr readFromMergeTree(MergeTreeWithSnapshot storage)
     auto data_parts = storage.merge_tree->getDataPartsVectorForInternalUsage();
     auto query_plan = std::make_unique<QueryPlan>();
     auto step = storage.merge_tree->reader.readFromParts(
-        data_parts, {}, storage.columns.getNames(), storage.snapshot, *query_info, global_context, 10000, 1);
+        RangesInDataParts{data_parts}, {}, storage.columns.getNames(), storage.snapshot, *query_info, global_context, 10000, 1);
     query_plan->addStep(std::move(step));
     return query_plan;
 }
 
-QueryPlanPtr joinPlan(QueryPlanPtr left, QueryPlanPtr right, String left_key, String right_key, size_t block_size = 8192)
+QueryPlanPtr joinPlan(QueryPlanPtr left, QueryPlanPtr right, String left_key, String right_key)
 {
     auto join = std::make_shared<TableJoin>(
         global_context->getSettingsRef(), global_context->getGlobalTemporaryVolume(), global_context->getTempDataOnDisk());
-    auto left_columns = left->getCurrentHeader().getColumnsWithTypeAndName();
-    auto right_columns = right->getCurrentHeader().getColumnsWithTypeAndName();
+    auto left_columns = left->getCurrentHeader()->getColumnsWithTypeAndName();
+    auto right_columns = right->getCurrentHeader()->getColumnsWithTypeAndName();
     join->setKind(JoinKind::Left);
     join->setStrictness(JoinStrictness::All);
-    join->setColumnsFromJoinedTable(right->getCurrentHeader().getNamesAndTypesList());
+    join->setColumnsFromJoinedTable(right->getCurrentHeader()->getNamesAndTypesList());
     join->addDisjunct();
     ASTPtr lkey = std::make_shared<ASTIdentifier>(left_key);
     ASTPtr rkey = std::make_shared<ASTIdentifier>(right_key);
@@ -824,7 +824,7 @@ QueryPlanPtr joinPlan(QueryPlanPtr left, QueryPlanPtr right, String left_key, St
     for (const auto & column : join->columnsFromJoinedTable())
         join->addJoinedColumn(column);
 
-    auto left_keys = left->getCurrentHeader().getNamesAndTypesList();
+    auto left_keys = left->getCurrentHeader()->getNamesAndTypesList();
     join->addJoinedColumnsAndCorrectTypes(left_keys, true);
     std::optional<ActionsDAG> left_convert_actions;
     std::optional<ActionsDAG> right_convert_actions;
@@ -846,7 +846,16 @@ QueryPlanPtr joinPlan(QueryPlanPtr left, QueryPlanPtr right, String left_key, St
     auto hash_join = std::make_shared<HashJoin>(join, right->getCurrentHeader());
 
     QueryPlanStepPtr join_step = std::make_unique<JoinStep>(
-        left->getCurrentHeader(), right->getCurrentHeader(), hash_join, block_size, 8192, 1, NameSet{}, false, false);
+        left->getCurrentHeader(),
+        right->getCurrentHeader(),
+        hash_join,
+        DEFAULT_BLOCK_SIZE,
+        DEFAULT_BLOCK_SIZE,
+        524288,
+        1,
+        NameSet{},
+        false,
+        false);
 
     std::vector<QueryPlanPtr> plans;
     plans.emplace_back(std::move(left));
