@@ -27,10 +27,6 @@ bool parseProtobuf(const uint8_t* buf, int bufLen, google::protobuf::Message* ms
 
 std::string substraitFromJsonToPb(std::string_view typeName, std::string_view json);
 
-std::string substraitFromPbToJson(
-    std::string_view typeName,
-    const uint8_t* data,
-    int32_t size,
-    std::optional<std::string> dumpFile);
+std::string substraitFromPbToJson(std::string_view typeName, const uint8_t* data, int32_t size);
 
 } // namespace gluten
