@@ -21,17 +21,16 @@
 
 #include "compute/ProtobufUtils.h"
 #include "compute/ResultIterator.h"
-#include "memory/ArrowMemoryPool.h"
 #include "memory/ColumnarBatch.h"
 #include "memory/MemoryManager.h"
 #include "operators/c2r/ColumnarToRow.h"
 #include "operators/r2c/RowToColumnar.h"
 #include "operators/serializer/ColumnarBatchSerializer.h"
-#include "operators/writer/ArrowWriter.h"
 #include "shuffle/ShuffleReader.h"
 #include "shuffle/ShuffleWriter.h"
 #include "substrait/plan.pb.h"
 #include "utils/ObjectStore.h"
+#include "utils/WholeStageDumper.h"
 
 namespace gluten {
 
@@ -42,13 +41,16 @@ struct SparkTaskInfo {
   int32_t partitionId{0};
   // Same as TID.
   int64_t taskId{0};
+  // virtual id for each backend internal use
+  int32_t vId{0};
 
   std::string toString() const {
-    return "[Stage: " + std::to_string(stageId) + " TID: " + std::to_string(taskId) + "]";
+    return "[Stage: " + std::to_string(stageId) + " TID: " + std::to_string(taskId) + " VID: " + std::to_string(vId) +
+        "]";
   }
 
   friend std::ostream& operator<<(std::ostream& os, const SparkTaskInfo& taskInfo) {
-    os << "[Stage: " << taskInfo.stageId << " TID: " << taskInfo.taskId << "]";
+    os << taskInfo.toString();
     return os;
   }
 };
@@ -67,6 +69,7 @@ class Runtime : public std::enable_shared_from_this<Runtime> {
       const std::unordered_map<std::string, std::string>& sessionConf = {});
   static void release(Runtime*);
   static std::optional<std::string>* localWriteFilesTempPath();
+  static std::optional<std::string>* localWriteFileName();
 
   Runtime(
       const std::string& kind,
@@ -80,11 +83,11 @@ class Runtime : public std::enable_shared_from_this<Runtime> {
     return kind_;
   }
 
-  virtual void parsePlan(const uint8_t* data, int32_t size, std::optional<std::string> dumpFile) {
+  virtual void parsePlan(const uint8_t* data, int32_t size) {
     throw GlutenException("Not implemented");
   }
 
-  virtual void parseSplitInfo(const uint8_t* data, int32_t size, std::optional<std::string> dumpFile) {
+  virtual void parseSplitInfo(const uint8_t* data, int32_t size, int32_t idx) {
     throw GlutenException("Not implemented");
   }
 
@@ -92,7 +95,6 @@ class Runtime : public std::enable_shared_from_this<Runtime> {
     throw GlutenException("Not implemented");
   }
 
-  // Just for benchmark
   ::substrait::Plan& getPlan() {
     return substraitPlan_;
   }
@@ -127,9 +129,9 @@ class Runtime : public std::enable_shared_from_this<Runtime> {
   }
 
   virtual std::shared_ptr<ShuffleWriter> createShuffleWriter(
-      int numPartitions,
-      std::unique_ptr<PartitionWriter> partitionWriter,
-      ShuffleWriterOptions options) {
+      int32_t numPartitions,
+      const std::shared_ptr<PartitionWriter>& partitionWriter,
+      const std::shared_ptr<ShuffleWriterOptions>& options) {
     throw GlutenException("Not implemented");
   }
 
@@ -147,22 +149,26 @@ class Runtime : public std::enable_shared_from_this<Runtime> {
     throw GlutenException("Not implemented");
   }
 
-  virtual void dumpConf(const std::string& path) {
-    throw GlutenException("Not implemented");
-  }
-
-  virtual std::shared_ptr<ArrowWriter> createArrowWriter(const std::string& path) {
-    throw GlutenException("Not implemented");
-  };
-
   const std::unordered_map<std::string, std::string>& getConfMap() {
     return confMap_;
   }
 
-  void setSparkTaskInfo(SparkTaskInfo taskInfo) {
+  virtual void setSparkTaskInfo(SparkTaskInfo taskInfo) {
     taskInfo_ = taskInfo;
   }
 
+  std::optional<SparkTaskInfo> getSparkTaskInfo() const {
+    return taskInfo_;
+  }
+
+  virtual void enableDumping() {
+    throw GlutenException("Not implemented");
+  }
+
+  virtual WholeStageDumper* getDumper() {
+    return dumper_.get();
+  }
+
   ObjectHandle saveObject(std::shared_ptr<void> obj) {
     return objStore_->save(obj);
   }
@@ -175,6 +181,8 @@ class Runtime : public std::enable_shared_from_this<Runtime> {
 
   ::substrait::Plan substraitPlan_;
   std::vector<::substrait::ReadRel_LocalFiles> localFiles_;
-  SparkTaskInfo taskInfo_;
+
+  std::optional<SparkTaskInfo> taskInfo_{std::nullopt};
+  std::shared_ptr<WholeStageDumper> dumper_{nullptr};
 };
 } // namespace gluten
