@@ -37,8 +37,6 @@ const std::string kAllowPrecisionLoss = "spark.sql.decimalOperations.allowPrecis
 
 const std::string kIgnoreMissingFiles = "spark.sql.files.ignoreMissingFiles";
 
-const std::string kDefaultSessionTimezone = "spark.gluten.sql.session.timeZone.default";
-
 const std::string kSparkOverheadMemory = "spark.gluten.memoryOverhead.size.in.bytes";
 
 const std::string kSparkOffHeapMemory = "spark.gluten.memory.offHeap.size.in.bytes";
@@ -48,6 +46,9 @@ const std::string kSparkTaskOffHeapMemory = "spark.gluten.memory.task.offHeap.si
 const std::string kMemoryReservationBlockSize = "spark.gluten.memory.reservationBlockSize";
 const uint64_t kMemoryReservationBlockSizeDefault = 8 << 20;
 
+const std::string kCheckUsageLeak = "spark.gluten.sql.columnar.backend.velox.checkUsageLeak";
+const bool kCheckUsageLeakDefault = true;
+
 const std::string kSparkBatchSize = "spark.gluten.sql.columnar.maxBatchSize";
 
 const std::string kParquetBlockSize = "parquet.block.size";
@@ -57,6 +58,14 @@ const std::string kParquetBlockRows = "parquet.block.rows";
 const std::string kParquetGzipWindowSize = "parquet.gzip.windowSize";
 const std::string kGzipWindowSize4k = "4096";
 
+const std::string kParquetZSTDCompressionLevel = "parquet.compression.codec.zstd.level";
+
+const std::string kParquetDataPageSize = "parquet.page.size";
+
+const std::string kParquetEnableDictionary = "parquet.enable.dictionary";
+
+const std::string kParquetWriterVersion = "parquet.writer.version";
+
 const std::string kParquetCompressionCodec = "spark.sql.parquet.compression.codec";
 
 const std::string kColumnarToRowMemoryThreshold = "spark.gluten.sql.columnarToRowMemoryThreshold";
@@ -67,8 +76,9 @@ const std::string kUGITokens = "spark.gluten.ugi.tokens";
 const std::string kShuffleCompressionCodec = "spark.gluten.sql.columnar.shuffle.codec";
 const std::string kShuffleCompressionCodecBackend = "spark.gluten.sql.columnar.shuffle.codecBackend";
 const std::string kShuffleSpillDiskWriteBufferSize = "spark.shuffle.spill.diskWriteBufferSize";
+const std::string kSortShuffleReaderDeserializerBufferSize =
+    "spark.gluten.sql.columnar.shuffle.sort.deserializerBufferSize";
 const std::string kQatBackendName = "qat";
-const std::string kIaaBackendName = "iaa";
 
 const std::string kSparkRedactionRegex = "spark.redaction.regex";
 const std::string kSparkRedactionString = "*********(redacted)";
@@ -77,6 +87,18 @@ const std::string kSparkLegacyTimeParserPolicy = "spark.sql.legacy.timeParserPol
 const std::string kShuffleFileBufferSize = "spark.shuffle.file.buffer";
 const std::string kSparkMapKeyDedupPolicy = "spark.sql.mapKeyDedupPolicy";
 
+const std::string kSparkLegacyStatisticalAggregate = "spark.sql.legacy.statisticalAggregate";
+
+const std::string kSparkJsonIgnoreNullFields = "spark.sql.jsonGenerator.ignoreNullFields";
+
+// cudf
+#ifdef GLUTEN_ENABLE_GPU
+const std::string kCudfEnabled = "spark.gluten.sql.columnar.cudf";
+const bool kCudfEnabledDefault = "true";
+const std::string kDebugCudf = "spark.gluten.sql.debug.cudf";
+const bool kDebugCudfDefault = "false";
+#endif
+
 std::unordered_map<std::string, std::string>
 parseConfMap(JNIEnv* env, const uint8_t* planData, const int32_t planDataLength);
 
