@@ -39,7 +39,13 @@ class MemoryManager {
     return kind_;
   }
 
-  virtual arrow::MemoryPool* getArrowMemoryPool() = 0;
+  // Get the default Arrow memory pool for this memory manager. This memory pool is held by the memory manager.
+  virtual arrow::MemoryPool* defaultArrowMemoryPool() = 0;
+
+  // Return the Arrow memory pool with the given name. The caller is responsible for managing the lifetime of the
+  // returned memory pool. Memory manager only holds the weak reference to the memory pool for collecting memory usage.
+  // If the memory pool with the given name does not exist, it will create a new one and return it.
+  virtual std::shared_ptr<arrow::MemoryPool> getOrCreateArrowMemoryPool(const std::string& name) = 0;
 
   virtual const MemoryUsageStats collectMemoryUsageStats() const = 0;
 
