@@ -25,11 +25,7 @@ class HashPartitioner final : public Partitioner {
  public:
   HashPartitioner(int32_t numPartitions) : Partitioner(numPartitions, true) {}
 
-  arrow::Status compute(
-      const int32_t* pidArr,
-      const int64_t numRows,
-      std::vector<uint32_t>& row2partition,
-      std::vector<uint32_t>& partition2RowCount) override;
+  arrow::Status compute(const int32_t* pidArr, const int64_t numRows, std::vector<uint32_t>& row2partition) override;
 
   arrow::Status compute(
       const int32_t* pidArr,
