@@ -17,7 +17,7 @@
 
 #pragma once
 
-#include "ShuffleMemoryPool.h"
+#include "memory/MemoryManager.h"
 #include "memory/Reclaimable.h"
 #include "shuffle/Options.h"
 #include "shuffle/Payload.h"
@@ -31,13 +31,22 @@ struct Evict {
 
 class PartitionWriter : public Reclaimable {
  public:
-  PartitionWriter(uint32_t numPartitions, PartitionWriterOptions options, arrow::MemoryPool* pool)
-      : numPartitions_(numPartitions), options_(std::move(options)), pool_(pool) {
-    payloadPool_ = std::make_unique<ShuffleMemoryPool>(pool);
-    codec_ = createArrowIpcCodec(options_.compressionType, options_.codecBackend, options_.compressionLevel);
+  PartitionWriter(uint32_t numPartitions, std::unique_ptr<arrow::util::Codec> codec, MemoryManager* memoryManager)
+      : numPartitions_(numPartitions), codec_(std::move(codec)), memoryManager_(memoryManager) {
+    payloadPool_ = memoryManager->getOrCreateArrowMemoryPool("PartitionWriter.cached_payload");
   }
 
-  virtual ~PartitionWriter() = default;
+  static inline std::string typeToString(PartitionWriterType type) {
+    switch (type) {
+      case PartitionWriterType::kLocal:
+        return "LocalPartitionWriter";
+      case PartitionWriterType::kRss:
+        return "RssPartitionWriter";
+    }
+    GLUTEN_UNREACHABLE();
+  }
+
+  ~PartitionWriter() override = default;
 
   virtual arrow::Status stop(ShuffleWriterMetrics* metrics) = 0;
 
@@ -46,21 +55,10 @@ class PartitionWriter : public Reclaimable {
       uint32_t partitionId,
       std::unique_ptr<InMemoryPayload> inMemoryPayload,
       Evict::type evictType,
-      bool reuseBuffers,
-      bool hasComplexType) = 0;
+      bool reuseBuffers) = 0;
 
-  virtual arrow::Status sortEvict(
-      uint32_t partitionId,
-      std::unique_ptr<InMemoryPayload> inMemoryPayload,
-      std::shared_ptr<arrow::Buffer> compressed,
-      bool isFinal) = 0;
-
-  std::optional<int64_t> getCompressedBufferLength(const std::vector<std::shared_ptr<arrow::Buffer>>& buffers) {
-    if (!codec_) {
-      return std::nullopt;
-    }
-    return BlockPayload::maxCompressedLength(buffers, codec_.get());
-  }
+  virtual arrow::Status
+  sortEvict(uint32_t partitionId, std::unique_ptr<InMemoryPayload> inMemoryPayload, bool isFinal) = 0;
 
   virtual arrow::Status evict(uint32_t partitionId, std::unique_ptr<BlockPayload> blockPayload, bool stop) = 0;
 
@@ -68,23 +66,18 @@ class PartitionWriter : public Reclaimable {
     return payloadPool_->bytes_allocated();
   }
 
-  PartitionWriterOptions& options() {
-    return options_;
-  }
-
  protected:
   uint32_t numPartitions_;
-  PartitionWriterOptions options_;
-  arrow::MemoryPool* pool_;
+  std::unique_ptr<arrow::util::Codec> codec_;
+  MemoryManager* memoryManager_;
 
   // Memory Pool used to track memory allocation of partition payloads.
   // The actual allocation is delegated to options_.memoryPool.
-  std::unique_ptr<ShuffleMemoryPool> payloadPool_;
-
-  std::unique_ptr<arrow::util::Codec> codec_;
+  std::shared_ptr<arrow::MemoryPool> payloadPool_;
 
   int64_t compressTime_{0};
   int64_t spillTime_{0};
   int64_t writeTime_{0};
 };
+
 } // namespace gluten
