@@ -21,6 +21,7 @@
 #include <arrow/io/interfaces.h>
 #include <arrow/memory_pool.h>
 
+#include "shuffle/Dictionary.h"
 #include "shuffle/Options.h"
 #include "shuffle/Utils.h"
 
@@ -36,8 +37,6 @@ class Payload {
 
   virtual arrow::Status serialize(arrow::io::OutputStream* outputStream) = 0;
 
-  virtual arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t index) = 0;
-
   virtual int64_t rawSize() = 0;
 
   int64_t getCompressTime() const {
@@ -56,10 +55,6 @@ class Payload {
     return numRows_;
   }
 
-  uint32_t numBuffers() {
-    return isValidityBuffer_ ? isValidityBuffer_->size() : 1;
-  }
-
   const std::vector<bool>* isValidityBuffer() const {
     return isValidityBuffer_;
   }
@@ -84,8 +79,7 @@ class BlockPayload final : public Payload {
       std::vector<std::shared_ptr<arrow::Buffer>> buffers,
       const std::vector<bool>* isValidityBuffer,
       arrow::MemoryPool* pool,
-      arrow::util::Codec* codec,
-      std::shared_ptr<arrow::Buffer> compressed);
+      arrow::util::Codec* codec);
 
   static arrow::Result<std::vector<std::shared_ptr<arrow::Buffer>>> deserialize(
       arrow::io::InputStream* inputStream,
@@ -101,25 +95,23 @@ class BlockPayload final : public Payload {
 
   arrow::Status serialize(arrow::io::OutputStream* outputStream) override;
 
-  arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t pos) override;
+  arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t pos);
 
   int64_t rawSize() override;
 
- protected:
+ private:
   BlockPayload(
       Type type,
       uint32_t numRows,
+      uint32_t numBuffers,
       std::vector<std::shared_ptr<arrow::Buffer>> buffers,
-      const std::vector<bool>* isValidityBuffer,
-      arrow::MemoryPool* pool,
-      arrow::util::Codec* codec)
-      : Payload(type, numRows, isValidityBuffer), buffers_(std::move(buffers)), pool_(pool), codec_(codec) {}
+      const std::vector<bool>* isValidityBuffer)
+      : Payload(type, numRows, isValidityBuffer), numBuffers_(numBuffers), buffers_(std::move(buffers)) {}
 
   void setCompressionTime(int64_t compressionTime);
 
+  uint32_t numBuffers_;
   std::vector<std::shared_ptr<arrow::Buffer>> buffers_;
-  arrow::MemoryPool* pool_;
-  arrow::util::Codec* codec_;
 };
 
 class InMemoryPayload final : public Payload {
@@ -127,28 +119,42 @@ class InMemoryPayload final : public Payload {
   InMemoryPayload(
       uint32_t numRows,
       const std::vector<bool>* isValidityBuffer,
-      std::vector<std::shared_ptr<arrow::Buffer>> buffers)
-      : Payload(Type::kUncompressed, numRows, isValidityBuffer), buffers_(std::move(buffers)) {}
+      const std::shared_ptr<arrow::Schema>& schema,
+      std::vector<std::shared_ptr<arrow::Buffer>> buffers,
+      bool hasComplexType = false)
+      : Payload(Type::kUncompressed, numRows, isValidityBuffer),
+        schema_(schema),
+        buffers_(std::move(buffers)),
+        hasComplexType_(hasComplexType) {}
 
   static arrow::Result<std::unique_ptr<InMemoryPayload>>
   merge(std::unique_ptr<InMemoryPayload> source, std::unique_ptr<InMemoryPayload> append, arrow::MemoryPool* pool);
 
   arrow::Status serialize(arrow::io::OutputStream* outputStream) override;
 
-  arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t index) override;
+  arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t index);
 
-  arrow::Result<std::unique_ptr<BlockPayload>> toBlockPayload(
-      Payload::Type payloadType,
-      arrow::MemoryPool* pool,
-      arrow::util::Codec* codec,
-      std::shared_ptr<arrow::Buffer> compressed = nullptr);
+  arrow::Result<std::unique_ptr<BlockPayload>>
+  toBlockPayload(Payload::Type payloadType, arrow::MemoryPool* pool, arrow::util::Codec* codec);
 
   arrow::Status copyBuffers(arrow::MemoryPool* pool);
 
   int64_t rawSize() override;
 
+  uint32_t numBuffers() const;
+
+  int64_t rawCapacity() const;
+
+  bool mergeable() const;
+
+  std::shared_ptr<arrow::Schema> schema() const;
+
+  arrow::Status createDictionaries(const std::shared_ptr<ShuffleDictionaryWriter>& dictionaryWriter);
+
  private:
+  std::shared_ptr<arrow::Schema> schema_;
   std::vector<std::shared_ptr<arrow::Buffer>> buffers_;
+  bool hasComplexType_;
 };
 
 class UncompressedDiskBlockPayload final : public Payload {
@@ -162,8 +168,6 @@ class UncompressedDiskBlockPayload final : public Payload {
       arrow::MemoryPool* pool,
       arrow::util::Codec* codec);
 
-  arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t index) override;
-
   arrow::Status serialize(arrow::io::OutputStream* outputStream) override;
 
   int64_t rawSize() override;
@@ -173,7 +177,6 @@ class UncompressedDiskBlockPayload final : public Payload {
   int64_t rawSize_;
   arrow::MemoryPool* pool_;
   arrow::util::Codec* codec_;
-  uint32_t readPos_{0};
 
   arrow::Result<std::shared_ptr<arrow::Buffer>> readUncompressedBuffer();
 };
@@ -189,8 +192,6 @@ class CompressedDiskBlockPayload final : public Payload {
 
   arrow::Status serialize(arrow::io::OutputStream* outputStream) override;
 
-  arrow::Result<std::shared_ptr<arrow::Buffer>> readBufferAt(uint32_t index) override;
-
   int64_t rawSize() override;
 
  private:
