@@ -17,18 +17,11 @@
 
 #pragma once
 
-#include <numeric>
-#include <utility>
-
-#include "memory/ArrowMemoryPool.h"
 #include "memory/ColumnarBatch.h"
 #include "memory/Reclaimable.h"
 #include "shuffle/Options.h"
 #include "shuffle/PartitionWriter.h"
 #include "shuffle/Partitioner.h"
-#include "shuffle/Partitioning.h"
-#include "shuffle/ShuffleMemoryPool.h"
-#include "utils/Compression.h"
 
 namespace gluten {
 
@@ -38,7 +31,9 @@ class ShuffleWriter : public Reclaimable {
 
   static constexpr int64_t kMaxMemLimit = 1LL * 1024 * 1024 * 1024;
 
-  static ShuffleWriterType stringToType(const std::string& type);
+  static ShuffleWriterType stringToType(const std::string& typeString);
+
+  static std::string typeToString(ShuffleWriterType type);
 
   virtual arrow::Status write(std::shared_ptr<ColumnarBatch> cb, int64_t memLimit) = 0;
 
@@ -46,8 +41,6 @@ class ShuffleWriter : public Reclaimable {
 
   int32_t numPartitions() const;
 
-  ShuffleWriterOptions& options();
-
   int64_t totalBytesWritten() const;
 
   int64_t totalBytesEvicted() const;
@@ -60,26 +53,27 @@ class ShuffleWriter : public Reclaimable {
 
   int64_t totalCompressTime() const;
 
-  virtual int64_t peakBytesAllocated() const;
+  virtual int64_t peakBytesAllocated() const = 0;
 
   virtual int64_t totalSortTime() const;
 
   virtual int64_t totalC2RTime() const;
 
+  double avgDictionaryFields() const;
+
+  int64_t dictionarySize() const;
+
   const std::vector<int64_t>& partitionLengths() const;
 
   const std::vector<int64_t>& rawPartitionLengths() const;
 
  protected:
-  ShuffleWriter(int32_t numPartitions, ShuffleWriterOptions options, arrow::MemoryPool* pool);
+  ShuffleWriter(int32_t numPartitions, Partitioning partitioning);
 
-  virtual ~ShuffleWriter() = default;
+  ~ShuffleWriter() override = default;
 
   int32_t numPartitions_;
-
-  ShuffleWriterOptions options_;
-
-  arrow::MemoryPool* pool_;
+  Partitioning partitioning_;
 
   ShuffleWriterMetrics metrics_{};
 };
