@@ -24,11 +24,7 @@ class SinglePartitioner final : public Partitioner {
  public:
   SinglePartitioner() : Partitioner(1, false) {}
 
-  arrow::Status compute(
-      const int32_t* pidArr,
-      const int64_t numRows,
-      std::vector<uint32_t>& row2partition,
-      std::vector<uint32_t>& partition2RowCount) override;
+  arrow::Status compute(const int32_t* pidArr, const int64_t numRows, std::vector<uint32_t>& row2partition) override;
 
   arrow::Status compute(
       const int32_t* pidArr,
