@@ -21,8 +21,6 @@
 
 namespace gluten {
 
-Spill::Spill(Spill::SpillType type) : type_(type) {}
-
 Spill::~Spill() {
   if (is_) {
     static_cast<void>(is_->Close());
@@ -50,7 +48,6 @@ void Spill::insertPayload(
     int64_t rawSize,
     arrow::MemoryPool* pool,
     arrow::util::Codec* codec) {
-  // TODO: Add compression threshold.
   switch (payloadType) {
     case Payload::Type::kUncompressed:
     case Payload::Type::kToBeCompressed:
@@ -77,10 +74,6 @@ void Spill::openForRead(uint64_t shuffleFileBufferSize) {
   }
 }
 
-Spill::SpillType Spill::type() const {
-  return type_;
-}
-
 void Spill::setSpillFile(const std::string& spillFile) {
   spillFile_ = spillFile;
 }
