@@ -30,13 +30,23 @@ void RssPartitionWriter::init() {
 }
 
 arrow::Status RssPartitionWriter::stop(ShuffleWriterMetrics* metrics) {
-  // Push data and collect metrics.
-  auto totalBytesEvicted = std::accumulate(bytesEvicted_.begin(), bytesEvicted_.end(), 0LL);
+  if (rssOs_ != nullptr && !rssOs_->closed()) {
+    if (compressedOs_ != nullptr) {
+      RETURN_NOT_OK(compressedOs_->Close());
+      compressTime_ = compressedOs_->compressTime();
+      spillTime_ -= compressTime_;
+    }
+    ARROW_ASSIGN_OR_RAISE(const auto buffer, rssOs_->Finish());
+    bytesEvicted_[lastEvictedPartitionId_] +=
+        rssClient_->pushPartitionData(lastEvictedPartitionId_, buffer->data_as<char>(), buffer->size());
+  }
+
   rssClient_->stop();
+
+  auto totalBytesEvicted = std::accumulate(bytesEvicted_.begin(), bytesEvicted_.end(), 0LL);
   // Populate metrics.
   metrics->totalCompressTime += compressTime_;
   metrics->totalEvictTime += spillTime_;
-  metrics->totalWriteTime += writeTime_;
   metrics->totalBytesEvicted += totalBytesEvicted;
   metrics->totalBytesWritten += totalBytesEvicted;
   metrics->partitionLengths = std::move(bytesEvicted_);
@@ -53,17 +63,44 @@ arrow::Status RssPartitionWriter::hashEvict(
     uint32_t partitionId,
     std::unique_ptr<InMemoryPayload> inMemoryPayload,
     Evict::type evictType,
-    bool reuseBuffers,
-    bool hasComplexType) {
-  return doEvict(partitionId, std::move(inMemoryPayload), nullptr);
+    bool reuseBuffers) {
+  return doEvict(partitionId, std::move(inMemoryPayload));
 }
 
-arrow::Status RssPartitionWriter::sortEvict(
-    uint32_t partitionId,
-    std::unique_ptr<InMemoryPayload> inMemoryPayload,
-    std::shared_ptr<arrow::Buffer> compressed,
-    bool isFinal) {
-  return doEvict(partitionId, std::move(inMemoryPayload), std::move(compressed));
+arrow::Status
+RssPartitionWriter::sortEvict(uint32_t partitionId, std::unique_ptr<InMemoryPayload> inMemoryPayload, bool isFinal) {
+  ScopedTimer timer(&spillTime_);
+  if (lastEvictedPartitionId_ != partitionId) {
+    if (lastEvictedPartitionId_ != -1) {
+      GLUTEN_DCHECK(rssOs_ != nullptr && !rssOs_->closed(), "rssOs_ should not be null");
+      if (compressedOs_ != nullptr) {
+        RETURN_NOT_OK(compressedOs_->Flush());
+      }
+
+      ARROW_ASSIGN_OR_RAISE(const auto buffer, rssOs_->Finish());
+      bytesEvicted_[lastEvictedPartitionId_] +=
+          rssClient_->pushPartitionData(lastEvictedPartitionId_, buffer->data_as<char>(), buffer->size());
+    }
+
+    ARROW_ASSIGN_OR_RAISE(
+        rssOs_, arrow::io::BufferOutputStream::Create(options_->pushBufferMaxSize, arrow::default_memory_pool()));
+    if (codec_ != nullptr) {
+      ARROW_ASSIGN_OR_RAISE(
+          compressedOs_,
+          ShuffleCompressedOutputStream::Make(
+              codec_.get(), options_->compressionBufferSize, rssOs_, arrow::default_memory_pool()));
+    }
+
+    lastEvictedPartitionId_ = partitionId;
+  }
+
+  rawPartitionLengths_[partitionId] += inMemoryPayload->rawSize();
+  if (compressedOs_ != nullptr) {
+    RETURN_NOT_OK(inMemoryPayload->serialize(compressedOs_.get()));
+  } else {
+    RETURN_NOT_OK(inMemoryPayload->serialize(rssOs_.get()));
+  }
+  return arrow::Status::OK();
 }
 
 arrow::Status RssPartitionWriter::evict(uint32_t partitionId, std::unique_ptr<BlockPayload> blockPayload, bool) {
@@ -74,18 +111,17 @@ arrow::Status RssPartitionWriter::evict(uint32_t partitionId, std::unique_ptr<Bl
   return arrow::Status::OK();
 }
 
-arrow::Status RssPartitionWriter::doEvict(
-    uint32_t partitionId,
-    std::unique_ptr<InMemoryPayload> inMemoryPayload,
-    std::shared_ptr<arrow::Buffer> compressed) {
+arrow::Status RssPartitionWriter::doEvict(uint32_t partitionId, std::unique_ptr<InMemoryPayload> inMemoryPayload) {
   rawPartitionLengths_[partitionId] += inMemoryPayload->rawSize();
   auto payloadType = codec_ ? Payload::Type::kCompressed : Payload::Type::kUncompressed;
   ARROW_ASSIGN_OR_RAISE(
-      auto payload,
-      inMemoryPayload->toBlockPayload(
-          payloadType, payloadPool_.get(), codec_ ? codec_.get() : nullptr, std::move(compressed)));
+      auto payload, inMemoryPayload->toBlockPayload(payloadType, payloadPool_.get(), codec_ ? codec_.get() : nullptr));
   // Copy payload to arrow buffered os.
-  ARROW_ASSIGN_OR_RAISE(auto rssBufferOs, arrow::io::BufferOutputStream::Create(options_.pushBufferMaxSize));
+  ARROW_ASSIGN_OR_RAISE(auto rssBufferOs, arrow::io::BufferOutputStream::Create(options_->pushBufferMaxSize));
+
+  static constexpr uint8_t kRssBlock = static_cast<uint8_t>(BlockType::kPlainPayload);
+  RETURN_NOT_OK(rssBufferOs->Write(&kRssBlock, sizeof(kRssBlock)));
+
   RETURN_NOT_OK(payload->serialize(rssBufferOs.get()));
   payload = nullptr; // Invalidate payload immediately.
 
