@@ -26,14 +26,17 @@
 
 namespace gluten {
 
+class RssPartitionWriterOutputStream;
+
 class RssPartitionWriter final : public PartitionWriter {
  public:
   RssPartitionWriter(
-      uint32_t numPartitions,
-      PartitionWriterOptions options,
-      arrow::MemoryPool* pool,
-      std::shared_ptr<RssClient> rssClient)
-      : PartitionWriter(numPartitions, std::move(options), pool), rssClient_(rssClient) {
+      const uint32_t numPartitions,
+      std::unique_ptr<arrow::util::Codec> codec,
+      MemoryManager* memoryManager,
+      const std::shared_ptr<RssPartitionWriterOptions>& options,
+      const std::shared_ptr<RssClient>& rssClient)
+      : PartitionWriter(numPartitions, std::move(codec), memoryManager), options_(options), rssClient_(rssClient) {
     init();
   }
 
@@ -41,14 +44,10 @@ class RssPartitionWriter final : public PartitionWriter {
       uint32_t partitionId,
       std::unique_ptr<InMemoryPayload> inMemoryPayload,
       Evict::type evictType,
-      bool reuseBuffers,
-      bool hasComplexType) override;
+      bool reuseBuffers) override;
 
-  arrow::Status sortEvict(
-      uint32_t partitionId,
-      std::unique_ptr<InMemoryPayload> inMemoryPayload,
-      std::shared_ptr<arrow::Buffer> compressed,
-      bool isFinal) override;
+  arrow::Status sortEvict(uint32_t partitionId, std::unique_ptr<InMemoryPayload> inMemoryPayload, bool isFinal)
+      override;
 
   arrow::Status evict(uint32_t partitionId, std::unique_ptr<BlockPayload> blockPayload, bool stop) override;
 
@@ -59,14 +58,17 @@ class RssPartitionWriter final : public PartitionWriter {
  private:
   void init();
 
-  arrow::Status doEvict(
-      uint32_t partitionId,
-      std::unique_ptr<InMemoryPayload> inMemoryPayload,
-      std::shared_ptr<arrow::Buffer> compressed);
+  arrow::Status doEvict(uint32_t partitionId, std::unique_ptr<InMemoryPayload> inMemoryPayload);
 
+  std::shared_ptr<RssPartitionWriterOptions> options_;
   std::shared_ptr<RssClient> rssClient_;
 
   std::vector<int64_t> bytesEvicted_;
   std::vector<int64_t> rawPartitionLengths_;
+
+  int32_t lastEvictedPartitionId_{-1};
+  std::shared_ptr<arrow::io::BufferOutputStream> rssOs_{nullptr};
+  std::shared_ptr<ShuffleCompressedOutputStream> compressedOs_{nullptr};
 };
+
 } // namespace gluten
