@@ -21,7 +21,7 @@
 
 namespace gluten {
 
-enum CodecBackend { NONE, QAT, IAA };
+enum CodecBackend { NONE, QAT };
 
 // BUFFER mode will preallocate max compressed buffer, and then compress each buffer to the max compressed buffer
 // ROWVECTOR mode will copy the buffers to a big buffer and then compress the big buffer
