@@ -20,9 +20,10 @@
 
 #include <arrow/c/bridge.h>
 #include <arrow/util/range.h>
+
 #include <benchmark/benchmark.h>
+
 #include <gflags/gflags.h>
-#include <operators/writer/ArrowWriter.h>
 
 #include "benchmarks/common/BenchmarkUtils.h"
 #include "compute/VeloxBackend.h"
@@ -30,15 +31,16 @@
 #include "config/GlutenConfig.h"
 #include "config/VeloxConfig.h"
 #include "operators/reader/FileReaderIterator.h"
-#include "operators/writer/VeloxArrowWriter.h"
+#include "operators/writer/VeloxColumnarBatchWriter.h"
 #include "shuffle/LocalPartitionWriter.h"
 #include "shuffle/VeloxShuffleWriter.h"
 #include "shuffle/rss/RssPartitionWriter.h"
 #include "utils/Exception.h"
+#include "utils/LocalRssClient.h"
 #include "utils/StringUtil.h"
+#include "utils/TestAllocationListener.h"
 #include "utils/Timer.h"
 #include "utils/VeloxArrowUtils.h"
-#include "utils/tests/LocalRssClient.h"
 #include "velox/exec/PlanNodeStats.h"
 
 using namespace gluten;
@@ -56,11 +58,9 @@ DEFINE_string(
     "rr",
     "Short partitioning name. Valid options are rr, hash, range, single, random (only for test purpose)");
 DEFINE_bool(rss, false, "Mocking rss.");
-DEFINE_string(
-    compression,
-    "lz4",
-    "Specify the compression codec. Valid options are lz4, zstd, qat_gzip, qat_zstd, iaa_gzip");
+DEFINE_string(compression, "lz4", "Specify the compression codec. Valid options are none, lz4, zstd");
 DEFINE_int32(shuffle_partitions, 200, "Number of shuffle split (reducer) partitions");
+DEFINE_bool(shuffle_dictionary, false, "Whether to enable dictionary encoding for shuffle write.");
 
 DEFINE_string(plan, "", "Path to input json file of the substrait plan.");
 DEFINE_string(
@@ -96,6 +96,7 @@ struct WriterMetrics {
   int64_t writeTime{0};
   int64_t compressTime{0};
 
+  int64_t dataSize{0};
   int64_t bytesSpilled{0};
   int64_t bytesWritten{0};
 };
@@ -160,71 +161,84 @@ void cleanupLocalDirs(const std::vector<std::string>& localDirs) {
   }
 }
 
-PartitionWriterOptions createPartitionWriterOptions() {
-  PartitionWriterOptions partitionWriterOptions{};
-  // Disable writer's merge.
-  partitionWriterOptions.mergeThreshold = 0;
-
-  // Configure compression.
-  if (FLAGS_compression == "lz4") {
-    partitionWriterOptions.codecBackend = CodecBackend::NONE;
-    partitionWriterOptions.compressionType = arrow::Compression::LZ4_FRAME;
-    partitionWriterOptions.compressionTypeStr = "lz4";
+void setCompressionTypeFromFlag(arrow::Compression::type& compressionType, CodecBackend& codecBackend) {
+  codecBackend = CodecBackend::NONE;
+  if (FLAGS_compression == "none") {
+    compressionType = arrow::Compression::UNCOMPRESSED;
+  } else if (FLAGS_compression == "lz4") {
+    compressionType = arrow::Compression::LZ4_FRAME;
   } else if (FLAGS_compression == "zstd") {
-    partitionWriterOptions.codecBackend = CodecBackend::NONE;
-    partitionWriterOptions.compressionType = arrow::Compression::ZSTD;
-    partitionWriterOptions.compressionTypeStr = "zstd";
-  } else if (FLAGS_compression == "qat_gzip") {
-    partitionWriterOptions.codecBackend = CodecBackend::QAT;
-    partitionWriterOptions.compressionType = arrow::Compression::GZIP;
-  } else if (FLAGS_compression == "qat_zstd") {
-    partitionWriterOptions.codecBackend = CodecBackend::QAT;
-    partitionWriterOptions.compressionType = arrow::Compression::ZSTD;
-  } else if (FLAGS_compression == "iaa_gzip") {
-    partitionWriterOptions.codecBackend = CodecBackend::IAA;
-    partitionWriterOptions.compressionType = arrow::Compression::GZIP;
+    compressionType = arrow::Compression::ZSTD;
+  } else {
+    throw GlutenException("Unrecognized compression type: " + FLAGS_compression);
   }
-  return partitionWriterOptions;
 }
 
-std::unique_ptr<PartitionWriter> createPartitionWriter(
-    Runtime* runtime,
-    PartitionWriterOptions options,
-    const std::string& dataFile,
-    const std::vector<std::string>& localDirs) {
+std::unique_ptr<arrow::util::Codec> createCodec() {
+  // Configure compression.
+  if (FLAGS_compression == "none") {
+    return nullptr;
+  }
+
+  arrow::Compression::type compressionType;
+  CodecBackend codecBackend;
+
+  setCompressionTypeFromFlag(compressionType, codecBackend);
+
+  return createArrowIpcCodec(compressionType, codecBackend);
+}
+
+std::shared_ptr<PartitionWriter>
+createPartitionWriter(Runtime* runtime, const std::string& dataFile, const std::vector<std::string>& localDirs) {
   std::unique_ptr<PartitionWriter> partitionWriter;
   if (FLAGS_rss) {
+    auto options = std::make_shared<RssPartitionWriterOptions>();
     auto rssClient = std::make_unique<LocalRssClient>(dataFile);
-    partitionWriter = std::make_unique<RssPartitionWriter>(
-        FLAGS_shuffle_partitions,
-        std::move(options),
-        runtime->memoryManager()->getArrowMemoryPool(),
-        std::move(rssClient));
-  } else {
-    partitionWriter = std::make_unique<LocalPartitionWriter>(
-        FLAGS_shuffle_partitions,
-        std::move(options),
-        runtime->memoryManager()->getArrowMemoryPool(),
-        dataFile,
-        localDirs);
+    return std::make_shared<RssPartitionWriter>(
+        FLAGS_shuffle_partitions, createCodec(), runtime->memoryManager(), options, std::move(rssClient));
   }
-  return partitionWriter;
+
+  auto options = std::make_shared<LocalPartitionWriterOptions>();
+  options->enableDictionary = FLAGS_shuffle_dictionary;
+  return std::make_unique<LocalPartitionWriter>(
+      FLAGS_shuffle_partitions, createCodec(), runtime->memoryManager(), options, dataFile, localDirs);
+}
+
+std::shared_ptr<ShuffleWriterOptions> createShuffleWriterOptions() {
+  std::shared_ptr<ShuffleWriterOptions> options;
+
+  switch (ShuffleWriter::stringToType(FLAGS_shuffle_writer)) {
+    case ShuffleWriterType::kHashShuffle:
+      options = std::make_shared<HashShuffleWriterOptions>();
+      break;
+    case ShuffleWriterType::kSortShuffle:
+      options = std::make_shared<SortShuffleWriterOptions>();
+      break;
+    case ShuffleWriterType::kRssSortShuffle:
+      options = std::make_shared<RssSortShuffleWriterOptions>();
+      break;
+    default:
+      throw GlutenException("Unsupported shuffle writer type: " + FLAGS_shuffle_writer);
+  }
+
+  options->partitioning = toPartitioning(FLAGS_partitioning);
+  return options;
 }
 
 std::shared_ptr<VeloxShuffleWriter> createShuffleWriter(
     Runtime* runtime,
-    std::unique_ptr<PartitionWriter> partitionWriter) {
-  auto options = ShuffleWriterOptions{};
-  options.partitioning = gluten::toPartitioning(FLAGS_partitioning);
-  if (FLAGS_rss || FLAGS_shuffle_writer == "rss_sort") {
-    options.shuffleWriterType = gluten::kRssSortShuffle;
-  } else if (FLAGS_shuffle_writer == "sort") {
-    options.shuffleWriterType = gluten::kSortShuffle;
-  }
+    std::shared_ptr<PartitionWriter> partitionWriter) {
+  auto shuffleWriterOptions = createShuffleWriterOptions();
   auto shuffleWriter =
-      runtime->createShuffleWriter(FLAGS_shuffle_partitions, std::move(partitionWriter), std::move(options));
+      runtime->createShuffleWriter(FLAGS_shuffle_partitions, std::move(partitionWriter), shuffleWriterOptions);
+  return std::dynamic_pointer_cast<VeloxShuffleWriter>(shuffleWriter);
+}
 
-  return std::reinterpret_pointer_cast<VeloxShuffleWriter>(shuffleWriter);
+std::shared_ptr<ShuffleReader> createShuffleReader(Runtime* runtime, const std::shared_ptr<arrow::Schema>& schema) {
+  auto readerOptions = ShuffleReaderOptions{};
+  readerOptions.shuffleWriterType = ShuffleWriter::stringToType(FLAGS_shuffle_writer),
+  setCompressionTypeFromFlag(readerOptions.compressionType, readerOptions.codecBackend);
+  return runtime->createShuffleReader(schema, readerOptions);
 }
 
 void populateWriterMetrics(
@@ -238,6 +252,8 @@ void populateWriterMetrics(
   if (splitTime > 0) {
     metrics.splitTime += splitTime;
   }
+  metrics.dataSize +=
+      std::accumulate(shuffleWriter->rawPartitionLengths().begin(), shuffleWriter->rawPartitionLengths().end(), 0LL);
   metrics.bytesWritten += shuffleWriter->totalBytesWritten();
   metrics.bytesSpilled += shuffleWriter->totalBytesEvicted();
 }
@@ -254,7 +270,7 @@ void setCpu(::benchmark::State& state) {
 
 void runShuffle(
     Runtime* runtime,
-    BenchmarkAllocationListener* listener,
+    TestAllocationListener* listener,
     const std::shared_ptr<gluten::ResultIterator>& resultIter,
     WriterMetrics& writerMetrics,
     ReaderMetrics& readerMetrics,
@@ -263,8 +279,7 @@ void runShuffle(
     const std::string& dataFileDir) {
   GLUTEN_ASSIGN_OR_THROW(auto dataFile, gluten::createTempShuffleFile(dataFileDir));
 
-  auto partitionWriterOptions = createPartitionWriterOptions();
-  auto partitionWriter = createPartitionWriter(runtime, partitionWriterOptions, dataFile, localDirs);
+  auto partitionWriter = createPartitionWriter(runtime, dataFile, localDirs);
   auto shuffleWriter = createShuffleWriter(runtime, std::move(partitionWriter));
   listener->setShuffleWriter(shuffleWriter.get());
 
@@ -284,16 +299,9 @@ void runShuffle(
 
   populateWriterMetrics(shuffleWriter, totalTime, writerMetrics);
 
-  if (readAfterWrite && cSchema) {
-    auto readerOptions = ShuffleReaderOptions{};
-    readerOptions.shuffleWriterType = shuffleWriter->options().shuffleWriterType;
-    readerOptions.compressionType = partitionWriterOptions.compressionType;
-    readerOptions.codecBackend = partitionWriterOptions.codecBackend;
-    readerOptions.compressionTypeStr = partitionWriterOptions.compressionTypeStr;
-
-    std::shared_ptr<arrow::Schema> schema =
-        gluten::arrowGetOrThrow(arrow::ImportSchema(reinterpret_cast<struct ArrowSchema*>(cSchema.get())));
-    auto reader = runtime->createShuffleReader(schema, readerOptions);
+  if (readAfterWrite && cSchema != nullptr) {
+    const auto schema = arrowGetOrThrow(arrow::ImportSchema(cSchema.get()));
+    const auto reader = createShuffleReader(runtime, schema);
 
     GLUTEN_ASSIGN_OR_THROW(auto in, arrow::io::ReadableFile::Open(dataFile));
     // Read all partitions.
@@ -302,6 +310,9 @@ void runShuffle(
       // Read and discard.
       auto cb = iter->next();
     }
+    // Call the dtor to collect the metrics.
+    iter.reset();
+
     readerMetrics.decompressTime = reader->getDecompressTime();
     readerMetrics.deserializeTime = reader->getDeserializeTime();
   }
@@ -343,6 +354,8 @@ void updateBenchmarkMetrics(
     state.counters["shuffle_split_time"] =
         benchmark::Counter(splitTime, benchmark::Counter::kAvgIterations, benchmark::Counter::OneK::kIs1000);
 
+    state.counters["shuffle_data_size"] = benchmark::Counter(
+        writerMetrics.dataSize, benchmark::Counter::kAvgIterations, benchmark::Counter::OneK::kIs1024);
     state.counters["shuffle_spilled_bytes"] = benchmark::Counter(
         writerMetrics.bytesSpilled, benchmark::Counter::kAvgIterations, benchmark::Counter::OneK::kIs1024);
     state.counters["shuffle_write_bytes"] = benchmark::Counter(
@@ -361,9 +374,6 @@ void setQueryTraceConfig(std::unordered_map<std::string, std::string>& configs)
   if (FLAGS_query_trace_max_bytes) {
     configs[kQueryTraceMaxBytes] = std::to_string(FLAGS_query_trace_max_bytes);
   }
-  if (FLAGS_query_trace_node_ids != "") {
-    configs[kQueryTraceNodeIds] = FLAGS_query_trace_node_ids;
-  }
   if (FLAGS_query_trace_task_reg_exp != "") {
     configs[kQueryTraceTaskRegExp] = FLAGS_query_trace_task_reg_exp;
   }
@@ -381,7 +391,9 @@ auto BM_Generic = [](::benchmark::State& state,
                      FileReaderType readerType) {
   setCpu(state);
 
-  auto listener = std::make_unique<BenchmarkAllocationListener>(FLAGS_memory_limit);
+  auto listener = std::make_unique<TestAllocationListener>();
+  listener->updateLimit(FLAGS_memory_limit);
+
   auto* listenerPtr = listener.get();
   auto* memoryManager = MemoryManager::create(kVeloxBackendKind, std::move(listener));
   auto runtime = runtimeFactory(memoryManager);
@@ -417,8 +429,9 @@ auto BM_Generic = [](::benchmark::State& state,
       std::vector<FileReaderIterator*> inputItersRaw;
       if (!dataFiles.empty()) {
         for (const auto& input : dataFiles) {
-          inputIters.push_back(FileReaderIterator::getInputIteratorFromFileReader(
-              readerType, input, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool().get()));
+          inputIters.push_back(
+              FileReaderIterator::getInputIteratorFromFileReader(
+                  readerType, input, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool()));
         }
         std::transform(
             inputIters.begin(),
@@ -429,9 +442,10 @@ auto BM_Generic = [](::benchmark::State& state,
             });
       }
       *Runtime::localWriteFilesTempPath() = FLAGS_write_path;
-      runtime->parsePlan(reinterpret_cast<uint8_t*>(plan.data()), plan.size(), std::nullopt);
-      for (auto& split : splits) {
-        runtime->parseSplitInfo(reinterpret_cast<uint8_t*>(split.data()), split.size(), std::nullopt);
+      runtime->parsePlan(reinterpret_cast<uint8_t*>(plan.data()), plan.size());
+      for (auto i = 0; i < splits.size(); i++) {
+        auto split = splits[i];
+        runtime->parseSplitInfo(reinterpret_cast<uint8_t*>(split.data()), split.size(), i);
       }
 
       auto resultIter = runtime->createResultIterator(veloxSpillDir, std::move(inputIters), runtime->getConfMap());
@@ -442,38 +456,33 @@ auto BM_Generic = [](::benchmark::State& state,
             runtime, listenerPtr, resultIter, writerMetrics, readerMetrics, false, shuffleSpillDirs, dataFileDir);
       } else {
         // May write the output into file.
-        auto veloxPlan = dynamic_cast<gluten::VeloxRuntime*>(runtime)->getVeloxPlan();
-
-        ArrowSchema cSchema;
-        toArrowSchema(veloxPlan->outputType(), runtime->memoryManager()->getLeafMemoryPool().get(), &cSchema);
-        GLUTEN_ASSIGN_OR_THROW(auto outputSchema, arrow::ImportSchema(&cSchema));
-        auto writer = std::make_shared<VeloxArrowWriter>(
-            FLAGS_save_output, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool().get());
-        state.PauseTiming();
-        if (!FLAGS_save_output.empty()) {
-          GLUTEN_THROW_NOT_OK(writer->initWriter(*(outputSchema.get())));
-        }
-        state.ResumeTiming();
+        std::shared_ptr<VeloxColumnarBatchWriter> writer{nullptr};
 
         while (resultIter->hasNext()) {
-          auto array = resultIter->next()->exportArrowArray();
+          auto cb = resultIter->next();
+
           state.PauseTiming();
-          auto maybeBatch = arrow::ImportRecordBatch(array.get(), outputSchema);
-          if (!maybeBatch.ok()) {
-            state.SkipWithError(maybeBatch.status().message().c_str());
-            return;
-          }
-          if (FLAGS_print_result) {
-            LOG(WARNING) << maybeBatch.ValueOrDie()->ToString();
-          }
+
           if (!FLAGS_save_output.empty()) {
-            GLUTEN_THROW_NOT_OK(writer->writeInBatches(maybeBatch.ValueOrDie()));
+            if (writer == nullptr) {
+              writer = std::make_shared<VeloxColumnarBatchWriter>(
+                  FLAGS_save_output, FLAGS_batch_size, runtime->memoryManager()->getAggregateMemoryPool());
+            }
+            GLUTEN_THROW_NOT_OK(writer->write(cb));
           }
+
+          if (FLAGS_print_result) {
+            auto rowVector =
+                VeloxColumnarBatch::from(runtime->memoryManager()->getLeafMemoryPool().get(), cb)->getRowVector();
+            LOG(WARNING) << rowVector->toString(0, 20);
+          }
+
+          state.ResumeTiming();
         }
 
         state.PauseTiming();
         if (!FLAGS_save_output.empty()) {
-          GLUTEN_THROW_NOT_OK(writer->closeWriter());
+          GLUTEN_THROW_NOT_OK(writer->close());
         }
         state.ResumeTiming();
       }
@@ -504,7 +513,9 @@ auto BM_ShuffleWriteRead = [](::benchmark::State& state,
                               FileReaderType readerType) {
   setCpu(state);
 
-  auto listener = std::make_unique<BenchmarkAllocationListener>(FLAGS_memory_limit);
+  auto listener = std::make_unique<TestAllocationListener>();
+  listener->updateLimit(FLAGS_memory_limit);
+
   auto* listenerPtr = listener.get();
   auto* memoryManager = MemoryManager::create(kVeloxBackendKind, std::move(listener));
   auto runtime = runtimeFactory(memoryManager);
@@ -521,7 +532,7 @@ auto BM_ShuffleWriteRead = [](::benchmark::State& state,
     ScopedTimer timer(&elapsedTime);
     for (auto _ : state) {
       auto resultIter = FileReaderIterator::getInputIteratorFromFileReader(
-          readerType, inputFile, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool().get());
+          readerType, inputFile, FLAGS_batch_size, runtime->memoryManager()->getLeafMemoryPool());
       runShuffle(
           runtime,
           listenerPtr,
@@ -561,8 +572,8 @@ int main(int argc, char** argv) {
   ::benchmark::Initialize(&argc, argv);
 
   // Init Velox backend.
-  auto backendConf = gluten::defaultConf();
-  auto sessionConf = gluten::defaultConf();
+  std::unordered_map<std::string, std::string> backendConf{};
+  std::unordered_map<std::string, std::string> sessionConf{};
   backendConf.insert({gluten::kDebugModeEnabled, std::to_string(FLAGS_debug_mode)});
   backendConf.insert({gluten::kGlogVerboseLevel, std::to_string(FLAGS_v)});
   backendConf.insert({gluten::kGlogSeverityLevel, std::to_string(FLAGS_minloglevel)});
@@ -619,7 +630,7 @@ int main(int argc, char** argv) {
   setQueryTraceConfig(backendConf);
 
   initVeloxBackend(backendConf);
-  memory::MemoryManager::testingSetInstance({});
+  memory::MemoryManager::testingSetInstance(memory::MemoryManager::Options{});
 
   // Parse substrait plan, split file and data files.
   std::string substraitJsonFile = FLAGS_plan;
