@@ -23,9 +23,7 @@
 #include <arrow/type.h>
 #include <arrow/type_fwd.h>
 #include <arrow/util/type_fwd.h>
-#include <boost/algorithm/string.hpp>
 #include <folly/executors/IOThreadPoolExecutor.h>
-#include <parquet/properties.h>
 
 #include "memory/ColumnarBatch.h"
 #include "memory/VeloxColumnarBatch.h"
@@ -89,9 +87,6 @@ class VeloxParquetDataSource : public VeloxDataSource {
       std::shared_ptr<arrow::Schema> schema)
       : VeloxDataSource(filePath, schema), filePath_(filePath), schema_(schema), pool_(std::move(veloxPool)) {}
 
-  static std::unique_ptr<facebook::velox::parquet::WriterOptions> makeParquetWriteOption(
-      const std::unordered_map<std::string, std::string>& sparkConfs);
-
   void init(const std::unordered_map<std::string, std::string>& sparkConfs) override;
   virtual void initSink(const std::unordered_map<std::string, std::string>& sparkConfs);
   void inspectSchema(struct ArrowSchema* out) override;
