@@ -25,23 +25,19 @@ namespace gluten {
 arrow::Result<std::shared_ptr<VeloxShuffleWriter>> VeloxShuffleWriter::create(
     ShuffleWriterType type,
     uint32_t numPartitions,
-    std::unique_ptr<PartitionWriter> partitionWriter,
-    ShuffleWriterOptions options,
-    std::shared_ptr<facebook::velox::memory::MemoryPool> veloxPool,
-    arrow::MemoryPool* arrowPool) {
+    const std::shared_ptr<PartitionWriter>& partitionWriter,
+    const std::shared_ptr<ShuffleWriterOptions>& options,
+    MemoryManager* memoryManager) {
   std::shared_ptr<VeloxShuffleWriter> shuffleWriter;
   switch (type) {
     case ShuffleWriterType::kHashShuffle:
-      return VeloxHashShuffleWriter::create(
-          numPartitions, std::move(partitionWriter), std::move(options), veloxPool, arrowPool);
+      return VeloxHashShuffleWriter::create(numPartitions, std::move(partitionWriter), options, memoryManager);
     case ShuffleWriterType::kSortShuffle:
-      return VeloxSortShuffleWriter::create(
-          numPartitions, std::move(partitionWriter), std::move(options), veloxPool, arrowPool);
+      return VeloxSortShuffleWriter::create(numPartitions, std::move(partitionWriter), options, memoryManager);
     case ShuffleWriterType::kRssSortShuffle:
-      return VeloxRssSortShuffleWriter::create(
-          numPartitions, std::move(partitionWriter), std::move(options), veloxPool, arrowPool);
+      return VeloxRssSortShuffleWriter::create(numPartitions, std::move(partitionWriter), options, memoryManager);
     default:
-      return arrow::Status::Invalid("Unsupported shuffle writer type: ", std::to_string(type));
+      return arrow::Status::Invalid("Unsupported shuffle writer type: ", typeToString(type));
   }
 }
 
