@@ -17,13 +17,13 @@
 
 #include "shuffle/VeloxSortShuffleWriter.h"
 
-#include <arrow/io/memory.h>
-
-#include "memory/ArrowMemory.h"
 #include "memory/VeloxColumnarBatch.h"
+#include "shuffle/RadixSort.h"
 #include "utils/Common.h"
 #include "utils/Timer.h"
 
+#include <arrow/io/memory.h>
+
 namespace gluten {
 namespace {
 
@@ -49,23 +49,27 @@ std::pair<uint32_t, uint32_t> extractPageNumberAndOffset(uint64_t compactRowId)
 
 arrow::Result<std::shared_ptr<VeloxShuffleWriter>> VeloxSortShuffleWriter::create(
     uint32_t numPartitions,
-    std::unique_ptr<PartitionWriter> partitionWriter,
-    ShuffleWriterOptions options,
-    std::shared_ptr<facebook::velox::memory::MemoryPool> veloxPool,
-    arrow::MemoryPool* arrowPool) {
-  std::shared_ptr<VeloxSortShuffleWriter> writer(new VeloxSortShuffleWriter(
-      numPartitions, std::move(partitionWriter), std::move(options), std::move(veloxPool), arrowPool));
-  RETURN_NOT_OK(writer->init());
-  return writer;
+    const std::shared_ptr<PartitionWriter>& partitionWriter,
+    const std::shared_ptr<ShuffleWriterOptions>& options,
+    MemoryManager* memoryManager) {
+  if (auto sortOptions = std::dynamic_pointer_cast<SortShuffleWriterOptions>(options)) {
+    std::shared_ptr<VeloxSortShuffleWriter> writer(
+        new VeloxSortShuffleWriter(numPartitions, partitionWriter, sortOptions, memoryManager));
+    RETURN_NOT_OK(writer->init());
+    return writer;
+  }
+  return arrow::Status::Invalid("Error casting ShuffleWriterOptions to SortShuffleWriterOptions.");
 }
 
 VeloxSortShuffleWriter::VeloxSortShuffleWriter(
     uint32_t numPartitions,
-    std::unique_ptr<PartitionWriter> partitionWriter,
-    ShuffleWriterOptions options,
-    std::shared_ptr<facebook::velox::memory::MemoryPool> veloxPool,
-    arrow::MemoryPool* pool)
-    : VeloxShuffleWriter(numPartitions, std::move(partitionWriter), std::move(options), std::move(veloxPool), pool) {}
+    const std::shared_ptr<PartitionWriter>& partitionWriter,
+    const std::shared_ptr<SortShuffleWriterOptions>& options,
+    MemoryManager* memoryManager)
+    : VeloxShuffleWriter(numPartitions, partitionWriter, options, memoryManager),
+      useRadixSort_(options->useRadixSort),
+      initialSortBufferSize_(options->initialSortBufferSize),
+      diskWriteBufferSize_(options->diskWriteBufferSize) {}
 
 arrow::Status VeloxSortShuffleWriter::write(std::shared_ptr<ColumnarBatch> cb, int64_t memLimit) {
   ARROW_ASSIGN_OR_RAISE(auto rv, getPeeledRowVector(cb));
@@ -102,20 +106,13 @@ arrow::Status VeloxSortShuffleWriter::reclaimFixedSize(int64_t size, int64_t* ac
 
 arrow::Status VeloxSortShuffleWriter::init() {
   ARROW_RETURN_IF(
-      options_.partitioning == Partitioning::kSingle,
+      partitioning_ == Partitioning::kSingle,
       arrow::Status::Invalid("VeloxSortShuffleWriter doesn't support single partition."));
   allocateMinimalArray();
   // In Spark, sortedBuffer_ memory and compressionBuffer_ memory are pre-allocated and counted into executor
   // memory overhead. To align with Spark, we use arrow::default_memory_pool() to avoid counting these memory in Gluten.
-  ARROW_ASSIGN_OR_RAISE(
-      sortedBuffer_, arrow::AllocateBuffer(options_.sortEvictBufferSize, arrow::default_memory_pool()));
-  rawBuffer_ = sortedBuffer_->mutable_data();
-  auto compressedBufferLength = partitionWriter_->getCompressedBufferLength(
-      {std::make_shared<arrow::Buffer>(rawBuffer_, options_.sortEvictBufferSize)});
-  if (compressedBufferLength.has_value()) {
-    ARROW_ASSIGN_OR_RAISE(
-        compressionBuffer_, arrow::AllocateBuffer(*compressedBufferLength, arrow::default_memory_pool()));
-  }
+  ARROW_ASSIGN_OR_RAISE(sortedBuffer_, arrow::AllocateBuffer(diskWriteBufferSize_, arrow::default_memory_pool()));
+  sortedBufferPtr_ = sortedBuffer_->mutable_data();
   return arrow::Status::OK();
 }
 
@@ -123,15 +120,12 @@ void VeloxSortShuffleWriter::initRowType(const facebook::velox::RowVectorPtr& rv
   if (UNLIKELY(!rowType_)) {
     rowType_ = facebook::velox::asRowType(rv->type());
     fixedRowSize_ = facebook::velox::row::CompactRow::fixedRowSize(rowType_);
-    if (fixedRowSize_) {
-      *fixedRowSize_ += sizeof(RowSizeType);
-    }
   }
 }
 
 arrow::Result<facebook::velox::RowVectorPtr> VeloxSortShuffleWriter::getPeeledRowVector(
     const std::shared_ptr<ColumnarBatch>& cb) {
-  if (options_.partitioning == Partitioning::kRange) {
+  if (partitioning_ == Partitioning::kRange) {
     auto veloxColumnBatch = VeloxColumnarBatch::from(veloxPool_.get(), cb);
     VELOX_CHECK_NOT_NULL(veloxColumnBatch);
     const int32_t numColumns = veloxColumnBatch->numColumns();
@@ -168,17 +162,16 @@ arrow::Status VeloxSortShuffleWriter::insert(const facebook::velox::RowVectorPtr
 
   facebook::velox::row::CompactRow row(vector);
 
-  if (!fixedRowSize_) {
+  if (fixedRowSize_.has_value()) {
+    rowSize_.resize(inputRows, fixedRowSize_.value());
+  } else {
     rowSize_.resize(inputRows);
     rowSizePrefixSum_.resize(inputRows + 1);
     rowSizePrefixSum_[0] = 0;
     for (auto i = 0; i < inputRows; ++i) {
-      auto rowSize = row.rowSize(i) + sizeof(RowSizeType);
-      rowSize_[i] = rowSize;
-      rowSizePrefixSum_[i + 1] = rowSizePrefixSum_[i] + rowSize;
+      rowSize_[i] = row.rowSize(i);
+      rowSizePrefixSum_[i + 1] = rowSizePrefixSum_[i] + rowSize_[i] + sizeof(RowSizeType);
     }
-  } else {
-    rowSize_.resize(inputRows, *fixedRowSize_);
   }
 
   facebook::velox::vector_size_t rowOffset = 0;
@@ -186,7 +179,8 @@ arrow::Status VeloxSortShuffleWriter::insert(const facebook::velox::RowVectorPtr
     auto remainingRows = inputRows - rowOffset;
     auto rows = maxRowsToInsert(rowOffset, remainingRows);
     if (rows == 0) {
-      auto minSizeRequired = fixedRowSize_ ? fixedRowSize_.value() : rowSize_[rowOffset];
+      auto minSizeRequired =
+          (fixedRowSize_.has_value() ? fixedRowSize_.value() : rowSize_[rowOffset]) + sizeof(RowSizeType);
       acquireNewBuffer(static_cast<uint64_t>(memLimit), minSizeRequired);
       rows = maxRowsToInsert(rowOffset, remainingRows);
       ARROW_RETURN_IF(
@@ -215,7 +209,7 @@ void VeloxSortShuffleWriter::insertRows(
     // size(RowSize) | bytes
     memcpy(currentPage_ + pageCursor_, &rowSize_[row], sizeof(RowSizeType));
     offsets[i] = pageCursor_ + sizeof(RowSizeType);
-    pageCursor_ += rowSize_[row];
+    pageCursor_ += rowSize_[row] + sizeof(RowSizeType);
     VELOX_DCHECK_LE(pageCursor_, currenPageSize_);
   }
   compact.serialize(offset, size, offsets.data(), currentPage_);
@@ -238,7 +232,7 @@ arrow::Status VeloxSortShuffleWriter::evictAllPartitions() {
   int32_t begin = 0;
   {
     ScopedTimer timer(&sortTime_);
-    if (options_.useRadixSort) {
+    if (useRadixSort_) {
       begin = RadixSort::sort(arrayPtr_, arraySize_, numRecords, kPartitionIdStartByteIndex, kPartitionIdEndByteIndex);
     } else {
       std::sort(arrayPtr_, arrayPtr_ + numRecords);
@@ -288,61 +282,58 @@ arrow::Status VeloxSortShuffleWriter::evictPartition(uint32_t partitionId, size_
   // Serialize [begin, end)
   int64_t offset = 0;
   char* addr;
-  uint32_t size;
+  uint32_t recordSize;
 
   auto index = begin;
   while (index < end) {
     auto pageIndex = extractPageNumberAndOffset(arrayPtr_[index]);
     addr = pageAddresses_[pageIndex.first] + pageIndex.second;
-    size = *(reinterpret_cast<RowSizeType*>(addr));
-    if (offset + size > options_.sortEvictBufferSize && offset > 0) {
+    recordSize = *(reinterpret_cast<RowSizeType*>(addr)) + sizeof(RowSizeType);
+    if (offset + recordSize > diskWriteBufferSize_ && offset > 0) {
       sortTime.stop();
-      RETURN_NOT_OK(evictPartitionInternal(partitionId, index - begin, rawBuffer_, offset));
+      RETURN_NOT_OK(evictPartitionInternal(partitionId, sortedBufferPtr_, offset));
       sortTime.start();
       begin = index;
       offset = 0;
     }
-    if (size > static_cast<uint32_t>(options_.sortEvictBufferSize)) {
+    if (recordSize > static_cast<uint32_t>(diskWriteBufferSize_)) {
       // Split large rows.
       sortTime.stop();
       RowSizeType bytes = 0;
       auto* buffer = reinterpret_cast<uint8_t*>(addr);
-      while (bytes < size) {
-        auto rawLength = std::min<RowSizeType>((uint32_t)options_.sortEvictBufferSize, size - bytes);
+      while (bytes < recordSize) {
+        auto rawLength = std::min<RowSizeType>(static_cast<uint32_t>(diskWriteBufferSize_), recordSize - bytes);
         // Use numRows = 0 to represent a part of row.
-        RETURN_NOT_OK(evictPartitionInternal(partitionId, 0, buffer + bytes, rawLength));
+        RETURN_NOT_OK(evictPartitionInternal(partitionId, buffer + bytes, rawLength));
         bytes += rawLength;
       }
       begin++;
       sortTime.start();
     } else {
       // Copy small rows.
-      gluten::fastCopy(rawBuffer_ + offset, addr, size);
-      offset += size;
+      gluten::fastCopy(sortedBufferPtr_ + offset, addr, recordSize);
+      offset += recordSize;
     }
     index++;
   }
   sortTime.stop();
   if (offset > 0) {
     VELOX_CHECK(index > begin);
-    RETURN_NOT_OK(evictPartitionInternal(partitionId, index - begin, rawBuffer_, offset));
+    RETURN_NOT_OK(evictPartitionInternal(partitionId, sortedBufferPtr_, offset));
   }
   sortTime_ += sortTime.realTimeUsed();
   return arrow::Status::OK();
 }
 
-arrow::Status VeloxSortShuffleWriter::evictPartitionInternal(
-    uint32_t partitionId,
-    int32_t numRows,
-    uint8_t* buffer,
-    int64_t rawLength) {
+arrow::Status VeloxSortShuffleWriter::evictPartitionInternal(uint32_t partitionId, uint8_t* buffer, int64_t rawLength) {
   VELOX_CHECK(rawLength > 0);
   auto payload = std::make_unique<InMemoryPayload>(
-      numRows,
+      0,
+      nullptr,
       nullptr,
       std::vector<std::shared_ptr<arrow::Buffer>>{std::make_shared<arrow::Buffer>(buffer, rawLength)});
   updateSpillMetrics(payload);
-  RETURN_NOT_OK(partitionWriter_->sortEvict(partitionId, std::move(payload), compressionBuffer_, stopped_));
+  RETURN_NOT_OK(partitionWriter_->sortEvict(partitionId, std::move(payload), stopped_));
   return arrow::Status::OK();
 }
 
@@ -354,9 +345,10 @@ facebook::velox::vector_size_t VeloxSortShuffleWriter::maxRowsToInsert(
     return 0;
   }
   auto remainingBytes = pages_.back()->size() - pageCursor_;
-  if (fixedRowSize_) {
+  if (fixedRowSize_.has_value()) {
     return std::min(
-        static_cast<facebook::velox::vector_size_t>(remainingBytes / (fixedRowSize_.value())), remainingRows);
+        static_cast<facebook::velox::vector_size_t>(remainingBytes / (fixedRowSize_.value() + sizeof(RowSizeType))),
+        remainingRows);
   }
   auto beginIter = rowSizePrefixSum_.begin() + 1 + offset;
   auto bytesWritten = rowSizePrefixSum_[offset];
@@ -413,10 +405,10 @@ void VeloxSortShuffleWriter::growArrayIfNecessary(uint32_t rows) {
 
 uint32_t VeloxSortShuffleWriter::newArraySize(uint32_t rows) {
   auto newSize = arraySize_;
-  auto usableCapacity = options_.useRadixSort ? newSize / 2 : newSize;
+  auto usableCapacity = useRadixSort_ ? newSize / 2 : newSize;
   while (offset_ + rows > usableCapacity) {
     newSize <<= 1;
-    usableCapacity = options_.useRadixSort ? newSize / 2 : newSize;
+    usableCapacity = useRadixSort_ ? newSize / 2 : newSize;
   }
   return newSize;
 }
@@ -444,8 +436,8 @@ int64_t VeloxSortShuffleWriter::totalC2RTime() const {
 }
 
 void VeloxSortShuffleWriter::allocateMinimalArray() {
-  auto array = facebook::velox::AlignedBuffer::allocate<char>(
-      options_.sortBufferInitialSize * sizeof(uint64_t), veloxPool_.get());
+  auto array =
+      facebook::velox::AlignedBuffer::allocate<char>(initialSortBufferSize_ * sizeof(uint64_t), veloxPool_.get());
   setUpArray(std::move(array));
 }
 
