@@ -59,7 +59,7 @@ const std::unordered_set<std::string> kRegexFunctions = {
     "rlike"};
 
 const std::unordered_set<std::string> kBlackList =
-    {"split_part", "factorial", "trunc", "sequence", "approx_percentile", "get_array_struct_fields", "map_from_arrays"};
+    {"split_part", "sequence", "approx_percentile", "map_from_arrays"};
 } // namespace
 
 bool SubstraitToVeloxPlanValidator::parseVeloxType(
@@ -133,10 +133,22 @@ bool SubstraitToVeloxPlanValidator::validateRound(
   // Velox has different result with Spark on negative scale.
   auto typeCase = arguments[1].value().literal().literal_type_case();
   switch (typeCase) {
-    case ::substrait::Expression_Literal::LiteralTypeCase::kI32:
-      return (arguments[1].value().literal().i32() >= 0);
-    case ::substrait::Expression_Literal::LiteralTypeCase::kI64:
-      return (arguments[1].value().literal().i64() >= 0);
+    case ::substrait::Expression_Literal::LiteralTypeCase::kI32: {
+      int32_t scale = arguments[1].value().literal().i32();
+      if (scale < 0) {
+        LOG_VALIDATION_MSG("Round scale validation failed: scale " + std::to_string(scale) + " is negative.");
+        return false;
+      }
+      return true;
+    }
+    case ::substrait::Expression_Literal::LiteralTypeCase::kI64: {
+      int64_t scale = arguments[1].value().literal().i64();
+      if (scale < 0) {
+        LOG_VALIDATION_MSG("Round scale validation failed: scale " + std::to_string(scale) + " is negative.");
+        return false;
+      }
+      return true;
+    }
     default:
       LOG_VALIDATION_MSG("Round scale validation is not supported for type case " + std::to_string(typeCase));
       return false;
@@ -199,7 +211,7 @@ bool SubstraitToVeloxPlanValidator::validateScalarFunction(
   }
 
   const auto& function =
-      SubstraitParser::findFunctionSpec(planConverter_.getFunctionMap(), scalarFunction.function_reference());
+      SubstraitParser::findFunctionSpec(planConverter_->getFunctionMap(), scalarFunction.function_reference());
   const auto& name = SubstraitParser::getNameBeforeDelimiter(function);
   std::vector<std::string> types = SubstraitParser::getSubFunctionTypes(function);
 
@@ -209,15 +221,6 @@ bool SubstraitToVeloxPlanValidator::validateScalarFunction(
   if (name == "extract") {
     return validateExtractExpr(params);
   }
-  if (name == "concat") {
-    for (const auto& type : types) {
-      if (type.find("struct") != std::string::npos || type.find("map") != std::string::npos ||
-          type.find("list") != std::string::npos) {
-        LOG_VALIDATION_MSG(type + " is not supported in concat.");
-        return false;
-      }
-    }
-  }
 
   // Validate regex functions.
   if (kRegexFunctions.find(name) != kRegexFunctions.end()) {
@@ -253,7 +256,19 @@ bool SubstraitToVeloxPlanValidator::isAllowedCast(const TypePtr& fromType, const
   }
 
   // Limited support for Timestamp to X.
-  if (fromType->isTimestamp() && !(toType->isDate() || toType->isVarchar())) {
+  if (fromType->isTimestamp()) {
+    if (toType->isDecimal()) {
+      return false;
+    }
+
+    if (toType->isBigint()) {
+      return true;
+    }
+
+    if (toType->isDate() || toType->isVarchar()) {
+      return true;
+    }
+
     return false;
   }
 
@@ -268,6 +283,9 @@ bool SubstraitToVeloxPlanValidator::isAllowedCast(const TypePtr& fromType, const
     if (fromType->isVarchar()) {
       return true;
     }
+    if (fromType->isBoolean()) {
+      return true;
+    }
     if (fromType->isTinyint() || fromType->isSmallint() || fromType->isInteger() || fromType->isBigint() ||
         fromType->isDouble() || fromType->isReal()) {
       return true;
@@ -275,8 +293,42 @@ bool SubstraitToVeloxPlanValidator::isAllowedCast(const TypePtr& fromType, const
     return false;
   }
 
-  // Limited support for Complex types.
-  if (fromType->isArray() || fromType->isMap() || fromType->isRow()) {
+  // For complex types recursively check that their children can be cast.
+  if (fromType->isArray() && toType->isArray()) {
+    const auto& toElem = toType->asArray().elementType();
+    const auto& fromElem = fromType->asArray().elementType();
+
+    return isAllowedCast(fromElem, toElem);
+  }
+
+  if (fromType->isMap() && toType->isMap()) {
+      const auto& fromKey = fromType->asMap().keyType();
+      const auto& fromValue = fromType->asMap().valueType();
+      const auto& toKey = toType->asMap().keyType();
+      const auto& toValue = toType->asMap().valueType();
+
+      return isAllowedCast(fromKey, toKey) && isAllowedCast(fromValue, toValue);
+  }
+
+  if (fromType->isRow() && toType->isRow()) {
+      const auto& fromChildren = fromType->asRow().children();
+      const auto& toChildren = toType->asRow().children();
+
+      if (fromChildren.size() != toChildren.size()) {
+        return false;
+      }
+
+      for (size_t childIdx = 0; childIdx < fromChildren.size(); ++childIdx) {
+        if (!isAllowedCast(fromChildren[childIdx], toChildren[childIdx])) {
+          return false;
+        }
+      }
+
+      return true;
+  }
+
+  // Casting a complex type to/from any other type is not allowed.
+  if (fromType->isArray() || fromType->isMap() || fromType->isRow() || toType->isArray() || toType->isMap() || toType->isRow()) {
     return false;
   }
 
@@ -449,7 +501,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::TopNRel& topNRel
     return false;
   }
 
-  auto [sortingKeys, sortingOrders] = planConverter_.processSortField(topNRel.sorts(), rowType);
+  auto [sortingKeys, sortingOrders] = planConverter_->processSortField(topNRel.sorts(), rowType);
   folly::F14FastSet<std::string> sortingKeyNames;
   for (const auto& sortingKey : sortingKeys) {
     auto result = sortingKeyNames.insert(sortingKey->name());
@@ -616,7 +668,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::WindowRel& windo
   funcSpecs.reserve(windowRel.measures().size());
   for (const auto& smea : windowRel.measures()) {
     const auto& windowFunction = smea.measure();
-    funcSpecs.emplace_back(planConverter_.findFuncSpec(windowFunction.function_reference()));
+    funcSpecs.emplace_back(planConverter_->findFuncSpec(windowFunction.function_reference()));
     SubstraitParser::parseType(windowFunction.output_type());
     for (const auto& arg : windowFunction.arguments()) {
       auto typeCase = arg.value().rex_type_case();
@@ -1017,7 +1069,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::JoinRel& joinRel
 
   if (joinRel.has_expression()) {
     std::vector<const ::substrait::Expression::FieldReference*> leftExprs, rightExprs;
-    planConverter_.extractJoinKeys(joinRel.expression(), leftExprs, rightExprs);
+    planConverter_->extractJoinKeys(joinRel.expression(), leftExprs, rightExprs);
   }
 
   if (joinRel.has_post_join_filter()) {
@@ -1047,6 +1099,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::CrossRel& crossR
   switch (crossRel.type()) {
     case ::substrait::CrossRel_JoinType_JOIN_TYPE_INNER:
     case ::substrait::CrossRel_JoinType_JOIN_TYPE_LEFT:
+    case ::substrait::CrossRel_JoinType_JOIN_TYPE_LEFT_SEMI:
       break;
     case ::substrait::CrossRel_JoinType_JOIN_TYPE_OUTER:
       if (crossRel.has_expression()) {
@@ -1091,8 +1144,8 @@ bool SubstraitToVeloxPlanValidator::validateAggRelFunctionType(const ::substrait
 
   for (const auto& smea : aggRel.measures()) {
     const auto& aggFunction = smea.measure();
-    const auto& funcStep = planConverter_.toAggregationFunctionStep(aggFunction);
-    auto funcSpec = planConverter_.findFuncSpec(aggFunction.function_reference());
+    const auto& funcStep = planConverter_->toAggregationFunctionStep(aggFunction);
+    auto funcSpec = planConverter_->findFuncSpec(aggFunction.function_reference());
     std::vector<TypePtr> types;
     bool isDecimal = false;
     types = SubstraitParser::sigToTypes(funcSpec);
@@ -1103,7 +1156,7 @@ bool SubstraitToVeloxPlanValidator::validateAggRelFunctionType(const ::substrait
     }
     auto baseFuncName =
         SubstraitParser::mapToVeloxFunction(SubstraitParser::getNameBeforeDelimiter(funcSpec), isDecimal);
-    auto funcName = planConverter_.toAggregationFunctionName(baseFuncName, funcStep);
+    auto funcName = planConverter_->toAggregationFunctionName(baseFuncName, funcStep);
     auto signaturesOpt = exec::getAggregateFunctionSignatures(funcName);
     if (!signaturesOpt) {
       LOG_VALIDATION_MSG("can not find function signature for " + funcName + " in AggregateRel.");
@@ -1187,7 +1240,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::AggregateRel& ag
     }
 
     const auto& aggFunction = smea.measure();
-    const auto& functionSpec = planConverter_.findFuncSpec(aggFunction.function_reference());
+    const auto& functionSpec = planConverter_->findFuncSpec(aggFunction.function_reference());
     funcSpecs.emplace_back(functionSpec);
     SubstraitParser::parseType(aggFunction.output_type());
     // Validate the size of arguments.
@@ -1279,7 +1332,7 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::AggregateRel& ag
 }
 
 bool SubstraitToVeloxPlanValidator::validate(const ::substrait::ReadRel& readRel) {
-  planConverter_.toVeloxPlan(readRel);
+  planConverter_->toVeloxPlan(readRel);
 
   // Validate filter in ReadRel.
   if (readRel.has_filter()) {
@@ -1370,8 +1423,8 @@ bool SubstraitToVeloxPlanValidator::validate(const ::substrait::RelRoot& relRoot
 bool SubstraitToVeloxPlanValidator::validate(const ::substrait::Plan& plan) {
   try {
     // Create plan converter and expression converter to help the validation.
-    planConverter_.constructFunctionMap(plan);
-    exprConverter_ = planConverter_.getExprConverter();
+    planConverter_->constructFunctionMap(plan);
+    exprConverter_ = planConverter_->getExprConverter();
 
     for (const auto& rel : plan.relations()) {
       if (rel.has_root()) {
