@@ -380,7 +380,7 @@ const ::substrait::Expression_Cast& VeloxToSubstraitExprConvertor::toSubstraitEx
   for (auto& arg : castExprInputs) {
     substraitCastExpr->mutable_input()->MergeFrom(toSubstraitExpr(arena, arg, inputType));
   }
-  if (castExpr->nullOnFailure()) {
+  if (castExpr->isTryCast()) {
     substraitCastExpr->set_failure_behavior(::substrait::Expression_Cast_FailureBehavior_FAILURE_BEHAVIOR_RETURN_NULL);
   } else {
     substraitCastExpr->set_failure_behavior(
