@@ -20,16 +20,11 @@
 #include "ConfigExtractor.h"
 #include <stdexcept>
 
+#include "config/VeloxConfig.h"
 #include "utils/Exception.h"
 #include "velox/connectors/hive/HiveConfig.h"
 #include "velox/connectors/hive/storage_adapters/s3fs/S3Config.h"
 
-namespace {
-
-const std::string kVeloxFileHandleCacheEnabled = "spark.gluten.sql.columnar.backend.velox.fileHandleCacheEnabled";
-const bool kVeloxFileHandleCacheEnabledDefault = false;
-} // namespace
-
 namespace gluten {
 
 std::string getConfigValue(
@@ -59,6 +54,17 @@ std::shared_ptr<facebook::velox::config::ConfigBase> getHiveConfig(
   const std::string kVeloxAwsSdkLogLevel = "spark.gluten.velox.awsSdkLogLevel";
   const std::string kVeloxAwsSdkLogLevelDefault = "FATAL";
 
+  // Whether to use proxy from env for s3 c++ client
+  const std::string kVeloxS3UseProxyFromEnv = "spark.gluten.velox.s3UseProxyFromEnv";
+  const std::string kVeloxS3UseProxyFromEnvDefault = "false";
+
+  // Payload signing policy
+  const std::string kVeloxS3PayloadSigningPolicy = "spark.gluten.velox.s3PayloadSigningPolicy";
+  const std::string kVeloxS3PayloadSigningPolicyDefault = "Never";
+
+  // Log location of AWS C++ SDK
+  const std::string kVeloxS3LogLocation = "spark.gluten.velox.s3LogLocation";
+
   const std::unordered_map<S3Config::Keys, std::pair<std::string, std::optional<std::string>>> sparkSuffixes = {
       {S3Config::Keys::kAccessKey, std::make_pair("access.key", std::nullopt)},
       {S3Config::Keys::kSecretKey, std::make_pair("secret.key", std::nullopt)},
@@ -68,10 +74,12 @@ std::shared_ptr<facebook::velox::config::ConfigBase> getHiveConfig(
       {S3Config::Keys::kMaxAttempts, std::make_pair("retry.limit", std::nullopt)},
       {S3Config::Keys::kRetryMode, std::make_pair("retry.mode", "legacy")},
       {S3Config::Keys::kMaxConnections, std::make_pair("connection.maximum", "15")},
-      {S3Config::Keys::kConnectTimeout, std::make_pair("connection.timeout", "200s")},
+      {S3Config::Keys::kSocketTimeout, std::make_pair("connection.timeout", "200s")},
+      {S3Config::Keys::kConnectTimeout, std::make_pair("connection.establish.timeout", "30s")},
       {S3Config::Keys::kUseInstanceCredentials, std::make_pair("instance.credentials", "false")},
       {S3Config::Keys::kIamRole, std::make_pair("iam.role", std::nullopt)},
       {S3Config::Keys::kIamRoleSessionName, std::make_pair("iam.role.session.name", "gluten-session")},
+      {S3Config::Keys::kEndpointRegion, std::make_pair("endpoint.region", std::nullopt)},
   };
 
   // get Velox S3 config key from Spark Suffix.
@@ -123,11 +131,19 @@ std::shared_ptr<facebook::velox::config::ConfigBase> getHiveConfig(
   setConfigIfPresent(S3Config::Keys::kSSLEnabled);
   setConfigIfPresent(S3Config::Keys::kPathStyleAccess);
   setConfigIfPresent(S3Config::Keys::kMaxConnections);
+  setConfigIfPresent(S3Config::Keys::kSocketTimeout);
   setConfigIfPresent(S3Config::Keys::kConnectTimeout);
+  setConfigIfPresent(S3Config::Keys::kEndpointRegion);
 
-  hiveConfMap[facebook::velox::filesystems::S3Config::kS3LogLevel] =
-      conf->get<std::string>(kVeloxAwsSdkLogLevel, kVeloxAwsSdkLogLevelDefault);
-  ;
+  hiveConfMap[S3Config::kS3LogLevel] = conf->get<std::string>(kVeloxAwsSdkLogLevel, kVeloxAwsSdkLogLevelDefault);
+  hiveConfMap[S3Config::baseConfigKey(S3Config::Keys::kUseProxyFromEnv)] =
+      conf->get<std::string>(kVeloxS3UseProxyFromEnv, kVeloxS3UseProxyFromEnvDefault);
+  hiveConfMap[S3Config::kS3PayloadSigningPolicy] =
+      conf->get<std::string>(kVeloxS3PayloadSigningPolicy, kVeloxS3PayloadSigningPolicyDefault);
+  auto logLocation = conf->get<std::string>(kVeloxS3LogLocation);
+  if (logLocation.has_value()) {
+    hiveConfMap[S3Config::kS3LogLocation] = logLocation.value();
+  };
 
   // Convert all Spark bucket configs to Velox bucket configs.
   for (const auto& [key, value] : conf->rawConfigs()) {
@@ -149,7 +165,7 @@ std::shared_ptr<facebook::velox::config::ConfigBase> getHiveConfig(
 #ifdef ENABLE_GCS
   // https://github.com/GoogleCloudDataproc/hadoop-connectors/blob/master/gcs/CONFIGURATION.md#api-client-configuration
   auto gsStorageRootUrl = conf->get<std::string>("spark.hadoop.fs.gs.storage.root.url");
-  if (gsStorageRootUrl.hasValue()) {
+  if (gsStorageRootUrl.has_value()) {
     std::string gcsEndpoint = gsStorageRootUrl.value();
 
     if (!gcsEndpoint.empty()) {
@@ -160,32 +176,32 @@ std::shared_ptr<facebook::velox::config::ConfigBase> getHiveConfig(
   // https://github.com/GoogleCloudDataproc/hadoop-connectors/blob/master/gcs/CONFIGURATION.md#http-transport-configuration
   // https://cloud.google.com/cpp/docs/reference/storage/latest/classgoogle_1_1cloud_1_1storage_1_1LimitedErrorCountRetryPolicy
   auto gsMaxRetryCount = conf->get<std::string>("spark.hadoop.fs.gs.http.max.retry");
-  if (gsMaxRetryCount.hasValue()) {
+  if (gsMaxRetryCount.has_value()) {
     hiveConfMap[facebook::velox::connector::hive::HiveConfig::kGcsMaxRetryCount] = gsMaxRetryCount.value();
   }
 
   // https://cloud.google.com/cpp/docs/reference/storage/latest/classgoogle_1_1cloud_1_1storage_1_1LimitedTimeRetryPolicy
   auto gsMaxRetryTime = conf->get<std::string>("spark.hadoop.fs.gs.http.max.retry-time");
-  if (gsMaxRetryTime.hasValue()) {
+  if (gsMaxRetryTime.has_value()) {
     hiveConfMap[facebook::velox::connector::hive::HiveConfig::kGcsMaxRetryTime] = gsMaxRetryTime.value();
   }
 
   // https://github.com/GoogleCloudDataproc/hadoop-connectors/blob/master/gcs/CONFIGURATION.md#authentication
   auto gsAuthType = conf->get<std::string>("spark.hadoop.fs.gs.auth.type");
-  if (gsAuthType.hasValue()) {
-    std::string type = gsAuthType.value();
-    if (type == "SERVICE_ACCOUNT_JSON_KEYFILE") {
-      auto gsAuthServiceAccountJsonKeyfile =
-          conf->get<std::string>("spark.hadoop.fs.gs.auth.service.account.json.keyfile");
-      if (gsAuthServiceAccountJsonKeyfile.hasValue()) {
-        hiveConfMap[facebook::velox::connector::hive::HiveConfig::kGcsCredentialsPath] =
-            gsAuthServiceAccountJsonKeyfile.value();
-      } else {
-        LOG(WARNING) << "STARTUP: conf spark.hadoop.fs.gs.auth.type is set to SERVICE_ACCOUNT_JSON_KEYFILE, "
-                        "however conf spark.hadoop.fs.gs.auth.service.account.json.keyfile is not set";
-        throw GlutenException("Conf spark.hadoop.fs.gs.auth.service.account.json.keyfile is not set");
-      }
+  auto gsAuthServiceAccountJsonKeyfile = conf->get<std::string>("spark.hadoop.fs.gs.auth.service.account.json.keyfile");
+  if (gsAuthType.has_value() && gsAuthType.value() == "SERVICE_ACCOUNT_JSON_KEYFILE") {
+    if (gsAuthServiceAccountJsonKeyfile.has_value()) {
+      hiveConfMap[facebook::velox::connector::hive::HiveConfig::kGcsCredentialsPath] =
+          gsAuthServiceAccountJsonKeyfile.value();
+    } else {
+      LOG(WARNING) << "STARTUP: conf spark.hadoop.fs.gs.auth.type is set to SERVICE_ACCOUNT_JSON_KEYFILE, "
+                      "however conf spark.hadoop.fs.gs.auth.service.account.json.keyfile is not set";
+      throw GlutenException("Conf spark.hadoop.fs.gs.auth.service.account.json.keyfile is not set");
     }
+  } else if (gsAuthServiceAccountJsonKeyfile.has_value()) {
+    LOG(WARNING) << "STARTUP: conf spark.hadoop.fs.gs.auth.service.account.json.keyfile is set, "
+                    "but conf spark.hadoop.fs.gs.auth.type is not SERVICE_ACCOUNT_JSON_KEYFILE";
+    throw GlutenException("Conf spark.hadoop.fs.gs.auth.type is missing or incorrect");
   }
 #endif
 
@@ -202,6 +218,26 @@ std::shared_ptr<facebook::velox::config::ConfigBase> getHiveConfig(
 
   hiveConfMap[facebook::velox::connector::hive::HiveConfig::kEnableFileHandleCache] =
       conf->get<bool>(kVeloxFileHandleCacheEnabled, kVeloxFileHandleCacheEnabledDefault) ? "true" : "false";
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kMaxCoalescedBytes] =
+      conf->get<std::string>(kMaxCoalescedBytes, "67108864"); // 64M
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kMaxCoalescedDistance] =
+      conf->get<std::string>(kMaxCoalescedDistance, "512KB"); // 512KB
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kPrefetchRowGroups] =
+      conf->get<std::string>(kPrefetchRowGroups, "1");
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kLoadQuantum] =
+      conf->get<std::string>(kLoadQuantum, "268435456"); // 256M
+  auto footerEstimatedSize = conf->get<std::string>(kDirectorySizeGuess, "32768"); // 32K
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kFooterEstimatedSize] =
+      conf->get<std::string>(kFooterEstimatedSize, footerEstimatedSize); // 32K
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kFilePreloadThreshold] =
+      conf->get<std::string>(kFilePreloadThreshold, "1048576"); // 1M
+
+  // read as UTC
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kReadTimestampPartitionValueAsLocalTime] = "false";
+
+  // Maps table field names to file field names using names, not indices.
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kParquetUseColumnNames] = "true";
+  hiveConfMap[facebook::velox::connector::hive::HiveConfig::kOrcUseColumnNames] = "true";
 
   return std::make_shared<facebook::velox::config::ConfigBase>(std::move(hiveConfMap));
 }
