@@ -25,9 +25,13 @@ ARCH=`uname -m`
 
 cd "$GLUTEN_DIR"
 
+if [ "$ARCH" = "aarch64" ]; then
+  export CPU_TARGET="aarch64"
+fi
+
 # build gluten with velox backend, prompt always respond y
 export PROMPT_ALWAYS_RESPOND=y
-./dev/buildbundle-veloxbe.sh --enable_vcpkg=OFF --build_tests=ON --build_benchmarks=ON --enable_s3=ON  --enable_hdfs=ON "$@"
+./dev/buildbundle-veloxbe.sh --enable_vcpkg=OFF --build_tests=ON --build_benchmarks=ON --enable_s3=ON --enable_hdfs=ON "$@"
 
 # make thirdparty package
 ./dev/build-thirdparty.sh
