@@ -18,7 +18,6 @@ package org.apache.gluten.jni;
 
 import org.apache.gluten.exception.GlutenException;
 
-import org.apache.spark.util.SparkShutdownManagerUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -26,35 +25,16 @@ import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
-import java.lang.reflect.Field;
-import java.lang.reflect.Method;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
-import java.util.ArrayList;
-import java.util.Collections;
 import java.util.HashSet;
-import java.util.Iterator;
-import java.util.LinkedHashSet;
-import java.util.List;
 import java.util.Set;
-import java.util.Vector;
-
-import scala.runtime.BoxedUnit;
 
 public class JniLibLoader {
   private static final Logger LOG = LoggerFactory.getLogger(JniLibLoader.class);
 
   private static final Set<String> LOADED_LIBRARY_PATHS = new HashSet<>();
-  private static final Set<String> REQUIRE_UNLOAD_LIBRARY_PATHS = new LinkedHashSet<>();
-
-  static {
-    SparkShutdownManagerUtil.addHookForLibUnloading(
-        () -> {
-          forceUnloadAll();
-          return BoxedUnit.UNIT;
-        });
-  }
 
   private final String workDir;
   private final Set<String> loadedLibraries = new HashSet<>();
@@ -63,13 +43,6 @@ public class JniLibLoader {
     this.workDir = workDir;
   }
 
-  public static synchronized void forceUnloadAll() {
-    List<String> loaded;
-    loaded = new ArrayList<>(REQUIRE_UNLOAD_LIBRARY_PATHS);
-    Collections.reverse(loaded); // use reversed order to unload
-    loaded.forEach(JniLibLoader::unloadFromPath);
-  }
-
   private static String toRealPath(String libPath) {
     String realPath = libPath;
     try {
@@ -83,7 +56,7 @@ public class JniLibLoader {
     }
   }
 
-  private static void loadFromPath0(String libPath, boolean requireUnload) {
+  private static void loadFromPath0(String libPath) {
     libPath = toRealPath(libPath);
     if (LOADED_LIBRARY_PATHS.contains(libPath)) {
       LOG.debug("Library in path {} has already been loaded, skipping", libPath);
@@ -92,66 +65,24 @@ public class JniLibLoader {
       LOADED_LIBRARY_PATHS.add(libPath);
       LOG.info("Library {} has been loaded using path-loading method", libPath);
     }
-    if (requireUnload) {
-      REQUIRE_UNLOAD_LIBRARY_PATHS.add(libPath);
-    }
   }
 
-  public static synchronized void loadFromPath(String libPath, boolean requireUnload) {
+  public static synchronized void loadFromPath(String libPath) {
     final File file = new File(libPath);
     if (!file.isFile() || !file.exists()) {
       throw new GlutenException("library at path: " + libPath + " is not a file or does not exist");
     }
-    loadFromPath0(file.getAbsolutePath(), requireUnload);
+    loadFromPath0(file.getAbsolutePath());
   }
 
-  public static synchronized void unloadFromPath(String libPath) {
-    if (!LOADED_LIBRARY_PATHS.remove(libPath)) {
-      LOG.warn("Library {} was not loaded or already unloaded:", libPath);
-      return;
-    }
-    LOG.info("Starting unload library path: {} ", libPath);
-    REQUIRE_UNLOAD_LIBRARY_PATHS.remove(libPath);
-    try {
-      ClassLoader classLoader = JniLibLoader.class.getClassLoader();
-      Field field = ClassLoader.class.getDeclaredField("nativeLibraries");
-      field.setAccessible(true);
-      Vector<Object> libs = (Vector<Object>) field.get(classLoader);
-      Iterator<Object> it = libs.iterator();
-      while (it.hasNext()) {
-        Object object = it.next();
-        Field[] fs = object.getClass().getDeclaredFields();
-        for (int k = 0; k < fs.length; k++) {
-          if (fs[k].getName().equals("name")) {
-            fs[k].setAccessible(true);
-            String verbosePath = fs[k].get(object).toString();
-            File verboseFile = new File(verbosePath);
-            String verboseFileName = verboseFile.getName();
-            File libFile = new File(libPath);
-            String libFileName = libFile.getName();
-
-            if (verboseFileName.equals(libFileName)) {
-              LOG.info("Finalizing library file: {}", libFileName);
-              Method finalize = object.getClass().getDeclaredMethod("finalize");
-              finalize.setAccessible(true);
-              finalize.invoke(object);
-            }
-          }
-        }
-      }
-    } catch (Throwable th) {
-      LOG.error("Unload native library error: ", th);
-    }
-  }
-
-  public synchronized void load(String libPath, boolean requireUnload) {
+  public synchronized void load(String libPath) {
     try {
       if (loadedLibraries.contains(libPath)) {
         LOG.debug("Library {} has already been loaded, skipping", libPath);
         return;
       }
       File file = moveToWorkDir(workDir, libPath);
-      loadWithLink(file.getAbsolutePath(), null, requireUnload);
+      loadWithLink(file.getAbsolutePath(), null);
       loadedLibraries.add(libPath);
       LOG.info("Successfully loaded library {}", libPath);
     } catch (IOException e) {
@@ -159,14 +90,13 @@ public class JniLibLoader {
     }
   }
 
-  public synchronized void loadAndCreateLink(
-      String libPath, String linkName, boolean requireUnload) {
+  public synchronized void loadAndCreateLink(String libPath, String linkName) {
     try {
       if (loadedLibraries.contains(libPath)) {
         LOG.debug("Library {} has already been loaded, skipping", libPath);
       }
       File file = moveToWorkDir(workDir, libPath);
-      loadWithLink(file.getAbsolutePath(), linkName, requireUnload);
+      loadWithLink(file.getAbsolutePath(), linkName);
       loadedLibraries.add(libPath);
       LOG.info("Successfully loaded library {}", libPath);
     } catch (IOException e) {
@@ -176,11 +106,11 @@ public class JniLibLoader {
 
   private File moveToWorkDir(String workDir, String libraryToLoad) throws IOException {
     // final File temp = File.createTempFile(workDir, libraryToLoad);
-    final Path libPath = Paths.get(workDir + "/" + libraryToLoad);
+    final Path libPath = Paths.get(workDir, libraryToLoad);
     if (Files.exists(libPath)) {
       Files.delete(libPath);
     }
-    final File temp = new File(workDir + "/" + libraryToLoad);
+    final File temp = libPath.toFile();
     if (!temp.getParentFile().exists()) {
       temp.getParentFile().mkdirs();
     }
@@ -197,9 +127,8 @@ public class JniLibLoader {
     return temp;
   }
 
-  private void loadWithLink(String libPath, String linkName, boolean requireUnload)
-      throws IOException {
-    loadFromPath0(libPath, requireUnload);
+  private void loadWithLink(String libPath, String linkName) throws IOException {
+    loadFromPath0(libPath);
     LOG.info("Library {} has been loaded", libPath);
     if (linkName != null) {
       Path target = Paths.get(libPath);
