@@ -16,12 +16,13 @@
  */
 package org.apache.gluten.memory.memtarget;
 
-import org.apache.gluten.config.GlutenConfig;
+import org.apache.gluten.config.GlutenCoreConfig;
 import org.apache.gluten.memory.MemoryUsageStatsBuilder;
 import org.apache.gluten.memory.memtarget.spark.TreeMemoryConsumers;
 
 import org.apache.spark.SparkEnv;
 import org.apache.spark.annotation.Experimental;
+import org.apache.spark.memory.MemoryMode;
 import org.apache.spark.memory.TaskMemoryManager;
 import org.apache.spark.util.SparkResourceUtil;
 import org.slf4j.Logger;
@@ -50,8 +51,8 @@ public final class MemoryTargets {
 
   @Experimental
   public static MemoryTarget dynamicOffHeapSizingIfEnabled(MemoryTarget memoryTarget) {
-    if (GlutenConfig.get().dynamicOffHeapSizingEnabled()) {
-      return new DynamicOffHeapSizingMemoryTarget(memoryTarget);
+    if (GlutenCoreConfig.get().dynamicOffHeapSizingEnabled()) {
+      return new DynamicOffHeapSizingMemoryTarget();
     }
 
     return memoryTarget;
@@ -62,8 +63,14 @@ public final class MemoryTargets {
       String name,
       Spiller spiller,
       Map<String, MemoryUsageStatsBuilder> virtualChildren) {
-    final TreeMemoryConsumers.Factory factory = TreeMemoryConsumers.factory(tmm);
-    if (GlutenConfig.get().memoryIsolation()) {
+    final MemoryMode mode;
+    if (GlutenCoreConfig.get().dynamicOffHeapSizingEnabled()) {
+      mode = MemoryMode.ON_HEAP;
+    } else {
+      mode = MemoryMode.OFF_HEAP;
+    }
+    final TreeMemoryConsumers.Factory factory = TreeMemoryConsumers.factory(tmm, mode);
+    if (GlutenCoreConfig.get().memoryIsolation()) {
       return TreeMemoryTargets.newChild(factory.isolatedRoot(), name, spiller, virtualChildren);
     }
     final TreeMemoryTarget root = factory.legacyRoot();
@@ -90,7 +97,7 @@ public final class MemoryTargets {
           LOGGER.info("Request for spilling on consumer {}...", consumer.name());
           // Note: Spill from root node so other consumers also get spilled.
           long spilled = TreeMemoryTargets.spillTree(root, Long.MAX_VALUE);
-          LOGGER.info("Consumer {} spilled {} bytes.", consumer.name(), spilled);
+          LOGGER.info("Consumer {} gets {} bytes from spilling.", consumer.name(), spilled);
         });
   }
 }
