@@ -52,8 +52,8 @@ object ConventionReq {
   ) extends ConventionReq
 
   val any: ConventionReq = of(RowType.Any, BatchType.Any)
-  val row: ConventionReq = ofRow(RowType.Is(Convention.RowType.VanillaRow))
-  val vanillaBatch: ConventionReq = ofBatch(BatchType.Is(Convention.BatchType.VanillaBatch))
+  val vanillaRow: ConventionReq = ofRow(RowType.Is(Convention.RowType.VanillaRowType))
+  val vanillaBatch: ConventionReq = ofBatch(BatchType.Is(Convention.BatchType.VanillaBatchType))
 
   def get(plan: SparkPlan): Seq[ConventionReq] = ConventionFunc.create().conventionReqOf(plan)
   def of(rowType: RowType, batchType: BatchType): ConventionReq = Impl(rowType, batchType)
