@@ -124,17 +124,23 @@ object SparkMemoryUtil {
       }
 
       override def visit(noopMemoryTarget: NoopMemoryTarget): String = {
-        prettyPrintStats("No-op memory manager stats: ", noopMemoryTarget)
+        prettyPrintStats("No-op memory target stats: ", noopMemoryTarget)
       }
 
       override def visit(
           dynamicOffHeapSizingMemoryTarget: DynamicOffHeapSizingMemoryTarget): String = {
-        dynamicOffHeapSizingMemoryTarget.delegated().accept(this)
+        prettyPrintStats(
+          "Dynamic off-heap sizing memory target stats: ",
+          dynamicOffHeapSizingMemoryTarget)
       }
 
       override def visit(retryOnOomMemoryTarget: RetryOnOomMemoryTarget): String = {
         retryOnOomMemoryTarget.target().accept(this)
       }
+
+      override def visit(globalOffHeapMemoryTarget: GlobalOffHeapMemoryTarget): String = {
+        prettyPrintStats("Global off-heap target stats: ", globalOffHeapMemoryTarget)
+      }
     })
   }
 
