@@ -17,14 +17,21 @@
 package org.apache.spark.util
 
 object SparkVersionUtil {
-  def majorMinorVersion(): (Int, Int) = {
-    VersionUtils.majorMinorVersion(org.apache.spark.SPARK_VERSION)
-  }
+  val lteSpark32: Boolean = compareMajorMinorVersion((3, 2)) <= 0
+  private val comparedWithSpark33 = compareMajorMinorVersion((3, 3))
+  private val comparedWithSpark35 = compareMajorMinorVersion((3, 5))
+  val eqSpark33: Boolean = comparedWithSpark33 == 0
+  val lteSpark33: Boolean = lteSpark32 || eqSpark33
+  val gteSpark33: Boolean = comparedWithSpark33 >= 0
+  val gteSpark35: Boolean = comparedWithSpark35 >= 0
 
   // Returns X. X < 0 if one < other, x == 0 if one == other, x > 0 if one > other.
-  def compareMajorMinorVersion(one: (Int, Int), other: (Int, Int)): Int = {
-    val base = 1000
-    assert(one._2 < base && other._2 < base)
-    one._1 * base + one._2 - (other._1 * base + other._2)
+  def compareMajorMinorVersion(other: (Int, Int)): Int = {
+    val (major, minor) = VersionUtils.majorMinorVersion(org.apache.spark.SPARK_VERSION)
+    if (major == other._1) {
+      minor - other._2
+    } else {
+      major - other._1
+    }
   }
 }
