@@ -60,11 +60,11 @@ object RasCluster {
         mutable.ListBuffer()
 
       override def contains(t: CanonicalNode[T]): Boolean = {
-        deDup.contains(t.toHashKey())
+        deDup.contains(t.toHashKey)
       }
 
       override def add(t: CanonicalNode[T]): Unit = {
-        val key = t.toHashKey()
+        val key = t.toHashKey
         assert(!deDup.contains(key))
         ras.metadataModel.verify(metadata, ras.metadataModel.metadataOf(t.self()))
         deDup += key
@@ -77,14 +77,12 @@ object RasCluster {
     }
   }
 
-  case class ImmutableRasCluster[T <: AnyRef] private (
-      ras: Ras[T],
-      override val nodes: Seq[CanonicalNode[T]])
-    extends RasCluster[T]
-
   object ImmutableRasCluster {
-    def apply[T <: AnyRef](ras: Ras[T], cluster: RasCluster[T]): ImmutableRasCluster[T] = {
-      ImmutableRasCluster(ras, cluster.nodes().toVector)
+    def apply[T <: AnyRef](ras: Ras[T], cluster: RasCluster[T]): RasCluster[T] = {
+      new Impl[T](ras, cluster.nodes().toSeq)
     }
+
+    private class Impl[T <: AnyRef](ras: Ras[T], override val nodes: Seq[CanonicalNode[T]])
+      extends RasCluster[T]
   }
 }
