@@ -187,33 +187,42 @@ object DpZipperAlgo {
 
       val xSolutions: mutable.Map[XKey[X, Y, XOutput, YOutput], XOutput] = mutable.Map()
 
-      def loop(): Unit = {
-        while (true) {
-          val xKeys: Set[XKey[X, Y, XOutput, YOutput]] =
-            algoDef.browseY(thisY).map(algoDef.keyOfX(_)).toSet
+      // Continuously tries to solve the unsolved children keys until all keys are filled with
+      // solutions.
+      def doExhaustively(onUnsolvedKeys: Set[XKey[X, Y, XOutput, YOutput]] => Unit): Unit = {
+        def browseXKeys(): Set[XKey[X, Y, XOutput, YOutput]] = {
+          algoDef.browseY(thisY).map(algoDef.keyOfX(_)).toSet
+        }
 
-          val xCount = xKeys.size
-          if (xCount == xSolutions.size) {
-            // We got enough children solutions.
-            return
-          }
+        var allXKeys = browseXKeys()
+        do {
+          val unsolvedKeys = allXKeys.filterNot(xKey => xSolutions.contains(xKey))
+          onUnsolvedKeys(unsolvedKeys)
+          allXKeys = browseXKeys()
+        } while (xSolutions.size < allXKeys.size)
+      }
 
-          xKeys.filterNot(xKey => xSolutions.contains(xKey)).foreach {
-            childXKey =>
-              val xOutputs = solveXRec(childXKey.x, xCycleDetector, newYCycleDetector)
-              val cm = xOutputs.cycleMemory()
-              cyclicXs ++= cm.cyclicXs
-              cyclicYs ++= cm.cyclicYs
-              sBuilder.addYAsBackDependencyOfX(thisY, childXKey.x)
-              xSolutions += childXKey -> xOutputs.output()
-              // Try applying adjustment
-              // to see if algo caller likes to add some Xs or to invalidate
-              // some of the registered solutions.
-              adjustment.exploreChildX(adjustmentPanel, childXKey.x)
+      doExhaustively {
+        _ =>
+          doExhaustively {
+            unsolvedChildXKeys =>
+              unsolvedChildXKeys.foreach {
+                childXKey =>
+                  val xOutputs = solveXRec(childXKey.x, xCycleDetector, newYCycleDetector)
+                  val cm = xOutputs.cycleMemory()
+                  cyclicXs ++= cm.cyclicXs
+                  cyclicYs ++= cm.cyclicYs
+                  sBuilder.addYAsBackDependencyOfX(thisY, childXKey.x)
+                  xSolutions += childXKey -> xOutputs.output()
+                  // Try applying adjustment
+                  // to see if algo caller likes to add some Xs or to invalidate
+                  // some of the registered solutions.
+                  adjustment.exploreChildX(adjustmentPanel, childXKey.x)
+              }
           }
           adjustment.exploreParentY(adjustmentPanel, thisY)
           // If an adjustment (this adjustment or children's) just invalidated one or more
-          // children of this element's solutions, the children's keys would be removed from
+          // children of this element's solutions, the children's keys would be removed from the
           // back-dependency list. We do a test here to trigger re-computation if some children
           // do get invalidated.
           xSolutions.keySet.foreach {
@@ -222,11 +231,8 @@ object DpZipperAlgo {
                 xSolutions -= childXKey
               }
           }
-        }
       }
 
-      loop()
-
       // Remove this element from cycle memory, if it's in it.
       cyclicYs -= algoDef.keyOfY(thisY)
 
@@ -262,33 +268,42 @@ object DpZipperAlgo {
 
       val ySolutions: mutable.Map[YKey[X, Y, XOutput, YOutput], YOutput] = mutable.Map()
 
-      def loop(): Unit = {
-        while (true) {
-          val yKeys: Set[YKey[X, Y, XOutput, YOutput]] =
-            algoDef.browseX(thisX).map(algoDef.keyOfY(_)).toSet
+      // Continuously tries to solve the unsolved children keys until all keys are filled with
+      // solutions.
+      def doExhaustively(onUnsolvedKeys: Set[YKey[X, Y, XOutput, YOutput]] => Unit): Unit = {
+        def browseYKeys(): Set[YKey[X, Y, XOutput, YOutput]] = {
+          algoDef.browseX(thisX).map(algoDef.keyOfY(_)).toSet
+        }
 
-          val yCount = yKeys.size
-          if (yCount == ySolutions.size) {
-            // We got enough children solutions.
-            return
-          }
+        var allYKeys = browseYKeys()
+        do {
+          val unsolvedKeys = allYKeys.filterNot(yKey => ySolutions.contains(yKey))
+          onUnsolvedKeys(unsolvedKeys)
+          allYKeys = browseYKeys()
+        } while (ySolutions.size < allYKeys.size)
+      }
 
-          yKeys.filterNot(yKey => ySolutions.contains(yKey)).foreach {
-            childYKey =>
-              val yOutputs = solveYRec(childYKey.y, newXCycleDetector, yCycleDetector)
-              val cm = yOutputs.cycleMemory()
-              cyclicXs ++= cm.cyclicXs
-              cyclicYs ++= cm.cyclicYs
-              sBuilder.addXAsBackDependencyOfY(thisX, childYKey.y)
-              ySolutions += childYKey -> yOutputs.output()
-              // Try applying adjustment
-              // to see if algo caller likes to add some Ys or to invalidate
-              // some of the registered solutions.
-              adjustment.exploreChildY(adjustmentPanel, childYKey.y)
+      doExhaustively {
+        _ =>
+          doExhaustively {
+            unsolvedChildYKeys =>
+              unsolvedChildYKeys.foreach {
+                childYKey =>
+                  val yOutputs = solveYRec(childYKey.y, newXCycleDetector, yCycleDetector)
+                  val cm = yOutputs.cycleMemory()
+                  cyclicXs ++= cm.cyclicXs
+                  cyclicYs ++= cm.cyclicYs
+                  sBuilder.addXAsBackDependencyOfY(thisX, childYKey.y)
+                  ySolutions += childYKey -> yOutputs.output()
+                  // Try applying adjustment
+                  // to see if algo caller likes to add some Ys or to invalidate
+                  // some of the registered solutions.
+                  adjustment.exploreChildY(adjustmentPanel, childYKey.y)
+              }
           }
           adjustment.exploreParentX(adjustmentPanel, thisX)
           // If an adjustment (this adjustment or children's) just invalidated one or more
-          // children of this element's solutions, the children's keys would be removed from
+          // children of this element's solutions, the children's keys would be removed from the
           // back-dependency list. We do a test here to trigger re-computation if some children
           // do get invalidated.
           ySolutions.keySet.foreach {
@@ -297,11 +312,8 @@ object DpZipperAlgo {
                 ySolutions -= childYKey
               }
           }
-        }
       }
 
-      loop()
-
       // Remove this element from cycle memory, if it's in it.
       cyclicXs -= algoDef.keyOfX(thisX)
 
