@@ -22,6 +22,8 @@ import org.apache.gluten.ras.memo.Closure
 import org.apache.gluten.ras.path.InClusterPath
 import org.apache.gluten.ras.property.PropertySet
 
+import java.util
+
 import scala.collection.mutable
 
 trait RuleApplier[T <: AnyRef] {
@@ -30,20 +32,21 @@ trait RuleApplier[T <: AnyRef] {
 }
 
 object RuleApplier {
-  def apply[T <: AnyRef](ras: Ras[T], closure: Closure[T], rule: RasRule[T]): RuleApplier[T] = {
+  def regular[T <: AnyRef](ras: Ras[T], closure: Closure[T], rule: RasRule[T]): RuleApplier[T] = {
     new RegularRuleApplier(ras, closure, rule)
   }
 
-  def apply[T <: AnyRef](
+  def enforcer[T <: AnyRef](
       ras: Ras[T],
       closure: Closure[T],
-      rule: EnforcerRule[T]): RuleApplier[T] = {
-    new EnforcerRuleApplier[T](ras, closure, rule)
+      constraintSet: PropertySet[T],
+      rule: RasRule[T]): RuleApplier[T] = {
+    new EnforcerRuleApplier[T](ras, closure, constraintSet, rule)
   }
 
   private class RegularRuleApplier[T <: AnyRef](ras: Ras[T], closure: Closure[T], rule: RasRule[T])
     extends RuleApplier[T] {
-    private val deDup = mutable.Map[RasClusterKey, mutable.Set[UnsafeHashKey[T]]]()
+    private val deDup = DeDup(ras)
 
     override def apply(icp: InClusterPath[T]): Unit = {
       if (!shape.identify(icp.path())) {
@@ -52,13 +55,9 @@ object RuleApplier {
       val cKey = icp.cluster()
       val path = icp.path()
       val plan = path.plan()
-      val appliedPlans = deDup.getOrElseUpdate(cKey, mutable.Set())
-      val pKey = ras.toHashKey(plan)
-      if (appliedPlans.contains(pKey)) {
-        return
+      deDup.run(cKey, plan) {
+        apply0(cKey, plan)
       }
-      apply0(cKey, plan)
-      appliedPlans += pKey
     }
 
     private def apply0(cKey: RasClusterKey, plan: T): Unit = {
@@ -67,7 +66,7 @@ object RuleApplier {
         equiv =>
           closure
             .openFor(cKey)
-            .memorize(equiv, ras.anyPropSet())
+            .memorize(equiv, ras.userConstraintSet())
       }
     }
 
@@ -77,11 +76,10 @@ object RuleApplier {
   private class EnforcerRuleApplier[T <: AnyRef](
       ras: Ras[T],
       closure: Closure[T],
-      rule: EnforcerRule[T])
+      constraintSet: PropertySet[T],
+      rule: RasRule[T])
     extends RuleApplier[T] {
-    private val deDup = mutable.Map[RasClusterKey, mutable.Set[UnsafeHashKey[T]]]()
-    private val constraint = rule.constraint()
-    private val constraintDef = constraint.definition()
+    private val deDup = DeDup(ras)
 
     override def apply(icp: InClusterPath[T]): Unit = {
       if (!shape.identify(icp.path())) {
@@ -90,18 +88,13 @@ object RuleApplier {
       val cKey = icp.cluster()
       val path = icp.path()
       val propSet = path.node().self().propSet()
-      if (propSet.get(constraintDef).satisfies(constraint)) {
+      if (propSet.satisfies(constraintSet)) {
         return
       }
       val plan = path.plan()
-      val pKey = ras.toHashKey(plan)
-      val appliedPlans = deDup.getOrElseUpdate(cKey, mutable.Set())
-      if (appliedPlans.contains(pKey)) {
-        return
+      deDup.run(cKey, plan) {
+        apply0(cKey, constraintSet, plan)
       }
-      val constraintSet = propSet.withProp(constraint)
-      apply0(cKey, constraintSet, plan)
-      appliedPlans += pKey
     }
 
     private def apply0(cKey: RasClusterKey, constraintSet: PropertySet[T], plan: T): Unit = {
@@ -116,4 +109,48 @@ object RuleApplier {
 
     override val shape: Shape[T] = rule.shape()
   }
+
+  private trait DeDup[T <: AnyRef] {
+    def run(cKey: RasClusterKey, plan: T)(computation: => Unit): Unit
+  }
+
+  private object DeDup {
+    def apply[T <: AnyRef](ras: Ras[T]): DeDup[T] = {
+      new Impl[T](ras)
+    }
+
+    private class Impl[T <: AnyRef](ras: Ras[T]) extends DeDup[T] {
+      private val layerOne = mutable.Map[RasClusterKey, java.util.IdentityHashMap[T, Object]]()
+      private val layerTwo = mutable.Map[RasClusterKey, mutable.Set[UnsafeHashKey[T]]]()
+
+      override def run(cKey: RasClusterKey, plan: T)(computation: => Unit): Unit = {
+        // L1 cache is built on the identity hash codes of the input query plans. If
+        // the cache is hit, which means the same plan object in this JVM was
+        // once applied for the computation. Return fast in that case.
+        val l1Plans = layerOne.getOrElseUpdate(cKey, new util.IdentityHashMap())
+        if (l1Plans.containsKey(plan)) {
+          // The L1 cache is hit.
+          return
+        }
+        // Add the plan object into L1 cache.
+        l1Plans.put(plan, new Object)
+
+        // L2 cache is built on the equalities of the input query plans. It internally
+        // compares plans through RAS API PlanMode#equals. If the cache is hit, which
+        // means an identical plan (but not necessarily the same one) was once applied
+        // for the computation. Return fast in that case.
+        val l2Plans = layerTwo.getOrElseUpdate(cKey, mutable.Set())
+        val pKey = ras.toHashKey(plan)
+        if (l2Plans.contains(pKey)) {
+          // The L2 cache is hit.
+          return
+        }
+        // Add the plan object into L2 cache.
+        l2Plans += pKey
+
+        // All cache missed, apply the computation on the plan.
+        computation
+      }
+    }
+  }
 }
