@@ -19,8 +19,6 @@ package org.apache.gluten.execution
 import org.apache.gluten.backendsapi.BackendsApiManager
 import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.expression.{ConverterUtils, ExpressionConverter}
-import org.apache.gluten.extension.ValidationResult
-import org.apache.gluten.sql.shims.SparkShimLoader
 import org.apache.gluten.substrait.`type`.ColumnTypeNode
 import org.apache.gluten.substrait.SubstraitContext
 import org.apache.gluten.substrait.extensions.ExtensionBuilder
@@ -31,7 +29,6 @@ import org.apache.spark.sql.catalyst.expressions._
 import org.apache.spark.sql.connector.read.InputPartition
 import org.apache.spark.sql.hive.HiveTableScanExecTransformer
 import org.apache.spark.sql.types.{StringType, StructField, StructType}
-import org.apache.spark.util.SerializableConfiguration
 
 import com.google.protobuf.StringValue
 import io.substrait.proto.NamedStruct
@@ -78,9 +75,6 @@ trait BasicScanExecTransformer extends LeafTransformSupport with BaseDataSource
           getProperties))
   }
 
-  val serializableHadoopConf: SerializableConfiguration = new SerializableConfiguration(
-    sparkContext.hadoopConfiguration)
-
   override protected def doValidateInternal(): ValidationResult = {
     var fields = schema.fields
 
@@ -100,7 +94,7 @@ trait BasicScanExecTransformer extends LeafTransformSupport with BaseDataSource
         fields,
         getRootFilePaths,
         getProperties,
-        Some(serializableHadoopConf))
+        sparkContext.hadoopConfiguration)
     if (!validationResult.ok()) {
       return validationResult
     }
@@ -130,7 +124,9 @@ trait BasicScanExecTransformer extends LeafTransformSupport with BaseDataSource
       attr =>
         if (getPartitionSchema.exists(_.name.equals(attr.name))) {
           new ColumnTypeNode(NamedStruct.ColumnType.PARTITION_COL)
-        } else if (SparkShimLoader.getSparkShims.isRowIndexMetadataColumn(attr.name)) {
+        } else if (
+          BackendsApiManager.getSparkPlanExecApiInstance.isRowIndexMetadataColumn(attr.name)
+        ) {
           new ColumnTypeNode(NamedStruct.ColumnType.ROWINDEX_COL)
         } else if (attr.isMetadataCol) {
           new ColumnTypeNode(NamedStruct.ColumnType.METADATA_COL)
@@ -143,7 +139,7 @@ trait BasicScanExecTransformer extends LeafTransformSupport with BaseDataSource
       .map(ExpressionConverter.replaceAttributeReference)
       .reduceLeftOption(And)
       .map(ExpressionConverter.replaceWithExpressionTransformer(_, output))
-    val filterNodes = transformer.map(_.doTransform(context.registeredFunction))
+    val filterNodes = transformer.map(_.doTransform(context))
     val exprNode = filterNodes.orNull
 
     // used by CH backend
