@@ -18,7 +18,6 @@ package org.apache.gluten.execution
 
 import org.apache.gluten.backendsapi.BackendsApiManager
 import org.apache.gluten.expression.ExpressionConverter
-import org.apache.gluten.extension.ValidationResult
 import org.apache.gluten.metrics.MetricsUpdater
 import org.apache.gluten.sql.shims.SparkShimLoader
 import org.apache.gluten.substrait.rel.LocalFilesNode.ReadFileFormat
@@ -88,14 +87,20 @@ abstract class BatchScanExecTransformerBase(
     keyGroupedPartitioning,
     ordering,
     table,
-    commonPartitionValues,
-    applyPartialClustering,
-    replicatePartitions)
+    commonPartitionValues = commonPartitionValues,
+    applyPartialClustering = applyPartialClustering,
+    replicatePartitions = replicatePartitions
+  )
   with BasicScanExecTransformer {
 
   // Note: "metrics" is made transient to avoid sending driver-side metrics to tasks.
   @transient override lazy val metrics: Map[String, SQLMetric] =
-    BackendsApiManager.getMetricsApiInstance.genBatchScanTransformerMetrics(sparkContext)
+    BackendsApiManager.getMetricsApiInstance.genBatchScanTransformerMetrics(
+      sparkContext) ++ customMetrics
+
+  def doPostDriverMetrics(): Unit = {
+    postDriverMetrics()
+  }
 
   // Similar to the problem encountered in https://github.com/oap-project/gluten/pull/3184,
   // we cannot add member variables to BatchScanExecTransformerBase, which inherits from case
