@@ -18,17 +18,14 @@ package org.apache.gluten.execution
 
 import org.apache.gluten.backendsapi.BackendsApiManager
 import org.apache.gluten.config.{GlutenConfig, GlutenNumaBindingInfo}
-import org.apache.gluten.exception.{GlutenException, GlutenNotSupportException}
+import org.apache.gluten.exception.GlutenException
 import org.apache.gluten.expression._
-import org.apache.gluten.extension.ValidationResult
 import org.apache.gluten.extension.columnar.transition.Convention
-import org.apache.gluten.logging.LogLevelUtil
 import org.apache.gluten.metrics.{GlutenTimeMetric, MetricsUpdater}
 import org.apache.gluten.substrait.`type`.{TypeBuilder, TypeNode}
 import org.apache.gluten.substrait.SubstraitContext
 import org.apache.gluten.substrait.plan.{PlanBuilder, PlanNode}
 import org.apache.gluten.substrait.rel.{LocalFilesNode, RelNode, SplitInfo}
-import org.apache.gluten.test.TestStats
 import org.apache.gluten.utils.SubstraitPlanPrinterUtil
 
 import org.apache.spark._
@@ -37,12 +34,12 @@ import org.apache.spark.softaffinity.SoftAffinity
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.{Attribute, SortOrder}
 import org.apache.spark.sql.catalyst.plans.physical.Partitioning
+import org.apache.spark.sql.catalyst.trees.TreeNodeTag
 import org.apache.spark.sql.connector.read.InputPartition
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.metric.SQLMetric
 import org.apache.spark.sql.utils.SparkInputMetricsUtil.InputMetricsWrapper
 import org.apache.spark.sql.vectorized.ColumnarBatch
-import org.apache.spark.util.SerializableConfiguration
 
 import com.google.common.collect.Lists
 import org.apache.hadoop.fs.viewfs.ViewFileSystemUtils
@@ -53,93 +50,14 @@ import scala.collection.mutable.ArrayBuffer
 
 case class TransformContext(outputAttributes: Seq[Attribute], root: RelNode)
 
-case class WholeStageTransformContext(root: PlanNode, substraitContext: SubstraitContext = null)
-
-/**
- * Base interface for a Gluten query plan that is also open to validation calls.
- *
- * Since https://github.com/apache/incubator-gluten/pull/2185.
- */
-trait ValidatablePlan extends GlutenPlan with LogLevelUtil {
-  protected def glutenConf: GlutenConfig = GlutenConfig.get
-
-  protected lazy val enableNativeValidation = glutenConf.enableNativeValidation
-
-  protected lazy val validationFailFast = glutenConf.validationFailFast
-
-  // Wraps a validation function f that can also throw a GlutenNotSupportException.
-  // Returns ValidationResult.failed if f throws a GlutenNotSupportException,
-  // otherwise returns the result of f.
-  protected def failValidationWithException(f: => ValidationResult)(
-      finallyBlock: => Unit = ()): ValidationResult = {
-    try {
-      f
-    } catch {
-      case e @ (_: GlutenNotSupportException | _: UnsupportedOperationException) =>
-        if (!e.isInstanceOf[GlutenNotSupportException]) {
-          logDebug(s"Just a warning. This exception perhaps needs to be fixed.", e)
-        }
-        val message = s"Validation failed with exception from: $nodeName, reason: ${e.getMessage}"
-        if (glutenConf.printStackOnValidationFailure) {
-          logOnLevel(glutenConf.validationLogLevel, message, e)
-        }
-        ValidationResult.failed(message)
-      case t: Throwable =>
-        throw t
-    } finally {
-      finallyBlock
-    }
-  }
-
-  /**
-   * Validate whether this SparkPlan supports to be transformed into substrait node in Native Code.
-   */
-  final def doValidate(): ValidationResult = {
-    val schemaValidationResult = BackendsApiManager.getValidatorApiInstance
-      .doSchemaValidate(schema)
-      .map {
-        reason =>
-          ValidationResult.failed(s"Found schema check failure for $schema, due to: $reason")
-      }
-      .getOrElse(ValidationResult.succeeded)
-    if (!schemaValidationResult.ok()) {
-      TestStats.addFallBackClassName(this.getClass.toString)
-      if (validationFailFast) {
-        logOnLevel(glutenConf.validationLogLevel, schemaValidationResult.reason())
-        return schemaValidationResult
-      }
-    }
-    val validationResult = failValidationWithException {
-      TransformerState.enterValidation
-      doValidateInternal()
-    } {
-      TransformerState.finishValidation
-    }
-    if (!validationResult.ok()) {
-      TestStats.addFallBackClassName(this.getClass.toString)
-    }
-    val result =
-      if (validationFailFast) validationResult
-      else ValidationResult.merge(schemaValidationResult, validationResult)
-    if (!result.ok()) {
-      logOnLevel(glutenConf.validationLogLevel, result.reason())
-    }
-    result
-  }
-
-  protected def doValidateInternal(): ValidationResult = ValidationResult.succeeded
-
-  private def logValidationMessage(msg: => String, e: Throwable): Unit = {
-    if (glutenConf.printStackOnValidationFailure) {
-      logOnLevel(glutenConf.validationLogLevel, msg, e)
-    } else {
-      logOnLevel(glutenConf.validationLogLevel, msg)
-    }
-  }
-}
+case class WholeStageTransformContext(
+    root: PlanNode,
+    substraitContext: SubstraitContext = null,
+    enableCudf: Boolean = false)
 
 /** Base interface for a query plan that can be interpreted to Substrait representation. */
 trait TransformSupport extends ValidatablePlan {
+
   override def batchType(): Convention.BatchType = {
     BackendsApiManager.getSettings.primaryBatchType
   }
@@ -153,6 +71,15 @@ trait TransformSupport extends ValidatablePlan {
       s"${this.getClass.getSimpleName} doesn't support doExecute")
   }
 
+  protected def isCudf: Boolean = getTagValue[Boolean](CudfTag.CudfTag).getOrElse(false)
+
+  // Use super.nodeName will cause exception scala 213 Super calls can only target methods
+  // for FileSourceScan.
+  override def nodeName: String =
+    if (isCudf) {
+      "Cudf" + getClass.getSimpleName.replaceAll("Exec$", "")
+    } else getClass.getSimpleName
+
   /**
    * Returns all the RDDs of ColumnarBatch which generates the input rows.
    *
@@ -210,6 +137,8 @@ trait TransformSupport extends ValidatablePlan {
 trait LeafTransformSupport extends TransformSupport with LeafExecNode {
   final override def columnarInputRDDs: Seq[RDD[ColumnarBatch]] = Seq.empty
   def getSplitInfos: Seq[SplitInfo]
+
+  /** Returns the partitions generated by this data source scan. */
   def getPartitions: Seq[InputPartition]
 }
 
@@ -236,9 +165,6 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
 
   val sparkConf: SparkConf = sparkContext.getConf
 
-  val serializableHadoopConf: SerializableConfiguration = new SerializableConfiguration(
-    sparkContext.hadoopConfiguration)
-
   val numaBindingInfo: GlutenNumaBindingInfo = GlutenConfig.get.numaBindingInfo
 
   @transient
@@ -335,7 +261,7 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
       PlanBuilder.makePlan(substraitContext, Lists.newArrayList(childCtx.root), outNames)
     }
 
-    WholeStageTransformContext(planNode, substraitContext)
+    WholeStageTransformContext(planNode, substraitContext, isCudf)
   }
 
   def doWholeStageTransform(): WholeStageTransformContext = {
@@ -382,14 +308,27 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
       wsCtx: WholeStageTransformContext,
       inputRDDs: ColumnarInputRDDsWrapper,
       pipelineTime: SQLMetric): RDD[ColumnarBatch] = {
+    val isKeyGroupPartition = leafTransformers.exists {
+      // TODO: May can apply to BatchScanExecTransformer without key group partitioning
+      case b: BatchScanExecTransformerBase if b.keyGroupedPartitioning.isDefined => true
+      case _ => false
+    }
 
     /**
      * If containing leaf exec transformer this "whole stage" generates a RDD which itself takes
      * care of [[LeafTransformSupport]] there won't be any other RDD for leaf operator. As a result,
      * genFirstStageIterator rather than genFinalStageIterator will be invoked
      */
-    val allInputPartitions = leafTransformers.map(_.getPartitions)
-    val allSplitInfos = getSplitInfosFromPartitions(leafTransformers)
+    val allInputPartitions = leafTransformers.map(
+      leafTransformer => {
+        if (isKeyGroupPartition) {
+          leafTransformer.asInstanceOf[BatchScanExecTransformerBase].getPartitionsWithIndex
+        } else {
+          Seq(leafTransformer.getPartitions)
+        }
+      })
+
+    val allSplitInfos = getSplitInfosFromPartitions(isKeyGroupPartition, leafTransformers)
 
     if (GlutenConfig.get.enableHdfsViewfs) {
       val viewfsToHdfsCache: mutable.Map[String, String] = mutable.Map.empty
@@ -400,7 +339,7 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
               val newPaths = ViewFileSystemUtils.convertViewfsToHdfs(
                 splitInfo.getPaths.asScala.toSeq,
                 viewfsToHdfsCache,
-                serializableHadoopConf.value)
+                sparkContext.hadoopConfiguration)
               splitInfo.setPaths(newPaths.asJava)
           }
       }
@@ -423,93 +362,70 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
         wsCtx.substraitContext.registeredRelMap,
         wsCtx.substraitContext.registeredJoinParams,
         wsCtx.substraitContext.registeredAggregationParams
-      )
+      ),
+      wsCtx.enableCudf
     )
 
-    SoftAffinity.updateFilePartitionLocations(Seq(allInputPartitions), rdd.id)
+    SoftAffinity.updateFilePartitionLocations(allInputPartitions, rdd.id)
+
+    leafTransformers.foreach {
+      case batchScan: BatchScanExecTransformerBase =>
+        batchScan.doPostDriverMetrics()
+      case _ =>
+    }
 
     rdd
   }
 
-  private def getSplitInfosFromPartitionSeqs(
-      leafTransformers: Seq[BatchScanExecTransformerBase]): Seq[Seq[SplitInfo]] = {
-    // If these are two batch scan transformer with keyGroupPartitioning,
-    // they have same partitionValues,
-    // but some partitions maybe empty for those partition values that are not present,
-    // otherwise, exchange will be inserted. We should combine the two leaf
-    // transformers' partitions with same index, and set them together in
-    // the substraitContext. We use transpose to do that, You can refer to
-    // the diagram below.
-    // leaf1  Seq(p11) Seq(p12, p13) Seq(p14) ... Seq(p1n)
-    // leaf2  Seq(p21) Seq(p22)      Seq()    ... Seq(p2n)
-    // transpose =>
-    // leaf1 | leaf2
-    //  Seq(p11)       |  Seq(p21)    => substraitContext.setSplitInfo([Seq(p11), Seq(p21)])
-    //  Seq(p12, p13)  |  Seq(p22)    => substraitContext.setSplitInfo([Seq(p12, p13), Seq(p22)])
-    //  Seq(p14)       |  Seq()    ...
-    //                ...
-    //  Seq(p1n)       |  Seq(p2n)    => substraitContext.setSplitInfo([Seq(p1n), Seq(p2n)])
+  private def getSplitInfosFromPartitions(
+      isKeyGroupPartition: Boolean,
+      leafTransformers: Seq[LeafTransformSupport]): Seq[Seq[SplitInfo]] = {
+    val allSplitInfos = if (isKeyGroupPartition) {
+      // If these are two batch scan transformer with keyGroupPartitioning,
+      // they have same partitionValues,
+      // but some partitions maybe empty for those partition values that are not present,
+      // otherwise, exchange will be inserted. We should combine the two leaf
+      // transformers' partitions with same index, and set them together in
+      // the substraitContext. We use transpose to do that, You can refer to
+      // the diagram below.
+      // leaf1  Seq(p11) Seq(p12, p13) Seq(p14) ... Seq(p1n)
+      // leaf2  Seq(p21) Seq(p22)      Seq()    ... Seq(p2n)
+      // transpose =>
+      // leaf1 | leaf2
+      //  Seq(p11)       |  Seq(p21)    => substraitContext.setSplitInfo([Seq(p11), Seq(p21)])
+      //  Seq(p12, p13)  |  Seq(p22)    => substraitContext.setSplitInfo([Seq(p12, p13), Seq(p22)])
+      //  Seq(p14)       |  Seq()    ...
+      //                ...
+      //  Seq(p1n)       |  Seq(p2n)    => substraitContext.setSplitInfo([Seq(p1n), Seq(p2n)])
+      leafTransformers.map(_.asInstanceOf[BatchScanExecTransformerBase].getSplitInfosWithIndex)
+    } else {
+      // If these are two leaf transformers, they must have same partitions,
+      // otherwise, exchange will be inserted. We should combine the two leaf
+      // transformers' partitions with same index, and set them together in
+      // the substraitContext. We use transpose to do that, You can refer to
+      // the diagram below.
+      // leaf1  p11 p12 p13 p14 ... p1n
+      // leaf2  p21 p22 p23 p24 ... p2n
+      // transpose =>
+      // leaf1 | leaf2
+      //  p11  |  p21    => substraitContext.setSplitInfo([p11, p21])
+      //  p12  |  p22    => substraitContext.setSplitInfo([p12, p22])
+      //  p13  |  p23    ...
+      //  p14  |  p24
+      //      ...
+      //  p1n  |  p2n    => substraitContext.setSplitInfo([p1n, p2n])
+      leafTransformers.map(_.getSplitInfos)
+    }
 
-    val allSplitInfos = leafTransformers.map(_.getSplitInfosWithIndex)
     val partitionLength = allSplitInfos.head.size
     if (allSplitInfos.exists(_.size != partitionLength)) {
       throw new GlutenException(
         "The partition length of all the leaf transformer are not the same.")
     }
-    if (GlutenConfig.get.enableHdfsViewfs) {
-      val viewfsToHdfsCache: mutable.Map[String, String] = mutable.Map.empty
-      allSplitInfos.foreach {
-        case splitInfo: LocalFilesNode =>
-          val newPaths = ViewFileSystemUtils.convertViewfsToHdfs(
-            splitInfo.getPaths.asScala.toSeq,
-            viewfsToHdfsCache,
-            serializableHadoopConf.value)
-          splitInfo.setPaths(newPaths.asJava)
-      }
-    }
 
     allSplitInfos.transpose
   }
 
-  private def generateWholeStageDatasourceRDD(
-      leafTransformers: Seq[BatchScanExecTransformerBase],
-      wsCtx: WholeStageTransformContext,
-      inputRDDs: ColumnarInputRDDsWrapper,
-      pipelineTime: SQLMetric): RDD[ColumnarBatch] = {
-
-    /**
-     * If containing leaf exec transformer this "whole stage" generates a RDD which itself takes
-     * care of [[LeafTransformSupport]] there won't be any other RDD for leaf operator. As a result,
-     * genFirstStageIterator rather than genFinalStageIterator will be invoked
-     */
-    val allInputPartitions = leafTransformers.map(_.getPartitionsWithIndex)
-    val allSplitInfos = getSplitInfosFromPartitionSeqs(leafTransformers)
-
-    val inputPartitions =
-      BackendsApiManager.getIteratorApiInstance.genPartitions(
-        wsCtx,
-        allSplitInfos,
-        leafTransformers)
-
-    val rdd = new GlutenWholeStageColumnarRDD(
-      sparkContext,
-      inputPartitions,
-      inputRDDs,
-      pipelineTime,
-      leafInputMetricsUpdater(),
-      BackendsApiManager.getMetricsApiInstance.metricsUpdatingFunction(
-        child,
-        wsCtx.substraitContext.registeredRelMap,
-        wsCtx.substraitContext.registeredJoinParams,
-        wsCtx.substraitContext.registeredAggregationParams
-      )
-    )
-
-    SoftAffinity.updateFilePartitionLocations(allInputPartitions, rdd.id)
-
-    rdd
-  }
-
   override def doExecuteColumnar(): RDD[ColumnarBatch] = {
     assert(child.isInstanceOf[TransformSupport])
     val pipelineTime: SQLMetric = longMetric("pipelineTime")
@@ -525,20 +441,7 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
 
     val leafTransformers = findAllLeafTransformers()
     if (leafTransformers.nonEmpty) {
-      val isKeyGroupPartition: Boolean = leafTransformers.exists {
-        // TODO: May can apply to BatchScanExecTransformer without key group partitioning
-        case b: BatchScanExecTransformerBase if b.keyGroupedPartitioning.isDefined => true
-        case _ => false
-      }
-      if (!isKeyGroupPartition) {
-        generateWholeStageRDD(leafTransformers, wsCtx, inputRDDs, pipelineTime)
-      } else {
-        generateWholeStageDatasourceRDD(
-          leafTransformers.map(_.asInstanceOf[BatchScanExecTransformerBase]),
-          wsCtx,
-          inputRDDs,
-          pipelineTime)
-      }
+      generateWholeStageRDD(leafTransformers, wsCtx, inputRDDs, pipelineTime)
     } else {
 
       /**
@@ -596,33 +499,6 @@ case class WholeStageTransformer(child: SparkPlan, materializeInput: Boolean = f
 
   override protected def withNewChildInternal(newChild: SparkPlan): WholeStageTransformer =
     copy(child = newChild, materializeInput = materializeInput)(transformStageId)
-
-  private def getSplitInfosFromPartitions(
-      leafTransformers: Seq[LeafTransformSupport]): Seq[Seq[SplitInfo]] = {
-    // If these are two leaf transformers, they must have same partitions,
-    // otherwise, exchange will be inserted. We should combine the two leaf
-    // transformers' partitions with same index, and set them together in
-    // the substraitContext. We use transpose to do that, You can refer to
-    // the diagram below.
-    // leaf1  p11 p12 p13 p14 ... p1n
-    // leaf2  p21 p22 p23 p24 ... p2n
-    // transpose =>
-    // leaf1 | leaf2
-    //  p11  |  p21    => substraitContext.setSplitInfo([p11, p21])
-    //  p12  |  p22    => substraitContext.setSplitInfo([p12, p22])
-    //  p13  |  p23    ...
-    //  p14  |  p24
-    //      ...
-    //  p1n  |  p2n    => substraitContext.setSplitInfo([p1n, p2n])
-    val allSplitInfos = leafTransformers.map(_.getSplitInfos)
-    val partitionLength = allSplitInfos.head.size
-    if (allSplitInfos.exists(_.size != partitionLength)) {
-      throw new GlutenException(
-        "The partition length of all the leaf transformer are not the same.")
-    }
-
-    allSplitInfos.transpose
-  }
 }
 
 /**
@@ -675,3 +551,7 @@ class ColumnarInputRDDsWrapper(columnarInputRDDs: Seq[RDD[ColumnarBatch]]) exten
     }
   }
 }
+
+object CudfTag {
+  val CudfTag = TreeNodeTag[Boolean]("org.apache.gluten.CudfTag")
+}
