@@ -109,6 +109,19 @@ object ExpressionConverter extends SQLConfHelper with Logging {
     }
   }
 
+  private def replaceFlattenedExpressionWithExpressionTransformer(
+      substraitName: String,
+      expr: Expression,
+      attributeSeq: Seq[Attribute],
+      expressionsMap: Map[Class[_], String]): ExpressionTransformer = {
+    val children =
+      expr.children.map(replaceWithExpressionTransformer0(_, attributeSeq, expressionsMap))
+    BackendsApiManager.getSparkPlanExecApiInstance.genFlattenedExpressionTransformer(
+      substraitName,
+      children,
+      expr)
+  }
+
   private def genRescaleDecimalTransformer(
       substraitName: String,
       b: BinaryArithmetic,
@@ -148,23 +161,29 @@ object ExpressionConverter extends SQLConfHelper with Logging {
         return BackendsApiManager.getSparkPlanExecApiInstance.genHiveUDFTransformer(
           expr,
           attributeSeq)
+      case i: StaticInvoke
+          if Seq("encode", "decode").contains(i.functionName) && i.objectName.endsWith(
+            "UrlCodec") =>
+        return GenericExpressionTransformer(
+          "url_" + i.functionName,
+          replaceWithExpressionTransformer0(i.arguments.head, attributeSeq, expressionsMap),
+          i)
+      case i: StaticInvoke if i.functionName.equals("isLuhnNumber") =>
+        return GenericExpressionTransformer(
+          ExpressionNames.LUHN_CHECK,
+          replaceWithExpressionTransformer0(i.arguments.head, attributeSeq, expressionsMap),
+          i)
+      case i: StaticInvoke
+          if Seq("encode", "decode").contains(i.functionName) && i.objectName.endsWith("Base64") =>
+        return BackendsApiManager.getSparkPlanExecApiInstance.genBase64StaticInvokeTransformer(
+          ExpressionNames.BASE64,
+          replaceWithExpressionTransformer0(i.arguments.head, attributeSeq, expressionsMap),
+          i
+        )
       case i: StaticInvoke =>
-        val objectName = i.staticObject.getName.stripSuffix("$")
-        if (objectName.endsWith("UrlCodec")) {
-          val child = i.arguments.head
-          i.functionName match {
-            case "decode" =>
-              return GenericExpressionTransformer(
-                ExpressionNames.URL_DECODE,
-                child.map(replaceWithExpressionTransformer0(_, attributeSeq, expressionsMap)),
-                i)
-            case "encode" =>
-              return GenericExpressionTransformer(
-                ExpressionNames.URL_ENCODE,
-                child.map(replaceWithExpressionTransformer0(_, attributeSeq, expressionsMap)),
-                i)
-          }
-        }
+        throw new GlutenNotSupportException(
+          s"Not supported to transform StaticInvoke with object: ${i.staticObject.getName}, " +
+            s"function: $i.functionName")
       case _ =>
     }
 
@@ -546,20 +565,19 @@ object ExpressionConverter extends SQLConfHelper with Logging {
           substraitExprName,
           m.children.map(replaceWithExpressionTransformer0(_, attributeSeq, expressionsMap)),
           m)
-      case timestampAdd if timestampAdd.getClass.getSimpleName.equals("TimestampAdd") =>
-        // for spark3.3
-        val extract = SparkShimLoader.getSparkShims.extractExpressionTimestampAddUnit(timestampAdd)
-        if (extract.isEmpty) {
-          throw new UnsupportedOperationException(s"Not support expression TimestampAdd.")
-        }
-        val add = timestampAdd.asInstanceOf[BinaryExpression]
-        TimestampAddTransformer(
+      case tsAdd: BinaryExpression if tsAdd.getClass.getSimpleName.equals("TimestampAdd") =>
+        BackendsApiManager.getSparkPlanExecApiInstance.genTimestampAddTransformer(
           substraitExprName,
-          extract.get.head,
-          replaceWithExpressionTransformer0(add.left, attributeSeq, expressionsMap),
-          replaceWithExpressionTransformer0(add.right, attributeSeq, expressionsMap),
-          extract.get.last,
-          add
+          replaceWithExpressionTransformer0(tsAdd.left, attributeSeq, expressionsMap),
+          replaceWithExpressionTransformer0(tsAdd.right, attributeSeq, expressionsMap),
+          tsAdd
+        )
+      case tsDiff: BinaryExpression if tsDiff.getClass.getSimpleName.equals("TimestampDiff") =>
+        BackendsApiManager.getSparkPlanExecApiInstance.genTimestampDiffTransformer(
+          substraitExprName,
+          replaceWithExpressionTransformer0(tsDiff.left, attributeSeq, expressionsMap),
+          replaceWithExpressionTransformer0(tsDiff.right, attributeSeq, expressionsMap),
+          tsDiff
         )
       case e: Transformable =>
         val childrenTransformers =
@@ -676,7 +694,7 @@ object ExpressionConverter extends SQLConfHelper with Logging {
       case arrayInsert if arrayInsert.getClass.getSimpleName.equals("ArrayInsert") =>
         // Since spark 3.4.0
         val children = SparkShimLoader.getSparkShims.extractExpressionArrayInsert(arrayInsert)
-        GenericExpressionTransformer(
+        BackendsApiManager.getSparkPlanExecApiInstance.genArrayInsertTransformer(
           substraitExprName,
           children.map(replaceWithExpressionTransformer0(_, attributeSeq, expressionsMap)),
           arrayInsert
@@ -742,6 +760,24 @@ object ExpressionConverter extends SQLConfHelper with Logging {
           substraitExprName,
           expr.children.map(replaceWithExpressionTransformer0(_, attributeSeq, expressionsMap)),
           j)
+      case s: StructsToJson =>
+        BackendsApiManager.getSparkPlanExecApiInstance.genToJsonTransformer(
+          substraitExprName,
+          replaceWithExpressionTransformer0(s.child, attributeSeq, expressionsMap),
+          s
+        )
+      case u: UnBase64 =>
+        BackendsApiManager.getSparkPlanExecApiInstance.genUnbase64Transformer(
+          substraitExprName,
+          replaceWithExpressionTransformer0(u.child, attributeSeq, expressionsMap),
+          u
+        )
+      case ce if BackendsApiManager.getSparkPlanExecApiInstance.expressionFlattenSupported(ce) =>
+        replaceFlattenedExpressionWithExpressionTransformer(
+          substraitExprName,
+          ce,
+          attributeSeq,
+          expressionsMap)
       case expr =>
         GenericExpressionTransformer(
           substraitExprName,
