@@ -21,7 +21,7 @@ import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.expression.ExpressionNames._
 import org.apache.gluten.sql.shims.SparkShimLoader
 
-import org.apache.spark.sql.catalyst.expressions.{StringTrimBoth, _}
+import org.apache.spark.sql.catalyst.expressions._
 import org.apache.spark.sql.catalyst.expressions.aggregate._
 import org.apache.spark.sql.catalyst.optimizer.NormalizeNaNAndZero
 import org.apache.spark.sql.execution.ScalarSubquery
@@ -211,6 +211,7 @@ object ExpressionMappings {
     Sig[StructsToJson](TO_JSON),
     Sig[JsonToStructs](FROM_JSON),
     Sig[JsonTuple](JSON_TUPLE),
+    Sig[JsonObjectKeys](JSON_OBJECT_KEYS),
     // Hash functions
     Sig[Murmur3Hash](MURMUR3HASH),
     Sig[XxHash64](XXHASH64),
@@ -342,19 +343,37 @@ object ExpressionMappings {
     Sig[NthValue](NTH_VALUE)
   )
 
+  private val RUNTIME_REPLACEABLE_SIGS: Seq[Sig] = Seq(
+    Sig[AssertTrue](ASSERT_TRUE),
+    Sig[NullIf](NULLIF),
+    Sig[Nvl](NVL),
+    Sig[Nvl2](NVL2),
+    Sig[Right](RIGHT)
+  ) ++ SparkShimLoader.getSparkShims.runtimeReplaceableExpressionMappings
+
+  def blacklistExpressionMap: Map[Class[_], String] = {
+    partitionExpressionMapByBlacklist._1
+  }
+
   def expressionsMap: Map[Class[_], String] = {
+    partitionExpressionMapByBlacklist._2
+  }
+
+  private def partitionExpressionMapByBlacklist: (Map[Class[_], String], Map[Class[_], String]) = {
     val blacklist = GlutenConfig.get.expressionBlacklist
-    val filtered = (defaultExpressionsMap ++ toMap(
-      BackendsApiManager.getSparkPlanExecApiInstance.extraExpressionMappings)).filterNot(
+    val (blacklistedExpr, filteredExpr) = (defaultExpressionsMap ++ toMap(
+      BackendsApiManager.getSparkPlanExecApiInstance.extraExpressionMappings)).partition(
       kv => blacklist.contains(kv._2))
-    filtered
+    (blacklistedExpr, filteredExpr)
   }
 
   // This is needed when generating function support status documentation for Spark built-in
   // functions.
   // Used by gluten/tools/scripts/gen-function-support-docs.py
   def listExpressionMappings(): Array[(String, String)] = {
-    expressionsMap.map(kv => (kv._1.getSimpleName, kv._2)).toArray
+    (expressionsMap ++ toMap(RUNTIME_REPLACEABLE_SIGS))
+      .map(kv => (kv._1.getSimpleName, kv._2))
+      .toArray
   }
 
   private lazy val defaultExpressionsMap: Map[Class[_], String] = {
