@@ -29,7 +29,7 @@ import org.apache.spark.internal.Logging
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.aggregate.{HashAggregateExec, ObjectHashAggregateExec, SortAggregateExec}
 import org.apache.spark.sql.execution.datasources.WriteFilesExec
-import org.apache.spark.sql.execution.datasources.v2.BatchScanExec
+import org.apache.spark.sql.execution.datasources.v2.{AppendDataExec, BatchScanExec, ReplaceDataExec}
 import org.apache.spark.sql.execution.exchange.{BroadcastExchangeExec, ShuffleExchangeExec}
 import org.apache.spark.sql.execution.joins._
 import org.apache.spark.sql.execution.window.WindowExec
@@ -136,7 +136,8 @@ object Validators {
         fail(p)
       case p: CartesianProductExec if !settings.supportCartesianProductExec() => fail(p)
       case p: TakeOrderedAndProjectExec if !settings.supportColumnarShuffleExec() => fail(p)
-      case p: CollectLimitExec if !settings.supportCollectLimitExec() => fail(p)
+      case p: AppendDataExec if !settings.supportAppendDataExec() => fail(p)
+      case p: ReplaceDataExec if !settings.supportReplaceDataExec() => fail(p)
       case _ => pass()
     }
   }
@@ -156,6 +157,7 @@ object Validators {
       case p: ShuffledHashJoinExec if !glutenConf.enableColumnarShuffledHashJoin => fail(p)
       case p: ShuffleExchangeExec if !glutenConf.enableColumnarShuffle => fail(p)
       case p: BroadcastExchangeExec if !glutenConf.enableColumnarBroadcastExchange => fail(p)
+      case p: AppendDataExec if !glutenConf.enableAppendData => fail(p)
       case p @ (_: LocalLimitExec | _: GlobalLimitExec) if !glutenConf.enableColumnarLimit =>
         fail(p)
       case p: GenerateExec if !glutenConf.enableColumnarGenerate => fail(p)
@@ -188,6 +190,7 @@ object Validators {
         fail(p)
       case p: RangeExec if !glutenConf.enableColumnarRange => fail(p)
       case p: CollectLimitExec if !glutenConf.enableColumnarCollectLimit => fail(p)
+      case p: CollectTailExec if !glutenConf.enableColumnarCollectTail => fail(p)
       case _ => pass()
     }
   }
@@ -213,7 +216,8 @@ object Validators {
         case p if HiveTableScanExecTransformer.isHiveTableScan(p) => pass()
         case filter: FilterExec =>
           val childIsScan = filter.child.isInstanceOf[FileSourceScanExec] ||
-            filter.child.isInstanceOf[BatchScanExec]
+            filter.child.isInstanceOf[BatchScanExec] || filter.child
+              .isInstanceOf[BasicScanExecTransformer]
           if (childIsScan) {
             pass()
           } else {
