@@ -16,15 +16,17 @@
  */
 package org.apache.spark.sql.execution
 
+import org.apache.gluten.exception.GlutenException
 import org.apache.gluten.execution.{GlutenPlan, WholeStageTransformer}
 import org.apache.gluten.utils.PlanUtil
 
-import org.apache.spark.sql.{AnalysisException, Dataset, SparkSession}
+import org.apache.spark.sql.{Column, Dataset, SparkSession}
 import org.apache.spark.sql.catalyst.plans.QueryPlan
 import org.apache.spark.sql.catalyst.plans.logical.{CommandResult, LogicalPlan}
 import org.apache.spark.sql.catalyst.util.StringUtils.PlanStringConcat
+import org.apache.spark.sql.classic.ClassicConversions._
 import org.apache.spark.sql.execution.ColumnarWriteFilesExec.NoopLeaf
-import org.apache.spark.sql.execution.adaptive.{AdaptiveSparkPlanExec, AQEShuffleReadExec, QueryStageExec}
+import org.apache.spark.sql.execution.adaptive.{AQEShuffleReadExec, AdaptiveSparkPlanExec, QueryStageExec}
 import org.apache.spark.sql.execution.columnar.InMemoryTableScanExec
 import org.apache.spark.sql.execution.command.{DataWritingCommandExec, ExecutedCommandExec}
 import org.apache.spark.sql.execution.datasources.WriteFilesExec
@@ -54,6 +56,11 @@ import scala.collection.mutable.ArrayBuffer
 // format: on
 object GlutenImplicits {
 
+  // TODO: remove this if we can suppress unused import error.
+  locally {
+    new ColumnConstructorExt(Column)
+  }
+
   case class FallbackSummary(
       numGlutenNodes: Int,
       numFallbackNodes: Int,
@@ -74,7 +81,7 @@ object GlutenImplicits {
     keys.zip(values).foreach {
       case (k, v) =>
         if (SQLConf.isStaticConfigKey(k)) {
-          throw new AnalysisException(s"Cannot modify the value of a static config: $k")
+          throw new GlutenException(s"Cannot modify the value of a static config: $k")
         }
         conf.setConfString(k, v)
     }
