@@ -17,76 +17,16 @@
 package org.apache.spark.sql.execution.datasources
 
 import org.apache.gluten.backendsapi.BackendsApiManager
-import org.apache.gluten.execution.{ColumnarToRowExecBase, GlutenPlan}
+import org.apache.gluten.execution.ColumnarToRowExecBase
 import org.apache.gluten.execution.datasource.GlutenFormatFactory
-import org.apache.gluten.extension.columnar.transition.{Convention, Transitions}
 
-import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.SparkSession
-import org.apache.spark.sql.catalyst.InternalRow
-import org.apache.spark.sql.catalyst.expressions.{Attribute, SortOrder}
-import org.apache.spark.sql.catalyst.plans.logical.{LogicalPlan, OrderPreservingUnaryNode}
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanExec
 import org.apache.spark.sql.execution.command.{CreateDataSourceTableAsSelectCommand, DataWritingCommand, DataWritingCommandExec}
 import org.apache.spark.sql.hive.execution.{CreateHiveTableAsSelectCommand, InsertIntoHiveDirCommand, InsertIntoHiveTable}
 import org.apache.spark.sql.sources.DataSourceRegister
-import org.apache.spark.sql.vectorized.ColumnarBatch
-
-private case class FakeRowLogicAdaptor(child: LogicalPlan) extends OrderPreservingUnaryNode {
-  override def output: Seq[Attribute] = child.output
-
-  // For spark 3.2.
-  protected def withNewChildInternal(newChild: LogicalPlan): FakeRowLogicAdaptor =
-    copy(child = newChild)
-}
-
-/**
- * Whether the child is columnar or not, this operator will convert the columnar output to FakeRow,
- * which is consumable by native parquet/orc writer
- *
- * This is usually used in data writing since Spark doesn't expose APIs to write columnar data as of
- * now.
- */
-case class FakeRowAdaptor(child: SparkPlan)
-  extends UnaryExecNode
-  with IFakeRowAdaptor
-  with GlutenPlan {
-  if (child.logicalLink.isDefined) {
-    setLogicalLink(FakeRowLogicAdaptor(child.logicalLink.get))
-  }
-
-  override def output: Seq[Attribute] = child.output
-
-  override def batchType(): Convention.BatchType = BackendsApiManager.getSettings.primaryBatchType
-
-  override def rowType0(): Convention.RowType = Convention.RowType.VanillaRow
-
-  override protected def doExecute(): RDD[InternalRow] = {
-    doExecuteColumnar().map(cb => new FakeRow(cb))
-  }
-
-  override def outputOrdering: Seq[SortOrder] = child match {
-    case aqe: AdaptiveSparkPlanExec => aqe.executedPlan.outputOrdering
-    case _ => child.outputOrdering
-  }
-
-  override protected def doExecuteColumnar(): RDD[ColumnarBatch] = {
-    if (child.supportsColumnar) {
-      child.executeColumnar()
-    } else {
-      val r2c = Transitions.toBatchPlan(child, BackendsApiManager.getSettings.primaryBatchType)
-      r2c.executeColumnar()
-    }
-  }
-
-  // For spark 3.2.
-  protected def withNewChildInternal(newChild: SparkPlan): FakeRowAdaptor =
-    copy(child = newChild)
-}
-
-case class MATERIALIZE_TAG()
 
 object GlutenWriterColumnarRules {
   // TODO: support ctas in Spark3.4, see https://github.com/apache/spark/pull/39220
@@ -99,10 +39,6 @@ object GlutenWriterColumnarRules {
     "org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat" -> "parquet"
   )
   private def getNativeFormat(cmd: DataWritingCommand): Option[String] = {
-    if (!BackendsApiManager.getSettings.enableNativeWriteFiles()) {
-      return None
-    }
-
     cmd match {
       case command: CreateDataSourceTableAsSelectCommand
           if !BackendsApiManager.getSettings.skipNativeCtas(command) =>
@@ -136,7 +72,8 @@ object GlutenWriterColumnarRules {
     child match {
       // if the child is columnar, we can just wrap & transfer the columnar data
       case c2r: ColumnarToRowExecBase =>
-        command.withNewChildren(Array(FakeRowAdaptor(c2r.child)))
+        command.withNewChildren(
+          Array(BackendsApiManager.getSparkPlanExecApiInstance.genColumnarToCarrierRow(c2r.child)))
       // If the child is aqe, we make aqe "support columnar",
       // then aqe itself will guarantee to generate columnar outputs.
       // So FakeRowAdaptor will always consumes columnar data,
@@ -144,7 +81,7 @@ object GlutenWriterColumnarRules {
       case aqe: AdaptiveSparkPlanExec =>
         command.withNewChildren(
           Array(
-            FakeRowAdaptor(
+            BackendsApiManager.getSparkPlanExecApiInstance.genColumnarToCarrierRow(
               AdaptiveSparkPlanExec(
                 aqe.inputPlan,
                 aqe.context,
@@ -152,7 +89,9 @@ object GlutenWriterColumnarRules {
                 aqe.isSubquery,
                 supportsColumnar = true
               ))))
-      case other => command.withNewChildren(Array(FakeRowAdaptor(other)))
+      case other =>
+        command.withNewChildren(
+          Array(BackendsApiManager.getSparkPlanExecApiInstance.genColumnarToCarrierRow(other)))
     }
   }
 
@@ -161,14 +100,22 @@ object GlutenWriterColumnarRules {
     override def apply(p: SparkPlan): SparkPlan = p match {
       case rc @ DataWritingCommandExec(cmd, child) =>
         // The same thread can set these properties in the last query submission.
-        val fields = child.output.toStructType.fields
         val format =
-          if (BackendsApiManager.getSettings.supportNativeWrite(fields)) {
+          if (
+            BackendsApiManager.getSettings.supportNativeWrite(child.schema.fields) &&
+            BackendsApiManager.getSettings.enableNativeWriteFiles()
+          ) {
             getNativeFormat(cmd)
           } else {
             None
           }
-        injectSparkLocalProperty(session, format)
+        val numStaticPartitions: Option[Int] = cmd match {
+          case cmd: InsertIntoHadoopFsRelationCommand =>
+            Some(cmd.staticPartitions.size)
+          case _ =>
+            None
+        }
+        injectSparkLocalProperty(session, format, numStaticPartitions)
         format match {
           case Some(_) =>
             injectFakeRowAdaptor(rc, child)
@@ -180,17 +127,28 @@ object GlutenWriterColumnarRules {
     }
   }
 
-  def injectSparkLocalProperty(spark: SparkSession, format: Option[String]): Unit = {
+  // TODO: This makes FileFormatWriter#write caller-sensitive.
+  //  Remove this workaround once we have a better solution.
+  def injectSparkLocalProperty(
+      spark: SparkSession,
+      format: Option[String],
+      numStaticPartitions: Option[Int]): Unit = {
     if (format.isDefined) {
       spark.sparkContext.setLocalProperty("isNativeApplicable", true.toString)
       spark.sparkContext.setLocalProperty("nativeFormat", format.get)
+      // The flag for static-only write is not used by Velox backend where
+      // the static partition write is already supported.
       spark.sparkContext.setLocalProperty(
         "staticPartitionWriteOnly",
         BackendsApiManager.getSettings.staticPartitionWriteOnly().toString)
+      spark.sparkContext.setLocalProperty(
+        "numStaticPartitionCols",
+        numStaticPartitions.getOrElse(0).toString)
     } else {
       spark.sparkContext.setLocalProperty("isNativeApplicable", null)
       spark.sparkContext.setLocalProperty("nativeFormat", null)
       spark.sparkContext.setLocalProperty("staticPartitionWriteOnly", null)
+      spark.sparkContext.setLocalProperty("numStaticPartitionCols", null)
     }
   }
 }
