@@ -17,15 +17,15 @@
 package org.apache.spark.sql.sources
 
 import org.apache.gluten.GlutenColumnarWriteTestSupport
+import org.apache.gluten.execution.GlutenPlan
 import org.apache.gluten.execution.SortExecTransformer
-import org.apache.gluten.extension.GlutenPlan
 
 import org.apache.spark.SparkConf
 import org.apache.spark.executor.OutputMetrics
 import org.apache.spark.scheduler.{SparkListener, SparkListenerTaskEnd}
 import org.apache.spark.sql._
 import org.apache.spark.sql.catalyst.TableIdentifier
-import org.apache.spark.sql.execution.{CommandResultExec, QueryExecution, SparkPlan}
+import org.apache.spark.sql.execution.{CommandResultExec, GlutenImplicits, QueryExecution, SparkPlan}
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.execution.command.DataWritingCommandExec
 import org.apache.spark.sql.execution.metric.SQLMetric
@@ -72,8 +72,9 @@ class GlutenInsertSuite
   }
 
   testGluten("insert partition table") {
-    withTable("pt") {
+    withTable("pt", "pt2") {
       spark.sql("CREATE TABLE pt (c1 int, c2 string) USING PARQUET PARTITIONED BY (pt string)")
+      spark.sql("CREATE TABLE pt2 (c1 int, c2 string) USING PARQUET PARTITIONED BY (pt string)")
 
       var taskMetrics: OutputMetrics = null
       val taskListener = new SparkListener {
@@ -113,19 +114,29 @@ class GlutenInsertSuite
         spark.sparkContext.removeSparkListener(taskListener)
         spark.listenerManager.unregister(queryListener)
       }
+
+      // check no fallback nodes
+      val df2 = spark.sql("INSERT INTO TABLE pt2 SELECT * FROM pt")
+      checkWriteFilesAndGetChild(df2)
+      val fallbackSummary = GlutenImplicits
+        .collectQueryExecutionFallbackSummary(spark, df2.queryExecution)
+      assert(fallbackSummary.numFallbackNodes == 0)
     }
   }
 
-  testGluten("Cleanup staging files if job is failed") {
-    withTable("t") {
-      spark.sql("CREATE TABLE t (c1 int, c2 string) USING PARQUET")
-      val table = spark.sessionState.catalog.getTableMetadata(TableIdentifier("t"))
+  ignoreGluten("Cleanup staging files if job failed") {
+    // Using a unique table name in this test. Sometimes, the table is not removed for some unknown
+    // reason, which can cause test failure (location already exists) if other following tests have
+    // the same table name.
+    withTable("tbl") {
+      spark.sql("CREATE TABLE tbl (c1 int, c2 string) USING PARQUET")
+      val table = spark.sessionState.catalog.getTableMetadata(TableIdentifier("tbl"))
       assert(new File(table.location).list().length == 0)
 
       intercept[Exception] {
         spark.sql(
           """
-            |INSERT INTO TABLE t
+            |INSERT INTO TABLE tbl
             |SELECT id, assert_true(SPARK_PARTITION_ID() = 1) FROM range(1, 3, 1, 2)
             |""".stripMargin
         )
