@@ -16,13 +16,12 @@
  */
 package org.apache.gluten.execution.datasource
 
-import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.SparkSession
-import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.execution.SparkPlan
-import org.apache.spark.sql.execution.datasources.{BlockStripes, FakeRow, OutputWriter}
+import org.apache.spark.sql.execution.datasources.{BlockStripes, OutputWriter}
 import org.apache.spark.sql.types.StructType
+import org.apache.spark.sql.vectorized.ColumnarBatch
 
 import org.apache.hadoop.fs.FileStatus
 import org.apache.hadoop.mapreduce.TaskAttemptContext
@@ -39,7 +38,7 @@ trait GlutenFormatWriterInjects {
       options: Map[String, String],
       files: Seq[FileStatus]): Option[StructType]
 
-  def executeWriterWrappedSparkPlan(plan: SparkPlan): RDD[InternalRow]
+  def getWriterWrappedSparkPlan(plan: SparkPlan): SparkPlan
 
   def nativeConf(
       options: Map[String, String],
@@ -50,10 +49,10 @@ trait GlutenFormatWriterInjects {
 
 trait GlutenRowSplitter {
   def splitBlockByPartitionAndBucket(
-      row: FakeRow,
+      batch: ColumnarBatch,
       partitionColIndice: Array[Int],
       hasBucket: Boolean,
-      reserve_partition_columns: Boolean = false): BlockStripes
+      reservePartitionColumns: Boolean = false): BlockStripes
 }
 
 object GlutenFormatFactory {
