@@ -119,7 +119,7 @@ abstract class FileSourceScanExecShim(
     }(t => driverMetrics("metadataTime") = NANOSECONDS.toMillis(t + optimizerMetadataTimeNs))
   }.toArray
 
-  private def isDynamicPruningFilter(e: Expression): Boolean =
+  protected def isDynamicPruningFilter(e: Expression): Boolean =
     e.find(_.isInstanceOf[PlanExpression[_]]).isDefined
 
   // We can only determine the actual partitions at runtime when a dynamic partition filter is
@@ -151,6 +151,10 @@ abstract class FileSourceScanExecShim(
     sendDriverMetrics()
     selected
   }
+
+  def getPartitionArray(): Array[PartitionDirectory] = {
+    dynamicallySelectedPartitions
+  }
 }
 
 abstract class ArrowFileSourceScanLikeShim(original: FileSourceScanExec)
