@@ -33,7 +33,7 @@ import org.apache.spark.sql.catalyst.expressions.codegen.{CodegenContext, ExprCo
 import org.apache.spark.sql.catalyst.plans.physical.HashPartitioning
 import org.apache.spark.sql.catalyst.util.{CaseInsensitiveMap, DateTimeUtils}
 import org.apache.spark.sql.errors.QueryExecutionErrors
-import org.apache.spark.sql.execution.{ProjectExec, SortExec, SparkPlan, SQLExecution, UnsafeExternalRowSorter}
+import org.apache.spark.sql.execution.{ColumnarToRowTransition, ProjectExec, SortExec, SparkPlan, SQLExecution, UnsafeExternalRowSorter}
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types.StringType
 import org.apache.spark.unsafe.types.UTF8String
@@ -107,8 +107,6 @@ object FileFormatWriter extends Logging {
    * @return
    *   The set of all partition paths that were updated during this write job.
    */
-
-  // scalastyle:off argcount
   def write(
       sparkSession: SparkSession,
       plan: SparkPlan,
@@ -119,41 +117,20 @@ object FileFormatWriter extends Logging {
       partitionColumns: Seq[Attribute],
       bucketSpec: Option[BucketSpec],
       statsTrackers: Seq[WriteJobStatsTracker],
-      options: Map[String, String]): Set[String] = write(
-    sparkSession = sparkSession,
-    plan = plan,
-    fileFormat = fileFormat,
-    committer = committer,
-    outputSpec = outputSpec,
-    hadoopConf = hadoopConf,
-    partitionColumns = partitionColumns,
-    bucketSpec = bucketSpec,
-    statsTrackers = statsTrackers,
-    options = options,
-    numStaticPartitionCols = 0
-  )
-
-  def write(
-      sparkSession: SparkSession,
-      plan: SparkPlan,
-      fileFormat: FileFormat,
-      committer: FileCommitProtocol,
-      outputSpec: OutputSpec,
-      hadoopConf: Configuration,
-      partitionColumns: Seq[Attribute],
-      bucketSpec: Option[BucketSpec],
-      statsTrackers: Seq[WriteJobStatsTracker],
-      options: Map[String, String],
-      numStaticPartitionCols: Int = 0): Set[String] = {
+      options: Map[String, String]): Set[String] = {
 
     val nativeEnabled =
       "true" == sparkSession.sparkContext.getLocalProperty("isNativeApplicable")
-    val staticPartitionWriteOnly =
-      "true" == sparkSession.sparkContext.getLocalProperty("staticPartitionWriteOnly")
+    val numStaticPartitionCols =
+      Option(sparkSession.sparkContext.getLocalProperty("numStaticPartitionCols"))
+        .map(_.toInt)
+        .getOrElse(0)
 
     if (nativeEnabled) {
-      logInfo("Use Gluten partition write for hive")
-      assert(plan.isInstanceOf[IFakeRowAdaptor])
+      logInfo(
+        s"Writing data with Gluten's native writer. The topmost node of the query plan to " +
+          s"write is: ${plan.nodeName}")
+      assert(plan.isInstanceOf[ColumnarToRowTransition])
     }
 
     val job = Job.getInstance(hadoopConf)
@@ -172,12 +149,10 @@ object FileFormatWriter extends Logging {
       case attr => attr
     }
 
-    val empty2NullPlan = if (staticPartitionWriteOnly && nativeEnabled) {
-      // Velox backend only support static partition write.
-      // And no need to add sort operator for static partition write.
-      plan
+    val empty2NullPlan = if (needConvert) {
+      ProjectExec(projectList, plan)
     } else {
-      if (needConvert) ProjectExec(projectList, plan) else plan
+      plan
     }
 
     val bucketIdExpression = bucketSpec.map {
@@ -256,13 +231,13 @@ object FileFormatWriter extends Logging {
       }
 
       val nativeFormat = sparkSession.sparkContext.getLocalProperty("nativeFormat")
-      (GlutenFormatFactory(nativeFormat).executeWriterWrappedSparkPlan(wrapped), None)
+      (GlutenFormatFactory(nativeFormat).getWriterWrappedSparkPlan(wrapped), None)
     }
 
     try {
-      val (rdd, concurrentOutputWriterSpec) = if (orderingMatched) {
-        if (!nativeEnabled || (staticPartitionWriteOnly && nativeEnabled)) {
-          (empty2NullPlan.execute(), None)
+      val (finalPlan, concurrentOutputWriterSpec) = if (orderingMatched) {
+        if (!nativeEnabled) {
+          (empty2NullPlan, None)
         } else {
           nativeWrap(empty2NullPlan)
         }
@@ -286,22 +261,19 @@ object FileFormatWriter extends Logging {
 
         if (concurrentWritersEnabled) {
           (
-            empty2NullPlan.execute(),
+            empty2NullPlan,
             Some(ConcurrentOutputWriterSpec(maxWriters, () => sortPlan.createSorter())))
         } else {
-          if (staticPartitionWriteOnly && nativeEnabled) {
-            // remove the sort operator for static partition write.
-            (empty2NullPlan.execute(), None)
+          if (!nativeEnabled) {
+            (sortPlan, None)
           } else {
-            if (!nativeEnabled) {
-              (sortPlan.execute(), None)
-            } else {
-              nativeWrap(sortPlan)
-            }
+            nativeWrap(sortPlan)
           }
         }
       }
 
+      val rdd = finalPlan.execute()
+
       // SPARK-23271 If we are attempting to write a zero partition rdd, create a dummy single
       // partition rdd to make sure we at least set up one write task to write the metadata.
       val rddWithNonEmptyPartitions = if (rdd.partitions.length == 0) {
