@@ -46,6 +46,8 @@ abstract class BatchScanExecShim(
 
   def hasUnsupportedColumns: Boolean = false
 
+  def postDriverMetrics(): Unit = {}
+
   override def doExecuteColumnar(): RDD[ColumnarBatch] = {
     throw new UnsupportedOperationException("Need to implement this method")
   }
@@ -98,6 +100,10 @@ abstract class BatchScanExecShim(
   }
 }
 
-abstract class ArrowBatchScanExecShim(original: BatchScanExec) extends DataSourceV2ScanExecBase {
-  @transient override lazy val partitions: Seq[InputPartition] = original.partitions
-}
+abstract class ArrowBatchScanExecShim(original: BatchScanExec)
+  extends BatchScanExecShim(
+    original.output,
+    original.scan,
+    original.runtimeFilters,
+    table = null
+  )
