@@ -146,10 +146,21 @@ abstract class BatchScanExecShim(
   }
 }
 
-abstract class ArrowBatchScanExecShim(original: BatchScanExec) extends DataSourceV2ScanExecBase {
-  @transient override lazy val inputPartitions: Seq[InputPartition] = original.inputPartitions
-
-  override def keyGroupedPartitioning: Option[Seq[Expression]] = original.keyGroupedPartitioning
+abstract class ArrowBatchScanExecShim(original: BatchScanExec)
+  extends BatchScanExecShim(
+    original.output,
+    original.scan,
+    original.runtimeFilters,
+    original.spjParams.keyGroupedPartitioning,
+    original.ordering,
+    original.table,
+    original.spjParams.commonPartitionValues,
+    original.spjParams.applyPartialClustering,
+    original.spjParams.replicatePartitions
+  ) {
+  override def scan: Scan = original.scan
 
   override def ordering: Option[Seq[SortOrder]] = original.ordering
+
+  override def output: Seq[Attribute] = original.output
 }
