@@ -16,19 +16,19 @@
  */
 package org.apache.gluten.integration.action
 
-import org.apache.commons.lang3.exception.ExceptionUtils
+import org.apache.gluten.integration.{QueryRunner, Suite}
 import org.apache.gluten.integration.QueryRunner.QueryResult
 import org.apache.gluten.integration.action.Actions.QuerySelector
 import org.apache.gluten.integration.action.TableRender.Field
 import org.apache.gluten.integration.action.TableRender.RowParser.FieldAppender.RowAppender
 import org.apache.gluten.integration.stat.RamStat
-import org.apache.gluten.integration.{QueryRunner, Suite}
+
 import org.apache.spark.sql.ConfUtils.ConfImplicits._
 import org.apache.spark.sql.SparkSession
 
 import java.util.concurrent.atomic.{AtomicInteger, AtomicLong}
+
 import scala.collection.mutable
-import scala.collection.mutable.ListBuffer
 
 class Parameterized(
     scale: Double,
@@ -41,25 +41,29 @@ class Parameterized(
     configDimensions: Seq[Parameterized.Dim],
     excludedCombinations: Seq[Set[Parameterized.DimKv]],
     metrics: Seq[String])
-    extends Action {
+  extends Action {
   import Parameterized._
 
   validateDims(configDimensions)
 
   private def validateDims(configDimensions: Seq[Dim]): Unit = {
-    if (configDimensions
-        .map(dim => {
-          dim.name
-        })
+    if (
+      configDimensions
+        .map(
+          dim => {
+            dim.name
+          })
         .toSet
-        .size != configDimensions.size) {
+        .size != configDimensions.size
+    ) {
       throw new IllegalStateException("Duplicated dimension name found")
     }
 
-    configDimensions.foreach { dim =>
-      if (dim.dimValues.map(dimValue => dimValue.name).toSet.size != dim.dimValues.size) {
-        throw new IllegalStateException("Duplicated dimension value found")
-      }
+    configDimensions.foreach {
+      dim =>
+        if (dim.dimValues.map(dimValue => dimValue.name).toSet.size != dim.dimValues.size) {
+          throw new IllegalStateException("Duplicated dimension value found")
+        }
     }
   }
 
@@ -74,24 +78,28 @@ class Parameterized(
         intermediateConf: Seq[(String, String)]): Unit = {
       if (dimOffset == dimCount) {
         // we got one coordinate
-        excludedCombinations.foreach { ec: Set[DimKv] =>
-          if (ec.forall { kv =>
-              intermediateCoordinate.contains(kv.k) && intermediateCoordinate(kv.k) == kv.v
-            }) {
-            println(s"Coordinate $intermediateCoordinate excluded by $ec.")
-            return
-          }
+        excludedCombinations.foreach {
+          ec: Set[DimKv] =>
+            if (
+              ec.forall {
+                kv => intermediateCoordinate.contains(kv.k) && intermediateCoordinate(kv.k) == kv.v
+              }
+            ) {
+              println(s"Coordinate $intermediateCoordinate excluded by $ec.")
+              return
+            }
         }
         coordinateMap(Coordinate(nextId.getAndIncrement(), intermediateCoordinate)) =
           intermediateConf
         return
       }
       val dim = configDimensions(dimOffset)
-      dim.dimValues.foreach { dimValue =>
-        fillCoordinates(
-          dimOffset + 1,
-          intermediateCoordinate + (dim.name -> dimValue.name),
-          intermediateConf ++ dimValue.conf)
+      dim.dimValues.foreach {
+        dimValue =>
+          fillCoordinates(
+            dimOffset + 1,
+            intermediateCoordinate + (dim.name -> dimValue.name),
+            intermediateConf ++ dimValue.conf)
       }
     }
 
@@ -108,78 +116,83 @@ class Parameterized(
     val testConf = suite.getTestConf()
 
     println("Prepared coordinates: ")
-    coordinates.keys.foreach { c =>
-      println(s"  ${c.id}: $c")
-    }
-    coordinates.foreach { entry =>
-      // register one session per coordinate
-      val coordinate = entry._1
-      val coordinateConf = entry._2
-      val conf = testConf.clone()
-      conf.setAllWarningOnOverriding(coordinateConf)
-      sessionSwitcher.registerSession(coordinate.toString, conf)
+    coordinates.keys.foreach(c => println(s"  ${c.id}: $c"))
+    coordinates.foreach {
+      entry =>
+        // register one session per coordinate
+        val coordinate = entry._1
+        val coordinateConf = entry._2
+        val conf = testConf.clone()
+        conf.setAllWarningOnOverriding(coordinateConf)
+        sessionSwitcher.registerSession(coordinate.toString, conf)
     }
 
     val runQueryIds = queries.select(suite).map(TestResultLine.QueryId(_))
 
-    val marks: Seq[TestResultLine.CoordMark] = coordinates.flatMap { entry =>
-      val coordinate = entry._1
-      sessionSwitcher.useSession(coordinate.toString, "Parameterized %s".format(coordinate))
-      runner.createTables(suite.tableCreator(), sessionSwitcher.spark())
+    val marks: Seq[TestResultLine.CoordMark] = coordinates.flatMap {
+      entry =>
+        val coordinate = entry._1
+        sessionSwitcher.useSession(coordinate.toString, "Parameterized %s".format(coordinate))
+        runner.createTables(suite.tableCreator(), sessionSwitcher.spark())
 
-      runQueryIds.flatMap { queryId =>
-        // warm up
-        (0 until warmupIterations).foreach { iteration =>
-          println(s"Warming up: Running query $queryId (iteration $iteration)...")
-          try {
-            Parameterized.warmUp(
-              runner,
-              sessionSwitcher.spark(),
-              queryId.id,
-              coordinate,
-              suite.desc())
-          } finally {
-            if (noSessionReuse) {
-              sessionSwitcher.renewSession()
-              runner.createTables(suite.tableCreator(), sessionSwitcher.spark())
+        runQueryIds.flatMap {
+          queryId =>
+            // warm up
+            (0 until warmupIterations).foreach {
+              iteration =>
+                println(s"Warming up: Running query $queryId (iteration $iteration)...")
+                try {
+                  Parameterized.warmUp(
+                    runner,
+                    sessionSwitcher.spark(),
+                    queryId.id,
+                    coordinate,
+                    suite.desc())
+                } finally {
+                  if (noSessionReuse) {
+                    sessionSwitcher.renewSession()
+                    runner.createTables(suite.tableCreator(), sessionSwitcher.spark())
+                  }
+                }
             }
-          }
-        }
 
-        // run
-        (0 until iterations).map { iteration =>
-          println(s"Running query $queryId with coordinate $coordinate (iteration $iteration)...")
-          val r =
-            try {
-              Parameterized.runQuery(
-                runner,
-                sessionSwitcher.spark(),
-                queryId.id,
-                coordinate,
-                suite.desc(),
-                explain,
-                metrics)
-            } finally {
-              if (noSessionReuse) {
-                sessionSwitcher.renewSession()
-                runner.createTables(suite.tableCreator(), sessionSwitcher.spark())
-              }
+            // run
+            (0 until iterations).map {
+              iteration =>
+                println(
+                  s"Running query $queryId with coordinate $coordinate (iteration $iteration)...")
+                val r =
+                  try {
+                    Parameterized.runQuery(
+                      runner,
+                      sessionSwitcher.spark(),
+                      queryId.id,
+                      coordinate,
+                      suite.desc(),
+                      explain,
+                      metrics)
+                  } finally {
+                    if (noSessionReuse) {
+                      sessionSwitcher.renewSession()
+                      runner.createTables(suite.tableCreator(), sessionSwitcher.spark())
+                    }
+                  }
+                TestResultLine.CoordMark(iteration, queryId, r)
             }
-          TestResultLine.CoordMark(iteration, queryId, r)
         }
-      }
     }.toSeq
 
     val results: Seq[TestResultLine] = marks
       .groupBy(m => (m.iteration, m.queryId))
       .toSeq
       .sortBy(_._1)
-      .map { e =>
-        val iteration = e._1._1
-        val queryId = e._1._2
-        val marks = e._2
-        val line = TestResultLine(queryId, marks.map(_.coord).toList)
-        line
+      .map {
+        e =>
+          val iteration = e._1._1
+          val queryId = e._1._2
+          val marks = e._2
+          val line = TestResultLine(queryId, marks.map(_.coord).toList)
+          line
       }
 
     val succeededCount = results.count(l => l.succeeded())
@@ -193,7 +206,8 @@ class Parameterized(
       "RAM statistics: JVM Heap size: %d KiB (total %d KiB), Process RSS: %d KiB\n",
       RamStat.getJvmHeapUsed(),
       RamStat.getJvmHeapTotal(),
-      RamStat.getProcessRamUsed())
+      RamStat.getProcessRamUsed()
+    )
 
     println("")
     println("Test report: ")
@@ -213,16 +227,18 @@ class Parameterized(
         Some(
           TestResultLine(
             TestResultLine.QueryId("all"),
-            coordinates.keys.map { c =>
-              TestResultLine.Coord(
-                c,
-                several
-                  .map(_.coord(c.id))
-                  .map(_.queryResult)
-                  .asSuccesses()
-                  .agg(s"coordinate $c")
-                  .get)
-            }.toSeq))
+            coordinates.keys.map {
+              c =>
+                TestResultLine.Coord(
+                  c,
+                  several
+                    .map(_.coord(c.id))
+                    .map(_.queryResult)
+                    .asSuccesses()
+                    .agg(s"coordinate $c")
+                    .get)
+            }.toSeq
+          ))
     }
     TestResultLines(coordinates.map(_._1.id).toSeq, configDimensions, metrics, succeeded ++ all)
       .print()
@@ -289,27 +305,34 @@ object Parameterized {
     case class CoordMark(iteration: Int, queryId: QueryId, coord: Coord)
 
     class Parser(coordIds: Seq[Int], metricNames: Seq[String])
-        extends TableRender.RowParser[TestResultLine] {
+      extends TableRender.RowParser[TestResultLine] {
       override def parse(rowAppender: RowAppender, line: TestResultLine): Unit = {
         val inc = rowAppender.incremental()
         inc.next().write(line.queryId)
         val coords = coordIds.map(id => line.coord(id))
         coords.foreach(coord => inc.next().write(coord.queryResult.succeeded()))
-        coords.foreach(coord =>
-          inc.next().write(coord.queryResult.asSuccessOption().map(_.runResult.rows.size)))
-        metricNames.foreach(metricName =>
-          coords.foreach(coord =>
+        coords.foreach(
+          coord => inc.next().write(coord.queryResult.asSuccessOption().map(_.runResult.rows.size)))
+        metricNames.foreach(
+          metricName =>
+            coords.foreach(
+              coord =>
+                inc
+                  .next()
+                  .write(
+                    coord.queryResult
+                      .asSuccessOption()
+                      .map(_.runResult.executorMetrics(metricName)))))
+        coords.foreach(
+          coord =>
             inc
               .next()
-              .write(coord.queryResult.asSuccessOption().map(_.runResult.executorMetrics(metricName)))))
-        coords.foreach(coord =>
-          inc
-            .next()
-            .write(coord.queryResult.asSuccessOption().map(_.runResult.planningTimeMillis)))
-        coords.foreach(coord =>
-          inc
-            .next()
-            .write(coord.queryResult.asSuccessOption().map(_.runResult.executionTimeMillis)))
+              .write(coord.queryResult.asSuccessOption().map(_.runResult.planningTimeMillis)))
+        coords.foreach(
+          coord =>
+            inc
+              .next()
+              .write(coord.queryResult.asSuccessOption().map(_.runResult.executionTimeMillis)))
       }
     }
   }
@@ -320,19 +343,19 @@ object Parameterized {
       metricNames: Seq[String],
       lines: Iterable[TestResultLine]) {
     def print(): Unit = {
-      val fields = ListBuffer[Field](Field.Leaf("Query ID"))
-      val coordFields = coordIds.map(id => Field.Leaf(id.toString))
-
-      fields.append(Field.Branch("Succeeded", coordFields))
-      fields.append(Field.Branch("Row Count", coordFields))
-      metricNames.foreach(metricName => fields.append(Field.Branch(metricName, coordFields)))
-      fields.append(Field.Branch("Planning Time (Millis)", coordFields))
-      fields.append(Field.Branch("Query Time (Millis)", coordFields))
-
+      val coordFields: Seq[Field] = coordIds.map(id => Field.Leaf(id.toString))
+      val fields: Seq[Field] =
+        Seq(Field.Leaf("Query ID")) ++
+          Seq(Field.Branch("Succeeded", coordFields), Field.Branch("Row Count", coordFields)) ++
+          metricNames.map(metricName => Field.Branch(metricName, coordFields)) ++
+          Seq(
+            Field.Branch("Planning Time (Millis)", coordFields),
+            Field.Branch("Query Time (Millis)", coordFields)
+          )
       val render =
         TableRender.create[TestResultLine](fields: _*)(
-          new TestResultLine.Parser(coordIds, metricNames))
-
+          new TestResultLine.Parser(coordIds, metricNames)
+        )
       lines.foreach(line => render.appendRow(line))
 
       render.print(System.out)
