@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.gluten.integration
 
 import org.apache.spark.sql.RunResult
@@ -33,15 +32,18 @@ package object action {
 
   implicit class CompletedOps(completed: Iterable[QueryRunner.Success]) {
     def agg(name: String): Option[QueryRunner.Success] = {
-      completed.reduceOption { (c1, c2) =>
-        QueryRunner.Success(
-          name,
-          RunResult(
-            c1.runResult.rows ++ c2.runResult.rows,
-            c1.runResult.planningTimeMillis + c2.runResult.planningTimeMillis,
-            c1.runResult.executionTimeMillis + c2.runResult.executionTimeMillis,
-            c1.runResult.sqlMetrics ++ c2.runResult.sqlMetrics,
-            (c1.runResult.executorMetrics, c2.runResult.executorMetrics).sumUp))
+      completed.reduceOption {
+        (c1, c2) =>
+          QueryRunner.Success(
+            name,
+            RunResult(
+              c1.runResult.rows ++ c2.runResult.rows,
+              c1.runResult.planningTimeMillis + c2.runResult.planningTimeMillis,
+              c1.runResult.executionTimeMillis + c2.runResult.executionTimeMillis,
+              c1.runResult.sqlMetrics ++ c2.runResult.sqlMetrics,
+              (c1.runResult.executorMetrics, c2.runResult.executorMetrics).sumUp
+            )
+          )
       }
     }
   }
