@@ -17,13 +17,14 @@
 package org.apache.gluten.integration.ds
 
 import org.apache.gluten.integration.{DataGen, ShimUtils, TypeModifier}
-import org.apache.spark.sql.{Column, Row, SaveMode, SparkSession}
 
+import org.apache.spark.sql.{Column, Row, SaveMode, SparkSession}
 import org.apache.spark.sql.types._
 
 import io.trino.tpcds._
 
 import java.io.File
+
 import scala.collection.JavaConverters._
 
 class TpcdsDataGen(
@@ -33,8 +34,8 @@ class TpcdsDataGen(
     dir: String,
     typeModifiers: List[TypeModifier] = List(),
     val genPartitionedData: Boolean)
-    extends Serializable
-    with DataGen {
+  extends Serializable
+  with DataGen {
 
   def writeParquetTable(t: Table): Unit = {
     val name = t.getName
@@ -97,23 +98,25 @@ class TpcdsDataGen(
     val tablePath = dir + File.separator + tableName
     spark
       .range(0, partitions, 1L, partitions)
-      .mapPartitions { itr =>
-        val id = itr.toArray
-        if (id.length != 1) {
-          throw new IllegalStateException()
-        }
-        val options = new Options()
-        options.scale = scale
-        options.parallelism = partitions
-        val session = options.toSession
-        val chunkSession = session.withChunkNumber(id(0).toInt + 1)
-        val results = Results.constructResults(t, chunkSession).asScala.toIterator
-        results.map { parentAndChildRow =>
-          // Skip child table when generating parent table,
-          // we generate every table individually no matter it is parent or child.
-          val array: Array[String] = parentAndChildRow.get(0).asScala.toArray
-          Row(array: _*)
-        }
+      .mapPartitions {
+        itr =>
+          val id = itr.toArray
+          if (id.length != 1) {
+            throw new IllegalStateException()
+          }
+          val options = new Options()
+          options.scale = scale
+          options.parallelism = partitions
+          val session = options.toSession
+          val chunkSession = session.withChunkNumber(id(0).toInt + 1)
+          val results = Results.constructResults(t, chunkSession).asScala.toIterator
+          results.map {
+            parentAndChildRow =>
+              // Skip child table when generating parent table,
+              // we generate every table individually no matter it is parent or child.
+              val array: Array[String] = parentAndChildRow.get(0).asScala.toArray
+              Row(array: _*)
+          }
       }(ShimUtils.getExpressionEncoder(stringSchema))
       .select(columns: _*)
       .write
@@ -166,7 +169,8 @@ object TpcdsDataGen {
         StructField("cs_net_paid_inc_tax", DecimalType(7, 2)),
         StructField("cs_net_paid_inc_ship", DecimalType(7, 2)),
         StructField("cs_net_paid_inc_ship_tax", DecimalType(7, 2)),
-        StructField("cs_net_profit", DecimalType(7, 2))))
+        StructField("cs_net_profit", DecimalType(7, 2))
+      ))
   }
 
   private def catalogReturnsSchema = {
@@ -198,7 +202,8 @@ object TpcdsDataGen {
         StructField("cr_refunded_cash", DecimalType(7, 2)),
         StructField("cr_reversed_charge", DecimalType(7, 2)),
         StructField("cr_store_credit", DecimalType(7, 2)),
-        StructField("cr_net_loss", DecimalType(7, 2))))
+        StructField("cr_net_loss", DecimalType(7, 2))
+      ))
   }
 
   private def inventorySchema = {
@@ -207,7 +212,8 @@ object TpcdsDataGen {
         StructField("inv_date_sk", LongType),
         StructField("inv_item_sk", LongType),
         StructField("inv_warehouse_sk", LongType),
-        StructField("inv_quantity_on_hand", LongType)))
+        StructField("inv_quantity_on_hand", LongType)
+      ))
   }
 
   private def storeSalesSchema = {
@@ -235,7 +241,8 @@ object TpcdsDataGen {
         StructField("ss_coupon_amt", DecimalType(7, 2)),
         StructField("ss_net_paid", DecimalType(7, 2)),
         StructField("ss_net_paid_inc_tax", DecimalType(7, 2)),
-        StructField("ss_net_profit", DecimalType(7, 2))))
+        StructField("ss_net_profit", DecimalType(7, 2))
+      ))
   }
 
   private def storeReturnsSchema = {
@@ -260,7 +267,8 @@ object TpcdsDataGen {
         StructField("sr_refunded_cash", DecimalType(7, 2)),
         StructField("sr_reversed_charge", DecimalType(7, 2)),
         StructField("sr_store_credit", DecimalType(7, 2)),
-        StructField("sr_net_loss", DecimalType(7, 2))))
+        StructField("sr_net_loss", DecimalType(7, 2))
+      ))
   }
 
   private def webSalesSchema = {
@@ -299,7 +307,8 @@ object TpcdsDataGen {
         StructField("ws_net_paid_inc_tax", DecimalType(7, 2)),
         StructField("ws_net_paid_inc_ship", DecimalType(7, 2)),
         StructField("ws_net_paid_inc_ship_tax", DecimalType(7, 2)),
-        StructField("ws_net_profit", DecimalType(7, 2))))
+        StructField("ws_net_profit", DecimalType(7, 2))
+      ))
   }
 
   private def webReturnsSchema = {
@@ -328,7 +337,8 @@ object TpcdsDataGen {
         StructField("wr_refunded_cash", DecimalType(7, 2)),
         StructField("wr_reversed_charge", DecimalType(7, 2)),
         StructField("wr_account_credit", DecimalType(7, 2)),
-        StructField("wr_net_loss", DecimalType(7, 2))))
+        StructField("wr_net_loss", DecimalType(7, 2))
+      ))
   }
 
   private def callCenterSchema = {
@@ -364,7 +374,8 @@ object TpcdsDataGen {
         StructField("cc_zip", StringType),
         StructField("cc_country", StringType),
         StructField("cc_gmt_offset", DecimalType(5, 2)),
-        StructField("cc_tax_percentage", DecimalType(5, 2))))
+        StructField("cc_tax_percentage", DecimalType(5, 2))
+      ))
   }
 
   private def catalogPageSchema = {
@@ -378,7 +389,8 @@ object TpcdsDataGen {
         StructField("cp_catalog_number", LongType),
         StructField("cp_catalog_page_number", LongType),
         StructField("cp_description", StringType),
-        StructField("cp_type", StringType)))
+        StructField("cp_type", StringType)
+      ))
   }
 
   private def customerSchema = {
@@ -401,7 +413,8 @@ object TpcdsDataGen {
         StructField("c_birth_country", StringType),
         StructField("c_login", StringType),
         StructField("c_email_address", StringType),
-        StructField("c_last_review_date", StringType)))
+        StructField("c_last_review_date", StringType)
+      ))
   }
 
   private def customerAddressSchema = {
@@ -419,7 +432,8 @@ object TpcdsDataGen {
         StructField("ca_zip", StringType),
         StructField("ca_country", StringType),
         StructField("ca_gmt_offset", DecimalType(5, 2)),
-        StructField("ca_location_type", StringType)))
+        StructField("ca_location_type", StringType)
+      ))
   }
 
   private def customerDemographicsSchema = {
@@ -433,7 +447,8 @@ object TpcdsDataGen {
         StructField("cd_credit_rating", StringType),
         StructField("cd_dep_count", LongType),
         StructField("cd_dep_employed_count", LongType),
-        StructField("cd_dep_college_count", LongType)))
+        StructField("cd_dep_college_count", LongType)
+      ))
   }
 
   private def dateDimSchema = {
@@ -466,7 +481,8 @@ object TpcdsDataGen {
         StructField("d_current_week", StringType),
         StructField("d_current_month", StringType),
         StructField("d_current_quarter", StringType),
-        StructField("d_current_year", StringType)))
+        StructField("d_current_year", StringType)
+      ))
   }
 
   private def householdDemographicsSchema = {
@@ -476,7 +492,8 @@ object TpcdsDataGen {
         StructField("hd_income_band_sk", LongType),
         StructField("hd_buy_potential", StringType),
         StructField("hd_dep_count", LongType),
-        StructField("hd_vehicle_count", LongType)))
+        StructField("hd_vehicle_count", LongType)
+      ))
   }
 
   private def incomeBandSchema = {
@@ -511,7 +528,8 @@ object TpcdsDataGen {
         StructField("i_units", StringType),
         StructField("i_container", StringType),
         StructField("i_manager_id", LongType),
-        StructField("i_product_name", StringType)))
+        StructField("i_product_name", StringType)
+      ))
   }
 
   private def promotionSchema = {
@@ -535,7 +553,8 @@ object TpcdsDataGen {
         StructField("p_channel_demo", StringType),
         StructField("p_channel_details", StringType),
         StructField("p_purpose", StringType),
-        StructField("p_discount_active", StringType)))
+        StructField("p_discount_active", StringType)
+      ))
   }
 
   private def reasonSchema = {
@@ -554,7 +573,8 @@ object TpcdsDataGen {
         StructField("sm_type", StringType),
         StructField("sm_code", StringType),
         StructField("sm_carrier", StringType),
-        StructField("sm_contract", StringType)))
+        StructField("sm_contract", StringType)
+      ))
   }
 
   private def storeSchema = {
@@ -588,7 +608,8 @@ object TpcdsDataGen {
         StructField("s_zip", StringType),
         StructField("s_country", StringType),
         StructField("s_gmt_offset", DecimalType(5, 2)),
-        StructField("s_tax_precentage", DecimalType(5, 2))))
+        StructField("s_tax_precentage", DecimalType(5, 2))
+      ))
   }
 
   private def timeDimSchema = {
@@ -603,7 +624,8 @@ object TpcdsDataGen {
         StructField("t_am_pm", StringType),
         StructField("t_shift", StringType),
         StructField("t_sub_shift", StringType),
-        StructField("t_meal_time", StringType)))
+        StructField("t_meal_time", StringType)
+      ))
   }
 
   private def warehouseSchema = {
@@ -622,7 +644,8 @@ object TpcdsDataGen {
         StructField("w_state", StringType),
         StructField("w_zip", StringType),
         StructField("w_country", StringType),
-        StructField("w_gmt_offset", DecimalType(5, 2))))
+        StructField("w_gmt_offset", DecimalType(5, 2))
+      ))
   }
 
   private def webPageSchema = {
@@ -641,7 +664,8 @@ object TpcdsDataGen {
         StructField("wp_char_count", LongType),
         StructField("wp_link_count", LongType),
         StructField("wp_image_count", LongType),
-        StructField("wp_max_ad_count", LongType)))
+        StructField("wp_max_ad_count", LongType)
+      ))
   }
 
   private def webSiteSchema = {
@@ -672,6 +696,7 @@ object TpcdsDataGen {
         StructField("web_zip", StringType),
         StructField("web_country", StringType),
         StructField("web_gmt_offset", StringType),
-        StructField("web_tax_percentage", DecimalType(5, 2))))
+        StructField("web_tax_percentage", DecimalType(5, 2))
+      ))
   }
 }
