@@ -16,7 +16,7 @@
  */
 package org.apache.gluten.integration.ds
 
-import org.apache.gluten.integration.{DataGen, Suite, TableCreator, TypeModifier}
+import org.apache.gluten.integration.{DataGen, Suite, TableCreator}
 import org.apache.gluten.integration.action.Action
 import org.apache.gluten.integration.ds.TpcdsSuite.{ALL_QUERY_IDS, HISTORY_WRITE_PATH, TPCDS_WRITE_RELATIVE_PATH}
 import org.apache.gluten.integration.metrics.MetricMapper
@@ -44,6 +44,7 @@ class TpcdsSuite(
     val disableWscg: Boolean,
     val shufflePartitions: Int,
     val scanPartitions: Int,
+    val decimalAsDouble: Boolean,
     val baselineMetricMapper: MetricMapper,
     val testMetricMapper: MetricMapper)
   extends Suite(
@@ -62,6 +63,7 @@ class TpcdsSuite(
     disableWscg,
     shufflePartitions,
     scanPartitions,
+    decimalAsDouble,
     baselineMetricMapper,
     testMetricMapper
   ) {
@@ -88,10 +90,6 @@ class TpcdsSuite(
     "/tpcds-queries"
   }
 
-  private def typeModifiers(): List[TypeModifier] = {
-    List()
-  }
-
   override private[integration] def allQueryIds(): Array[String] = ALL_QUERY_IDS
 
   override private[integration] def desc(): String = "TPC-DS"
