@@ -16,52 +16,57 @@
  */
 package org.apache.gluten.integration.h
 
+import org.apache.gluten.integration.{DataGen, Suite, TableCreator}
 import org.apache.gluten.integration.action.Action
 import org.apache.gluten.integration.h.TpchSuite.{HISTORY_WRITE_PATH, TPCH_WRITE_RELATIVE_PATH}
 import org.apache.gluten.integration.metrics.MetricMapper
-import org.apache.gluten.integration.{DataGen, Suite, TableCreator, TypeModifier}
-import org.apache.log4j.Level
+
 import org.apache.spark.SparkConf
 
+import org.apache.log4j.Level
+
 import java.io.File
 
 class TpchSuite(
-                 val masterUrl: String,
-                 val actions: Array[Action],
-                 val testConf: SparkConf,
-                 val baselineConf: SparkConf,
-                 val extraSparkConf: Map[String, String],
-                 val logLevel: Level,
-                 val errorOnMemLeak: Boolean,
-                 val dataDir: String,
-                 val enableUi: Boolean,
-                 val enableHsUi: Boolean,
-                 val hsUiPort: Int,
-                 val disableAqe: Boolean,
-                 val disableBhj: Boolean,
-                 val disableWscg: Boolean,
-                 val shufflePartitions: Int,
-                 val scanPartitions: Int,
-                 val baselineMetricMapper: MetricMapper,
-                 val testMetricMapper: MetricMapper)
-    extends Suite(
-      masterUrl,
-      actions,
-      testConf,
-      baselineConf,
-      extraSparkConf,
-      logLevel,
-      errorOnMemLeak,
-      enableUi,
-      enableHsUi,
-      hsUiPort,
-      disableAqe,
-      disableBhj,
-      disableWscg,
-      shufflePartitions,
-      scanPartitions,
-      baselineMetricMapper,
-      testMetricMapper) {
+    val masterUrl: String,
+    val actions: Array[Action],
+    val testConf: SparkConf,
+    val baselineConf: SparkConf,
+    val extraSparkConf: Map[String, String],
+    val logLevel: Level,
+    val errorOnMemLeak: Boolean,
+    val dataDir: String,
+    val enableUi: Boolean,
+    val enableHsUi: Boolean,
+    val hsUiPort: Int,
+    val disableAqe: Boolean,
+    val disableBhj: Boolean,
+    val disableWscg: Boolean,
+    val shufflePartitions: Int,
+    val scanPartitions: Int,
+    val decimalAsDouble: Boolean,
+    val baselineMetricMapper: MetricMapper,
+    val testMetricMapper: MetricMapper)
+  extends Suite(
+    masterUrl,
+    actions,
+    testConf,
+    baselineConf,
+    extraSparkConf,
+    logLevel,
+    errorOnMemLeak,
+    enableUi,
+    enableHsUi,
+    hsUiPort,
+    disableAqe,
+    disableBhj,
+    disableWscg,
+    shufflePartitions,
+    scanPartitions,
+    decimalAsDouble,
+    baselineMetricMapper,
+    testMetricMapper
+  ) {
 
   override protected def historyWritePath(): String = HISTORY_WRITE_PATH
 
@@ -84,10 +89,6 @@ class TpchSuite(
     "/tpch-queries"
   }
 
-  private def typeModifiers(): List[TypeModifier] = {
-    List()
-  }
-
   override private[integration] def allQueryIds(): Array[String] = TpchSuite.ALL_QUERY_IDS
 
   override private[integration] def desc(): String = "TPC-H"
