@@ -117,11 +117,7 @@ for (t <- sorted) {
   println(t)
   val fileContents = Source.fromFile(t).getLines.filter(!_.startsWith("--")).mkString(" ")
   println(fileContents)
-  try {
-    time{spark.sql(fileContents).show}
-    // spark.sql(fileContents).explain
-    Thread.sleep(2000)
-  } catch {
-    case e: Exception => None
-  }
+  time{spark.sql(fileContents).collectAsList()}
+  // spark.sql(fileContents).explain
+  Thread.sleep(2000)
 }
